@@ -10,11 +10,15 @@
 #include <ctype.h>
 #include "mail.h"
 
+#include "malloc.h"
+#include "libc.h"
+#include "libz.h"
+
 #define	REMOTE_FROM	"remote from "	/* standard magic phrase */
 
 #define	MAXHOPS		500	/* maximum number of >From lines */
 
-char *somewhere	= "uunet";	/* default remote host (for uunet et al) */
+const char *somewhere = "uunet"; /* default remote host (for uunet et al) */
 
 /*
  *  This program considers:
@@ -40,11 +44,12 @@ char *somewhere	= "uunet";	/* default remote host (for uunet et al) */
 #define PRINTABLE(a,b)	((a != NULL && *a != '\0') ? a : b)
 
 struct from_ {
-	char	*address;
-	char	*date;
-	char	*remotehost;
+	const char	*address;
+	const char	*date;
+	const char	*remotehost;
 };
 
+
 extern char *optarg;
 extern int optind;
 extern char *getenv();
@@ -54,7 +59,9 @@ extern char *strchr();
 
 int D_alloc = 0;	/* Memory usage debugging */
 
-char *progname;
+const char *progname;
+extern struct from_ *copyfrom_ __((struct from_ *));
+extern struct from_ *breakdown __((char *fromline, int len));
 
 int
 main(argc, argv)
@@ -62,14 +69,10 @@ main(argc, argv)
 	char *argv[];
 {
 	int n, flmax, i, debug, c, errflg;
-	char *cp, buf[BUFSIZ];
+	char buf[BUFSIZ];
+	const char *cp;
 	struct from_ *sc, *fl[MAXHOPS];
 	FILE *mfp;
-	extern unsigned char *linebuf;
-	extern int getline();
-	extern int runastrusteduser(), getrest();
-	extern void initline(), prversion();
-	extern struct from_ *breakdown(), *copyfrom_();
 
 	progname = argv[0];
 	debug = 0;
@@ -102,13 +105,13 @@ main(argc, argv)
 	flmax = 0;
 	while ((n = getline(stdin)) > 0) {
 		if (((*linebuf == '>' && (++linebuf, --n)) || 1)
-		    && strncmp("From ", (char *)linebuf, 5) == 0) {
+		    && strncmp("From ", linebuf, 5) == 0) {
 			if (debug) {
 				printf("Found From_ line: '");
-				fwrite((char *)linebuf, 1, n-1, stdout);
+				fwrite(linebuf, 1, n-1, stdout);
 				printf("'\n");
 			}
-			if ((sc = breakdown((char *)linebuf, n-1)) == NULL)
+			if ((sc = breakdown(linebuf, n-1)) == NULL)
 				exit(1);	/* message printed below */
 			if (flmax >= MAXHOPS) {
 				fprintf(stderr,"%s: too many hops\n",progname);
@@ -148,7 +151,7 @@ main(argc, argv)
 		fprintf(mfp, "todsn ORCPT=rfc822;");
 		s = argv[optind];
 		while (*s) {
-		  u_char c = *s;
+		  int c = (*s) & 0xFF;
 		  if ('!' <= c && c <= '~' && c != '+' && c != '=')
 		    putc(c,mfp);
 		  else
@@ -194,7 +197,8 @@ main(argc, argv)
 			}
 		}
 
-		if ((cp = fl[--i]->address) == NULL) {	/* egad! */
+		cp = fl[--i]->address;
+		if (cp == NULL) {	/* egad! */
 			fprintf(stderr,"%s: malformed From_ line\n", progname);
 			if (!debug) mail_abort(mfp);
 			exit(1);
@@ -212,7 +216,8 @@ main(argc, argv)
 
 	fwrite((char *)linebuf, 1, n, mfp);
 
-	if ((n = getrest()) > 0)
+	n = linegetrest();
+	if (n > 0)
 		fwrite((char *)linebuf, 1, n, mfp);
 	while ((n = fread(buf, 1, sizeof buf, stdin)) > 0)
 		fwrite(buf, 1, n, mfp);
@@ -299,21 +304,22 @@ copyfrom_(fp)
 	struct from_ *fp;
 {
 	struct from_ *nfp;
-	extern char *emalloc();
 
 	nfp = (struct from_ *)emalloc(sizeof (struct from_));
 	if ((nfp->address = fp->address) != NULL) {
-		nfp->address = emalloc((unsigned int) strlen(fp->address)+1);
-		strcpy(nfp->address, fp->address);
+	  char *wcp = emalloc((unsigned int) strlen(fp->address)+1);
+	  strcpy(wcp, fp->address);
+	  nfp->address = wcp;
 	}
 	if ((nfp->date = fp->date) != NULL) {
-		nfp->date = emalloc((unsigned int) strlen(fp->date)+1);
-		strcpy(nfp->date, fp->date);
+	  char *wcpd = emalloc((unsigned int) strlen(fp->date)+1);
+	  strcpy(wcpd, fp->date);
+	  nfp->date = wcpd;
 	}
 	if ((nfp->remotehost = fp->remotehost) != NULL) {
-		nfp->remotehost =
-			emalloc((unsigned int) strlen(fp->remotehost)+1);
-		strcpy(nfp->remotehost, fp->remotehost);
+	  char *wcp = emalloc((unsigned int) strlen(fp->remotehost)+1);
+	  strcpy(wcp, fp->remotehost);
+	  nfp->remotehost = wcp;
 	}
 	return nfp;
 }
