@@ -12,6 +12,7 @@
 #include <stdio.h>
 #include <sysexits.h>
 #include <ctype.h>
+#include <string.h>
 #include <pwd.h>
 #include "mail.h"
 #include "zmsignal.h"
@@ -21,34 +22,37 @@
 #include <sys/file.h>
 #include "zsyslog.h"
 
+#include "malloc.h"
+#include "libz.h"
+#include "libc.h"
+
+
 #define	SMTPSERVER	"smtpserver"
 #define	ROUTER		"router"
 #define	SCHEDULER	"scheduler"
 #define MAILQ		"mailq"
 #define	NEWALIASES	"newaliases"
 
-char *zmailer = "ZMailer";
+const char *zmailer = "ZMailer";
 char *verbfile = NULL;
 
-extern FILE *deadletter(), *fopen(), *fdopen();
-extern char *emalloc();
+extern FILE * deadletter __((int uid));
 extern char *optarg;
 #ifndef strchr
 extern char *strchr();
 extern char *strrchr();
 #endif
 extern char *strtok();
-extern char *getenv();
-extern char *getlogin();
-extern int errno;
-extern int optind;
-extern int runastrusteduser(), runasrootuser();
-extern void prversion(), doabort();
-extern int mail_priority;
-extern void check_and_print_to __((FILE *, char*, char*));
-extern int is_xtext_string __((char *));
+/* extern char *getenv();
+   extern char *getlogin(); */
+extern int  errno;
+extern int  optind;
+extern void doabort __((int));
+extern int  mail_priority;
+extern void check_and_print_to __((FILE *, const char*, const char*));
+extern int  is_xtext_string __((const char *));
 
-char *progname;
+const char *progname;
 
 int D_alloc = 0;	/* memory usage debugging */
 
@@ -58,14 +62,14 @@ int D_alloc = 0;	/* memory usage debugging */
 #endif
 
 FILE	*mfp = NULL;
-extern char *postoffice; /* At mail_open() */
-
 
 #define RFC821_822QUOTE(newcp,cp) \
 	if (cp && strchr(cp,'\\') != NULL && *cp != '"') {	\
-	  char *s1 = cp, *s2;					\
+	  const char *s1 = cp;					\
+	  char *s2;						\
 	  /* For this we can add at most 2 new quote chars */	\
-	  s2 = newcp = emalloc(strlen(cp)+4);			\
+	  s2 = emalloc(strlen(cp)+4);				\
+	  newcp = s2;						\
 	  *s2++ = '"';						\
 	  while (*s1) {						\
 	    if (*s1 == '@')					\
@@ -81,17 +85,20 @@ extern char *postoffice; /* At mail_open() */
 	  cp = newcp;						\
 	}
 
+extern int main __((int argc, const char *argv[]));
 int
 main(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int	c, errflg, n, pid, bpflag, uid, truid, external, verbose;
 	int	speaksmtp, daemon_flg, interactive, aliases, printq, dotiseof;
 	int	binary, vfd;
 	char	*ebp, *s, *cp, errbuf[BUFSIZ];
-	char	*from, *fullname, *mailbin, *mailshare, *mav[10];
-	char	*configfile, *pooption, *path, *mailpriority;
+	const char *from, *mailbin, *mailshare, *fullname;
+	const char *mav[10];
+	const char *configfile, *mailpriority, *pooption;
+	char * path;
 	FILE	*vfp;
 	struct passwd *pwd;
 	char	*LC_ctype;
@@ -103,16 +110,18 @@ main(argc, argv)
 	char	*notificationopt = NULL;
 	char	*returnopt = NULL;
 	char	*envidstr = NULL;
-	char	*newcp = NULL;
+	const char * newcp = NULL;
 
-	if ((cp = strrchr(argv[0], '/')) != NULL)
+	cp = strrchr(argv[0], '/');
+	if (cp != NULL)
 		progname = ++cp;
 	else
 		progname = argv[0];
 	umask(022);
 
 	mailpriority = getenv("MAILPRIORITY");
-	if (mailpriority) mail_priority = atoi(mailpriority);
+	if (mailpriority)
+	  mail_priority = atoi(mailpriority);
 
 	/* Pick up the sender's idea about C-chartype.. */
 	LC_ctype = getenv("LC_CTYPE");
@@ -129,7 +138,11 @@ main(argc, argv)
 	configfile = pooption = NULL;
 	if (printq || aliases)
 		goto otherprog;
-	while ((c = getopt(argc, argv,"B:C:EF:JN:OP:R:UV:b:d:e:r:f:h:imno:p:q:stvx")) != EOF) {
+	while (1) {
+		c = getopt(argc, (char*const*)argv,
+			   "B:C:EF:JN:OP:R:UV:b:d:e:r:f:h:imno:p:q:stvx");
+		if (c == EOF)
+			break;
 		switch (c) {
 		case 'B':
 			/* Sendmail 8.7 compability:
@@ -191,7 +204,7 @@ main(argc, argv)
 			break;
 		case 'C':	/* specify configuration file */
 			configfile = emalloc((unsigned)(2+strlen(optarg)+1));
-			sprintf(configfile, "-f%s", optarg);
+			sprintf((char*)configfile, "-f%s", optarg);
 			break;
 		case 'o':	/* options */
 			switch (*optarg) {
@@ -199,7 +212,7 @@ main(argc, argv)
 				if (getuid() != geteuid() && getuid() != 0)
 					break;
 				pooption = emalloc((unsigned)(2+strlen(optarg+1)+1));
-				sprintf(pooption, "-P%s", optarg+1);
+				sprintf((char*)pooption, "-P%s", optarg+1);
 				postoffice = pooption + 2 /* "-P" */;
 				break;
 			case 'd':	/* -odb -- ignore quietly */
@@ -233,15 +246,15 @@ main(argc, argv)
 		case 'N':
 			notificationopt = optarg;
 			{
-			  char *s = optarg;
+			  s = optarg;
 			  while (*s) {
-			    if (strncasecmp(s,"never",5)==0)
+			    if (cistrncmp(s,"never",5)==0)
 			      s += 5;
-			    else if (strncasecmp(s,"success",7)==0)
+			    else if (cistrncmp(s,"success",7)==0)
 			      s += 7;
-			    else if (strncasecmp(s,"failure",7)==0)
+			    else if (cistrncmp(s,"failure",7)==0)
 			      s += 7;
-			    else if (strncasecmp(s,"delay",5)==0)
+			    else if (cistrncmp(s,"delay",5)==0)
 			      s += 5;
 			    if (*s == ',') {
 			      ++s;
@@ -256,8 +269,8 @@ main(argc, argv)
 			break;
 		case 'R':
 			returnopt = optarg;
-			if (strcasecmp(returnopt,"full") != 0 &&
-			    strcasecmp(returnopt,"hdrs") != 0) {
+			if (cistrcmp(returnopt,"full") != 0 &&
+			    cistrcmp(returnopt,"hdrs") != 0) {
 			  fprintf(stderr,"sendmail: illegal -R -option parameter: '%s'\n",optarg);
 			  exit(EX_USAGE);
 			}
@@ -323,21 +336,21 @@ otherprog:
 		mailshare = MAILSHARE;
 	path = NULL;
 	if (speaksmtp) {
-		char *argv[30+1];
+		const char *av[30+1];
 		path = emalloc((unsigned)(strlen(mailbin)+1+strlen(SMTPSERVER)+1));
 		sprintf(path, "%s/%s", mailbin, SMTPSERVER);
-		argv[0] = "smtp-in";
-		argv[1] = "-i";
+		av[0] = "smtp-in";
+		av[1] = "-i";
 		n = 2;
 		cp = getzenv("SMTPOPTIONS"); /* Normal smtp-server options */
 		/* pass on suidness */
 		s = strtok(cp, " \t\"'");
 		do {
-		  argv[n++] = s;
+		  av[n++] = s;
 		  s = strtok(NULL, " \t\"'");
 		} while (s != NULL && n < 30);
-		argv[n] = NULL;
-		execv(path, argv);
+		av[n] = NULL;
+		execv(path, (char*const*)av);
 		perror(path);
 	} else if (daemon_flg) {
 		setuid(getuid());
@@ -347,13 +360,13 @@ otherprog:
 		}
 		if ((pid = fork()) > 0) {
 			mav[1] = "-dk";
-			execv(ROUTER, mav);
+			execv(ROUTER, (char*const*)mav);
 			perror(ROUTER);
 			exit(EX_UNAVAILABLE);
 		} else if (pid == 0) {
 			mav[0] = "scheduler";
 			mav[1] = "-d";
-			execv(SCHEDULER, mav);
+			execv(SCHEDULER, (char*const*)mav);
 			perror(SCHEDULER);
 			exit(EX_UNAVAILABLE);
 		}
@@ -364,17 +377,17 @@ otherprog:
 		mav[1] = "-i";
 		path = emalloc((unsigned)(strlen(mailbin)+1+strlen(ROUTER)+1));
 		sprintf(path, "%s/%s", mailbin, ROUTER);
-		execv(path, mav);
+		execv(path, (char*const*)mav);
 		perror(path);
 		exit(EX_UNAVAILABLE);
 	} else if (aliases) {
 		setuid(getuid());
 		mav[1] = emalloc((unsigned)(2+strlen(mailbin)+1
 					     +strlen(NEWALIASES)+1));
-		sprintf(mav[1], "-f%s/%s", mailbin, NEWALIASES);
+		sprintf((char*)mav[1], "-f%s/%s", mailbin, NEWALIASES);
 		path = emalloc((unsigned)(strlen(mailbin)+1+strlen(ROUTER)+1));
 		sprintf(path, "%s/%s", mailbin, ROUTER);
-		execv(path, mav);
+		execv(path, (char*const*)mav);
 		perror(path);
 		exit(EX_UNAVAILABLE);
 	} else if (printq) {
@@ -384,11 +397,11 @@ otherprog:
 		argv[0] = MAILQ;
 		if (bpflag)
 			argv[1] = NULL;
-		execv(path, argv);
+		execv(path, (char*const*)argv);
 		perror(path);
 		exit(EX_UNAVAILABLE);
 	} else {	/* ahh, normal, at last! */
-		RETSIGTYPE (*oldsig)();
+		RETSIGTYPE (*oldsig) __((int));
 		SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 		if (oldsig != SIG_IGN)
 			SIGNAL_HANDLE(SIGINT, doabort);
@@ -487,7 +500,7 @@ otherprog:
 		if (from != NULL && *from != '\0')
 		  fprintf(mfp, "from %s\n", from);
 
-		if (newcp) free(newcp); newcp = NULL;
+		if (newcp) free((void*)newcp); newcp = NULL;
 
 		if (envidstr)
 		  fprintf(mfp, "envid %s\n", envidstr);
@@ -638,7 +651,7 @@ otherprog:
 		/* NOTREACHED */
 	}
 	if (path != NULL)
-	  free(path);
+	  free((void*)path);
 	exit(EX_UNAVAILABLE);
 	/* NOTREACHED */
 	return 0;
@@ -669,7 +682,8 @@ deadletter(uid)
 }
 
 void
-doabort()
+doabort(dummy)
+int dummy;
 {
 	if (mfp != NULL)
 		mail_abort(mfp);
@@ -685,10 +699,11 @@ doabort()
 void
 check_and_print_to(mfp, addr, notify)
 	FILE *mfp;
-	char *addr, *notify;
+	const char *addr, *notify;
 {
-	char *copy = NULL, *printme = addr, *from, *to;
-	char *s, *newcp = NULL;
+	const char *copy = NULL, *printme = addr, *from;
+	const char *s, *newcp = NULL;
+	char *to;
 
 	if (addr == NULL) {
 	  fprintf(stderr, "sendmail: Argument botch: NULL address\n");
@@ -696,10 +711,10 @@ check_and_print_to(mfp, addr, notify)
 	}
 	if (strchr(addr,'\n')) {
 	  fprintf(stderr, "sendmail: LF in  to<SPC> address: %s\n",addr);
-	  copy = emalloc(strlen(addr)+1); /* +1 just to be sure */
-	  if (copy == NULL) return;	  /* should never happen... */
+	  to = emalloc(strlen(addr)+1); /* +1 just to be sure */
+	  if (to == NULL) return;	  /* should never happen... */
 
-	  to = copy;
+	  copy = to;
 	  for (from = addr; *from; ++from)
 	    if (*from != '\n') *to++ = *from;
 	  *to = 0;
@@ -724,8 +739,8 @@ check_and_print_to(mfp, addr, notify)
 	  fprintf(mfp," NOTIFY=%s", notify);
 	putc('\n',mfp);
 	fprintf(mfp, "to %s\n", printme);
-	if (copy) free(copy);
-	if (newcp) free(newcp);
+	if (copy) free((void*)copy);
+	if (newcp) free((void*)newcp);
 }
 
 #if 0
@@ -738,7 +753,7 @@ tmalloc(n)
 #endif
 
 int is_xtext_string(str)
-char *str;
+const char *str;
 {
 	/* Verify that the input is valid RFC 1981 XTEXT string! */
 
