@@ -15,6 +15,5 @@ struct dnsresult {
 /* dnsgetrr.c */
 extern int	getrr     __((char *, int *, int, int, int, FILE *));
 extern int	getrrtype __((char *, int *, int, int, int, FILE *));
-extern struct hostent *gethostbyname2_r __((const char *, int, struct dnsresult *));
-extern struct hostent *gethostbyaddr_r __((const char *, int, int, struct dnsresult *));
-
+extern struct hostent *gethostbyname2_rz __((const char *, int, struct dnsresult *));
+extern struct hostent *gethostbyaddr_rz __((const char *, int, int, struct dnsresult *));
