@@ -1,9 +1,9 @@
 #ifndef	__HOSTENV__
 #define	__HOSTENV__
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+/* #ifdef HAVE_CONFIG_H */
+#include "config.h" /* We have this always.. */
+/* #endif */
 
 #ifdef HAVE_FCNTL_H
 # define FCNTL_H <fcntl.h>
