@@ -11,30 +11,32 @@
 #define	_LISTUTILS_H
 
 /*
- * A LIST is a struct conscell with flags 0, the list is kept under dtpr.
- * A STRING is a struct conscell with flags !0, the string is kept under string.
- * The value of a LIST is a linked list of (struct conscell) of either type.
+ * A LIST is a conscell with flags 0, the list is kept under dtpr.
+ * A STRING is a conscell with flags !0, the string is kept under string.
+ * The value of a LIST is a linked list of (conscell) of either type.
  * The value of a C variable must not be assumed to be either a LIST of STRING.
  * NIL is a LIST with null dtpr.
  */
 
-struct conscell {
-	struct conscell *next;
+typedef struct _conscell {
+	struct _conscell *next;
 	union {
-		struct conscell *u_dtpr;
-		unsigned char	*u_string;
+		struct _conscell *u_dtpr;
+		char		 *u_string;
+		const char	 *cu_string;
 	} u;
 #ifdef __alpha	/* Alpha doesn't take nicely stores into too small objects.
 		   The hardware has store support only for 32/64/128-bit
-		   objects. */
+		   objects. (Well, newer EV5/EV56/EV6 processors have byte
+		   and short store/read operations too..) */
 	int		flags;	/* if 0 is a list, otherwise a string */
 	int		pflags;	/* if 1 x->prev->cdr == x */
 #else
 	short		flags;	/* if 0 is a list, otherwise a string */
 	short		pflags;	/* if 1 x->prev->cdr == x */
 #endif
-	struct conscell *prev;	/* points to whichever dtpr points to here */
-};
+	struct _conscell *prev;	/* points to whichever dtpr points to here */
+} conscell;
 
 #define	NEWSTRING	0001	/* newly allocated string (free() when GC) */
 #define	CONSTSTRING	0002	/* constant, mutually exclusive with above */
@@ -54,6 +56,7 @@ struct conscell {
 
 #define	dtpr		u.u_dtpr
 #define	string		u.u_string
+#define	cstring		u.cu_string
 
 
 #define	car(X)		(X)->dtpr
@@ -75,7 +78,11 @@ struct conscell {
 
 /* YOU MUST BE VERY CAREFUL ABOUT CALLING MACROS FROM WITHIN MACROS!!! */
 /* BE VERY CAREFUL ABOUT USING EXPRESSIONS WITH SIDEEFFECTS INSIDE MACRO ARGS */
-/* #define newcell()	(struct conscell *)tmalloc(sizeof (struct conscell)) */
+/* #define newcell()	(conscell *)tmalloc(sizeof (conscell)) */
+
+#ifndef newcell
+extern conscell * newcell __((void));
+#endif
 #define copycell(X)	(tmp = newcell(), *tmp = *(X), tmp)
 /* nconc(list, list) -> old (,@list ,@list) */
 #define nconc(X,Y)	(car(X) != 0 ? cdr(s_last(car(X))) = (Y) \
@@ -89,40 +96,27 @@ struct conscell {
 #define s_push(X,Y)	(cdr(X) = car(Y), car(Y) = (X), (Y))
 #define newstring(X)	(tmp = newcell(), tmp->string = (X), \
     tmp->flags = NEWSTRING, cdr(tmp) = 0, tmp->prev = 0, tmp->pflags = 0, tmp)
-#define conststring(X)	(tmp = newcell(), tmp->string = (X), \
+#define conststring(X)	(tmp = newcell(), tmp->cstring = (X), \
     tmp->flags = CONSTSTRING, cdr(tmp) = 0, tmp->prev = 0, tmp->pflags = 0, tmp)
 #define NIL		ncons(0)
 
-
-/* variables.c */
-extern struct conscell *v_find(), *v_expand();
-extern struct conscell *envarlist;
-
-/* sh.c */
-extern struct conscell *commandline;
-
 /* listutils.c */
-extern struct conscell *s_last(), *s_nth();
-extern struct conscell *s_read(), *s_catstring(), *s_listify();
-extern void s_grind(), s_set_prev();
-extern struct conscell *s_pushstack(), *s_popstack();
 #ifdef	MALLOC_TRACE
 #ifndef	s_copy_tree
-extern struct conscell *__s_copy_tree();
-extern void __s_free_tree();
+extern conscell *__s_copy_tree __((conscell *, const char *, const char *));
+extern void __s_free_tree __((conscell *, const char *, const char *));
 #define s_copy_tree(l)		__s_copy_tree((l), __FILE__, __LINE__)
 #define s_free_tree(l)		__s_free_tree((l), __FILE__, __LINE__)
 #endif	/* !s_copy_tree */
 #else	/* !MALLOC_TRACE */
-extern struct conscell *s_copy_tree();
-extern void s_free_tree();
+extern conscell *s_copy_tree __((conscell *));
+extern void      s_free_tree __((conscell *));
 #endif	/* MALLOC_TRACE */
 
 /* LISPic memory allocator, and other stuff.. */
-extern struct conscell * newcell();
-extern int cons_garbage_collect();
-extern int consvar_register(/* const struct conscell* */);
-extern void *consvar_mark();
-extern void consvar_release(/* void* */);
+extern int cons_garbage_collect __((void));
+extern int consvar_register __(( const conscell* ));
+extern void *consvar_mark __(( void * ));
+extern void consvar_release __(( void* ));
 
 #endif	/* _LISTUTILS_H */
