@@ -54,14 +54,15 @@
 #define	DOLLAR '\007'		/* What $ in the config file translates to */
 #define	AMPERSAND '\010'	/* What & in the config file translates to */
 
-#define	CROSSBAR	((u_char *)"crossbar")
-#define ROUTER		((u_char *)"router")
-#define SERVER		((u_char *)"server")
+#define	CROSSBAR	"crossbar"
+#define ROUTER		"router"
+#define SERVER		"server"
+#define FREEZER		"freezer"
 
 #define	DEFER		"defer"
 #define	DEFER_IO_ERROR	"IO:error"
 
-#define DEFERHDR	((u_char *)"header_defer")
+#define DEFERHDR	"header_defer"
 
 #define DBLOOKUPNAME	"%dblookup"
 
@@ -87,13 +88,13 @@
 #define	FYI_NOSENDER		4
 
 
-extern u_char	*linebuf;
+extern char	*linebuf;
 
-#define	CISTREQ(x, y)		(cistrcmp((char *)(x), (char *)(y)) == 0)
-#define	CISTREQN(x, y, n)	(cistrncmp((char *)(x), (char *)(y), n) == 0)
-#define	STREQ(x, y)		(strcmp((char *)(x), (char *)(y)) == 0)
-#define	STREQN(x, y, n)		(strncmp((char *)(x), (char *)(y), n) == 0)
-#define TOKENLEN(t) ((t)->t_len > 0 ? (int) (t)->t_len : strlen((char *)(t)->t_pname))
+#define	CISTREQ(x, y)		(cistrcmp((const char *)(x), (const char *)(y)) == 0)
+#define	CISTREQN(x, y, n)	(cistrncmp((const char *)(x), (const char *)(y), n) == 0)
+#define	STREQ(x, y)		(strcmp((const char *)(x), (const char *)(y)) == 0)
+#define	STREQN(x, y, n)		(strncmp((const char *)(x), (const char *)(y), n) == 0)
+#define TOKENLEN(t) ((t)->t_len > 0 ? (int) (t)->t_len : strlen((const char *)(t)->t_pname))
 
 #if	defined(lint) && defined(putc)
 #undef	putc
@@ -108,19 +109,19 @@ typedef enum {
 
 struct addr {
 	AddrComponent	p_type;
-	struct token	*p_tokens;
+	token822	*p_tokens;
 	struct addr	*p_next;
 };
 
 typedef enum { newAddress, BadAddress } AddressStamp;
 
 struct notary {
-	char	*envid;
-	char	*dsn;
+	const char	*envid;
+	const char	*dsn;
 };
 
 struct address {
-	u_char		*a_pname;		/* printable representation */
+	const char	*a_pname;		/* printable representation */
 	struct addr	*a_tokens;		/* RFC822 token list */
 	AddressStamp	a_stamp;		/* what this is */
 	/* XX: this should be changed to a list of tags */
@@ -134,17 +135,17 @@ struct address {
 struct received {
 	struct address	*r_from;	
 	struct address	*r_by;
-	struct token	*r_via;
-	struct token	*r_with;
+	token822	*r_via;
+	token822	*r_with;
 	struct address	*r_id;
 	struct address	*r_for;
-	struct token	*r_convert;
+	token822	*r_convert;
 	time_t		r_time;
 };
 
 union misc {					/* what a header can be */
 	struct address	*a;
-	struct token	*t;
+	token822	*t;
 	struct received	*r;
 	time_t		d; 
 };
@@ -176,11 +177,12 @@ typedef enum {	normal,		/* nil */
 		eVia,		/* received via <medium> */
 		eWith,		/* received with <protocol> */
 		eBodytype,	/* The user has given some sendmail -B -option */
+		eComment,	/* just ignorable comments */
 		eEnvEnd		/* Last of the envelope entries "env-end" */
 } HeaderClass;
 
 struct headerinfo {
-	char		*hdr_name;
+	const char	*hdr_name;
 	HeaderSemantics	semantics;	/* information on header type */
 	HeaderUserType	user_type;	/* sender / recipient address */
 	HeaderClass	class;		/* resent- or envelope header */
@@ -192,19 +194,19 @@ typedef enum {
 } HeaderStamp;
 
 struct header {
-	char		*h_pname;	/* printable representation */
+	const char	*h_pname;	/* printable representation */
 	union misc	h_contents;	/* contents depending on type */
 	struct headerinfo *h_descriptor; /* characteristics of header */
-	struct token	*h_lines;	/* original header lines */
+	token822	*h_lines;	/* original header lines */
 	HeaderStamp	h_stamp;	/* what this is */
 	struct header	*h_next;
 };
 
 #if 0
 struct triple {
-	u_char		*channel;	/* channel message arrived on */
-	u_char		*host;		/* host the mesg arrived from */
-	u_char		*user;		/* user address wrt that host */
+	const char	*channel;	/* channel message arrived on */
+	const char	*host;		/* host the mesg arrived from */
+	const char	*user;		/* user address wrt that host */
 };
 
 struct msgident {
@@ -224,17 +226,17 @@ struct envelope {
 	time_t		e_nowtime;	/* time message processed	*/
 	int		e_resent;	/* flag indicating resent msg	*/
 	int		e_trusted;	/* is local agent trusted?	*/
-	struct conscell	*e_from_trusted;  /* given (channel,host,user)	*/
-	struct conscell	*e_from_resolved; /* found (channel,host,user)	*/
-	char		*e_file;	/* message file name		*/
+	conscell	*e_from_trusted;  /* given (channel,host,user)	*/
+	conscell	*e_from_resolved; /* found (channel,host,user)	*/
+	const char	*e_file;	/* message file name		*/
 	FILE		*e_fp;		/* message file pointer		*/
-	char		*e_messageid;	/* message id string		*/
+	const char	*e_messageid;	/* message id string		*/
 };
 
 #ifdef	RFC976
-#define HDR_SCANNER(x)	scan822(&(x), strlen((char *)x), (u_char)'!', (u_char)'%', 0, (struct token **)0)
+#define HDR_SCANNER(x)	scan822(&(x), strlen((const char *)x), '!', '%', 0, (token822 **)0)
 #else	/* !RFC976 */
-#define HDR_SCANNER(x)	scan822(&(x), strlen((char *)x), (u_char)0, (u_char)0, 0, (struct token **)0)
+#define HDR_SCANNER(x)	scan822(&(x), strlen((const char *)x), 0, 0, 0, (token822 **)0)
 #endif	/* RFC976 */
 
 /* #include "libz.h" */
@@ -245,23 +247,10 @@ struct envelope {
 #define	getgrnam	zgetgrnam
 #endif	/* USE_GETPWNAM */
 
-extern struct token * scan822 __((u_char **cpp, long n, int c1, int c2,
-				  int allowcomments, struct token **tlistp));
-extern struct token	*makeToken(), *copyToken();
-extern struct header	*makeHeader();
-extern struct header	*copySender(), *copyRecipient();
-extern union misc	hdr_scanparse();
-extern u_char		*hash_get(), *tbl_find(), *hash_first(), *hash_next();
-extern u_char		**hash_getp();
-extern u_int		hash_make(), tbl_loc();
-extern char		*pname(), *formatToken(), *formatAddr();
-extern int		hash_put(), blockmem();
-extern void		push(), pop(), squirrel(), killrouter(), die();
-
 /* lib/allocate.c: */
 extern void      tfree __((const memtypes memtype));
 extern univptr_t getlevel __((const memtypes memtype));
-extern void      setlevel __((const memtypes memtype, univptr_t up));
+extern void      setlevel __((const memtypes memtype, const univptr_t up));
 extern char *    strsave __((const char *s));
 extern char *    strnsave __((const char *s, const size_t n));
 extern memtypes	 stickymem;
@@ -274,24 +263,26 @@ extern univptr_t smalloc __((const memtypes memtype, const size_t n));
  * having them all over the place.
  */
 
+#if 0
 /* list of hash table id's and the corresponding (log 2 of) table sizes */
-
 struct htbl_init {
 	u_int	*idp;
 	u_int	log2size;
 };
+#endif
 
 /* initialization of builtin functions */
 
+#if 0
 typedef enum { internal, interpreted, program, retvalue } funcType;
-
 struct func_init {
-	char		*fnname;
-	char		*(*fnaddr)();
+	const char	*fnname;
+	const char	*(*fnaddr)();
 	funcType	fntype;
 };
+#endif
 
 struct sptree_init {
 	struct sptree	**spta;			/* address of sptree * */
-	char		*incore_name;		/* "name" of incore database */
+	const char	*incore_name;		/* "name" of incore database */
 };
