@@ -58,18 +58,18 @@ extern void ___cfree __proto((univptr_t, const char *, int));
 
 #else /* MALLOC_TRACE */
 
-extern univptr_t malloc __proto((size_t));
-extern univptr_t calloc __proto((size_t, size_t));
-extern univptr_t realloc __proto((univptr_t, size_t));
-extern univptr_t valloc __proto((size_t));
+extern univptr_t malloc   __proto((size_t));
+extern univptr_t calloc   __proto((size_t, size_t));
+extern univptr_t realloc  __proto((univptr_t, size_t));
+extern univptr_t valloc   __proto((size_t));
 extern univptr_t memalign __proto((size_t, size_t));
-extern univptr_t emalloc __proto((size_t));
-extern univptr_t ecalloc __proto((size_t, size_t));
+extern univptr_t emalloc  __proto((size_t));
+extern univptr_t ecalloc  __proto((size_t, size_t));
 extern univptr_t erealloc __proto((univptr_t, size_t));
-extern char *strdup __proto((const char *));
-extern char *strsave __proto((const char *));
-extern char *strnsave __proto((const char *, const size_t));
-extern void free __proto((univptr_t));
+extern char * strdup   __proto((const char *));
+extern char * strsave  __proto((const char *));
+extern char * strnsave __proto((const char *, const size_t));
+extern void free  __proto((univptr_t));
 extern void cfree __proto((univptr_t));
 
 #endif /* MALLOC_TRACE */
@@ -80,7 +80,7 @@ extern void mal_heapdump __proto((FILE *));
 extern void mal_leaktrace __proto((int));
 extern void mal_sbrkset __proto((int));
 extern void mal_slopset __proto((int));
-extern void mal_statsdump __proto(());
+extern void mal_statsdump __proto((FILE*));
 extern void mal_setstatsfile __proto((FILE *));
 extern void mal_trace __proto((int));
 extern int  mal_verify __proto((int));
