@@ -12,11 +12,11 @@
 
 struct address {
 	struct address	*link;		/* next sender / sender for this rcpt */
-	char		*channel;
-	char		*host;
-	char		*user;
-	char		*misc;		/* expected to be uid privilege */
-	char		**routermxes;	/* [mea] hostpointers from router */
+	const char	*channel;
+	const char	*host;
+	const char	*user;
+	const char	*misc;		/* expected to be uid privilege */
+	const char	**routermxes;	/* [mea] hostpointers from router */
 };
 
 #define _DSN_NOTIFY_SUCCESS	0x001
@@ -39,8 +39,8 @@ struct address {
 struct rcpt {
 	struct rcpt	*next;
 	struct address	*addr;		/* addr.link is the sender address */
-	char		*orcpt;		/* DSN ORCPT= string */
-	char		*notify;	/* DSN NOTIFY= flags */
+	const char	*orcpt;		/* DSN ORCPT= string */
+	const char	*notify;	/* DSN NOTIFY= flags */
 	int		notifyflgs;
 	char		***newmsgheader; /* message header line pointer ptr
 					   that points to an address of
@@ -60,11 +60,11 @@ struct rcpt {
 };
 
 struct ctldesc {
-	char		*msgfile;	/* message file name */
-	char		*logident;	/* message id for logging */
-	char		*verbose;	/* file for verbose logging */
-	char		*envid;		/* DSN ENVID data */
-	char		*dsnretmode;	/* DSN RET=-mode */
+	const char	*msgfile;	/* message file name */
+	const char	*logident;	/* message id for logging */
+	const char	*verbose;	/* file for verbose logging */
+	const char	*envid;		/* DSN ENVID data */
+	const char	*dsnretmode;	/* DSN RET=-mode */
 	time_t		msgmtime;	/* Message file arrival time */
 	long		msgbodyoffset;	/* offset of message body in msgfile */
 	long		msgsizeestimate; /* Estimate of the msg size */
@@ -72,7 +72,7 @@ struct ctldesc {
 	int		ctlfd;		/* control file I/O descriptor */
 	int		ctlid;		/* control file id (inode number) */
 	char		*ctlmap;	/* control file mmap() block */
-	char		*contents;	/* message file data */
+	const char	*contents;	/* message file data */
 	long		contentsize;	/* message file size */
 	long		*offset;	/* array of indices into contents */
 	struct address	*senders;	/* list of sender addresses */
@@ -83,8 +83,8 @@ struct ctldesc {
 	char		***msgheaders;	/* pointer to all msg headers */
 	char		***msgheaderscvt; /* converted headers */
 #ifdef	HAVE_MMAP
-	char		*let_buffer;	/* MMAP()ed memory area containing */
-	char		*let_end;	/* the mail -- and its end..	   */
+	const char	*let_buffer;	/* MMAP()ed memory area containing */
+	const char	*let_end;	/* the mail -- and its end..	   */
 #endif
 };
 
@@ -115,34 +115,22 @@ struct mimestate {
 };
 
 
-#ifndef __
-# ifndef __STDC__
-#  define __(x) ()
-# else
-#  define __(x) x
-# endif
-#endif
-#ifdef __GNUC__
-#   define const    __const
-#   define volatile __volatile
-#else
-#   define const
-#   define volatile
-#endif
-
 /* ctlopen.c: */
 extern void            ctlfree __((struct ctldesc *dp, void *anyp));
 extern void           *ctlrealloc __((struct ctldesc *dp, void *anyp, size_t size));
-extern struct ctldesc *ctlopen __((char *file,char *channel, char *host, int *exitflag, int (*selectaddr)(), void *saparam, int (*matchrouter)(), void *mrparam));
+extern struct ctldesc *ctlopen __((const char *file, const char *channel, const char *host, int *exitflag, int (*selectaddr)(const char *, const char *, void *), void *saparam, int (*matchrouter)(const char *, struct address *, void *), void *mrparam));
 extern void            ctlclose __((struct ctldesc *dp));
-extern int	       ctlsticky __((char *spec_host,char *addr_host, void *cbparam));
+extern int	       ctlsticky __((const char *spec_host, const char *addr_host, void *cbparam));
 
 /* diagnostic.c: */
-extern char           *notaryacct __((int rc, char *okstr));
+extern const char     *notaryacct __((int rc, const char *okstr));
 		/* NOTARY: addres / action / status / diagnostic  */
-extern void 	       notaryreport __((const char*,const char*,const char*,const char*));
+extern void 	       notaryreport __((const char*, const char*, const char*, const char*));
+extern void            notary_setxdelay __((int));
+extern void            notary_setwtt __(( const char *host ));
+extern void            notary_setwttip __(( const char *ip ));
 #if defined(HAVE_STDARG_H)
-extern void	       diagnostic __((struct rcpt *rp, int rc, int timeout, char *fmt, ... ));
+extern void	       diagnostic __((struct rcpt *rp, int rc, int timeout, const char *fmt, ... ));
 #else
 extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 #endif
@@ -153,19 +141,20 @@ extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 #endif
 
 /* emptyline.c: */
-extern int	       emptyline __((char *line, int size));
+extern int	       emptyline __(( char *line, int size ));
 
+#if 0 /* actually better to include "libz.h" for this */
 /* esyslib.c: */
 extern int cistrcmp  __((const char *s1, const char *s2));
-extern int cistrncmp __((const char *s1, const char *s2, const int n));
-extern int efstat __((int fd, struct stat *stbuf));
-extern int emkdir __((char *dirpath, int mode));
-extern int eopen __((char *s, int f, int m));
-extern int epipe __((int fdarr[2]));
-extern int eread __((int fd, char *buf, int len));
+extern int cistrncmp __((const char *s1, const char *s2, int n));
+extern int efstat  __((int fd, struct stat *stbuf));
+extern int emkdir  __((const char *dirpath, int mode));
+extern int eopen   __((const char *s, int f, int m));
+extern int epipe   __((int fdarr[2]));
+extern int eread   __((int fd, char *buf, int len));
+extern int estat   __((const char *filename, struct stat *stbuf));
+extern int eunlink __((const char *filename));
 extern int errno;
-extern int estat __((char *filename, struct stat *stbuf));
-extern int eunlink __((char *filename));
 extern int optind;
 extern int embytes;
 extern int emcalls;
@@ -174,26 +163,24 @@ extern unsigned emsleeptime;
    extern univptr_t erealloc __((univptr_t buf, size_t len)); */
 #ifdef S_IWUSR /* Defined on <sys/stat.h> */
 extern int efstat __((int fd, struct stat *stbuf));
-extern int estat __((char *path, struct stat *stbuf));
 #endif
 extern long elseek __((int fd, off_t pos, int action));
-extern int elink __((char *file1, char *file2));
-extern int eunlink __((char *file));
+extern int elink __((const char *file1, const char *file2));
 extern int eclose __((int fd));
-extern int echdir __((char *path));
-extern int emkdir __((char *path, int mode));
-extern int ermdir __((char *path));
+extern int echdir __((const char *path));
+extern int ermdir __((const char *path));
 extern int erename __((char *from, char *to));
+#endif
 
 /* lockaddr.c: */
 extern int lockaddr __((int fd, char *map, long offset, int was, int new, const char *file, const char *host, const int mypid));
 
 /* markoff.c: */
-extern int markoff __((char *filecontents, int bytesleft, long offsets[], char *filename));
+extern int markoff __((char *filecontents, int bytesleft, long offsets[], const char *filename));
 
 /* mimeheaders.c: */
 #if defined(HAVE_STDARG_H)
-extern int append_header __((struct rcpt *rp, char *fmt, ...));
+extern int append_header __((struct rcpt *rp, const char *fmt, ...));
 #else
 extern int append_header __(());
 #endif
@@ -202,40 +189,47 @@ extern struct ct_data  *parse_content_type __((char **ct_linep));
 extern void output_content_type __((struct rcpt *rp, struct ct_data *ct, char **oldpos));
 extern int check_conv_prohibit __((struct rcpt *rp));
 extern int cte_check __((struct rcpt *rp));
-extern char **has_header __((struct rcpt *rp, char *keystr));
+extern char **has_header __((struct rcpt *rp, const char *keystr));
 extern void delete_header __((struct rcpt *rp, char **hdrp));
-extern int downgrade_charset __((struct rcpt *rp, FILE *verboselog));
+extern int  downgrade_charset __((struct rcpt *rp, FILE *verboselog));
 extern void downgrade_headers __((struct rcpt *rp, int downgrade, FILE *verboselog));
 extern int qp_to_8bit __((struct rcpt *rp));
 
 /* mime2headers.c */
-extern int headers_to_mime2 __((struct rcpt *rp, char *defcharset, FILE *verboselog));
-
+extern int headers_to_mime2 __((struct rcpt *rp, const char *defcharset, FILE *verboselog));
+extern int headers_need_mime2 __(( struct rcpt *rp ));
+ 
 
 /* writeheaders.c: */
-extern int writeheaders __((struct rcpt *rp, FILE *fp, char *newline, int use_cvt, int maxwidth));
+extern int writeheaders __((struct rcpt *rp, FILE *fp, const char *newline, int use_cvt, int maxwidth));
 
 /* buildbndry.c: */
 extern char *mydomain __((void));
-extern char *buildboundary __((char *what));
+extern char *buildboundary __((const char *what));
 
 extern int getout;
-extern RETSIGTYPE wantout __(());
+extern RETSIGTYPE wantout __((int));
 
 /* warning.c */
-extern int warning __(());
+#ifdef HAVE_STDARG_H
+extern void warning __((const char *fmt, ...));
+#else
+extern void warning __(());
+#endif
 
 /* lib/skip821address.c */
-extern char *skip821address __((char *s));
+extern char *skip821address __((const char *s));
 
 /* lib/taspoolid.c */
-extern void taspoolid __((char *buf, int buflen, time_t mtime, char *path));
+extern void taspoolid __((char *buf, int buflen, time_t mtime, const char *path));
 
 /* tasyslog.c */
 extern void tatimestr __((char *buf, int dt));
-extern void tasyslog __((struct rcpt *rp, int xdelay, char *wtthost, char *wttip, char *stat, char *msg));
+extern void tasyslog __((struct rcpt *rp, int xdelay, const char *wtthost, const char *wttip, const char *stats, const char *msg));
 
 #ifndef CISTREQ
-#define  CISTREQ(x, y)      (cistrcmp((char *)(x), (char *)(y)) == 0)
-#define  CISTREQN(x, y, n)  (cistrncmp((char *)(x), (char *)(y), n) == 0)
+#define  CISTREQ(x, y)      (cistrcmp((const char *)(x), (const char *)(y)) == 0)
+#define  CISTREQN(x, y, n)  (cistrncmp((const char *)(x), (const char *)(y), n) == 0)
 #endif
+
+extern int getmyuucename __((char *, int));
