@@ -12,9 +12,9 @@ typedef enum {
 } TokenType;
 
 struct token {
-	u_char		*t_pname;		/* printable representation */
-	u_long		t_len;			/* length (0 if malloc'ed) */
-	TokenType	t_type;			/* information on token type */
+	const char	*t_pname;		/* printable representation */
+	u_long		 t_len;			/* length (0 if malloc'ed) */
+	TokenType	 t_type;		/* information on token type */
 	struct token	*t_next;
 };
 
