@@ -14,15 +14,15 @@
 #undef	MALLOC_TRACE
 #endif
 
+
 #include "mailer.h"
+#include "libz.h"
 
 int	embytes = 0;
 int	emcalls = 0;
 memtypes stickymem = MEM_PERM;
 extern int D_alloc;
 
-extern char *nbytes();
-
 #if defined(sparc) || defined(__sparc__)
 #define NALIGN	8	/* SPARC wants doubles at 8-byte boundary, always!
 			   The ZMailer does not use floating point, but
@@ -85,6 +85,8 @@ static u_long stackmem = 0;	/* bitmap: intend to use block memory as a stack */
 /*
  * Inline macros for allocating memory
  */
+#if 0
+
 #define	GETALIGNED(n,i,cp)	if (blockinuse[(int)(i)] == NULL || \
 			    blockinuse[(int)(i)]->cur+(n) >= blockinuse[(int)(i)]->endp) \
 					moremem((n), (i)); \
@@ -97,6 +99,44 @@ static u_long stackmem = 0;	/* bitmap: intend to use block memory as a stack */
 				(cp) = blockinuse[(int)(i)]->cur; \
 				blockinuse[(int)(i)]->cur += (n)
 
+#else
+
+#define GETALIGNED(n,i,cp) cp = GETALIGNED_((n),(i))
+
+#ifndef __GNUC__
+# define __inline
+#endif
+
+static __inline void * GETALIGNED_ __((u_int n, u_int i));
+static __inline void *
+GETALIGNED_(n,i)
+u_int n, i;
+{
+  char *cp;
+  if (blockinuse[i] == NULL ||
+      blockinuse[i]->cur+ n >= blockinuse[i]->endp)
+    moremem(n, i);
+  cp = ALIGN(blockinuse[i]->cur);
+  blockinuse[i]->cur = cp + n;
+  return (void *)cp;
+}
+
+#define	GETMEMORY(n,i,cp) cp = GETMEMORY_((n),(i))
+
+static __inline void * GETMEMORY_ __((u_int n, u_int i));
+static __inline void *
+GETMEMORY_(n,i)
+u_int n, i;
+{
+  char *cp;
+  if (blockinuse[i] == NULL ||
+      blockinuse[i]->cur+ n >= blockinuse[i]->endp)
+    moremem(n, i);
+  cp = blockinuse[i]->cur;
+  blockinuse[i]->cur += n;
+  return (void *)cp;
+}
+#endif
 
 /*
  * Statistics for calls to moremem()
@@ -121,7 +161,8 @@ moremem(n, i)
 	
 	mmcalls[i]++;
 
-	if ((stackflag = (stackmem & (1<<i))))
+	stackflag = (stackmem & (1<<i));
+	if (stackflag != 0)
 		bp = blockinuse[i];
 	else
 		bp = blockhead[i];
@@ -194,6 +235,7 @@ moremem(n, i)
 	}
 }
 
+#if 0
 int
 blockmem(memtype, up)
 	const int memtype;
@@ -202,10 +244,11 @@ blockmem(memtype, up)
 	register struct block *bp;
 
 	for (bp = blockhead[memtype]; bp != NULL; bp = bp->next)
-		if (INBLOCK(bp, ((char*)up)))
-			return 1;
+	  if (INBLOCK(bp, ((const char*)up)))
+	    return 1;
 	return 0;
 }
+#endif
 
 /*
  * malloc() replacement that allocates out of temporary storage.
@@ -259,8 +302,8 @@ memcontents()
 	memtypes memtype;
 
 	for (memtype = MEM_PERM; memtype < MEMTYPES ; ++memtype)
-		if (blockhead[memtype])
-			memstats(memtype);
+	  if (blockhead[memtype])
+	    memstats(memtype);
 }
 
 /*
@@ -275,24 +318,24 @@ tfree(memtype)
 
 	bp = blockfull[memtype];
 	while (bp != NULL) {
-		bpn = bp->next;
-		bp->next = blockhead[memtype];
-		blockhead[memtype] = bp;
-		bp = bpn;
+	  bpn = bp->next;
+	  bp->next = blockhead[memtype];
+	  blockhead[memtype] = bp;
+	  bp = bpn;
 	}
 	blockfull[memtype] = NULL;
 
 	if (D_alloc)
-		memstats(memtype);
+	  memstats(memtype);
 	for (bp = blockhead[memtype]; bp != NULL; bp = bp->next)
-		bp->cur = &bp->area[0];
+	  bp->cur = &bp->area[0];
 	if (D_alloc) {
-		printf("Memory refreshes for %d memory: %d\n",
-		       memtype, mmcalls[memtype]);
-		printf("Requiring global memory: %d\n",
-		       mmmallocs[memtype]);
-		printf("Total permanent %d memory: %d bytes from %d calls\n",
-		       memtype, embytes, emcalls);
+	  printf("Memory refreshes for %d memory: %d\n",
+		 memtype, mmcalls[memtype]);
+	  printf("Requiring global memory: %d\n",
+		 mmmallocs[memtype]);
+	  printf("Total permanent %d memory: %d bytes from %d calls\n",
+		 memtype, embytes, emcalls);
 	}
 	mmcalls[memtype] = mmmallocs[memtype] = 0;
 }
@@ -308,7 +351,7 @@ getlevel(memtype)
 {
 	stackmem |= (1<<memtype);
 	if (blockinuse[memtype] == NULL)
-		moremem(0, memtype);
+	  moremem(0, memtype);
 	return (univptr_t)blockinuse[memtype]->cur;
 }
 
@@ -320,30 +363,30 @@ getlevel(memtype)
 void
 setlevel(memtype, up)
 	const memtypes memtype;
-	univptr_t up;
+	const univptr_t up;
 {
 	register struct block *bp;
-	register char *s = (char *) up;
+	register const char *s = (const char *) up;
 
 	if (!(stackmem & (1<<memtype)) || blockfull[memtype] != NULL) {
-		printf("Memory type %d is not usable as a stack!\n",
-		       memtype);
-		return;
+	  printf("Memory type %d is not usable as a stack!\n",
+		 memtype);
+	  return;
 	}
 	if (INBLOCK(blockinuse[memtype], s))
-		blockinuse[memtype]->cur = s;
+	  blockinuse[memtype]->cur = (char*) s;
 	else {
-		for (bp = blockhead[memtype]; bp != NULL; bp = bp->next)
-			if (INBLOCK(bp, s))
-				break;
-		if (bp == NULL) {
-			printf("Illegal pointer into %d memory!\n", memtype);
-			return;
-		}
-		bp->cur = s;
-		blockinuse[memtype] = bp;
-		for (bp = bp->next; bp != NULL; bp = bp->next)
-			bp->cur = &bp->area[0];
+	  for (bp = blockhead[memtype]; bp != NULL; bp = bp->next)
+	    if (INBLOCK(bp, s))
+	      break;
+	  if (bp == NULL) {
+	    printf("Illegal pointer into %d memory!\n", memtype);
+	    return;
+	  }
+	  bp->cur = (char*) s;
+	  blockinuse[memtype] = bp;
+	  for (bp = bp->next; bp != NULL; bp = bp->next)
+	    bp->cur = &bp->area[0];
 	}
 }
 
@@ -362,7 +405,7 @@ strnsave(s, n)
 	if (stickymem == MEM_MALLOC) {
 		cp = emalloc(n+1);
 	} else {
-		GETMEMORY((u_int)(n+1), stickymem, cp);
+		GETMEMORY(n+1, stickymem, cp); /* lengthy macro.. */
 	}
 	memcpy(cp, s, n);
 	*(cp+n) = '\0';
