@@ -11,24 +11,27 @@
 #undef	MALLOC_TRACE
 #endif
 #include "hostenv.h"
-#include "mailer.h"
 #include <errno.h>
 #include <fcntl.h>
+#include "mailer.h"
+#include "libz.h"
 
 extern char	*progname;
 extern int	errno;
-extern char	*strerror();
+extern char	*strerror __((int));
 
 int
 eopen(s, f, m)
-	char	*s;
+	const char *s;
 	int	f, m;
 {
 	int	r;
 
-	if ((r = open(s, f, m)) < 0) {
-		fprintf(stderr, "%s: open: %s: %s\n",
-			progname, s, strerror(errno));
+	r = open(s, f, m);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: open: %s: %s\n", progname, s, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
@@ -40,9 +43,11 @@ eread(fd, buf, len)
 {
 	int	r;
 
-	if ((r = read(fd, buf, len)) < 0) {
-		fprintf(stderr, "%s: read: %s\n",
-			progname, strerror(errno));
+	r = read(fd, buf, len);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: read: %s\n", progname, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
@@ -53,9 +58,11 @@ epipe(fdarr)
 {
 	int	r;
 
-	if ((r = pipe(fdarr)) < 0) {
-		fprintf(stderr, "%s: pipe: %s\n",
-			progname, strerror(errno));
+	r = pipe(fdarr);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: pipe: %s\n", progname, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
@@ -67,23 +74,27 @@ efstat(fd, stbuf)
 {
 	int	r;
 
-	if ((r = fstat(fd, stbuf)) < 0) {
-		fprintf(stderr, "%s: fstat(%d): %s\n", progname,
-			fd, strerror(errno));
+	r = fstat(fd, stbuf);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: fstat(%d): %s\n", progname, fd,strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
 
 int
 estat(path, stbuf)
-	char	*path;
+	const char	*path;
 	struct stat	*stbuf;
 {
 	int	r;
 
-	if ((r = stat(path, stbuf)) < 0) {
-		fprintf(stderr, "%s: stat(%s): %s\n", progname,
-			path, strerror(errno));
+	r = stat(path, stbuf);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: stat(%s): %s\n",progname,path,strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
@@ -95,35 +106,44 @@ elseek(fd, pos, action)
 {
 	off_t	r;
 
-	if ((r = lseek(fd, pos, action)) == -1) {
-		fprintf(stderr, "%s: lseek(%d, %ld, %d): %s\n", progname,
-			fd, pos, action, strerror(errno));
+	r = lseek(fd, pos, action);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: lseek(%d, %ld, %d): %s\n", progname,
+		  fd, pos, action, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
 
 int
 elink(file1, file2)
-	char *file1, *file2;
+	const char *file1, *file2;
 {
 	int	r;
 
-	if ((r = link(file1, file2)) < 0) {
-		fprintf(stderr, "%s: link(%s, %s): %s\n", progname,
-			file1, file2, strerror(errno));
+	r = link(file1, file2);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: link(%s, %s): %s\n", progname,
+		  file1, file2, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
 
 int
 eunlink(file)
-	char *file;
+	const char *file;
 {
 	int	r;
 
-	if ((r = unlink(file)) < 0) {
-		fprintf(stderr, "%s: unlink(%s): %s\n", progname,
-			file, strerror(errno));
+	r = unlink(file);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: unlink(%s): %s\n", progname,
+		  file, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
@@ -134,29 +154,35 @@ eclose(fd)
 {
 	int	r;
 
-	if ((r = close(fd)) < 0) {
-		fprintf(stderr, "%s: close(%d): %s\n", progname,
-			fd, strerror(errno));
+	r = close(fd);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: close(%d): %s\n", progname,
+		  fd, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
 
 int
 echdir(file)
-	char *file;
+	const char *file;
 {
 	int	r;
 
-	if ((r = chdir(file)) < 0) {
-		fprintf(stderr, "%s: chdir(%s): %s\n", progname,
-			file, strerror(errno));
+	r = chdir(file);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: chdir(%s): %s\n", progname,
+		  file, strerror(errno));
+	  errno = serrno;
 	}
 	return r;
 }
 
 int
 emkdir(file, mode)
-	char *file;
+	const char *file;
 	int mode;
 {
 	int	r;
@@ -171,9 +197,12 @@ emkdir(file, mode)
 	}
 #else	/* !USE_BINMKDIR */
 
-	if ((r = mkdir(file, mode)) < 0) {
-		fprintf(stderr, "%s: mkdir(%s, 0%o): %s\n", progname,
-			file, mode, strerror(errno));
+	r = mkdir(file, mode);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: mkdir(%s, 0%o): %s\n", progname,
+		  file, mode, strerror(errno));
+	  errno = serrno;
 	}
 #endif	/* USE_BINMKDIR */
 	return r;
@@ -181,7 +210,7 @@ emkdir(file, mode)
 
 int
 ermdir(file)
-	char *file;
+	const char *file;
 {
 	int	r;
 #ifdef	USE_BINRMDIR
@@ -195,9 +224,12 @@ ermdir(file)
 	}
 #else	/* !USE_BINRMDIR */
 
-	if ((r = rmdir(file)) < 0) {
-		fprintf(stderr, "%s: rmdir(%s): %s\n", progname,
-			file, strerror(errno));
+	r = rmdir(file);
+	if (r < 0) {
+	  int serrno = errno;
+	  fprintf(stderr, "%s: rmdir(%s): %s\n", progname,
+		  file, strerror(errno));
+	  errno = serrno;
 	}
 #endif	/* USE_BINRMDIR */
 	return r;
@@ -205,30 +237,35 @@ ermdir(file)
 
 int
 erename(from, to)
-	char *from, *to;
+	const char *from, *to;
 {
 
 #ifdef	HAVE_RENAME
-
 	if (rename(from, to) < 0) {
+	  int serrno = errno;
 	  fprintf(stderr, "%s: rename(%s,%s): %s\n", progname,
 		  from, to, strerror(errno));
+	  errno = serrno;
 	  return -1;
 	}
 
 #else	/* !HAVE_RENAME */
 	
 	if ((unlink(to) < 0 && errno != ENOENT) || (link(from, to) < 0)) {
+	  int serrno = errno;
 	  fprintf(stderr, "%s: rename(%s,%s): %s\n", progname,
 		  from, to, strerror(errno));
+	  errno = serrno;
 	  return -1;
 	}
 
 	if (unlink(from) < 0) {
+	  int serrno = errno;
 	  if (errno != ENOENT)
 	    fprintf(stderr, "%s: rename(%s,%s): %s\n", progname,
 		    from, to, strerror(errno));
 	  unlink(to);
+	  errno = serrno;
 	  return -1;
 	}
 #endif	/* !HAVE_RENAME */
@@ -238,7 +275,7 @@ erename(from, to)
 
 int
 eqrename(from, to)
-	char *from, *to;
+	const char *from, *to;
 {
 #ifdef	HAVE_RENAME_we_use_link_unlink_to_avoid_races_in_routers
 
@@ -253,7 +290,9 @@ eqrename(from, to)
 	}
 
 	if (unlink(from) < 0) {
+	  int serrno = errno;
 	  unlink(to);
+	  errno = serrno;
 	  return -1;
 	}
 #endif	/* !HAVE_RENAME */
