@@ -9,6 +9,7 @@
 #include "mailer.h"
 #include <sys/file.h>
 #include "../libsh/io.h"
+#include "libz.h"
 
 #ifndef	L_INCR
 #define	L_INCR	1
@@ -20,12 +21,12 @@
  *  allocated buffer known only within this module. Code external to here
  *  should treat linebuf as a read-only (constant) buffer.
  */
-static u_char	*getline_block;		/* malloc()'ed input buffer */
-static u_char	*getline_bend;		/* end of line pointer */
+static char	*getline_block;		/* malloc()'ed input buffer */
+static char	*getline_bend;		/* end of line pointer */
 static u_int	getline_blen;		/* bytes available to consume */
 static u_int	getline_bsize;		/* size of malloc()'ed buffer */
 
-u_char	*linebuf = NULL;	/* where to start consuming */
+char	*linebuf = NULL;	/* where to start consuming */
 
 /*
  * This routine should be called between opening a file
@@ -40,7 +41,7 @@ initline(blksize)
 		getline_bsize = blksize;
 		if (getline_bsize < BUFSIZ)
 			getline_bsize = BUFSIZ;
-		getline_block = (u_char *)emalloc(getline_bsize);
+		getline_block = (char *)emalloc(getline_bsize);
 	}
 	getline_blen = 0;
 }
@@ -62,45 +63,47 @@ int
 getline(fp)
 	FILE *fp;
 {
-	register u_char	*cp;
+	register char	*cp;
 	register int	n;
 
 	/* assert getline_block != NULL */
-	if (getline_blen <= 0 || getline_bend >= getline_block + getline_bsize) {
-		getline_blen = fread((char *)getline_block, 1, (int)getline_bsize, fp);
-		if (getline_blen == 0)
-			return 0;
-		getline_bend = getline_block;
+	if (0 >= getline_blen ||
+	    getline_bend >= (getline_block + getline_bsize)) {
+	  getline_blen = fread(getline_block, 1, getline_bsize, fp);
+	  if (getline_blen == 0)
+	    return 0;
+	  getline_bend = getline_block;
 	}
 	while (1) {
-		/* look for end of line in what remains of the input buffer */
-		for (cp = getline_bend, n = 0; n < getline_blen; ++n, ++cp)
-		  if (*cp == '\n') {
-		    linebuf = getline_bend;
-		    getline_bend = ++cp;
-		    getline_blen -= ++n;
-		    return n;
-		  }
-		/* copy the partial line to the beginning of the input buffer */
-		if (getline_bend > getline_block)
-		  memcpy((char *)getline_block, (char *)getline_bend, (int)getline_blen);
-		/* get some more bytes which will hopefully contain a newline */
-		n = getline_bsize - getline_blen;
-		if (n <= 0) {		/* grow the buffer */
-			n = getline_bsize;
-			getline_bsize *= 2;
-			getline_block = (u_char *)erealloc((char *)getline_block, getline_bsize);
-		}
-		n = fread((char *)(getline_block+getline_blen), 1, n, fp);
-		if (n == 0) {
-			/* the file doesn't terminate in a newline */
-			n = getline_blen;
-			getline_blen = 0;
-			linebuf = getline_block;
-			return n;
-		}
-		getline_blen += n;
-		getline_bend = getline_block;
+	  /* look for end of line in what remains of the input buffer */
+	  for (cp = getline_bend, n = 0; n < getline_blen; ++n, ++cp)
+	    if (*cp == '\n') {
+	      linebuf = getline_bend;
+	      getline_bend = ++cp;
+	      getline_blen -= ++n;
+	      return n;
+	    }
+	  /* copy the partial line to the beginning of the input buffer */
+	  if (getline_bend > getline_block)
+	    memcpy(getline_block, getline_bend, getline_blen);
+	  /* get some more bytes which will hopefully contain a newline */
+	  n = getline_bsize - getline_blen;
+	  if (n <= 0) {		/* grow the buffer */
+	    n = getline_bsize;
+	    getline_bsize *= 2;
+	    getline_block = (char *)erealloc(getline_block,
+					     getline_bsize);
+	  }
+	  n = fread(getline_block + getline_blen, 1, n, fp);
+	  if (n == 0) {
+	    /* the file doesn't terminate in a newline */
+	    n = getline_blen;
+	    getline_blen = 0;
+	    linebuf = getline_block;
+	    return n;
+	  }
+	  getline_blen += n;
+	  getline_bend = getline_block;
 	}
 	/* NOTREACHED */
 }
@@ -113,7 +116,7 @@ getline(fp)
  */
 
 int
-getrest()
+linegetrest()
 {
 	/* assert getline_block != NULL */
 	if (getline_blen <= 0 || getline_bend >= getline_block + getline_bsize)
@@ -127,7 +130,7 @@ getrest()
  */
 
 long
-offset(fp)
+lineoffset(fp)
 	FILE *fp;
 {
 	return ftell(fp) - getline_blen;
