@@ -7,18 +7,17 @@
  *	<mea@nic.funet.fi> (and elsewere), copyright 1992-1997
  */
 
-#include <stdio.h>
-#include "hostenv.h"
+#include "mailer.h"
+#include "libz.h"
+#include "libsh.h"
 
-static char *Copyright = "Copyright 1992 Rayan S. Zachariassen\n\
+static const char *Copyright = "Copyright 1992 Rayan S. Zachariassen\n\
 Copyright 1992-1997 Matti Aarnio";
 
 void
-prversion(progname)
-	char *progname;
+prversion(prgname)
+	const char *prgname;
 {
-	extern char *Version, *CC_user, *CC_pwd;
-
-	fprintf(stderr, "ZMailer %s (%s)\n  %s:%s\n%s\n", progname,
+	fprintf(stderr, "ZMailer %s (%s)\n  %s:%s\n%s\n", prgname,
 		Version, CC_user, CC_pwd, Copyright);
 }
