@@ -5,6 +5,7 @@
 
 #include "hostenv.h"
 #include "mailer.h"
+#include "libz.h"
 
 /* Start of the scanner */
 
@@ -74,13 +75,11 @@ _8,	_8,	_8,	_8,	_8,	_8,	_8,	_8	/* 370 - 377 */
 
 int
 hdr_status(cp, lbuf, n, octo)
-	register u_char	*cp, *lbuf;
+	register const char *cp, *lbuf;
 	int	n, octo;
 {
-	u_char *cpin = cp;
-
 	if (*cp == ' ' || *cp == '\t') {
-	  while ((cp < lbuf + n) && (rfc_ctype[*cp] & (_w|_l)))
+	  while ((cp < lbuf + n) && (rfc_ctype[(*cp) & 0xFF] & (_w|_l)))
 	    ++cp;
 	  if (cp == lbuf + n)
 	    /* a line containing only whitespace is EOH */
@@ -89,7 +88,7 @@ hdr_status(cp, lbuf, n, octo)
 	  return 0;
 	}
 	if (!octo) {
-	  while ((cp < lbuf + n) && (rfc_ctype[*cp] & _h))
+	  while ((cp < lbuf + n) && (rfc_ctype[(*cp) & 0xFF] & _h))
 	    ++cp;
 	  if ((cp < lbuf + n) &&
 	      (*cp == ':') /*&& (cp > cpin)*/)	/* header line */
@@ -112,7 +111,7 @@ hdr_status(cp, lbuf, n, octo)
 	      quote = 0;
 	    else if (c == '"')
 	      quote = '"';
-	    else if (!quote && !(rfc_ctype[c] & _h))
+	    else if (!quote && !(rfc_ctype[c & 0xFF] & _h))
 	      break;
 	    ++cp;
 	  }
@@ -122,7 +121,7 @@ hdr_status(cp, lbuf, n, octo)
 	}
 }
 
-#define MKERROR(msg,prev)	tn = makeToken((u_char *)(msg), strlen(msg)); \
+#define MKERROR(msg,prev)	tn = makeToken((msg), strlen(msg)); \
 				tn->t_type = Error; \
 				tn->t_next = (prev); \
 				(prev) = tn;
@@ -135,15 +134,13 @@ hdr_status(cp, lbuf, n, octo)
 
 u_long
 _hdr_compound(cp, n, cstart, cend, type, tp, tlist, tlistp)
-	register u_char	*cp;
+	register const char *cp;
 	long	n;
-	u_char	cstart, cend;
+	int	cstart, cend;
 	TokenType	type;
 	struct token	*tp, **tlist, **tlistp;
 {
-	int i;
 	struct token *tn;
-	char msgbuf[50];
 
 	if (*cp != cstart)
 		abort(); /* Sanity check!  Call fault! */
@@ -151,7 +148,6 @@ _hdr_compound(cp, n, cstart, cend, type, tp, tlist, tlistp)
 nextline:
 	for (; n > 0 && *cp != cend; ++cp, --n) {
 		if (*cp == cstart && type == Comment) {
-			i = n;
 			n = _hdr_compound(cp, n, cstart, cend,
 					  type, tp, tlist, tlistp) +1;
 			cp = (*tlistp)->t_pname + TOKENLEN(*tlistp) - n;
@@ -171,6 +167,8 @@ nextline:
 	/* we either found cend, or ran off the end, either may be within
 	   a recursion */
 	if (n == 0) { /* we ran off the end */
+		char msgbuf[50];
+
 		if (tlistp != NULL && *tlistp != NULL
 		    && (*tlistp)->t_next != NULL) {
 			/* compound token is continued on next line */
@@ -194,19 +192,19 @@ nextline:
 
 /* Unfold (see RFC822) the contents of a compound token */
 
-u_char *
+const char *
 _unfold(start, end, t)
-	u_char *start, *end;
+	const char *start, *end;
 	struct token *t;
 {
-	u_char *s, *cp;
+	char *s, *cp;
 	/*
 	 * Since we will be ignoring CRLF and copying everything else,
 	 * we know the max length is going to be (end - start) (sic).
 	 * However, I like being conservative about these things...
 	 */
 
-	s = cp = (u_char *)tmalloc((u_int)(end-start+1));
+	s = cp = (char *)tmalloc((u_int)(end-start+1));
 	while (start < end) {
 		/* != 'cuz *end is ')' or '"' and lines are randomly alloced */
 		if (*start == '\n') {
@@ -239,87 +237,87 @@ _unfold(start, end, t)
  * for efficiency this was changed to avoid function call overhead. The tokens
  * returned are classified by type (TokenType enum class).
  */
-struct token * scan822(cpp, n, c1, c2, allowcomments, tlistp)
-	u_char	**cpp;			/* pointer to pointer to text */
-	long	n;			/* number of characters to scan */
+struct token * scan822(cpp, nn, c1, c2, allowcomments, tlistp)
+	const char **cpp;		/* pointer to pointer to text */
+	size_t	nn;			/* number of characters to scan */
 	int	c1, c2;			/* temporary specials */
 	int allowcomments;		/* #prefix tokens are comments to EOT */
 	struct token **tlistp;		/* continuation line tokens if any */
 {
-	register u_char	*cp;
-	static struct token	t;
+	register const char *cp;
+	static struct token  t;
 	struct token	*tlist, *tp, *tn, *ot;
 	char	msgbuf[50];
 	short	ct, sc1, sc2;
+	int n = (int) nn;
 
 	if (n == 0)
 		return NULL;
 	sc1 = sc2 = '\0';
 	if (c1 != '\0') {
-		sc1 = rfc_ctype[c1];
+		sc1 = rfc_ctype[c1 & 0xFF];
 		rfc_ctype[c1] |= _s;
 	}
 	if (c2 != '\0') {
-		sc2 = rfc_ctype[c2];
-		rfc_ctype[c2] |= _s;
+		sc2 = rfc_ctype[c2 & 0xFF];
+		rfc_ctype[c2 & 0xFF] |= _s;
 	}
 	tlist = NULL;
 	do {
 		cp = *cpp;
-		ct = rfc_ctype[*cp];
+		ct = rfc_ctype[(*cp) & 0xFF];
 		t.t_len = n;
 		t.t_pname = cp;
 		if (ct & _w) {		/* LWSP without the CR LF part */
-			while (--n > 0 && (rfc_ctype[*++cp] & _w))
-				continue;
+			while (--n > 0 && (rfc_ctype[(*++cp) & 0xFF] & _w))
+			  continue;
 			t.t_type = Space;
 		} else if (ct & _r) {	/* >= 1 CR followed by LFs is a fold */
-			while (--n > 0 && (rfc_ctype[*++cp] & _r))
-				continue;
-			if (n == 0 || !(rfc_ctype[*cp] & _l)) {
-				strcpy(msgbuf, "CR without LF (newline)");
-				MKERROR(msgbuf,tlist);
-			} else if (n > 1 && (rfc_ctype[*cp] & _l)) {
-				while (--n > 0 && (rfc_ctype[*++cp] & _l))
-					continue;
-				strcpy(msgbuf,
-					"too many newlines (LFs) in field[1]");
-				MKERROR(msgbuf,tlist);
+			while (--n > 0 && (rfc_ctype[(*++cp) & 0xFF] & _r))
+			  continue;
+			if (n == 0 || !(rfc_ctype[(*cp) & 0xFF] & _l)) {
+			  strcpy(msgbuf, "CR without LF (newline)");
+			  MKERROR(msgbuf,tlist);
+			} else if (n > 1 && (rfc_ctype[(*cp) & 0xFF] & _l)) {
+			  while (--n > 0 && (rfc_ctype[(*++cp) & 0xFF] & _l))
+			    continue;
+			  strcpy(msgbuf,"too many newlines (LFs) in field[1]");
+			  MKERROR(msgbuf,tlist);
 			}
 			t.t_type = Fold;
 		} else if (ct & _l) {	/* >= 1 LFs without CR is a fold too */
-			while (--n > 0 && (rfc_ctype[*++cp] & _l))
-				continue;
+			while (--n > 0 && (rfc_ctype[(*++cp) & 0xFF] & _l))
+			  continue;
 			strcpy(msgbuf,"too many newlines (LFs) in field[2]");
 			MKERROR(msgbuf,tlist);
 			t.t_type = Fold;
 		} else if ((ct & _s) && (*cp=='(' || *cp=='"' || *cp=='[')) {
 			TokenType	type;
-			u_char	cend;
+			char	cend;
 
 			if (*cp == '"') {
-				cend = *cp;
-				type = String;
+			  cend = *cp;
+			  type = String;
 			} else if (*cp == '[') {
-				cend = ']';
-				type = DomainLiteral;
+			  cend = ']';
+			  type = DomainLiteral;
 			} else {
-				cend = ')';
-				type = Comment;
+			  cend = ')';
+			  type = Comment;
 			}
 			ot = (tlistp == NULL ? NULL : *tlistp);
 			n = _hdr_compound(cp, n, *cp, cend,
 					  type, &t, &tlist, tlistp);
 			if (ot != NULL && tlistp != NULL && ot != *tlistp) {
-				/* a compound token crossed line boundary */
-				(*cpp) = (*tlistp)->t_pname +
-					TOKENLEN(*tlistp) - n;
-				/* copy from ++cp up to and excluding *cpp */
-				t.t_pname = _unfold(++cp, (*cpp)-1, ot);
-				t.t_len = strlen((char *)t.t_pname);
-				/* compensate for calculations below */
-				(*cpp) -= t.t_len;
-				t.t_len += n;
+			  /* a compound token crossed line boundary */
+			  (*cpp) = ((*tlistp)->t_pname +
+				    TOKENLEN(*tlistp) - n);
+			  /* copy from ++cp up to and excluding *cpp */
+			  t.t_pname = _unfold(++cp, (*cpp)-1, ot);
+			  t.t_len   = strlen(t.t_pname);
+			  /* compensate for calculations below */
+			  (*cpp)  -= t.t_len;
+			  t.t_len += n;
 			} else {
 				if (t.t_pname != NULL)
 					/* magic sign, no ending char */
@@ -334,13 +332,15 @@ struct token * scan822(cpp, n, c1, c2, allowcomments, tlistp)
 			--n;
 			t.t_type = Special;
 		} else if (!(ct & (_c|_8))) {	/* atom */
-			while (--n > 0 && !(rfc_ctype[*++cp]&(_w|_s|_c|_l|_r)))
+			while (--n > 0 &&
+			       !(rfc_ctype[(*++cp) & 0xFF]&(_w|_s|_c|_l|_r)))
 				continue;
 			t.t_type = Atom;
 		} else {
 			int bit8 = 0;
-			while (--n > 0 && (rfc_ctype[*++cp] & (_c|_8)))
-				if (rfc_ctype[*cp] & _8) {
+			while (--n > 0 &&
+			       (rfc_ctype[(*++cp) & 0xFF] & (_c|_8)))
+				if (rfc_ctype[(*cp) & 0xFF] & _8) {
 					bit8 = 1;
 					break;
 				}
@@ -364,8 +364,8 @@ struct token * scan822(cpp, n, c1, c2, allowcomments, tlistp)
 		if (t.t_len > 0)
 			tlist = copyToken(&t);
 		else {
-			t.t_pname = (u_char *)"";
-			t.t_len = 0;
+			t.t_pname = "";
+			t.t_len   = 0;
 			tlist = copyToken(&t);
 		}
 	} while (n > 0);
@@ -376,7 +376,7 @@ struct token * scan822(cpp, n, c1, c2, allowcomments, tlistp)
 		tlist->t_next = tp;
 		tp = tlist;
 	}
-	if (c1 != '\0') rfc_ctype[c1] = sc1;
-	if (c2 != '\0') rfc_ctype[c2] = sc2;
+	if (c1 != '\0') rfc_ctype[c1 & 0xFF] = sc1;
+	if (c2 != '\0') rfc_ctype[c2 & 0xFF] = sc2;
 	return tp;
 }
