@@ -27,12 +27,12 @@
     by Matti Aarnio <mea@nic.funet.fi> 1997
 */
 
-#include "hostenv.h"
+#include "mailer.h"
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#if defined(__svr4__) && defined(__sun)
+#if (defined(__svr4__) || defined(__SVR4)) && defined(__sun)
 # define BSD_COMP /* Damn Solaris, and its tricks... */
 #endif
 #include <sys/ioctl.h>
@@ -56,14 +56,9 @@
 #include <arpa/inet.h>
 #include <string.h>
 
-extern char  *getzenv     __((const char *));
+#include "libz.h"
 
-/* hostent.c */
-extern void   hp_init     __((struct hostent *hp));
-extern void   hp_setalist __((struct hostent *hp, void **));
-extern char **hp_getaddr  __((void));
-extern char **hp_nextaddr __((void));
-extern void   hp_addr_randomize __((struct hostent *hp));
+extern char  *getzenv     __((const char *));
 
 /*
 **  LOADIFADDRESSES -- load interface-specific addresses
@@ -104,13 +99,17 @@ struct sockaddr ***sockaddrp;
 	struct sockaddr **sap;
 
 	sap = (void*)malloc(sizeof(struct sockaddr*) * (ifcount + 2));
-	if (! sap) {
-	  close(s);
-	  free(interfacebuf);
+	if (! sap || !interfacebuf) {
+	  if (interfacebuf)
+	    free(interfacebuf);
+	  if (sap)
+	    free(sap);
 	  return -3; /* UAARGH! */
 	}
 
+#if defined(AF_INET6) && defined(INET6)
 other_socktype:
+#endif
 
 	s = socket(af, SOCK_DGRAM, 0);
 	if (s < 0) {
@@ -173,7 +172,7 @@ other_socktype:
 	  /* Known address families ? */
 
 	  if (ifr->ifr_addr.sa_family != AF_INET
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	      &&
 	      ifr->ifr_addr.sa_family != AF_INET6
 #endif
@@ -220,7 +219,7 @@ other_socktype:
 	    memcpy(si4, &ifr->ifr_addr, sizeof(struct sockaddr_in));
 	    sap[ifcount] = (struct sockaddr*)si4;
 	  }
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	  if (sa->sa_family == AF_INET6) {
 	    struct sockaddr_in6 *si6 = (void*)malloc(sizeof(*si6));
 	    if (si6 == NULL)
@@ -238,7 +237,7 @@ other_socktype:
 	sap[ifcount] = NULL;
 	close(s);
 
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	if (af != AF_INET6) {
 	  af = AF_INET6;
 	  goto other_socktype;
@@ -259,22 +258,21 @@ other_socktype:
 static             int    nmyaddrs = 0;
 static struct sockaddr  ** myaddrs = NULL;
 
+static void stashmyaddress __((const char *));
 static void
 stashmyaddress(host)
-	char *host;
+	const char *host;
 {
 	int naddrs;
 	struct hostent *hp, hent;
 	union {
 	  struct in_addr ia4;
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	  struct in6_addr ia6;
 #endif
 	} au;
 	int addrsiz, af, rc;
 	void *addrs[2];
-	const int addrlen = 4; /* IPv4 length.. */
-	int nmyaddr0;
 
 	if (host == NULL) return;
 
@@ -291,7 +289,7 @@ stashmyaddress(host)
 #define IN6ADDRSZ 16
 #endif
 
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	  if (strncasecmp(host,"[ipv6.",6)==0) {
 	    af = AF_INET6;
 	    addrsiz = IN6ADDRSZ;
@@ -311,10 +309,10 @@ stashmyaddress(host)
 
 	  hp = &hent;
 	  /* don't really care about gethostbyaddr() here */
-	  hp->h_name = host;
+	  hp->h_name     = (char*)host;
 	  hp->h_addrtype = af;
-	  hp->h_aliases = NULL;
-	  hp->h_length = addrsiz;
+	  hp->h_aliases  = NULL;
+	  hp->h_length   = addrsiz;
 	  addrs[0] = (void *)&au;
 	  addrs[1] = NULL;
 	  hp_setalist(hp, addrs);
@@ -325,7 +323,7 @@ stashmyaddress(host)
 	    ++naddrs;
 	  if (hp->h_addrtype == AF_INET)
 	    addrsiz = sizeof(struct sockaddr_in);
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	  else if (hp->h_addrtype == AF_INET6)
 	    addrsiz = sizeof(struct sockaddr_in6);
 #endif
@@ -353,7 +351,7 @@ stashmyaddress(host)
 	    si->sin_family = AF_INET;
 	    memcpy(&si->sin_addr.s_addr, *hp_getaddr(), hp->h_length);
 	  }
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	  if (hp->h_addrtype == AF_INET6) {
 	    struct sockaddr_in6 *si6;
 	    si6 = malloc(sizeof(*si6));
@@ -372,7 +370,7 @@ stashmyaddress(host)
 
 void
 stashmyaddresses(host)
-char *host;
+const char *host;
 {
 	char *s1, *s2, *zenv;
 	struct sockaddr **sa;
@@ -437,7 +435,7 @@ matchmyaddress(sa)
 	  si = (struct sockaddr_in *)sa;
 
 #ifndef INADDR_LOOPBACK /* FreeBSD does not have this in  <netinet/in.h> ! */
-# define INADDR_LOOPBACK 0x7f000001
+# define INADDR_LOOPBACK 0x7f000001UL
 #endif
 #define INADDR_LOOPBACKNET (IN_CLASSA_NET & INADDR_LOOPBACK)
 
@@ -456,7 +454,7 @@ matchmyaddress(sa)
 		memcmp(sa, myaddrs[i],
 		       sizeof(struct sockaddr_in)) == 0)
 	      return 1;
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
 	    if (sa->sa_family == AF_INET6 &&
 		memcmp(sa, myaddrs[i],
 		       sizeof(struct sockaddr_in6)) == 0)
@@ -468,45 +466,13 @@ matchmyaddress(sa)
 }
 
 int
-matchmyaddresses(hp)
-	struct hostent *hp;
+matchmyaddresses(ai)
+	struct addrinfo *ai;
 {
-	char **alist;
-	int i;
-	struct sockaddr_in si;
-#ifdef AF_INET6
-	struct sockaddr_in6 si6;
-#endif
-#ifndef	h_addr	/* no h_addr macro -- presumably 4.2BSD or earlier.. */
-	char *fakealist[2];
-
-	alist = fakealist;
-	fakealist[0] = hp->h_addr;
-	fakealist[1] = NULL;
-#else
-	alist = hp->h_addr_list;
-#endif
-	memset(&si,0,sizeof(si));
-	si.sin_family = AF_INET;
-#ifdef AF_INET6
-	memset(&si6,0,sizeof(si6));
-	si6.sin6_family = AF_INET;
-#endif
-
-	while (alist && *alist) {
-	  if (hp->h_addrtype == AF_INET) {
-	    memcpy(&si.sin_addr.s_addr, *alist, hp->h_length);
-	    if ((i = matchmyaddress((struct sockaddr*)&si)))
+	for ( ; ai ; ai = ai->ai_next ) {
+	  int i = matchmyaddress(ai->ai_addr);
+	  if (i != 0)
 	      return i;
-	  }
-#ifdef AF_INET6
-	  if (hp->h_addrtype == AF_INET) {
-	    memcpy(&si6.sin6_addr.s6_addr, *alist, hp->h_length);
-	    if ((i = matchmyaddress((struct sockaddr*)&si6)))
-	      return i;
-	  }
-#endif
-	  ++alist;
 	}
 	return 0;
 }
@@ -534,7 +500,7 @@ char *argv[];
       inet_ntop(AF_INET, &((struct sockaddr_in **) sa)[i]->sin_addr, buf, sizeof(buf));
       printf("IPv4: [%s]\n", buf);
       break;
-#ifdef AF_INET6
+#if defined(AF_INET6) && defined(INET6)
     case AF_INET6:
       inet_ntop(AF_INET6, &((struct sockaddr_in6 **) sa)[i]->sin6_addr, buf, sizeof(buf));
       printf("IPv6: [ipv6.%s]\n");
