@@ -33,11 +33,16 @@
 #undef	sp_install
 #endif
 
-extern struct spblk *_sp_enq();		/* insert item into the tree */
-extern struct spblk *_sp_deq();	/* return and remove lowest item in subtree */
-extern void _splay();			/* reorganize tree */
-extern struct spblk *sp_fhead();	/* fast non-splaying head */
-extern struct spblk *sp_fnext();	/* fast non-splaying next */
+extern struct spblk * _sp_enq __((struct spblk *, struct sptree *));
+			/* insert item into the tree */
+extern struct spblk * _sp_deq __((struct spblk **));
+			/* return and remove lowest item in subtree */
+extern void _splay __((struct spblk *, struct sptree *));
+			/* reorganize tree */
+extern struct spblk * sp_fhead __((struct sptree *));
+			/* fast non-splaying head */
+extern struct spblk * sp_fnext __((struct spblk *));
+			/* fast non-splaying next */
 
 /* USER SUPPLIED: char *emalloc() */
 
@@ -84,7 +89,7 @@ sp_lookup(key, q)
 struct spblk *
 sp_install(key, data, mark, q)
 	register spkey_t key;
-	register unsigned char *data;
+	register const void *data;
 	register long mark;
 	register struct sptree *q;
 {
@@ -96,6 +101,7 @@ sp_install(key, data, mark, q)
 		n->leftlink = NULL;
 		n->rightlink = NULL;
 		n->uplink = NULL;
+		q->eltscnt += 1;
 		(void) _sp_enq(n, q);
 	}
 
@@ -139,7 +145,7 @@ sp_fhead(q)
 
 void
 sp_scan(f, n, q)
-	register int (*f)();
+	register int (*f) __((struct spblk *));
 	register struct spblk *n;
 	register struct sptree *q;
 {
@@ -151,7 +157,7 @@ sp_scan(f, n, q)
 	}
 }
 
-
+static void _sp_chop __((struct spblk *));
 static void
 _sp_chop(n)
 	register struct spblk *n;
@@ -176,6 +182,7 @@ sp_null(q)
 	if (q->root == NULL)
 		return;
 	_sp_chop(q->root);
+	q->eltscnt = 0;
 	q->lookups = 0;
 	q->lkpcmps = 0;
 	q->enqs = 0;
@@ -230,7 +237,7 @@ sp_fnext(n)
 } /* sp_fnext */
 
 
-char *
+const char *
 sp_stats(q)
 	struct sptree *q;
 {
@@ -246,8 +253,8 @@ sp_stats(q)
 	elen = q->enqs ? (float)q->enqcmps/q->enqs : 0;
 	sloops = q->splays ? (float)q->splayloops/q->splays : 0;
 
-	(void) sprintf(buf, "f(%d %4.2f) i(%d %4.2f) s(%d %4.2f)",
-	    q->lookups, llen, q->enqs, elen, q->splays, sloops);
+	sprintf(buf, "f(%d %4.2f) i(%d %4.2f) s(%d %4.2f)",
+		q->lookups, llen, q->enqs, elen, q->splays, sloops);
 
 	return buf;
 }
@@ -352,6 +359,7 @@ sp_delete(n, q)
 		q->root = x;
 	}
 	free((char *)n);
+	q->eltscnt -= 1;
 } /* sp_delete */
 
 
@@ -410,13 +418,14 @@ sp_init()
 
 	q = (struct sptree *) emalloc(sizeof(*q));
 
+	q->eltscnt = 0;
 	q->lookups = 0;
 	q->lkpcmps = 0;
-	q->enqs = 0;
+	q->enqs    = 0;
 	q->enqcmps = 0;
-	q->splays = 0;
+	q->splays  = 0;
 	q->splayloops = 0;
-	q->root = NULL;
+	q->root    = NULL;
 	return q;
 }
 
