@@ -8,10 +8,13 @@
  * that can be used as an index into various tables.
  */
 
-#include <stdio.h>
 #include "hostenv.h"
+#include <stdio.h>
+#include <sys/types.h>
+
 #include "splay.h"
 #include "malloc.h"
+#include "libz.h"
 
 #ifdef	symbol
 #undef	symbol
@@ -45,6 +48,7 @@
 
 static long CrcTable[128];
 
+static void crcinit __((void));
 static void
 crcinit()
 {       
@@ -62,7 +66,7 @@ crcinit()
 
 
 struct syment {
-	u_char	*name;
+	const char    *name;
 	struct syment *next;
 };
 
@@ -70,12 +74,13 @@ struct sptree *spt_symtab = NULL;
 
 spkey_t
 symbol(s)
-	u_char	*s;
+	const void *s;
 {
-	register u_char	*ucp;
+	register const char *ucp;
 	register spkey_t key;
 	register struct syment *se, *pe;
-	static u_char *idname = (u_char *)"";
+	char *newname;
+	static const char *idname = "";
 	static spkey_t id, idkey = 1;
 	struct spblk *spl;
 
@@ -90,14 +95,15 @@ symbol(s)
 	for (ucp = s; *ucp != '\0'; ++ucp)
 		key = (key >> 7) ^ CrcTable[(key ^ *ucp) & 0x7f];
 
-	if (key == idkey && strcmp((char *)s, (char *)idname) == 0)
+	if (key == idkey && strcmp(s, (const char *)idname) == 0)
 		return id;
 	idkey = key;
 	pe = NULL;
-	if ((spl = sp_lookup(key, spt_symtab)) != NULL) {
+	spl = sp_lookup(key, spt_symtab);
+	if (spl != NULL) {
 		se = (struct syment *)spl->data;
 		do {
-			if (strcmp((char *)se->name, (char *)s) == 0) {
+			if (strcmp(se->name, s) == 0) {
 				idname = se->name;
 				id = (spkey_t)se;
 				return id;
@@ -107,14 +113,15 @@ symbol(s)
 		} while (se != NULL);
 	}
 	se = (struct syment *)emalloc(sizeof (struct syment));
-	se->name = (u_char *)emalloc((u_int)(strlen((char *)s)+1));
-	(void) strcpy((char *)se->name, (char *)s);
+	newname = (char *) emalloc((u_int)(strlen(s)+1));
+	strcpy(newname, s);
+	se->name = newname;
 	idname = se->name;
 	se->next = NULL;
 	if (pe != NULL)
 		pe->next = se;
 	else
-		(void) sp_install(key, (u_char *)se, 0, spt_symtab);
+		(void) sp_install(key, (const void *)se, 0, spt_symtab);
 	id = (spkey_t)se;
 	return id;
 }
@@ -123,11 +130,11 @@ symbol(s)
  * Return a printable string representation of the symbol whose key is passed.
  */
 
-char *
+const char *
 pname(id)
 	spkey_t id;
 {
-	return (char *)((struct syment *)id)->name;
+	return (const char *)((struct syment *)id)->name;
 }
 
 #ifdef	MALLOC_TRACE
