@@ -21,15 +21,21 @@ char *filename;
   char *s = buf;
   char *fn;
 
-  /* At first fill in the base-64 encoded mtime .. */
+  /* Start with a capital 'S' .. */
+  *s++ = 'S';
+
+  /* .. then fill in the base-64 encoded mtime .. */
   for (i = 5; i >= 0; --i)
     *s++ = base64chars[ (mtime >> (i*6)) & 63 ];
 
   fn = strrchr(filename,'/');
   if (!fn) fn = filename;
+  else ++fn;
+
+  /* .. and finally attach the inode-number part of the spool file name. */
 
   s = strchr(fn,'-'); if (s) *s = 0;
-  sprintf(buf+6,"%.*s", buflen-7, fn); /* Truncate if longer than we have
-					  room for it! */
+  sprintf(buf+7,"%.*s", buflen-8, fn); /* Truncate if longer than we have
+					  room for! */
   if (s) *s = '-';
 }
