@@ -11,16 +11,17 @@
 #include "mailer.h"
 #include "../libsh/io.h"
 #include <ctype.h>
+#include "libz.h"
 
 struct token *
 makeToken(s, n)
-	u_char	*s;
-	register int	n;
+	const char	*s;
+	register u_int	n;
 {
 	register struct token	*t;
 
 	t = (struct token *)tmalloc(sizeof (struct token));
-	t->t_pname = (u_char *)strnsave((char *)s, n);
+	t->t_pname = strnsave(s, n);
 	t->t_len = 0;
 	t->t_type = Empty;
 	t->t_next = 0;
@@ -48,11 +49,10 @@ copyToken(t)
 	}
 
 	ct = (struct token *)tmalloc(sizeof (struct token));
-	if (stickymem != MEM_TEMP) {
-		ct->t_pname = (u_char *)strnsave((char *)t->t_pname, tlen);
-	} else {
-		ct->t_pname = t->t_pname;
-	}
+	if (stickymem != MEM_TEMP)
+	  ct->t_pname = strnsave((const char *)t->t_pname, tlen);
+	else
+	  ct->t_pname = t->t_pname;
 	ct->t_len = tlen;
 
 	ct->t_type = t->t_type;
@@ -60,11 +60,11 @@ copyToken(t)
 	return ct;
 }
 
-char *
+const char *
 formatToken(t)
 	register struct token *t;
 {
-	char *name;
+	const char *name;
 	int len;
 	static char buf[256];
 
@@ -90,7 +90,7 @@ formatToken(t)
 	return buf;
 }
 
-char *
+const char *
 formatAddr(d)
 	AddrComponent d;
 {
@@ -111,11 +111,12 @@ formatAddr(d)
 
 int
 printToken(buf, eob, t, tend, quotespecials)
-	u_char *buf, *eob;
+	char *buf, *eob;
 	register struct token *t, *tend;
 	int quotespecials;
 {
-	register u_char *cp, *s;
+	register char *cp;
+	register const char *s;
 	register int len;
 
 	--eob;		/* make space for terminating NUL */
@@ -139,11 +140,12 @@ printToken(buf, eob, t, tend, quotespecials)
 
 int
 printdToken(bufp, buflenp, t, tend, quotespecials)
-	u_char **bufp;
+	char **bufp;
 	register struct token *t, *tend;
 	int quotespecials, *buflenp;
 {
-	register u_char *cp, *s, *buf = *bufp;
+	register char *cp, *buf = *bufp;
+	register const char *s;
 	register int len, buflen = *buflenp;
 
 	for (cp = buf; t != NULL && t != tend; t = t->t_next) {
@@ -218,8 +220,8 @@ fprintFold(fp, t, col)
 	struct token *t;
 	int col;
 {
-	int len, sawlwsp;
-	u_char *cp, *ncp;
+	int len;
+	const char *cp, *ncp;
 
 	len = TOKENLEN(t);
 	if (t->t_type == DomainLiteral)
@@ -235,39 +237,46 @@ fprintFold(fp, t, col)
 		if (col < LINELEN && len > (LINELEN-col)) {
 			/* there is a breakpoint */
 			for (ncp = cp + (LINELEN-col); ncp > cp; --ncp) {
-				if (isascii(*ncp)
-				    && (isspace(*ncp) || *ncp == ','))
+				if (isascii((*ncp)&0xFF)
+				    && (isspace((*ncp)&0xFF) || *ncp == ','))
 					break;
 			}
 			if (ncp == cp) {
 				for (ncp = cp + (LINELEN-col); ncp < cp + len;
 								++ncp)
-					if (isascii(*ncp)
-					    && (isspace(*ncp) || *ncp == ','))
+					if (isascii((*ncp)&0xFF)
+					    && (isspace((*ncp)&0xFF) ||
+						*ncp == ','))
 						break;
 			}
 			/* found breakpoint */
 		} else
 			ncp = cp + len;
 		while (cp < ncp && len > 0) {
-			if (isascii(*cp) && isspace(*cp)) {
-				++col, putc(' ', fp);
-				while (cp < ncp && len > 0 && isascii(*cp) && isspace(*cp))
-					--len, ++cp;
-			} else {
-				++col, putc(*cp, fp);
-				--len, ++cp;
-			}
+		  int c = (*cp) & 0xFF;
+		  if (isascii(c) && isspace(c)) {
+		    ++col, putc(' ', fp);
+		    while (cp < ncp && len > 0) {
+		      c = (*cp) & 0xFF;
+		      if (!(isascii(c) && isspace(c)))
+			break;
+		      --len, ++cp;
+		    }
+		  } else {
+		    ++col, putc(*cp, fp);
+		    --len, ++cp;
+		  }
 		}
 		if (len > 0) {
-			/* gobble LWSP at beginning of line */
-			while (len > 0 && isascii(*cp) && isspace(*cp))
-				--len, ++cp;
-			if (len > 0) {
-				putc('\n', fp);
-				putc('\t', fp);
-				col = 8;
-			}
+		  /* gobble LWSP at beginning of line */
+		  while (len > 0 &&
+			 isascii((*cp)&0xFF) && isspace((*cp)&0xFF))
+		    --len, ++cp;
+		  if (len > 0) {
+		    putc('\n', fp);
+		    putc('\t', fp);
+		    col = 8;
+		  }
 		}
 	} while (len > 0);
 		
