@@ -13,14 +13,23 @@ static FILE *fopenlog = NULL;
 
 extern int errno;
 
+#ifdef __STDC__
+# define __(x) x
+#else
+# define __(x) ()
+#endif
+
+
+static void fopen_init __((void));
 static void fopen_init()
 {
   fopenlog = stderr;
   initialized = 1;
 }
 
+static void fopenerr __((const char *, const char *, int, int));
 static void fopenerr(name,msg,dumpfps,oerrno)
-char *name, *msg;
+const char *name, *msg;
 int dumpfps, oerrno;
 {
 	if (!fopenlog) abort();
@@ -42,7 +51,7 @@ int dumpfps, oerrno;
 
 
 FILE *__fopen(filename, type)
-     char *filename, *type;
+	const char *filename, *type;
 {
 	FILE *fp = fopen(filename, type);
 	int i;
@@ -69,8 +78,8 @@ FILE *__fopen(filename, type)
 }
 
 FILE *__freopen(filename,type,stream)
-     char *filename, *type;
-     FILE *stream;
+	const char *filename, *type;
+	FILE *stream;
 {
 	FILE *fp = freopen(filename, type, stream);
 	int i;
@@ -97,8 +106,8 @@ FILE *__freopen(filename,type,stream)
 }
 
 FILE *__fdopen(fd,type)
-     int fd;
-     char *type;
+	int fd;
+	const char *type;
 {
 	FILE *fp = fdopen(fd,type);
 	int i;
@@ -125,7 +134,7 @@ FILE *__fdopen(fd,type)
 }
 
 int __fclose(stream)
-     FILE *stream;
+	FILE *stream;
 {
 	int i;
 
