@@ -17,14 +17,9 @@
 #include <sys/socket.h>
 #include "mail.h"
 
-extern char *fullname();
-extern char *getenv();
-#ifndef strchr
-extern char *strchr();
-extern char *strrchr();
-#endif
-extern char *whathost();
-extern int errno;
+#include "listutils.h"
+#include "libc.h"
+#include "libsh.h"
 
 /*
  * Standard routines that may be used by any program to submit mail.
@@ -63,7 +58,7 @@ extern int errno;
 /* array of message file name associated with a file descriptor */
 static char **mail_file = NULL;
 static int mail_nfiles  = 0;
-char *postoffice;	/* may be extern or local */
+const char *postoffice;	/* may be extern or local */
 
 /*
   Define sending mail priority.
@@ -83,12 +78,13 @@ FILE *
 _mail_fopen(filenamep)
 	char **filenamep;
 {
-	char *path, *cp, *suffix, *post;
+	const char *suffix, *post;
+	char *path, *cp;
 	FILE *fp;
 	int fd, eno;
 
 	if (postoffice == NULL && (postoffice = getzenv("POSTOFFICE")) == NULL)
-		postoffice = POSTOFFICE;
+	  postoffice = POSTOFFICE;
 	path = mail_alloc(strlen(postoffice)+strlen(*filenamep)+2);
 	sprintf(path, "%s/%s", postoffice, *filenamep);
 	for (cp = *filenamep; *cp != '\0' && *cp != '%'; ++cp)
@@ -133,7 +129,8 @@ mail_link(from, tonamep)
 	const char *from;
 	char **tonamep;
 {
-	char *path, *cp, *suffix, *post;
+	char *path, *cp;
+	const char *suffix, *post;
 	int eno;
 
 	if (postoffice == NULL && (postoffice = getzenv("POSTOFFICE")) == NULL)
@@ -177,27 +174,27 @@ FILE *
 mail_open(type)
 	const char *type;
 {
-	char *scratch, *message, *cp;
+	char *scratch, *message;
+	const char *cp;
 	FILE *fp;
 	int eno;
 	struct stat stbuf;
 	char namebuf[BUFSIZ];
-	static char *host = NULL;
+	static const char *host = NULL;
 	
 	/* Create a file, any file, in the PUBLIC directory */
 
 	if (host == NULL)
 		host = mail_host();
-	cp = host == NULL ? "I" : host ;
+	cp = (host == NULL) ? "I" : host ;
 	scratch = mail_alloc(strlen(PUBLICDIR)+strlen(cp)+3+1+10);
 
-	sprintf(scratch, "%s/%7s:%d%%", PUBLICDIR,
-		host == NULL ? "I" : host, getpid());
+	sprintf(scratch, "%s/%7s:%d%%", PUBLICDIR, cp, getpid());
 
-	if ((fp = _mail_fopen(&scratch)) == NULL) {
+	fp = _mail_fopen(&scratch);
+	if (fp == NULL) {
 		eno = errno;
-		fprintf(stderr,
-			"mail_fopen(\"%s\", \"w+\"): errno %d\n",
+		fprintf(stderr, "mail_fopen(\"%s\", \"w+\"): errno %d\n",
 			scratch, errno);
 		mail_free(scratch);
 		errno = eno;
@@ -223,7 +220,7 @@ mail_open(type)
 	if (type == NULL)
 		type = MSG_RFC822;
 	message = mail_alloc(strlen(PUBLICDIR)+strlen(type)+1+1+10);
-	sprintf(message, "%s/%d%%%s", PUBLICDIR, stbuf.st_ino, type);
+	sprintf(message, "%s/%d%%%s", PUBLICDIR, (int)stbuf.st_ino, type);
 #endif
 	if (mail_link(scratch, &message) < 0) {
 		eno = errno;
@@ -244,7 +241,8 @@ mail_open(type)
 	  if (mail_file == NULL) {
 	    mail_file = (char**)mail_alloc((u_int)(sizeof(char*) * nfile));
 	  } else {
-	    mail_file = (char**)mail_realloc(mail_file,(sizeof(char*) * nfile));
+	    mail_file = (char**)mail_realloc((char*)mail_file,
+					     (sizeof(char*) * nfile));
 	  }
 	  while (mail_nfiles < nfile) {
 	    mail_file[mail_nfiles] = NULL;
@@ -259,19 +257,30 @@ mail_open(type)
 	if (type != NULL && *type != '\0')
 		fprintf(fp, "type %s\n", type);
 #endif
-	if ((cp = getenv("FULLNAME")) != NULL)
+	cp = getenv("FULLNAME");
+	if (cp != NULL)
 		fprintf(fp, "fullname %s\n",
 			fullname(cp, namebuf, sizeof namebuf, (char *)NULL));
-	if ((cp = getenv("PRETTYLOGIN")) != NULL)
+	cp = getenv("PRETTYLOGIN");
+	if (cp != NULL)
 		fprintf(fp, "loginname %s\n", cp);
 	/*
 	 * If the postoffice lives elsewhere, put our hostname
 	 * in the Received-from header, to aid in message tracing.
 	 */
-	if ((getzenv("MAILSERVER") != NULL
-	|| ((host = whathost(message)) != NULL && strcmp(host,"localhost"))) &&
-	    getmyhostname(namebuf, sizeof namebuf) == 0)
-		fprintf(fp, "rcvdfrom %s\n", namebuf);
+	host = whathost(message);
+	if (getzenv("MAILSERVER") != NULL ||
+	    (host != NULL && strcmp(host,"localhost") != 0))
+	  if (getmyhostname(namebuf, sizeof namebuf) == 0) {
+	    cp = getenv("LOGNAME");
+	    if (cp == NULL)
+	      cp = getenv("USERNAME");
+	    if (cp == NULL)
+	      cp = getenv("USER");
+	    if (cp == NULL)
+	      cp = "\"??\"";
+	    fprintf(fp, "rcvdfrom %s@%s\n", cp, namebuf);
+	  }
 	return fp;
 }
 
@@ -326,8 +335,8 @@ _mail_close_(fp,inop, mtimep)
 	time_t *mtimep;
 {
 	char **messagep, *message, *nmessage, *msgbase;
-	char *routerdir;
-	char *routerdirs, *s = NULL;
+	const char *routerdir;
+	char *s = NULL;
 	struct stat stb;
 
 	if (postoffice == NULL) {
@@ -356,6 +365,7 @@ _mail_close_(fp,inop, mtimep)
 	 * on NFS mounted postoffices if you want to guarantee not losing
 	 * data without being told about it.
 	 */
+
 	if (fflush(fp) != 0
 #ifdef HAVE_FSYNC
 	    || fsync(FILENO(fp)) < 0
@@ -382,7 +392,7 @@ _mail_close_(fp,inop, mtimep)
 	  if (routerdirs) {
 	    int i = mail_priority;
 	    char *rd = routerdirs;
-	    char *ord = routerdir;
+	    const char *ord = routerdir;
 #ifdef HAVE_ALLOCA
 	    nmessage = alloca(strlen(postoffice)+
 			      strlen(routerdirs)+3+strlen(msgbase));
@@ -459,9 +469,9 @@ _mail_close_(fp,inop, mtimep)
 	mail_free(message);
 
 	if (inop != NULL)
-	  *inop = stb.st_ino;
+	  *inop   = (int)    stb.st_ino;
 	if (mtimep != NULL)
-	  *mtimep = stb.st_mtime;
+	  *mtimep = (time_t) stb.st_mtime;
 
 	return 0;
 }
@@ -475,7 +485,7 @@ _mail_close_(fp,inop, mtimep)
 int
 mail_close_alternate(fp,where,suffix)
 	FILE *fp;
-	char *where, *suffix;
+	const char *where, *suffix;
 {
 	char **messagep, *message, *nmessage, *msgbase;
 	int eno;
