@@ -4,8 +4,8 @@
  */
 
 #include "hostenv.h"
-#ifndef	HAVE_SYS_SIGLIST
-char *sys_siglist[] = {
+#ifndef	HAVE_SYS_SIGLIST /* No  sys_siglist[]  at the libc ? */
+const char *sys_siglist[] = {
 	"Signal 0",
 	"Signal 1",
 	"Signal 2",
@@ -42,7 +42,7 @@ char *sys_siglist[] = {
 #endif	/* HAVE_SYS_SIGLIST */
 
 #ifndef HAVE_STRSIGNAL
-#ifndef HAVE_EXTERN_SYS_SIGLIST
+#ifndef SYS_SIGLIST_DECLARED /* Not declared anywhere ? */
 extern const char *sys_siglist[];
 #endif
 
