@@ -10,7 +10,7 @@
 # include <stdio.h>
 # include <string.h>
 # include <unistd.h>
-# include <sys/fcntl.h>
+# include <fcntl.h>
 
 #else /* ! STDHEADERS */
 
@@ -48,10 +48,9 @@ extern univptr_t memset proto((univptr_t, int, memsize_t));
 
 #ifndef __GNUC__	/* clash with builtin, garn */
 extern univptr_t memcpy proto((univptr_t, const univptr_t, memsize_t));
-#endif
-
 extern char *strcpy proto((char *, const char *));
 extern memsize_t strlen proto((const char *));
+#endif
 
 /* UNIX -- unistd.h */
 extern int write proto((int /*fd*/, const char * /*buf*/, int /*nbytes*/));
