@@ -73,7 +73,9 @@ static char rcsid[] = "$Id: res_send.c,v 8.8 1996/05/09 05:59:13 vixie Exp $";
 #include <sys/param.h>
 #include <sys/time.h>
 #include <sys/socket.h>
+#if defined(_AIX) /* AIX 3.2.5e5 requires this */
 #include <sys/select.h>
+#endif
 #include <sys/uio.h>
 #include <netinet/in.h>
 #include "arpa/nameser.h"
@@ -108,7 +110,7 @@ static int vc = 0;	/* is the socket a virtual ciruit? */
 #define	FD_SET(n, p)	((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
 #define	FD_CLR(n, p)	((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
 #define	FD_ISSET(n, p)	((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
-#define FD_ZERO(p)	bzero((char *)(p), sizeof(*(p)))
+#define FD_ZERO(p)	memset((char *)(p), 0, sizeof(*(p)))
 #endif
 
 /* XXX - this should be done in portability.h */
