@@ -15,6 +15,10 @@
 #include "io.h"		/* redefines stdio routines */
 #include "shconfig.h"
 
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
+
 #ifdef  HAVE_WAITPID
 # include <sys/wait.h>
 #else
@@ -41,91 +45,91 @@ extern int wait();
 #define	HZ	60
 #endif	/* HZ */
 
-extern struct conscell *sh_car(), *sh_cdr(), *sh_list(), *sh_grind();
-extern struct conscell *sh_elements(), *sh_setf(), *sh_get(), *sh_length();
-extern struct conscell *sh_return(),*sh_returns(), *sh_last();
-
-extern int sh_read(), sh_echo(), sh_read(), sh_test(), sh_include();
-extern int sh_cd(), sh_colon(), sh_hash(), sh_bce(), sh_eval(), sh_export();
-extern int sh_getopts(), sh_shift(), sh_umask(), sh_set(), sh_unset();
-extern int sh_wait(), sh_times(), sh_trap(), sh_type(), sh_builtin();
-extern int sh_sleep(), sh_expr(), sh_true(), sh_false();
-
-
-extern char *ifs;
-extern char *optarg;
-extern char *path_hash();
-extern char *prepath();
-extern char *strerror();
-extern char *strsignal();
-#ifndef strchr
-extern char *strchr();
-#endif
 extern int errno;
-extern int eval();
-extern int loadeval();
-extern int optind, opterr;
-extern struct codedesc *interpret();
-extern struct conscell *envarlist;
-extern struct osCmd *globalcaller;
-extern void assign();
-extern void functype();
-extern void ifs_flush();
-extern void path_flush();
-extern void v_export();
-extern void v_set();
-extern void v_touched();
+
+#define CSARGS2 __((conscell *, conscell *))
+
+extern conscell *sh_car		CSARGS2;
+static conscell *sh_cdr		CSARGS2;
+static conscell *sh_list	CSARGS2;
+static conscell *sh_grind	CSARGS2;
+static conscell *sh_elements	CSARGS2;
+static conscell *sh_setf	CSARGS2;
+static conscell *sh_get		CSARGS2;
+static conscell *sh_length	CSARGS2;
+static conscell *sh_last	CSARGS2;
+
+#define CSARGV2 __((int argc, const char *argv[]))
+
+static int sh_read	CSARGV2;
+static int sh_echo	CSARGV2;
+static int sh_cd	CSARGV2;
+static int sh_colon	CSARGV2;
+static int sh_hash	CSARGV2;
+static int sh_bce	CSARGV2;
+static int sh_eval	CSARGV2;
+static int sh_export	CSARGV2;
+static int sh_getopts	CSARGV2;
+static int sh_shift	CSARGV2;
+static int sh_umask	CSARGV2;
+static int sh_set	CSARGV2;
+static int sh_unset	CSARGV2;
+static int sh_wait	CSARGV2;
+static int sh_times	CSARGV2;
+static int sh_type	CSARGV2;
+static int sh_sleep	CSARGV2;
+static int sh_true	CSARGV2;
+static int sh_false	CSARGV2;
 
 
 struct shCmd builtins[] = {
-{	"car",		0,		sh_car,		SH_ARGV		},
-{	"first",	0,		sh_car,/*sic*/	SH_ARGV		},
-{	"cdr",		0,		sh_cdr,		SH_ARGV		},
-{	"rest",		0,		sh_cdr,/*sic*/	SH_ARGV		},
-{	"last",		0,		sh_last,	SH_ARGV		},
-{	"list",		0,		sh_list,	SH_ARGV		},
-{	"grind",	0,		sh_grind,	SH_ARGV		},
-{	"elements",	0,		sh_elements,	SH_ARGV		},
-{	"setf",		0,		sh_setf,	SH_ARGV		},
-{	"get",		0,		sh_get,		SH_ARGV		},
-{	"length",	0,		sh_length,	SH_ARGV		},
-{	"[",		sh_test,	0,		0		},
-{	"test",		sh_test,	0,		0		},
-{	"echo",		sh_echo,	0,		0		},
-{	"cd",		sh_cd,		0,		0		},
-{	"hash",		sh_hash,	0,		0		},
-{	"read",		sh_read,	0,		0		},
-{	":",		sh_colon,	0,		0		},
-{	".",		sh_include,	0,		0		},
-{	"break",	sh_bce,		0,		SH_INTERNAL	},
-{	"continue",	sh_bce,		0,		SH_INTERNAL	},
-{	"return",	0,		sh_return,	SH_ARGV		},
-{	"returns",	0,		sh_returns,	SH_ARGV		},
-{	"exit",		sh_bce,		0,		SH_INTERNAL	},
-{	"expr",		sh_expr,	0,		SH_ARGV		},
-{	"eval",		sh_eval,	0,		0		},
-{	"export",	sh_export,	0,		0		},
-{	"getopts",	sh_getopts,	0,		0		},
-{	"shift",	sh_shift,	0,		0		},
-{	"umask",	sh_umask,	0,		0		},
-{	"set",		sh_set,		0,		0		},
-{	"unset",	sh_unset,	0,		0		},
-{	"wait",		sh_wait,	0,		0		},
-{	"times",	sh_times,	0,		0		},
-{	"trap",		sh_trap,	0,		0		},
-{	"type",		sh_type,	0,		0		},
-{	"builtin",	sh_builtin,	0,		0		},
-{	"sleep",	sh_sleep,	0,		0		},
-{	"true",		sh_true,	0,		0		},
-{	"false",	sh_false,	0,		0		},
-{	0,		0,		0,		0		},
+{	"car",		NULL,	sh_car,		NULL,	SH_ARGV		},
+{	"first",	NULL,	sh_car,/*sic*/	NULL,	SH_ARGV		},
+{	"cdr",		NULL,	sh_cdr,		NULL,	SH_ARGV		},
+{	"rest",		NULL,	sh_cdr,/*sic*/	NULL,	SH_ARGV		},
+{	"last",		NULL,	sh_last,	NULL,	SH_ARGV		},
+{	"list",		NULL,	sh_list,	NULL,	SH_ARGV		},
+{	"grind",	NULL,	sh_grind,	NULL,	SH_ARGV		},
+{	"elements",	NULL,	sh_elements,	NULL,	SH_ARGV		},
+{	"setf",		NULL,	sh_setf,	NULL,	SH_ARGV		},
+{	"get",		NULL,	sh_get,		NULL,	SH_ARGV		},
+{	"length",	NULL,	sh_length,	NULL,	SH_ARGV		},
+{	"[",		sh_test,	NULL,	NULL,	0		},
+{	"test",		sh_test,	NULL,	NULL,	0		},
+{	"echo",		sh_echo,	NULL,	NULL,	0		},
+{	"cd",		sh_cd,		NULL,	NULL,	0		},
+{	"hash",		sh_hash,	NULL,	NULL,	0		},
+{	"read",		sh_read,	NULL,	NULL,	0		},
+{	":",		sh_colon,	NULL,	NULL,	0		},
+{	".",		sh_include,	NULL,	NULL,	0		},
+{	"break",	sh_bce,		NULL,	NULL,	SH_INTERNAL	},
+{	"continue",	sh_bce,		NULL,	NULL,	SH_INTERNAL	},
+{	"return",	NULL,	NULL,	sh_return,	SH_ARGV		},
+{	"returns",	NULL,	NULL,	sh_returns,	SH_ARGV		},
+{	"exit",		sh_bce,		NULL,	NULL,	SH_INTERNAL	},
+{	"eval",		sh_eval,	NULL,	NULL,	0		},
+{	"export",	sh_export,	NULL,	NULL,	0		},
+{	"getopts",	sh_getopts,	NULL,	NULL,	0		},
+{	"shift",	sh_shift,	NULL,	NULL,	0		},
+{	"umask",	sh_umask,	NULL,	NULL,	0		},
+{	"set",		sh_set,		NULL,	NULL,	0		},
+{	"unset",	sh_unset,	NULL,	NULL,	0		},
+{	"wait",		sh_wait,	NULL,	NULL,	0		},
+{	"times",	sh_times,	NULL,	NULL,	0		},
+{	"trap",		sh_trap,	NULL,	NULL,	0		},
+{	"type",		sh_type,	NULL,	NULL,	0		},
+{	"builtin",	sh_builtin,	NULL,	NULL,	0		},
+{	"sleep",	sh_sleep,	NULL,	NULL,	0		},
+{	"true",		sh_true,	NULL,	NULL,	0		},
+{	"false",	sh_false,	NULL,	NULL,	0		},
+{	NULL,		NULL,		NULL,	NULL,	0		},
 };
 
-struct conscell *
+conscell *
 sh_car(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *tmp;
+	conscell *tmp;
 
 	il = cdar(avl);
 	if (il == NULL)
@@ -145,9 +149,9 @@ sh_car(avl, il)
 	return car(il);
 }
 
-struct conscell *
+static conscell *
 sh_cdr(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
@@ -166,9 +170,9 @@ sh_cdr(avl, il)
 	return il;
 }
 
-struct conscell *
+static conscell *
 sh_last(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
@@ -186,26 +190,26 @@ sh_last(avl, il)
 	return il;
 }
 
-struct conscell *
+static conscell *
 sh_list(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
 	car(avl) = cdar(avl);
 	return avl;
 }
 
-struct conscell *
+static conscell *
 sh_grind(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
 	return cdar(avl);
 }
 
-struct conscell *
+static conscell *
 sh_elements(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *p;
+	conscell *p;
 
 	if ((p = cdar(avl)) == NULL || !LIST(p))
 		return p;
@@ -214,11 +218,11 @@ sh_elements(avl, il)
 	return cadar(avl);
 }
 
-struct conscell *
+static conscell *
 sh_setf(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *place, *value, *tmp = NULL;
+	conscell *place, *value, *tmp = NULL;
 	memtypes oval = stickymem;
 
 	/*
@@ -289,11 +293,11 @@ sh_setf(avl, il)
 	return cddar(avl);
 }
 
-struct conscell *
+static conscell *
 sh_get(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *plist, *key, *d, *tmp;
+	conscell *plist, *key, *d, *tmp;
 
 	if ((plist = cdar(avl)) == NULL
 	    || (key = cddar(avl)) == NULL
@@ -303,14 +307,15 @@ sh_get(avl, il)
 		return NULL;
 	}
 	if (STRING(plist)) {
-		if ((d = v_find(plist->string)) == NULL) {
+		d = v_find((const char *)plist->string);
+		if (d == NULL) {
 			/* (setq plist '(key nil)) */
 			d = conststring(key->string);
 			cdr(d) = NIL;
 			d = ncons(d);
 			assign(plist, d, (struct osCmd *)NULL);
 			/* setf preparation */
-			cdar(d)->prev = cadr(v_find(plist->string));
+			cdar(d)->prev = cadr(v_find((const char*)plist->string));
 			cdar(d)->pflags = 1;
 			return cdar(d);
 		}
@@ -325,7 +330,7 @@ sh_get(avl, il)
 		/* if (STRING(d))
 			printf("comparing '%s' and '%s'\n",
 				d->string, key->string);	*/
-		if (STRING(d) && strcmp((char *)d->string, (char *)key->string) == 0) {
+		if (STRING(d) && strcmp(d->string, key->string) == 0) {
 			d = copycell(cdr(d));
 			cdr(d) = NULL;
 			d->pflags |= 04;
@@ -360,12 +365,12 @@ sh_get(avl, il)
 	return d;
 }
 
-struct conscell *
+static conscell *
 sh_length(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
 	char buf[10];
-	struct conscell *tmp;
+	conscell *tmp;
 	int len = 0;
 
 	if ((il = cdar(avl)) && LIST(il)) {
@@ -374,14 +379,14 @@ sh_length(avl, il)
 	}
 	sprintf(buf, "%d", len);
 
-	avl = newstring((u_char *)strsave(buf));
+	avl = newstring(strsave(buf));
 	return avl;
 }
 
 /* returns -- return ALWAYS a string [mea] */
-struct conscell *
+conscell *
 sh_returns(avl, il, statusp)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 	int *statusp;
 {
 	int n;
@@ -392,13 +397,15 @@ sh_returns(avl, il, statusp)
 	else if (il == NULL)
 		return NULL;
 	else if (STRING(il)) {
-		for (cp = (char *)il->string; *cp != 0; ++cp) {
-			if (!isascii(*cp) || !isdigit(*cp))
-				break;
+		for (cp = il->string; *cp != 0; ++cp) {
+		  int c = (*cp) & 0xFF;
+		  if (!isascii(c) || !isdigit(c))
+		    break;
 		}
 		if (*cp)
 			return il;
-		if ((n = atoi((char *)il->string)) < 0) {
+		n = atoi(il->string);
+		if (n < 0) {
 			fprintf(stderr, "%s: %s: %d\n", car(avl)->string,
 					NEGATIVE_VALUE, n);
 			n = 1;
@@ -410,9 +417,9 @@ sh_returns(avl, il, statusp)
 	return NULL;
 }
 
-struct conscell *
+conscell *
 sh_return(avl, il, statusp)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 	int *statusp;
 {
 	int n;
@@ -423,13 +430,15 @@ sh_return(avl, il, statusp)
 	else if (il == NULL)
 		return NULL;
 	else if (STRING(il)) {
-		for (cp = (char *)il->string; *cp != 0; ++cp) {
-			if (!isascii(*cp) || !isdigit(*cp))
-				break;
+		for (cp = il->string; *cp != 0; ++cp) {
+		  int c = (*cp) & 0xFF;
+		  if (!isascii(c) || !isdigit(c))
+		    break;
 		}
-		if (*cp)
+		if (*cp != 0)
 			return il;
-		if ((n = atoi((char *)il->string)) < 0) {
+		n = atoi(il->string);
+		if (n < 0) {
 			fprintf(stderr, "%s: %s: %d\n", car(avl)->string,
 					NEGATIVE_VALUE, n);
 			n = 1;
@@ -443,10 +452,10 @@ sh_return(avl, il, statusp)
 
 /*-*/
 
-int
+static int
 sh_colon(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	return 0;
 }
@@ -454,15 +463,15 @@ sh_colon(argc, argv)
 int
 sh_builtin(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	return 0;
 }
 
-int
+static int
 sh_echo(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int n;
 
@@ -484,13 +493,14 @@ sh_echo(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_cd(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	char *dir, *cddir, *path;
-	struct conscell *d;
+	const char *dir;
+	char *cddir, *path;
+	conscell *d;
 	u_int pathlen;
 
 	if (argc == 1) {
@@ -500,7 +510,7 @@ sh_cd(argc, argv)
 			fprintf(stderr, "%s: %s\n", argv[0], NO_HOME_DIRECTORY);
 			return 1;
 		}
-		dir = (char *)cdr(d)->string;
+		dir = cdr(d)->string;
 	} else if (argc == 2) {
 		/* cd argv[1] */
 		dir = argv[1];
@@ -514,7 +524,7 @@ sh_cd(argc, argv)
 		(dir[1] == '.' && (dir[2] == '\0' || dir[2] == '/'))))) &&
 	    (d = v_find(CDPATH)) != NULL &&
 	    cdr(d) != NULL && STRING(cdr(d))) {
-		cddir = (char *)cdr(d)->string;
+		cddir = cdr(d)->string;
 		pathlen = strlen(cddir)+strlen(dir)+1+1;
 		path = tmalloc(pathlen);
 		while (cddir != NULL) {
@@ -532,10 +542,10 @@ sh_cd(argc, argv)
 	return 1;
 }
 
-int
+static int
 sh_hash(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	if (argc == 1) {
 		printf(NO_HASHING_INFORMATION);
@@ -552,13 +562,13 @@ sh_hash(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_read(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	static char *buf = NULL;
-	static int bufsize;
+	static u_int bufsize;
 	char *cp, *value = NULL, *bp;
 	int flag, offset;
 
@@ -651,13 +661,15 @@ eoinput:
 int
 sh_include(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	int fd, status;
-	struct conscell *d;
+	int fd;
+	int status;
+	conscell *d;
 	struct stat stbuf;
 	u_int pathlen;
 	char *dir, *buf, *path = NULL;
+	const char *rpath = NULL;
 
 	if (argc < 2 || argv[1][0] == '\0') {
 		fprintf(stderr, USAGE_INCLUDE, argv[0]);
@@ -667,18 +679,21 @@ sh_include(argc, argv)
 	if (strchr(argv[1], '/') == NULL) {
 		d = v_find(PATH);
 		if (d != NULL && cdr(d) != NULL && STRING(cdr(d))) {
-			dir = (char *)cdr(d)->string;
+			dir = cdr(d)->string;
 			pathlen = strlen(dir)+strlen(argv[1])+1+1;
 			path = tmalloc(pathlen);
+			rpath = path;
 			while (dir != NULL) {
 				dir = prepath(dir, argv[1], path, pathlen);
-				if ((fd = open(path, O_RDONLY, 0)) >= 0)
+				rpath = path;
+				fd = open(rpath, O_RDONLY, 0);
+				if (fd >= 0)
 					break;
 			}
 		}
 	} else {
-		path = argv[1];
-		fd = open(path, O_RDONLY, 0);
+		rpath = argv[1];
+		fd = open(rpath, O_RDONLY, 0);
 	}
 	if (fd < 0) {
 		fprintf(stderr, "%s: %s\n", argv[1], NOT_FOUND);
@@ -689,26 +704,27 @@ sh_include(argc, argv)
 		close(fd);
 		return 1;
 	}
-	status = loadeval(fd, path, &stbuf);
+	status = (int) loadeval(fd, rpath, &stbuf);
 	if (status >= 0) {
 		/* loadeval closes fd */
 		return status;
 	}
-	buf = tmalloc(stbuf.st_size+1);
-	if ((status = read(fd, buf, stbuf.st_size)) != stbuf.st_size) {
+	buf = tmalloc((u_int)stbuf.st_size + 1);
+	status = (int) read(fd, buf, (int)stbuf.st_size);
+	if (status != (int) stbuf.st_size) {
 		perror("read");
 		close(fd);
 		return 1;
 	}
 	close(fd);
-	buf[stbuf.st_size] = '\0';
-	return eval(buf, argv[1], path);
+	buf[(u_int)stbuf.st_size] = '\0';
+	return eval(buf, argv[1], rpath);
 }
 
-int
+static int
 sh_bce(argc, argv)		/* handle break, continue, and exit */
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int n;
 
@@ -730,14 +746,14 @@ sh_bce(argc, argv)		/* handle break, continue, and exit */
 	return n;
 }
 
-int
+static int
 sh_eval(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int len, i;
 	int status;
-	u_char *table, *eotable;
+	void *table, *eotable;
 	char *cp, *buf;
 
 	for (len = 0, i = 1; i < argc; ++i) {
@@ -763,18 +779,18 @@ sh_eval(argc, argv)
 			if (isset('V'))
 				table = optimize(1, table, &eotable);
 		}
-		interpret(table, eotable, (u_char *)NULL, globalcaller,
+		interpret(table, eotable, NULL, globalcaller,
 			  &status, (struct codedesc *)NULL);
 	}
 	return status;
 }
 
-int
+static int
 sh_export(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	struct conscell *elist, *glist, *scope;
+	conscell *elist, *glist, *scope;
 
 	if (argc > 1) {
 		/* export the listed variables */
@@ -793,8 +809,7 @@ sh_export(argc, argv)
 			continue;
 		for (glist = car(scope); glist != NULL; glist = cddr(glist)) {
 			if (STRING(glist) &&
-			    strcmp((char *)glist->string,
-				   (char *)elist->string) == 0)
+			    strcmp(glist->string, elist->string) == 0)
 				break;
 		}
 		if (glist == NULL)
@@ -803,14 +818,16 @@ sh_export(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_getopts(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	char buf[20], *optstring, **av, *name;
+	char buf[20];
+	const char *optstring, *name;
+	const char **av;
 	int c;
-	struct conscell *d;
+	conscell *d;
 
 	if (argc < 3) {
 		fprintf(stderr, USAGE_GETOPTS, argv[0]);
@@ -824,13 +841,17 @@ sh_getopts(argc, argv)
 	if (argc == 1) {
 		for (d = cdar(globalcaller->argv); d != NULL ; d = cdr(d))
 			++argc;
-		argv = av = (char **)tmalloc((argc+1)*sizeof (char *));
+		av = (const char **)tmalloc((argc+1)*sizeof (char *));
+		argv = av;
 		for (d = car(globalcaller->argv); d != NULL ; d = cdr(d))
-			if (STRING(d))
-				*av++ = (char *)d->string;
+			if (STRING(d)) {
+				*av++ = d->string;
+				++argc;
+			}
 		*av = NULL;
 	}
-	c = getopt(argc, argv, optstring);	/* this MUST be our getopt() */
+	c = getopt(argc, (char*const*)argv,
+		   optstring);	/* this MUST be our getopt() */
 	sprintf(buf, "%d", optind);
 	v_set(OPTIND, buf);
 	if (c == EOF)
@@ -842,13 +863,13 @@ sh_getopts(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_shift(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int n;
-	struct conscell *d;
+	conscell *d;
 
 	if (argc > 2 || (argc == 2 && !isdigit(argv[1][0]))) {
 		fprintf(stderr, USAGE_SHIFT, argv[0]);
@@ -865,13 +886,13 @@ sh_shift(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_umask(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int mask;
-	char *cp;
+	const char *cp;
 
 	if (argc > 2 || (argc == 2 && !isdigit(argv[1][0]))) {
 		fprintf(stderr, USAGE_UMASK, argv[0]);
@@ -885,20 +906,22 @@ sh_umask(argc, argv)
 	}
 	mask = 0;
 	for (cp = argv[1]; *cp != '\0'; ++cp)
-		mask = mask*8 + (*cp - '0');		/* XX: ebcdic?? */
+		mask = mask * 8 + (*cp - '0');		/* XX: ebcdic?? */
 	umask(mask);
 	return 0;
 }
 
-int
+static int
 sh_set(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int n, i;
 	memtypes oval;
-	char **kk, *cp, *ep;
-	struct conscell *d, *pd = NULL, *scope, *tmp;
+	const char *cp;
+	char **kk, *ep;
+	conscell *d, *pd = NULL, *scope, *tmp;
+	char ebuf[32];
 
 	if (argc == 1) {
 		/* print variable values */
@@ -937,7 +960,8 @@ sh_set(argc, argv)
 	--argc, ++argv;
 	if (argv[0][0] == '-' || argv[0][0] == '+') {
 		i = (argv[0][0] == '-');
-		for (cp = ep = &argv[0][1]; *cp != '\0'; ++cp) {
+		ep = ebuf;
+		for (cp = &argv[0][1]; *cp != '\0'; ++cp) {
 			switch (*cp) {
 			case 'a':/* automatically export changed variables */
 			case 'e':/* exit on error exit status of any command */
@@ -958,14 +982,15 @@ sh_set(argc, argv)
 				break;
 			case 'k':	/* we do not support the k option */
 			default:	/* complain */
-				*ep++ = *cp;
+				if (ep < (ebuf + sizeof(ebuf)-2))
+				  *ep++ = *cp;
 				break;
 			}
 		}
-		if (ep > &argv[0][1]) {
+		if (ep > ebuf) {
 			*ep = '\0';
 			fprintf(stderr, "%s: %s: %s\n",
-					argv[-1], BAD_OPTIONS, argv[0]);
+					argv[-1], BAD_OPTIONS, ebuf);
 		}
 		--argc, ++argv;
 	}
@@ -978,7 +1003,7 @@ sh_set(argc, argv)
 		pd = car(globalcaller->argv);
 	/* XX: possible memory leak here! */
 	while (argc-- > 0) {
-		d = newstring((u_char *)strsave(*argv));
+		d = newstring(strsave(*argv));
 		if (pd != NULL)
 			cdr(pd) = d;
 		++argv;
@@ -988,15 +1013,15 @@ sh_set(argc, argv)
 	return 0;
 }
 
-STATIC char *sacred[] = { PATH, PS1, PS2, IFS, MAILCHECK, 0 };
+static const char *sacred[] = { PATH, PS1, PS2, IFS, MAILCHECK, 0 };
 
-int
+static int
 sh_unset(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	struct conscell *d, *pd = NULL, *scope, *next;
-	char **cpp, *av0;
+	conscell *d, *pd = NULL, *scope, *next;
+	const char **cpp, *av0;
 
 	if (argc == 1) {
 		fprintf(stderr, USAGE_UNSET, argv[0]);
@@ -1036,10 +1061,10 @@ sh_unset(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_wait(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int retcode, pid;
 	int status = 0;
@@ -1065,10 +1090,10 @@ sh_wait(argc, argv)
 	return retcode;
 }
 
-int
+static int
 sh_times(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct tms foo;
 
@@ -1084,10 +1109,10 @@ sh_times(argc, argv)
 	return 0;
 }
 
-int
+static int
 sh_type(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	char *dir, *path;
 	struct shCmd *shcmdp;
@@ -1111,7 +1136,8 @@ sh_type(argc, argv)
 			continue;
 		}
 		/* unix command */
-		if ((dir = path_hash(*argv)) != NULL) {
+		dir = path_hash(*argv);
+		if (dir != NULL) {
 			pathlen = strlen(dir)+strlen(*argv)+1+1;
 			path = tmalloc(pathlen);
 			dir = prepath(dir, *argv, path, pathlen);
@@ -1124,10 +1150,10 @@ sh_type(argc, argv)
 }
 
 
-int
+static int
 sh_sleep(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	if (argc != 2 || atoi(argv[1]) <= 0) {
 		fprintf(stderr, USAGE_SLEEP, argv[0]);
@@ -1137,40 +1163,18 @@ sh_sleep(argc, argv)
 	return 0;
 }
 
-int
-sh_expr(argc,argv)
-	int argc;
-	char *argv[];
-{
-#ifndef	MAILER
-	fprintf(stderr, "%s: Not implemented w/o Zmailer\n",argv[0]);
-	return 1;
-#else
-	regexp *re = NULL;
-
-	if (argc != 4 || argv[2][0] != ':') {
-		fprintf(stderr, USAGE_EXPR, argv[0]);
-		return 1;
-	}
-/* XX: expr  code! */
-	re = NULL;
-	fprintf(stderr,"%s: Not yet implemented!\n",argv[0]);
-	return 1;
-#endif
-}
-
-int
+static int
 sh_true(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	return 0;
 }
 
-int
+static int
 sh_false(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	return 1;
 }
