@@ -20,7 +20,11 @@
 #include <sys/file.h>
 #include "shconfig.h"
 
-extern int vsiodoprnt();
+#define REALSTDIO
+#include "io.h"
+
+STATIC int vsiodoprnt __((struct siobuf *, const char *, va_list));
+
 #define NO_FLOAT_CONVERSION
 #ifndef NO_FLOAT_CONVERSION
 extern int cvt();
@@ -49,29 +53,27 @@ long std_ftell(fp) FILE *fp; { return ftell(fp); }
 
 char *std_gets(s) char *s; { return gets(s); }
 
-char *std_fgets(s, n, fp) char *s; int n; FILE *fp; { return fgets(s, n, fp); }
+char *std_fgets(s, n, fp) char *s; u_int n; FILE *fp; { return fgets(s, n, fp); }
 
-int std_puts(s) char *s; { return puts(s); }
+int std_puts(s) const char *s; { return puts(s); }
 
-int std_fputs(s, fp) char *s; FILE *fp; { return fputs(s, fp); }
+int std_fputs(s, fp) const char *s; FILE *fp; { return fputs(s, fp); }
 
 #ifdef HAVE_STDARG_H
 #if (defined(__GNUC__) && defined(sun) && !defined(__svr4__))
 extern int printf(); /* Sun 4.1.x with old GCC */
 #endif
-int (*std_printf) __((char *fmt, ...)) = (int(*)__((char *, ...)))printf;
+int (*std_printf) __((const char *fmt, ...)) = (int(*)__((const char *, ...)))printf;
 #else
 #if (defined(__GNUC__) && defined(sun) && !defined(__svr4__))
-extern int printf __((char *fmt, ...));
+extern int printf __((const char *fmt, ...));
 #endif
-int (*std_printf) __((char *fmt, ...)) = (int (*) __((char *fmt, ...)))printf;
+int (*std_printf) __((const char *fmt, ...)) = (int (*) __((const char *fmt, ...)))printf;
 #endif
 
-int std_fread(b,s,n,fp) char *b; int s, n; FILE *fp;{ return fread(b,s,n,fp); }
+int std_fread(b,s,n,fp) char *b; u_int s, n; FILE *fp;{ return fread(b,s,n,fp); }
 
-int std_fwrite(b,s,n,fp) char *b; int s, n; FILE *fp;{ return fwrite(b,s,n,fp); }
-
-#include "io.h"
+int std_fwrite(b,s,n,fp) const char *b; u_int s, n; FILE *fp;{ return fwrite(b,s,n,fp); }
 
 /* functionally equivalent (to stdio) routines to deal with string buffers */
 
@@ -87,7 +89,7 @@ siogets(s)
 	while (siop->sb_ptr + siop->sb_cnt < siop->sb_base + siop->sb_bufsiz) {
 	  if ((ch = (int)*siop->sb_ptr++) == '\0' || ch == '\n')
 	    break;
-	  *s++ = (char)ch;
+	  *s++ = ch;
 	}
 	*s = '\0';
 	return (s == os) ? NULL : os;
@@ -96,7 +98,7 @@ siogets(s)
 char *
 siofgets(s, n, fp)
 	register char *s;
-	register int n;
+	register u_int n;
 	FILE *fp;
 {
 	register struct siobuf *siop = siofds[FILENO(fp)];
@@ -107,7 +109,7 @@ siofgets(s, n, fp)
 	while (--n > 0 && siop->sb_ptr + siop->sb_cnt < siop->sb_base + siop->sb_bufsiz) {
 	  if ((ch = (int)*siop->sb_ptr++) == '\0')
 	    break;
-	  *s++ = (char)ch;
+	  *s++ = ch;
 	  if (ch == '\n')
 	    break;
 	}
@@ -117,7 +119,7 @@ siofgets(s, n, fp)
 
 int
 sioputs(s)
-	register char *s;
+	register const char *s;
 {
 	register struct siobuf *siop = siofds[FILENO(stdout)];
 
@@ -133,7 +135,7 @@ sioputs(s)
 
 int
 siofputs(s, fp)
-	register char *s;
+	register const char *s;
 	FILE *fp;
 {
 	register struct siobuf *siop = siofds[FILENO(fp)];
@@ -143,7 +145,7 @@ siofputs(s, fp)
 	while (*s != '\0') {
 	  if ((--siop->sb_cnt) <= 0)
 	    siomore(siop);
-	  *siop->sb_ptr++ = (u_char)*s++;
+	  *siop->sb_ptr++ = *s++;
 	}
 	return 0;
 }
@@ -176,11 +178,11 @@ vfprintf(fp, fmt, ap)
 int
 #ifdef HAVE_STDARG_H
 #ifdef __STDC__
-siofprintf(FILE *fp, char *fmt, ...)
+siofprintf(FILE *fp, const char *fmt, ...)
 #else /* Non ANSI-C */
 siofprintf(fp, fmt)
 	FILE *fp;
-	char *fmt;
+	const char *fmt;
 #endif
 #else
 siofprintf(va_alist)
@@ -210,10 +212,10 @@ siofprintf(va_alist)
 int
 #ifdef HAVE_STDARG_H
 #ifdef __STDC__
-sioprintf(char *fmt, ...)
+sioprintf(const char *fmt, ...)
 #else /* Non ANSI-C */
 sioprintf(fmt)
-	char *fmt;
+	const char *fmt;
 #endif
 #else
 sioprintf(va_alist)
@@ -225,7 +227,7 @@ sioprintf(va_alist)
 #ifdef HAVE_STDARG_H
 	va_start(ap, fmt);
 #else
-	char *fmt;
+	const char *fmt;
 
 	va_start(ap);
 	fmt = va_arg(ap, char *);
@@ -241,7 +243,7 @@ sioprintf(va_alist)
 int
 siofread(b, s, n, fp)
 	char *b;
-	int s, n;
+	u_int s, n;
 	FILE *fp;
 {
 	struct siobuf *siop = siofds[FILENO(fp)];
@@ -257,8 +259,8 @@ siofread(b, s, n, fp)
 
 int
 siofwrite(b, s, n, fp)
-	register char *b;
-	register int s, n;
+	register const char *b;
+	register u_int s, n;
 	FILE *fp;
 {
 	register struct siobuf *siop = siofds[FILENO(fp)];
@@ -270,7 +272,7 @@ siofwrite(b, s, n, fp)
 	while (s-- > 0) {
 	  if ((--siop->sb_cnt) <= 0)
 	    siomore(siop);
-	  *siop->sb_ptr++ = (u_char)*b++;
+	  *siop->sb_ptr++ = *b++;
 	}
 	return n;
 }
@@ -282,13 +284,13 @@ int
 siomore(siop)
 	struct siobuf *siop;
 {
-	u_char *prevbase;
+	char *prevbase;
 
 	if (siop->sb_base == NULL) {
 	  /* first time around, we allocate off scratch memory */
 	  siop->sb_bufsiz = CLICK;
 	  /* note: base needn't be aligned */
-	  siop->sb_base = (u_char *)emalloc(siop->sb_bufsiz);
+	  siop->sb_base = (char *)emalloc(siop->sb_bufsiz);
 	  siop->sb_flag |= O_CREAT;
 	  siop->sb_ptr = siop->sb_base;
 	  siop->sb_cnt = siop->sb_bufsiz;
@@ -302,11 +304,11 @@ siomore(siop)
 	  siop->sb_bufsiz *= 2;
 	  if (siop->sb_flag & O_CREAT) {
 	    /* ahh, it already is a malloc'ed buffer */
-	    siop->sb_base = (u_char*)erealloc((char *)siop->sb_base,
-					      siop->sb_bufsiz);
+	    siop->sb_base = (char *)erealloc(siop->sb_base,
+					     siop->sb_bufsiz);
 	  } else {
 	    /* copy existing buffer into malloc'ed ditto */
-	    siop->sb_base = (u_char *)emalloc(siop->sb_bufsiz);
+	    siop->sb_base = (char *)emalloc(siop->sb_bufsiz);
 	    memcpy(siop->sb_base, prevbase, siop->sb_ptr - prevbase);
 	    siop->sb_flag |= O_CREAT;
 	  }
@@ -352,18 +354,20 @@ STATIC char sccsid[] = "@(#)doprnt.c	5.35 (Berkeley) 6/27/88";
 #define	BUF		(MAXEXP+MAXFRACT+1)	/* + decimal point */
 
 #if 1
+static void sio_PUTC __((int ch, struct siobuf *siop));
+
 static void sio_PUTC(ch,siop)
 register int ch;
 register struct siobuf *siop;
 {
 	if ((--siop->sb_cnt) <= 0)
 	  siomore(siop);
-	*(siop->sb_ptr++) = (u_char) ch;
+	*(siop->sb_ptr++) = ch;
 }
 #define PUTC(ch) sio_PUTC(ch,siop)
 #else
 #define	PUTC(ch)	(void) ((((--siop->sb_cnt) <= 0) ? siomore(siop) : 0), \
-				(int)(*(siop->sb_ptr++) = (u_char)(ch)))
+				(int)(*(siop->sb_ptr++) = (ch)))
 #endif
 #define	ARG() \
 	_ulong = flags&LONGINT ? va_arg(argp, long) : \
@@ -383,11 +387,11 @@ register struct siobuf *siop;
 #define	ZEROPAD		0x20		/* zero (as opposed to blank) pad */
 #define	HEXPREFIX	0x40		/* add 0x or 0X prefix */
 
-static
-char *
+static const char *mymemchr __((const char *s, int, int));
+static const char *
 mymemchr(s, c, n)
-	register char *s, c;
-	register int n;
+	register const char *s;
+	register int c, n;
 {
 	while (n > 0 && *s != c)
 		--n, ++s;
@@ -396,17 +400,18 @@ mymemchr(s, c, n)
 	return NULL;
 }
 
-int
+
+STATIC int
 vsiodoprnt(siop, fmt0, argp)
 	register struct siobuf *siop;
-	u_char *fmt0;
+	const char *fmt0;
 	va_list argp;
 {
-	register u_char *fmt;	/* format string */
+	register const char *fmt; /* format string */
 	register int ch;	/* character from fmt */
 	register int cnt;	/* return value accumulator */
 	register int n;		/* random handy integer */
-	register char *t;	/* buffer pointer */
+	register const char *t;	/* buffer pointer */
 	double _double;		/* double precision arguments %[eEfgG] */
 	u_long _ulong;		/* integer arguments %[diouxX] */
 	int base;		/* base for [diouxX] conversion */
@@ -419,8 +424,10 @@ vsiodoprnt(siop, fmt0, argp)
 	int size;		/* size of converted field or string */
 	int width;		/* width from format (%8d), or 0 */
 	char sign;		/* sign prefix (' ', '+', '-', or \0) */
+#ifndef NO_FLOAT_CONVERSION
 	char softsign;		/* temporary negative sign for floats */
-	char *digs;		/* digits for [diouxX] conversion */
+#endif
+	const char *digs;	/* digits for [diouxX] conversion */
 	char buf[BUF];		/* space for %c, %[diouxX], %[eEfgG] */
 
 	if ((siop->sb_flag & O_WRONLY) == 0)
@@ -506,7 +513,8 @@ rflag:		switch (*++fmt) {
 			flags |= LONGINT;
 			goto rflag;
 		case 'c':
-			*(t = buf) = va_arg(argp, int);
+			*buf = va_arg(argp, int);
+			t = buf;
 			size = 1;
 			sign = '\0';
 			goto pforw;
@@ -599,7 +607,8 @@ rflag:		switch (*++fmt) {
 			base = 16;
 			goto nosign;
 		case 's':
-			if (!(t = va_arg(argp, char *)))
+			t = va_arg(argp, char *);
+			if (t == NULL)
 				t = "(null)";
 			if (prec >= 0) {
 				/*
@@ -607,9 +616,10 @@ rflag:		switch (*++fmt) {
 				 * NUL in the first `prec' characters, and
 				 * strlen() will go further.
 				 */
-				char *p, *mymemchr();
+				const char *p;
 
-				if ((p = mymemchr(t, 0, prec))) {
+				p = mymemchr(t, 0, prec);
+				if (p != NULL) {
 					size = p - t;
 					if (size > prec)
 						size = prec;
@@ -653,13 +663,15 @@ number:			if ((dprec = prec) >= 0)
 			 */
 			t = buf + BUF;
 			if (_ulong != 0 || prec != 0) {
+				char *tt = buf + BUF;
 				do {
-					*--t = digs[_ulong % base];
+					*(--tt) = digs[_ulong % base];
 					_ulong /= base;
 				} while (_ulong);
 				digs = "0123456789abcdef";
-				if (flags & ALT && base == 8 && *t != '0')
-					*--t = '0'; /* octal leading 0 */
+				if (flags & ALT && base == 8 && *tt != '0')
+					*(--tt) = '0'; /* octal leading 0 */
+				t = tt;
 			}
 			size = buf + BUF - t;
 
@@ -698,7 +710,7 @@ pforw:
 				PUTC(sign);
 			if (flags & HEXPREFIX) {
 				PUTC('0');
-				PUTC((char)*fmt);
+				PUTC(*fmt);
 			}
 			/* right-adjusting zero padding */
 			if ((flags & (LADJUST|ZEROPAD)) == ZEROPAD)
@@ -711,7 +723,7 @@ pforw:
 			/* the string or number proper */
 			if (siop->sb_cnt - (n = size) > 0) {
 				siop->sb_cnt -= n;
-				memcpy((char *)siop->sb_ptr, t, n);
+				memcpy(siop->sb_ptr, t, n);
 				siop->sb_ptr += n;
 			} else
 				while (--n >= 0)
@@ -729,7 +741,7 @@ pforw:
 		case '\0':	/* "%?" prints ?, unless ? is NULL */
 			return (cnt);
 		default:
-			PUTC((char)*fmt);
+			PUTC(*fmt);
 			cnt++;
 		}
 	}
@@ -742,7 +754,7 @@ cvt(number, prec, flags, signp, fmtch, startp, endp)
 	double number;
 	register int prec;
 	int flags;
-	u_char fmtch;
+	char fmtch;
 	char *signp, *startp, *endp;
 {
 	register char *p, *t;
@@ -786,7 +798,7 @@ cvt(number, prec, flags, signp, fmtch, startp, endp)
 				} while (--prec && fract);
 			if (fract)
 				startp = round(fract, (int *)NULL, startp,
-				    t - 1, (char)0, signp);
+					       t - 1, 0, signp);
 		}
 		for (; prec--; *t++ = '0');
 		break;
@@ -838,7 +850,7 @@ eformat:	if (expcnt) {
 				} while (--prec && fract);
 			if (fract)
 				startp = round(fract, &expcnt, startp,
-				    t - 1, (char)0, signp);
+					       t - 1, 0, signp);
 		}
 		/* if requires more precision */
 		for (; prec--; *t++ = '0');
@@ -908,7 +920,7 @@ eformat:	if (expcnt) {
 			}
 			if (fract)
 				startp = round(fract, (int *)NULL, startp,
-				    t - 1, (char)0, signp);
+					       t - 1, 0, signp);
 		}
 		/* alternate format, adds 0's for precision, else trim 0's */
 		if (flags&ALT)
@@ -971,7 +983,7 @@ char *
 exponent(p, exp, fmtch)
 	register char *p;
 	register int exp;
-	u_char fmtch;
+	char fmtch;
 {
 	register char *t;
 	char expbuf[MAXEXP];
