@@ -13,8 +13,9 @@
 #include "io.h"
 #include "shconfig.h"
 
-int lastbgpid = 0;
+#include "libsh.h"
 
+int lastbgpid = 0;
 
 /*
  * We're finished forking things, now report to the user.
@@ -39,7 +40,7 @@ jc_report(pgrp)
 void
 jc_newproc(pgrpp, pid, argc, argv)
 	int *pgrpp, pid, argc;
-	char *argv[];
+	const char *argv[];
 {
 	if (*pgrpp == 0)
 		*pgrpp = pid;
