@@ -8,6 +8,7 @@
  */
 
 #include "hostenv.h"
+#include "listutils.h"
 #ifdef	MAILER
 #include "sift.h"
 #endif	/* MAILER */
@@ -17,9 +18,8 @@
 #include "io.h"
 #include "shconfig.h"
 
-extern FILE *runiofp;
-extern int cistrcmp();
-extern struct conscell *envarlist;
+#include "libz.h"
+#include "libsh.h"
 
 /*
  * Cdr down a linked list to retrieve its last element.  This is not
@@ -27,9 +27,9 @@ extern struct conscell *envarlist;
  * our representation of lists.
  */
 
-struct conscell *
+conscell *
 s_last(list)
-	register struct conscell *list;
+	register conscell *list;
 {
 	if (list == NULL)
 		return NULL;
@@ -40,7 +40,7 @@ s_last(list)
 
 int
 s_equal1(l1, l2)
-	register struct conscell *l1, *l2;
+	register conscell *l1, *l2;
 {
 	if (l1 == NULL)
 		return (l2 == NULL);
@@ -55,7 +55,7 @@ s_equal1(l1, l2)
 
 int
 s_equal(l1, l2)
-	register struct conscell *l1, *l2;
+	register conscell *l1, *l2;
 {
 	if (l1 == NULL)
 		return l2 == NULL;
@@ -96,9 +96,9 @@ s_equal(l1, l2)
  * The N'th element of a list.
  */
 
-struct conscell *
+conscell *
 s_nth(list, n)
-	register struct conscell *list;
+	register conscell *list;
 	register int n;
 {
 	if (list == NULL || STRING(list))
@@ -117,7 +117,7 @@ s_nth(list, n)
 
 void
 s_set_prev(prev, list)
-	register struct conscell *prev, *list;
+	register conscell *prev, *list;
 {
 	while (list != NULL) {
 		list->prev = prev;
@@ -137,13 +137,15 @@ s_set_prev(prev, list)
  * if the string was read in as is?
  */
 
+STATIC int s_isname __((const char *s));
 STATIC int
 s_isname(s)
-	register u_char *s;
+	register const char *s;
 {
 	while (*s != '\0') {
-		if (!isascii((char)*s) || !isprint((char)*s) || *s == '\\'
-		    || *s == ' ' || *s == '\'' || *s == '"' || *s == '`') {
+	  int c = (*s) & 0xFF;
+	  if (!isascii(c) || !isprint(c) || c == '\\' || c  ==  ' ' ||
+	      c == '\''   || c == '"'  || *s == '`') {
 			return 0;
 		} else
 			++s;
@@ -155,12 +157,13 @@ s_isname(s)
  * Print a string in quoted form.
  */
 
+STATIC void s_pname __((const char *s, FILE *fp));
 STATIC void
 s_pname(s, fp)
-	register u_char *s;
+	register const char *s;
 	FILE *fp;
 {
-	u_char *base = s;
+	const char *base = s;
 
 	if (*s != '\'')
 		putc('\'', fp);
@@ -186,7 +189,7 @@ s_pname(s, fp)
 
 void
 s_grind(list, fp)
-	struct conscell *list;
+	conscell *list;
 	FILE *fp;
 {
 	if (list == NULL) {
@@ -221,7 +224,7 @@ s_grind(list, fp)
 
 void
 _grind(list)
-	struct conscell *list;
+	conscell *list;
 {
 	s_grind(list, stderr);
 	putc('\n', stderr);
@@ -231,12 +234,12 @@ _grind(list)
  * Squish a linked list of buffers into a single buffer.
  */
 
-struct conscell *
+conscell *
 s_catstring(s)
-	struct conscell *s;
+	conscell *s;
 {
-	u_char *cp, *buf;
-	struct conscell *sp, *tmp;
+	char *cp, *buf;
+	conscell *sp, *tmp;
 	int len, quoted;
 
 	if (cdr(s) == NULL)
@@ -244,13 +247,13 @@ s_catstring(s)
 	len = 0;
 	for (sp = s; sp != NULL; sp = cdr(sp))
 		if (sp->string)
-			len += strlen((char *)sp->string);
+			len += strlen(sp->string);
 	quoted = 0;
-	cp = buf = (u_char *)tmalloc(len+1);
+	cp = buf = (char *)tmalloc(len+1);
 	for (sp = s; sp != NULL; sp = cdr(sp)) {
 		if (sp->string) {
-			strcpy((char *)cp, (char *)sp->string);
-			cp += strlen((char *)cp);
+			strcpy(cp, sp->string);
+			cp += strlen(cp);
 			quoted += ISQUOTED(sp);
 		}
 	}
@@ -265,15 +268,15 @@ s_catstring(s)
  * Construct a list structure representing the data on the input file pointer.
  */
 
-struct conscell *
+conscell *
 s_read(fp)
 	FILE *fp;
 {
 	register int ch;
-	register u_char *bp;
-	register struct conscell **listp;
-	u_char	ech, buf[8096];
-	struct conscell *list;
+	register char *bp;
+	register conscell **listp;
+	char	ech, buf[8096];
+	conscell *list;
 
 	if (feof(fp))
 		return NULL;
@@ -317,7 +320,7 @@ s_read(fp)
 			*bp++ = ch;
 		}
 		*bp = '\0';
-		list->string = (u_char *)strnsave((char *)buf, bp - buf);
+		list->string = strnsave(buf, bp - buf);
 		break;
 	default:	/* normal symbol */
 		list = newcell();
@@ -326,8 +329,8 @@ s_read(fp)
 		list->prev = 0;
 		list->pflags = 0;
 		*bp++ = ch;
-		while ((ch = getc(fp)) != EOF && isascii((u_char)ch)
-		       && !isspace(ch) && ch != '(' && ch != ')') {
+		while ((ch = getc(fp)) != EOF && isascii(ch) &&
+		       !isspace(ch) && ch != '(' && ch != ')') {
 			if (ch == '\\')
 				if ((ch = getc(fp)) == EOF)
 					break;
@@ -336,7 +339,7 @@ s_read(fp)
 		if (ch != EOF)
 			ungetc(ch, fp);
 		*bp = '\0';
-		list->string = (u_char *)strnsave((char *)buf, bp - buf);
+		list->string = strnsave(buf, bp - buf);
 		break;
 	}
 	/* putc('>', runiofp);s_grind(list, runiofp); putc('\n', runiofp); */
@@ -347,43 +350,43 @@ s_read(fp)
  * Turn an argc,argv type parameter spec into a list of the argv's.
  */
 
-struct conscell *
+conscell *
 s_listify(ac, av)
 	register int ac;
-	register char *av[];
+	register const char *av[];
 {
-	register struct conscell **pl, *tmp;
-	struct conscell *l;
+	register conscell **pl, *tmp;
+	conscell *l;
 
 	for (l = NIL, pl = &car(l); ac-- > 0 && *av != NULL; pl = &cdr(*pl))
-		*pl = conststring((u_char *)*av++);
+		*pl = conststring(*av++);
 	*pl = NULL;
 	return l;
 }
 
 /* push a command string onto the input stack */
 
-struct conscell *
+conscell *
 s_pushstack(l, s)
-	struct conscell *l;
-	char *s;
+	conscell *l;
+	const char *s;
 {
-	struct conscell *d, *tmp;
+	conscell *d, *tmp;
 	memtypes oval = stickymem;
 
 	stickymem = MEM_MALLOC;
-	d = newstring((u_char *)strsave(s));
-	cdr(d) = conststring((u_char *)" \n");
+	d = newstring(strsave(s));
+	cdr(d) = conststring(" \n");
 	cddr(d) = l;
 	stickymem = oval;
 	return d;
 }
 
-struct conscell *
+conscell *
 s_popstack(l)
-	struct conscell *l;
+	conscell *l;
 {
-	struct conscell *d;
+	conscell *d;
 
 	d = l;
 	l = cdr(l);
@@ -392,8 +395,8 @@ s_popstack(l)
 	return l;
 }
 
-struct conscell *
+conscell *
 newcell()
 {
-	return (struct conscell*)tmalloc(sizeof(struct conscell));
+	return (conscell*)tmalloc(sizeof(conscell));
 }
