@@ -14,18 +14,18 @@ int D_assign = 0;
 int funclevel = 0;
 
 #include "listutils.h"
-struct conscell **return_valuep = 0;
+conscell **return_valuep = 0;
 #endif	/* MAILER */
 
+#include "libsh.h"
+
+extern int optind;
+
 int
 main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	extern int optind;
-	extern void zshinit(), trapexit();
-	extern int zshtoplevel();
-
 	/* mal_debug(3); */
 	zshinit(argc, argv);
 	/* mal_leaktrace(1); */
