@@ -3,8 +3,9 @@
  *	This will be free software, but only when it is finished.
  */
 
-#include "hostenv.h"
-#include <stdio.h>
+#include "hostenv.h" /* Usage of 'mailer.h' here is dangerous, because
+			'sh.h' includes 'regex.h', which barfs at the
+			definition of 'string' ... */
 #if HAVE_STRING_H || STDC_HEADERS
 # include <string.h>
 #else
@@ -13,20 +14,16 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/file.h>
+#include "sh.h"
 #include "shconfig.h"
 #include "flags.h"
-#include "interpret.h"
-#include "listutils.h"
 #include "io.h"		/* redefines stdio routines */
 #include "splay.h"
 
-extern char *getenv();
-extern int errno;
-extern int pathcmp();
-extern struct conscell *envarlist;
+#include "libz.h"
+#include "libsh.h"
+
 extern struct sptree *spt_searchpath;
-extern void sp_null();
-extern void *tmalloc();
 
 /*
  * Take a path specification, e.g. ":/bin:/usr/bin" and iterate through it
@@ -37,9 +34,10 @@ extern void *tmalloc();
 
 char *
 prepath(pathspec, name, buf, buflen)
-	register char *pathspec, *name;
+	register char *pathspec;
+	register const char *name;
 	char *buf;
-	register int buflen;
+	register unsigned int buflen;
 {
 	register char *cp;
 
@@ -81,13 +79,13 @@ prepath(pathspec, name, buf, buflen)
 
 char *
 path_hash(command)
-	char *command;
+	const char *command;
 {
 	char *dir, *odir, *path;
 	spkey_t n;
-	struct conscell *d;
+	conscell *d;
 	struct spblk *spl;
-	u_int pathlen;
+	int pathlen;
 
 	if (!isset('h'))
 		return NULL;
@@ -95,7 +93,7 @@ path_hash(command)
 		return NULL;
 	if (strchr(command, '/') != NULL)
 		return NULL;
-	n = symbol((u_char *)command);
+	n = symbol(command);
 	if ((spl = sp_lookup(n, spt_searchpath)) != NULL
 	    && (dir = (char *)spl->data) != NULL)
 		return dir;
@@ -144,7 +142,7 @@ execvp(command, argv)
 	char *const *argv;
 {
 	char *dir, *odir, *path;
-	struct conscell *d;
+	conscell *d;
 	struct spblk *spl;
 	u_int pathlen;
 
@@ -170,9 +168,9 @@ execvp(command, argv)
 		path = emalloc(pathlen);
 #endif
 		while (dir != NULL) {
-			dir = prepath(dir, command, path, pathlen);
-			/* printf("execv '%s'\n", path); */
-			execv(path, argv);
+		  dir = prepath(dir, command, path, pathlen);
+		  /* printf("execv '%s'\n", path); */
+		  execv(path, argv);
 		}
 #ifndef	xxUSE_ALLOCA
 		free(path);
@@ -181,8 +179,7 @@ execvp(command, argv)
 	if ((d = v_find("PATH")) != NULL && cdr(d) != NULL && STRING(cdr(d))) {
 		if (odir != NULL) {
 			/* Hmm, wasn't in the hashed location, clear that out */
-			spl = sp_lookup(symbol((u_char *)command),
-					spt_searchpath);
+			spl = sp_lookup(symbol(command), spt_searchpath);
 			if (spl != NULL)
 				sp_delete(spl, spt_searchpath);
 			/*
@@ -198,9 +195,9 @@ execvp(command, argv)
 		path = emalloc(pathlen);
 #endif
 		while (dir != NULL && dir != odir) {
-			dir = prepath(dir, command, path, pathlen);
-			/* printf("reexecv '%s'\n", path); */
-			execv(path, argv);
+		  dir = prepath(dir, command, path, pathlen);
+		  /* printf("reexecv '%s'\n", path); */
+		  execv(path, argv);
 		}
 #ifndef	xxUSE_ALLOCA
 		free(path);
@@ -215,7 +212,7 @@ execv(command, argv)
 	const char *command;
 	char *const *argv;
 {
-	register struct conscell *scope, *l;
+	register conscell *scope, *l;
 	register int n, len;
 	register char **envp, *buf;
 
@@ -227,7 +224,7 @@ execv(command, argv)
 		if (STRING(l))
 			++n, len += strlen((char *)l->string);
 	envp = (char **)tmalloc((n/2 + 1) * sizeof (char *));
-	buf = (char *)tmalloc(len + n /* terminating NUL */ + n /* = */);
+	buf  = (char *)tmalloc(len + n /* terminating NUL */ + n /* = */);
 	for (n = 0, l = car(scope); l != NULL; l = cddr(l)) {
 		if (!(STRING(l) && cdr(l) != NULL && STRING(cdr(l))))
 			continue;
@@ -246,16 +243,16 @@ execv(command, argv)
 
 	if (errno == ENOEXEC) {	/* maybe the kernel doesn't understand #! */
 		int nargs;
-		char **av;
+		const char **av;
 
 		for (nargs = 0; argv[nargs] != NULL; ++nargs)
 			continue;
-		av = (char **)tmalloc((nargs + 2) * sizeof argv[0]);
+		av = (const char **)tmalloc((nargs + 2) * sizeof argv[0]);
 		av[0] = "sh";
 		for (nargs = 0; argv[nargs] != NULL; ++nargs)
 			av[nargs+1] = argv[nargs];
 		av[nargs+1] = NULL;
-		execve("/bin/sh", av, envp);
+		execve("/bin/sh", (char*const*)av, envp);
 		errno = ENOEXEC;
 	}
 	return n;
