@@ -12,7 +12,7 @@
 #include "listutils.h"
 #include "shconfig.h"
 
-extern int funcall();
+#include "libsh.h"
 
 STATIC char *ps1 = NULL;
 STATIC char *ps2 = NULL;
@@ -27,7 +27,7 @@ prompt_print()
 void
 prompt_flush()
 {
-	struct conscell *d = v_find(PS1);
+	conscell *d = v_find(PS1);
 
 	if (d == NULL || cdr(d) == NULL || LIST(cdr(d))
 	    || *(ps1 = (char *)cdr(d)->string) == '\0')
@@ -44,7 +44,7 @@ prompt2_print()
 void
 prompt2_flush()
 {
-	struct conscell *d = v_find(PS2);
+	conscell *d = v_find(PS2);
 
 	if (d == NULL || cdr(d) == NULL || LIST(cdr(d))
 	    || *(ps2 = (char *)cdr(d)->string) == '\0')
