@@ -5,26 +5,28 @@
 
 #ifndef	Z_SIFT_H
 #define	Z_SIFT_H
+#include "listutils.h"
 #include "token.h"
-#include "regex.h"
 #include "tregexp.h"
+#include "regex.h"
 
 struct vaccess {	/* in a list of this structure */
 	struct vaccess	*next;
-	struct conscell	*l;		/* points at variable name in :env */
+	conscell	*l;		/* points at variable name in :env */
 };
+extern struct vaccess * v_accessed;
 
 typedef struct regexp {
 	regex_t		re;
-	char		*pattern;
-	char		**match;
+	const char	*pattern;
+	const char	**match;
 } regexp;
 
 struct si_retab {
 	struct si_retab	*next;
 	regexp		*rep;
 	tregexp		*trep;
-	char		**match;
+	const char	**match;
 	struct token	*startp[NSUBEXP];
 	struct token	*endp[NSUBEXP];
 };
@@ -32,7 +34,7 @@ struct si_retab {
 struct siftinfo {
 	int		kind;		/* 0: StringSift, !0: TokenSift */
 	struct token	*tlist;		/* token list for tsift expression */
-	char		*str;		/* string for sift expression */
+	const char	*str;		/* string for sift expression */
 	struct vaccess	*accessed;	/* variables dependencies of expr. */
 	int		label;		/* label to go to when reevaluating */
 	regexp		*program;	/* compiled regular expression stack */
@@ -40,9 +42,4 @@ struct siftinfo {
 	struct si_retab	*subexps;	/* linked list of subexpressions */
 };
 
-extern int v_record;			/* record variable accesses */
-extern int v_changed;			/* antecedent variable changed */
-extern struct vaccess *v_accessed;	/* list of variables accessed */
-extern void v_written();		/* a variable value changed */
-
 #endif	/* Z_SIFT_H */
