@@ -6,6 +6,7 @@
 #include "hostenv.h"
 #include <stdio.h>
 #include <ctype.h>
+#include "listutils.h"
 #include "sh.h"
 #include "shconfig.h"
 
@@ -13,16 +14,15 @@ static		/* This will apply to the array in the .sst file */
 #include "sh.sst.c"
 #include "sh.entry"
 
-extern char *InputTokenName();
-extern int	interrupted;
-extern int	zshinput();
+#include "libsh.h"
 
-STATIC u_char	*base, *nbase, *out;
+STATIC const char * InputTokenName __((int));
+STATIC char	*base, *nbase, *out;
 STATIC u_int	outlen, outsize;
-#define PATCH(Y)	*out++ = (u_char)(Y)
+#define PATCH(Y)	*out++ = (Y)
 #define	OUTPUT(X)	((outlen <= 0 ? \
 			    (outsize *= 2, \
-			     nbase = (u_char *)erealloc((char *)base, outsize),\
+			     nbase = (char *)erealloc(base, outsize),\
 			     out += (nbase - base), base = nbase, \
 			     outlen = outsize/2 - 1) \
 			  : --outlen), \
@@ -34,8 +34,8 @@ STATIC u_int	outlen, outsize;
 struct smallSymbol shsymbol[CHARSETSIZE];
 
 struct kwdefn {
-	char	*name;
-	Keyword	kw;
+	const char *name;
+	Keyword	    kw;
 } kwlist[] = {	/* most used first order */
 {	"in",		kIn		},
 #ifdef	MAILER
@@ -97,7 +97,8 @@ STATIC InputTokens	nextInputToken;
 STATIC InputTokens	savedToken;
 STATIC char inputTokenBuffer[3], acceptedTokenText[3];
 STATIC char *inputCharStack = NULL;
-STATIC int inputCharStackSize = 0, charStackIndex = -1;
+STATIC u_int inputCharStackSize = 0;
+STATIC int   charStackIndex = -1;
 
 /* Line Counters */
 
@@ -116,60 +117,62 @@ STATIC struct smallSymbol ws_symbols[] = {
 {	tLetter,	tSyntaxError	}		/* tab */
 };
 
-STATIC char ws_chars[] = " \t";
+STATIC const char ws_chars[] = " \t";
 
 void
-ShInitIFS(ifs)
-	u_char *ifs;
+ShInitIFS(uIFS)
+	const char *uIFS;
 {
-	register u_char *cp;
+	register const char *cp;
 	int count;
-	static u_char *saveifs = (u_char *)ws_chars;
+	static const char *saveifs = ws_chars;
+	char *saveifs_w;
 	static struct smallSymbol *savesymbol = ws_symbols;
 
 	if (savesymbol != NULL) {
 		for (cp = saveifs, count = 0; *cp != '\0'; ++cp)
-			if (*cp != '\n') {
-				shsymbol[*cp].name = savesymbol[count].name;
-				shsymbol[*cp].name2 = savesymbol[count].name2;
-				++count;
-			}
-		if (saveifs != (u_char *)ws_chars) {
-			free((char *)savesymbol);
-			free((char *)saveifs);
+		  if (*cp != '\n') {
+		    shsymbol[(*cp) & 0xFF].name  = savesymbol[count].name;
+		    shsymbol[(*cp) & 0xFF].name2 = savesymbol[count].name2;
+		    ++count;
+		  }
+		if (saveifs != ws_chars) {
+			free((void *)savesymbol);
+			free((void *)saveifs);
 		}
 	}
-	if (*ifs == '\0')
-		ifs = (u_char *)ws_chars;
-	for (cp = ifs, count = 0; *cp != '\0'; ++cp)
+	if (*uIFS == '\0')
+		uIFS = ws_chars;
+	for (cp = uIFS, count = 0; *cp != '\0'; ++cp)
 		if (*cp != '\n')
 			++count;
 	savesymbol = (struct smallSymbol *)emalloc(count *
 						   sizeof (struct smallSymbol));
-	for (cp = ifs, count = 0; *cp != '\0'; ++cp)
+	for (cp = uIFS, count = 0; *cp != '\0'; ++cp)
 		if (*cp != '\n') {
-			savesymbol[count].name = shsymbol[*cp].name;
-			savesymbol[count].name2 = shsymbol[*cp].name2;
-			shsymbol[*cp].name = tWhiteSpace;
-			shsymbol[*cp].name2 = tSyntaxError;
+			savesymbol[count].name  = shsymbol[(*cp) & 0xFF].name;
+			savesymbol[count].name2 = shsymbol[(*cp) & 0xFF].name2;
+			shsymbol[(*cp) & 0xFF].name = tWhiteSpace;
+			shsymbol[(*cp) & 0xFF].name2 = tSyntaxError;
 			++count;
 		}
 	/* Using strsave() has hidden dangers due to  stickymem  value! */
-	saveifs = (u_char *)emalloc(strlen((char*)ifs)+1);
-	strcpy((char *)saveifs, (char *)ifs);
+	saveifs_w = (char *)emalloc(strlen(uIFS)+1);
+	strcpy(saveifs_w, uIFS);
+	saveifs = saveifs_w;
 }
 
 void
 ShInit()
 {
 	register int i;
-	register u_char *cp;
+	register const u_char *cp;
 
 	for (i = 0; i < (sizeof shsymbol/sizeof shsymbol[0]); ++i) {
-		shsymbol[i].name = tLetter;
+		shsymbol[i].name  = tLetter;
 		shsymbol[i].name2 = tSyntaxError;
 	}
-	for (cp = (u_char*)"0123456789"; *cp != '\0'; ++cp)
+	for (cp = (const u_char *)"0123456789"; *cp != '\0'; ++cp)
 		shsymbol[*cp].name = tDigit;
 #if 0
 	for (cp = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; *cp != '\0'; ++cp)
@@ -223,7 +226,7 @@ ShInit()
 
 STATIC struct errmsgs {
 	ErrorCodes	e;
-	char		*m;
+	const char	*m;
 } emsglist[] = {
 { eSyntaxError,			"syntax error"				},
 { ePrematureEndOfFile,		"unexpected end of file"		},
@@ -247,15 +250,16 @@ STATIC struct errmsgs {
 { eIllegalTokenUnbalancedList,	"illegal token within list"		},
 };
 
-STATIC char *errfilename;	/* set from inputname in SslWalker */
+STATIC const char *errfilename;   /* set from inputname in SslWalker */
 
+STATIC void sslw_error __((ErrorCodes));
 STATIC void
 sslw_error(errCode)
 	ErrorCodes errCode;
 {
 	int i;
 	char *cp, *bos;
-	char *msg = "unknown error";
+	const char *msg = "unknown error";
 
 	if (errCode == eNoError)
 		abort();
@@ -334,7 +338,7 @@ ungetbuf(buf, len)
 		DECPOS(inputTokenBuffer[1],nextLineChar);
 	}
 	inputCharStack[++charStackIndex] = inputTokenBuffer[0];
-	if (shsymbol[(u_char)inputTokenBuffer[0]].name == tNewLine)
+	if (shsymbol[inputTokenBuffer[0] & 0xFF].name == tNewLine)
 		--nextLineNumber;
 	/* push buffer */
 	for (i = 0; i < len; ++i) {
@@ -363,6 +367,7 @@ if (charStackIndex < 0) /* Make sure it is above zero.. */
  * indicators and the values and text associated with input tokens.  
  */
 
+STATIC void AcceptInputToken __((int));
 STATIC void
 AcceptInputToken(flag)
 	register int	flag;
@@ -394,32 +399,32 @@ AcceptInputToken(flag)
 	NEXTCHAR(flag);
 	inputTokenBuffer[0] = c;
 	inputTokenBuffer[1] = '\0';
-	nextInputToken = shsymbol[c].name;
+	nextInputToken = shsymbol[c & 0xFF].name;
 	if (nextInputToken == tNewLine) {
 		/* Update Line Counter and Set Flag */
 		newInputLine = 1;
 		nextLineChar = 0;
 		++nextLineNumber;
-	} else if (shsymbol[c].name2 != tSyntaxError) {
+	} else if (shsymbol[c & 0xFF].name2 != tSyntaxError) {
 		/* A two-character symbol */
 		NEXTCHAR(1);
 		if (c == inputTokenBuffer[0]) {
-			nextInputToken = shsymbol[c].name2;
+			nextInputToken = shsymbol[c & 0xFF].name2;
 			inputTokenBuffer[1] = c;
 			inputTokenBuffer[2] = '\0';
 		} else {
 			PUSHCHAR(c);
 		}
 	} else if (counter[(int)countBackQuoteNestingLevel] > 0
-		   && shsymbol[c].name == tBackSlash) {
+		   && shsymbol[c & 0xFF].name == tBackSlash) {
 		cp = backbuf;
 		for (i = 0; i < counter[(int)countBackQuoteNestingLevel]; ++i) {
-			if (shsymbol[c].name != tBackSlash)
+			if (shsymbol[c & 0xFF].name != tBackSlash)
 				break;
 			NEXTCHAR(flag);
 			*cp++ = c;	/* onto slash stack */
 		}
-		if (shsymbol[c].name == tBackQuote) {
+		if (shsymbol[c & 0xFF].name == tBackQuote) {
 			/* ignore the stuff we pushed - that's easy */
 			inputTokenBuffer[0] = c;
 			inputTokenBuffer[1] = '\0';
@@ -448,6 +453,7 @@ traceit:
 
 /* Emit an output token to the output stream */
 
+STATIC void EmitOutputToken __(( OutputTokens ));
 STATIC void
 EmitOutputToken(emittedToken) 
 	register OutputTokens	emittedToken;
@@ -513,6 +519,7 @@ EmitOutputToken(emittedToken)
  * been matched.
  */
 
+STATIC void SslSyntaxError __((TableOperation));
 STATIC void
 SslSyntaxError(opcode)
 	TableOperation opcode;
@@ -549,7 +556,7 @@ SslSyntaxError(opcode)
 	}
 }
 
-char *
+STATIC const char *
 InputTokenName(in)
 	int in;
 {
@@ -561,7 +568,8 @@ InputTokenName(in)
 	return "unknown input token";
 }
 
-STATIC char *
+STATIC const char * TableOperationName __((TableOperation));
+STATIC const char *
 TableOperationName(op)
 	TableOperation	op;
 {
@@ -615,6 +623,7 @@ TableOperationName(op)
 
 STATIC int level = 0;
 
+STATIC void SslTrace __((TableOperation));
 STATIC void
 SslTrace(opcode)
 	TableOperation opcode;
@@ -639,6 +648,8 @@ SslTrace(opcode)
 }
 
 
+STATIC void SslFailure __((FailureCodes, TableOperation));
+
 STATIC void
 SslFailure(failCode, opcode)
 	FailureCodes failCode;
@@ -672,6 +683,8 @@ SslFailure(failCode, opcode)
  * if a match is found and false otherwise.
  */
 
+STATIC int SslChoice __((int));
+
 STATIC int
 SslChoice(choiceTag)
 	register int choiceTag;
@@ -712,11 +725,11 @@ SslChoice(choiceTag)
 }
 
 
-u_char *
+void *
 SslWalker(inputname, tfp, eotp)
-	char *inputname;	/* string used for error messages */
+	const char *inputname;	/* string used for error messages */
 	FILE *tfp;
-	u_char **eotp;
+	void **eotp;
 {
 	static int	inited = 0;
 	struct kwdefn	*kwp;
@@ -784,7 +797,7 @@ SslWalker(inputname, tfp, eotp)
 
 	/* Initialize Input/Output */
 	outlen = outsize = (1<<8)-8; /* Smallish start size */
-	out = base = (u_char *)emalloc(outsize); /* External buffer */
+	out = base = (char *)emalloc(outsize); /* External buffer */
 	nextInputToken = tNewLine;
 	nextLineNumber = lineNumber = 1;
 	nextLineChar = lineChar = 0;
@@ -915,7 +928,7 @@ SslWalker(inputname, tfp, eotp)
 			if (buffer == NULL || buffer[0] == '\0')
 				break;
 			ungetbuf(buffer, bufferIndex);
-			nextInputToken = shsymbol[(u_char)buffer[0]].name;
+			nextInputToken = shsymbol[buffer[0] & 0xFF].name;
 			break;
 
 		/* Buffer mechanism */
@@ -937,10 +950,10 @@ SslWalker(inputname, tfp, eotp)
 			if (bufferIndex + 1
 			    + (acceptedTokenText[1] != '\0') >= bufferSize) {
 				if (buffer == NULL) {
-					bufferSize = 128-24,
+					bufferSize = 128-24;
 					buffer = emalloc(bufferSize);
 				} else {
-					bufferSize *= 2,
+					bufferSize *= 2;
 					buffer = erealloc(buffer, bufferSize);
 				}
 			}
