@@ -15,20 +15,17 @@
 #include "io.h"
 #include "shconfig.h"
 
+#include "listutils.h"
+#include "libsh.h"
+
 int test_debug = 0;
 
-extern int testeval();
-extern int tellme();
-
-#ifdef	HAVE_LSTAT
-extern int lstat();
-#else
-extern int stat();
-#endif
+extern int testeval __((const char **avp[]));
+extern int tellme   __((const char **avp[], int));
 
 #define dprintf		if (test_debug) printf
 
-STATIC char *testprogname;
+STATIC const char *testprogname;
 
 STATIC int testgv[20];		/* shunting-yard stack (operand values) */
 STATIC int testgc = -1;		/* top of said stack */
@@ -39,12 +36,13 @@ STATIC short prec[CHARSETSIZE];
  * Shunting yard galore.
  */
 
+STATIC int testparse __((const char **avp[]));
 STATIC int
 testparse(avp)
-	char **avp[];
+	const char **avp[];
 {
 	int	c;
-	char	**av;
+	const char **av;
 
 	dprintf("testparse:");
 	for (av = *avp; *av != NULL; av++)
@@ -53,7 +51,8 @@ testparse(avp)
 
 	if (**avp == NULL)
 		return 0;
-	switch (c = ***avp) {
+	c = ((***avp) & 0xFF);
+	switch (c) {
 	case '(':
 		++*avp, testparse(avp);
 		if (**avp != NULL && ***avp == ')') {
@@ -65,7 +64,8 @@ testparse(avp)
 		break;
 	case '!':
 		++*avp;
-		if (**avp == NULL || prec[*((unsigned char *)**avp)]) goto err;
+		if (**avp == NULL || prec[(*((char *)**avp)) & 0xFF])
+			goto err;
 		testparse(avp);
 		if (testgc < 0) goto err;
 		testgv[testgc] = !testgv[testgc];
@@ -76,34 +76,36 @@ testparse(avp)
 		switch (*(**avp+1)) {
 		case 'a':
 			++*avp;
-			if (**avp == NULL || prec[*((unsigned char *)**avp)]) goto err;
+			if (**avp == NULL || prec[(*((char *)**avp)) & 0xFF])
+				goto err;
 			testparse(avp);
 			if (**avp != NULL &&
-			    (testgc < 1 || !prec[*((unsigned char *)**avp)]))
+			    (testgc < 1 || !prec[(*((char *)**avp)) & 0xFF]))
 				goto err;
 
 			while (**avp != NULL &&
-			       prec[*((unsigned char *)**avp)] > prec[c])
+			       prec[(*((char *)**avp)) & 0xFF] > prec[c])
 				testparse(avp);
-			testgv[testgc-1] &= testgv[testgc];
+			testgv[testgc - 1] &= testgv[testgc];
 			testgc--;
 			dprintf("[%d] &= [%d] = %d\n",
-				      testgc, testgc+1, testgv[testgc]);
+				      testgc, testgc + 1, testgv[testgc]);
 			break;
 		case 'o':
 			++*avp;
-			if (**avp == NULL ||
-			    prec[*((unsigned char *)**avp)]) goto err;
+			if (**avp == NULL || prec[(*((char *)**avp)) & 0xFF])
+			  goto err;
 			testparse(avp);
-			if (**avp != NULL && (testgc < 1 || !prec[*((unsigned char *)**avp)]))
+			if (**avp != NULL &&
+			    (testgc < 1 || !prec[(*((char *)**avp)) & 0xFF]))
 				goto err;
 			while (**avp != NULL &&
-			       prec[*((unsigned char *)**avp)] > prec[c])
+			       prec[(*((char *)**avp)) & 0xFF] > prec[c])
 				testparse(avp);
-			testgv[testgc-1] |= testgv[testgc];
+			testgv[testgc - 1] |= testgv[testgc];
 			testgc--;
 			dprintf("[%d] |= [%d] = %d\n",
-				      testgc, testgc+1, testgv[testgc]);
+				      testgc, testgc + 1, testgv[testgc]);
 			break;
 		default: /* push value onto stack */
 			testgv[++testgc] = testeval(avp);
@@ -132,6 +134,7 @@ err:
 	return 1;
 }
 
+STATIC int fildes __((char **avp[]));
 STATIC int
 fildes(avp)
 	char **avp[];
@@ -145,8 +148,9 @@ fildes(avp)
 	return isatty(fd);
 }
 
+STATIC int strng __((char **avp[], int));
 STATIC int
-string(avp, i)
+strng(avp, i)
 	char **avp[];
 	int i;
 {
@@ -191,9 +195,9 @@ STATIC struct flags {
 { 'g',	stat,	0,		~S_IFMT,	S_ISGID			},
 { 'k',	stat,	0,		~S_IFMT,	S_ISVTX			},
 { 't',	fildes,	0,		0,		0			},
-{ 'l',	string,	0,		~0,		~0			},
-{ 'n',	string,	0,		~0,		~0			},
-{ 'z',	string,	1,		~0,		~0			},
+{ 'l',	strng,	0,		~0,		~0			},
+{ 'n',	strng,	0,		~0,		~0			},
+{ 'z',	strng,	1,		~0,		~0			},
 };
 
 #define	N1	(value < 0 ? atoi(cp) : value)
@@ -206,9 +210,9 @@ STATIC struct flags {
 
 int
 testeval(avp)
-	char **avp[];
+	const char **avp[];
 {
-	register char *cp;
+	register const char *cp;
 	int	i, value = -1;
 
 	cp = **avp;
@@ -268,7 +272,7 @@ gotvalue:
 
 int
 tellme(avp, i)
-	char **avp[];
+	const char **avp[];
 	int i;
 {
 	struct stat stbuf;
@@ -284,7 +288,7 @@ tellme(avp, i)
 		}
 		++*avp;
 		if (flarr[i].i1)
-			return stbuf.st_size > 0;
+			return ((int)stbuf.st_size > 0);
 		return stbuf.st_mode & flarr[i].i2 & flarr[i].i3;
 	}
 	return (flarr[i].func)(avp, flarr[i].i1, flarr[i].i2, flarr[i].i3);
@@ -297,7 +301,7 @@ tellme(avp, i)
 int
 sh_test(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	if (argc == 1)
 		return 1;
