@@ -14,32 +14,23 @@
 #include "zmsignal.h"
 #include "flags.h"
 #include "malloc.h"
-#include "listutils.h"
+/* #include "listutils.h" */
+#include "sh.h"
 #include "io.h"			/* redefines stdio routines */
 #include "shconfig.h"
 
+#include "libsh.h"
+
 #ifndef SIGCHLD
 #define SIGCHLD SIGCLD
 #endif  /* SIGCHLD */
 
-extern char *progname;
-#ifndef strchr
-extern char *strchr();
-extern char *strrchr();
-#endif
-extern int  leaux();
-extern int  magic_number;
-extern long bin_magic;
-extern struct codedesc *interpret();
-extern struct osCmd *globalcaller, avcmd;
-extern u_char *SslWalker(), *optimize();
-
 /*
  * The script to execute for a particular trap is stored as a string in
  * malloc()'ed storage, with a pointer to it in the traps[] array.
  */
 
-char *traps[NSIG];
+const char *traps[NSIG];
 
 /*
  * The effect of a signal is to increment a count of seen but unprocessed
@@ -73,7 +64,7 @@ int interrupted;		/* XX: we saw an interrupt */
  * stored in the orig_handler[] array.
  */
 
-RETSIGTYPE (*orig_handler[NSIG])();
+RETSIGTYPE (*orig_handler[NSIG]) __((int));
 
 /*
  * Indeed, that is what the trapsnap() function does.
@@ -118,9 +109,11 @@ trap_handler(sig)
 #define FCSUFFIX	".fc"
 #define FCSUBDIR	"fc"
 
-static char *
+STATIC char * makefc __((const char *, char *));
+STATIC char *
 makefc(path, buf)
-	char *path, *buf;
+	const char *path;
+	char *buf;
 {
 	register char *cp;
 	struct stat stbuf;
@@ -155,10 +148,10 @@ makefc(path, buf)
 
 int
 eval(script, scriptname, savefile)
-	char *script, *scriptname, *savefile;
+	const char *script, *scriptname, *savefile;
 {
 	int status;
-	u_char *table, *eotable;
+	void *table, *eotable;
 	FILE *fp;
 	char *fcfile;
 	char *buf = NULL;
@@ -188,8 +181,8 @@ eval(script, scriptname, savefile)
 			    (fp = fopen(fcfile, "w")) != NULL) {
 				fprintf(fp, "#!zsh -l%d,%ld\n",
 					magic_number, bin_magic);
-				std_fwrite(table, sizeof *table,
-					   eotable - table, fp);
+				std_fwrite(table, 1,
+					   (char*)eotable - (char*)table, fp);
 				if (fclose(fp) == EOF) {
 					fprintf(stderr,
 						"%s: write to %s failed\n",
@@ -211,7 +204,7 @@ eval(script, scriptname, savefile)
 int
 loadeval(fcfd, path, srcstbufp)
 	int fcfd;
-	char *path;
+	const char *path;
 	struct stat *srcstbufp;
 {
 	int status;
@@ -222,7 +215,8 @@ loadeval(fcfd, path, srcstbufp)
 	char *buf = emalloc(strlen(path)+9);
 #endif
 
-	if ((fcfile = makefc(path,buf)) == NULL) {
+	fcfile = makefc(path,buf);
+	if (fcfile == NULL) {
 #ifndef	USE_ALLOCA
 		free(buf);
 #endif
@@ -238,17 +232,18 @@ loadeval(fcfd, path, srcstbufp)
 int
 leaux(fcfd, path, srcstbufp)
 	int fcfd;
-	char *path;
+	const char *path;
 	struct stat *srcstbufp;
 {
 	FILE *fp;
 	int status, len, checknum;
 	long checkbin;
-	u_char *table;
+	void *table;
 	struct stat objstbuf;
 	char buf[80];
 
-	if ((fp = fopen(path, "r")) == NULL)
+	fp = fopen(path, "r");
+	if (fp == NULL)
 		return -1;
 	if (std_fgets(buf, sizeof buf, fp) == NULL) {
 		fclose(fp);
@@ -282,18 +277,19 @@ leaux(fcfd, path, srcstbufp)
 		unlink(path);
 		return -1;
 	}
-	len = objstbuf.st_size - ftell(fp);
-	table = (u_char *)emalloc(len);
-	if (std_fread(table, sizeof *table, len / sizeof *table, fp) != len / sizeof *table) {
+	len = (int)(objstbuf.st_size - ftell(fp));
+	table = (void *)emalloc(len);
+	if (std_fread(table, 1, len, fp) != len) {
 		fclose(fp);
 		fprintf(stderr, "%s: read of %d failed on %s\n",
 			progname, len, path);
 		return -1;
 	}
 	fclose(fp);
-	close(fcfd);
+	if (fcfd >= 0)
+	  close(fcfd);
 	status = 0;
-	interpret(table, table + len, (u_char *)NULL,
+	interpret(table, (char*)table + len, NULL,
 		  globalcaller == NULL ? &avcmd : globalcaller,
 		  &status, (struct codedesc *)NULL);
 	return status;
@@ -312,7 +308,7 @@ trapped()
 	static int intrap = 0;
 
 	if (!sprung)
-		return;
+	  return;
 	/*
 	 * We must reset the sprung flag before calling aval(), or we will
 	 * almost certainly get a recursive invocation of this routine from
@@ -327,18 +323,17 @@ trapped()
 	 * then maybe get run twice.  Ergo we need some semaphore here.
 	 */
 	if (intrap) {
-		/* more signals have arrived, be sure not to miss them */
-		++intrap;
-		return;
+	  /* more signals have arrived, be sure not to miss them */
+	  ++intrap;
+	  return;
 	}
 	for (++intrap; intrap > 0; --intrap) {
-		for (i = 1; i < NSIG; ++i)
-			while (spring[i] > 0) {
-				if (traps[i] != NULL)
-					eval(traps[i],
-					     "trap", (char *)NULL);
-				--spring[i];
-			}
+	  for (i = 1; i < NSIG; ++i)
+	    while (spring[i] > 0) {
+	      if (traps[i] != NULL)
+		eval(traps[i], "trap", (char *)NULL);
+	      --spring[i];
+	    }
 	}
 }
 
@@ -352,9 +347,9 @@ trapexit(n)
 	int n;
 {
 	if (traps[0] != NULL) {
-		char *cmd = traps[0];
-		traps[0] = NULL;
-		eval(cmd, "exit trap", (char *)NULL);
+	  const char *cmd = traps[0];
+	  traps[0] = NULL;
+	  eval(cmd, "exit trap", (char *)NULL);
 	}
 #ifdef	MALLOC_TRACE
 	mal_dumpleaktrace(stderr);
@@ -371,10 +366,10 @@ trapexit(n)
 int
 sh_trap(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int i;
-	char *av0, *script;
+	const char *av0, *script;
 	
 	if (argc == 1) {
 		/* just print the known traps */
@@ -412,10 +407,11 @@ sh_trap(argc, argv)
 			continue;
 		}
 		if (traps[i] != NULL)
-			free(traps[i]);
+			free((void*)traps[i]);
 		if (script != NULL && *script != '\0') {
-			traps[i] = (char *)emalloc(strlen(script)+1);
-			strcpy(traps[i], script);
+			char *tp = (char *)emalloc(strlen(script)+1);
+			strcpy(tp, script);
+			traps[i] = tp;
 			/* enable that signal */
 			if (i > 0 && orig_handler[i] != SIG_IGN)
 				SIGNAL_HANDLE(i, trap_handler);
