@@ -11,6 +11,8 @@
 extern char *strchr();
 #endif
 
+#include "libz.h"
+
 /*
  * tregcomp, tregexec, and tregsub -- regerror is in errors.c
  *
@@ -145,9 +147,9 @@ extern char *strchr();
  * Utility definitions.
  */
 #ifndef CHARBITS
-#define	UCHARAT(p)	((int)*(unsigned char *)(p))
+#define	UCHARAT(p)	((*(p)) & 0xFF)
 #else
-#define	UCHARAT(p)	((int)*(p)&CHARBITS)
+#define	UCHARAT(p)	((*(p)) & CHARBITS)
 #endif
 
 #define	FAIL(m)  { tregerror(m, regprog); return NULL; }
@@ -155,8 +157,8 @@ extern char *strchr();
 #define	ISMULT(c)	((c) == '*' || (c) == '+' || (c) == '?')
 #define	META	"^$.[()|?+*\\"
 
-#define	CICHEQ(a,b) (((isascii(*(a)) && isupper(*(a))) ? tolower(*(a)) : *(a)) \
-		  == ((isascii(*(b)) && isupper(*(b))) ? tolower(*(b)) : *(b)))
+#define	CICHEQ(a,b) (((isascii(UCHARAT(a)) && isupper(UCHARAT(a))) ? tolower(UCHARAT(a)) : UCHARAT(a)) \
+		  == ((isascii(UCHARAT(b)) && isupper(UCHARAT(b))) ? tolower(UCHARAT(b)) : UCHARAT(b)))
 
 /*
  * Flags to be passed up and down.
@@ -169,7 +171,7 @@ extern char *strchr();
 /*
  * Global work variables for tregcomp().
  */
-STATIC char *regparse;		/* Input-scan pointer. */
+STATIC const char *regparse;	/* Input-scan pointer. */
 STATIC int regnpar;		/* () count. */
 STATIC char regdummy;
 STATIC char *regcode;		/* Code-emit pointer; &regdummy = don't. */
@@ -179,27 +181,29 @@ STATIC tregexp *regprog;
 /*
  * Forward declarations for tregcomp()'s friends.
  */
-STATIC char *reg();
-STATIC char *regbranch();
-STATIC char *regpiece();
-STATIC char *regatom();
-STATIC char *regnode();
-STATIC char *regnext();
-STATIC void regc();
-STATIC void reginsert();
-STATIC void regtail();
-STATIC void regoptail();
+STATIC const char *reg       __((int, int *));
+STATIC const char *regatom   __((int *));
+STATIC const char *regbranch __((int *));
+STATIC       char *regnode   __((int));
+STATIC const char *regpiece  __((int *));
+STATIC       int   regmatch  __((const char *));
+STATIC       int   regrepeat __((const char *));
+STATIC       int   regtoken  __((struct token **, const char *, int));
+STATIC       int   regtry    __((tregexp *, struct token *));
+STATIC       void  regc      __((int));
+STATIC       void  reginsert __((int, const char *));
+STATIC       void  regoptail __((const char *, const char *));
+STATIC       void  regtail   __((const char *, const char *));
+STATIC const char *regnext   __((const char *));
+
 
 extern int funclevel;
-extern int ci2strncmp();
 extern char *progname;
-extern int fprintToken();
-extern int printdToken();
 
 #ifdef DEBUG
 extern int D_regnarrate, D_compare, D_matched;
-void tregdump();
-STATIC char *regprop();
+void tregdump __((tregexp *));
+STATIC char *regprop __((const char *));
 char *progp;
 #endif
 
@@ -220,11 +224,11 @@ char *progp;
  */
 tregexp *
 tregcomp(exp)
-char *exp;
+const char *exp;
 {
 	register tregexp *prog;
-	register char *scan;
-	register char *longest;
+	register const char *scan;
+	register const char *longest;
 	register int len;
 	int flags;
 
@@ -258,7 +262,8 @@ char *exp;
 	regnpar = 1;
 	regcode = prog->program;
 	regc(MAGIC);
-	if (reg(0, &flags, prog) == NULL) {
+	/* if (reg(0, &flags, prog) == NULL) ... */
+	if (reg(0, &flags) == NULL) {
 		free((char *)prog);
 		return NULL;
 	}
@@ -314,14 +319,14 @@ char *exp;
  * is a trifle forced, but the need to tie the tails of the branches to what
  * follows makes it hard to avoid.
  */
-STATIC char *
+STATIC const char *
 reg(paren, flagp)
 	int paren;			/* Parenthesized? */
 	int *flagp;
 {
-	register char *ret;
-	register char *br;
-	register char *ender;
+	register const char *ret;
+	register const char *br;
+	register const char *ender;
 	register int parno = 0;
 	int flags;
 
@@ -392,13 +397,13 @@ reg(paren, flagp)
  *
  * Implements the concatenation operator.
  */
-STATIC char *
+STATIC const char *
 regbranch(flagp)
 int *flagp;
 {
-	register char *ret;
-	register char *chain;
-	register char *latest;
+	register const char *ret;
+	register const char *chain;
+	register const char *latest;
 	int flags;
 
 	*flagp = WORST;		/* Tentatively. */
@@ -431,13 +436,13 @@ int *flagp;
  * It might seem that this node could be dispensed with entirely, but the
  * endmarker role is not redundant.
  */
-STATIC char *
+STATIC const char *
 regpiece(flagp)
 int *flagp;
 {
-	register char *ret;
-	register char op;
-	register char *next;
+	register const char *ret;
+	register       char op;
+	register const char *next;
 	int flags;
 
 	ret = regatom(&flags);
@@ -460,23 +465,23 @@ int *flagp;
 		/* Emit x* as (x&|), where & means "self". */
 		reginsert(BRANCH, ret);			/* Either x */
 		regoptail(ret, regnode(BACK));		/* and loop */
-		regoptail(ret, ret);			/* back */
-		regtail(ret, regnode(BRANCH));		/* or */
-		regtail(ret, regnode(NOTHING));		/* null. */
+		regoptail(ret, ret);			/* back     */
+		regtail(ret, regnode(BRANCH));		/* or       */
+		regtail(ret, regnode(NOTHING));		/* null.    */
 	} else if (op == '+' && (flags&SIMPLE))
 		reginsert(PLUS, ret);
 	else if (op == '+') {
 		/* Emit x+ as x(&|), where & means "self". */
-		next = regnode(BRANCH);			/* Either */
+		next = regnode(BRANCH);			/* Either    */
 		regtail(ret, next);
 		regtail(regnode(BACK), ret);		/* loop back */
-		regtail(next, regnode(BRANCH));		/* or */
-		regtail(ret, regnode(NOTHING));		/* null. */
+		regtail(next, regnode(BRANCH));		/* or        */
+		regtail(ret, regnode(NOTHING));		/* null.     */
 	} else if (op == '?') {
 		/* Emit x? as (x|) */
-		reginsert(BRANCH, ret);			/* Either x */
-		regtail(ret, regnode(BRANCH));		/* or */
-		next = regnode(NOTHING);		/* null. */
+		reginsert(BRANCH, ret);			/* Either x  */
+		regtail(ret, regnode(BRANCH));		/* or        */
+		next = regnode(NOTHING);		/* null.     */
 		regtail(ret, next);
 		regoptail(ret, next);
 	}
@@ -495,11 +500,11 @@ int *flagp;
  * faster to run.  Backslashed characters are exceptions, each becoming a
  * separate node; the code is simpler that way and it's not worth fixing.
  */
-STATIC char *
+STATIC const char *
 regatom(flagp)
 	int *flagp;
 {
-	register char *ret;
+	register const char *ret;
 	int flags;
 
 	*flagp = WORST;		/* Tentatively. */
@@ -611,7 +616,7 @@ regatom(flagp)
  */
 STATIC char *			/* Location. */
 regnode(op)
-	char op;
+	int op;
 {
 	register char *ret;
 	register char *ptr;
@@ -622,13 +627,13 @@ regnode(op)
 		return(ret);
 	}
 
-	ptr = ret;
-	*ptr++ = op;
-	*ptr++ = '\0';		/* Null "next" pointer. */
-	*ptr++ = '\0';
+	 ptr    = ret;
+	*ptr++  = op;
+	*ptr++  = '\0';		/* Null "next" pointer. */
+	*ptr++  = '\0';
 	regcode = ptr;
 
-	return(ret);
+	return (ret);
 }
 
 /*
@@ -636,7 +641,7 @@ regnode(op)
  */
 STATIC void
 regc(b)
-	char b;
+	int b;
 {
 	if (regcode != &regdummy)
 		*regcode++ = b;
@@ -651,12 +656,12 @@ regc(b)
  */
 STATIC void
 reginsert(op, opnd)
-	char op;
-	char *opnd;
+	int         op;
+	const char *opnd;
 {
-	register char *src;
-	register char *dst;
-	register char *place;
+	register  char * src;
+	register  char * dst;
+	register  char * place;
 
 	if (regcode == &regdummy) {
 		regsize += 3;
@@ -669,7 +674,8 @@ reginsert(op, opnd)
 	while (src > opnd)
 		*--dst = *--src;
 
-	place = opnd;		/* Op node, where operand used to be. */
+	/* Yeah, we WRITE here! */
+	place   = (char*)opnd;	/* Op node, where operand used to be. */
 	*place++ = op;
 	*place++ = '\0';
 	*place++ = '\0';
@@ -680,12 +686,12 @@ reginsert(op, opnd)
  */
 STATIC void
 regtail(p, val)
-	char *p;
-	char *val;
+	const char *p;
+	const char *val;
 {
-	register char *scan;
-	register char *temp;
-	register int offset;
+	const char * scan;
+	char * wp;
+	int offset;
 
 	if (p == &regdummy)
 		return;
@@ -693,6 +699,7 @@ regtail(p, val)
 	/* Find last node. */
 	scan = p;
 	for (;;) {
+		register const char * temp;
 		temp = regnext(scan);
 		if (temp == NULL)
 			break;
@@ -703,8 +710,10 @@ regtail(p, val)
 		offset = scan - val;
 	else
 		offset = val - scan;
-	*(scan+1) = (offset>>8)&0377;
-	*(scan+2) = offset&0377;
+	/* Ok, we DO modify this.. */
+	wp = (char*)scan;
+	*(wp+1) = (offset >> 8) & 0xFF;
+	*(wp+2) =  offset       & 0xFF;
 }
 
 /*
@@ -712,8 +721,8 @@ regtail(p, val)
  */
 STATIC void
 regoptail(p, val)
-	char *p;
-	char *val;
+	const char *p;
+	const char *val;
 {
 	/* "Operandless" and "op != BRANCH" are synonymous in practice. */
 	if (p == NULL || p == &regdummy || OP(p) != BRANCH)
@@ -735,14 +744,6 @@ STATIC struct token **regstartp;	/* Pointer to startp array. */
 STATIC struct token **regendp;		/* Ditto for endp. */
 STATIC short regatomize[10] = { 0 };	/* if on turn DomainLiteral->Atom */
 
-/*
- * Forwards.
- */
-STATIC int regtry();
-STATIC int regmatch();
-STATIC int regrepeat();
-STATIC int regtoken();
-
 /*
  - tregexec - match a tregexp against a string
  */
@@ -784,13 +785,14 @@ tregexec(prog, tstring)
 	if (prog->regmust != NULL) {
 		int rmlen, len;
 		struct token *t;
-		char *cp;
 
 		for (s = tstring; s != NULL; s = s->t_next) {
+			const char *cp;
 			rmlen = prog->regmlen;
 			for (t = s, cp = prog->regmust; t != NULL && rmlen > 0;
 			     cp += len) {
-				if ((len = regtoken(&t, cp, rmlen)) == 0)
+				len = regtoken(&t, cp, rmlen);
+				if (len == 0)
 					break;
 				rmlen -= len;
 			}
@@ -845,7 +847,7 @@ success:
 STATIC int
 regtoken(tp, s, maxlen)
 	struct token **tp;		/* the token to match with */
-	char *s;			/* the string to match with */
+	const char *s;			/* the string to match with */
 	int maxlen;			/* how long the string is */
 {
 	register int	len, endbytes;
@@ -868,7 +870,7 @@ regtoken(tp, s, maxlen)
 		return 0;
 	if (endbytes && *(s+len) != ']')
 		return 0;
-	if (ci2strncmp((char *)(*tp)->t_pname, s, len) == 0) {
+	if (ci2strncmp((*tp)->t_pname, s, len) == 0) {
 		*tp = (*tp)->t_next;
 		return len + endbytes;
 	}
@@ -901,7 +903,7 @@ regtry(prog, tstring)
 	progp = prog->program;
 	if (regmatch(prog->program + 1)) {
 		prog->startp[0] = tstring;
-		prog->endp[0] = reginput;
+		prog->endp[0]   = reginput;
 		return(1);
 	} else
 		return(0);
@@ -919,21 +921,21 @@ regtry(prog, tstring)
  */
 STATIC int			/* 0 failure, 1 success */
 regmatch(prog)
-	char *prog;
+	const char *prog;
 {
-	register char *scan;	/* Current node. */
-	char *next;		/* Next node. */
+	register const char *scan;	/* Current node. */
+	const char *next;		/* Next node.    */
 
 	scan = prog;
 #ifdef DEBUG
 	if (scan != NULL && D_regnarrate)
-		fprintf(stderr, "%d%s(\n", scan-progp, regprop(scan));
+		fprintf(stderr, "%d%s(\n", scan - progp, regprop(scan));
 #endif
 	while (scan != NULL) {
 #ifdef DEBUG
 		if (D_regnarrate) {
 			fprintf(stderr, "%d%s...'",
-					scan-progp, regprop(scan));
+					scan - progp, regprop(scan));
 			if (reginput != NULL)
 				fprintToken(stderr, reginput, 0);
 			else
@@ -960,7 +962,7 @@ regmatch(prog)
 		case EXACTLY: {
 				struct token *t;
 				register int len, toklen;
-				register char *opnd;
+				register const char *opnd;
 
 				opnd = OPERAND(scan);
 				if (*opnd == ']' && previnput != NULL
@@ -1096,7 +1098,7 @@ regmatch(prog)
 			break;
 		case STAR:
 		case PLUS: {
-				register char *nextch;
+				register const char *nextch;
 				register int no, i;
 				register struct token *save, *saveprev;
 				register int min;
@@ -1151,11 +1153,11 @@ regmatch(prog)
  */
 STATIC int
 regrepeat(p)
-	char *p;
+	const char *p;
 {
-	register int count = 0;
+	register int    count = 0;
 	register struct token *scan;
-	register char *opnd;
+	register const  char  *opnd;
 
 	scan = reginput;
 	opnd = OPERAND(p);
@@ -1197,9 +1199,9 @@ regrepeat(p)
 /*
  - regnext - dig the "next" pointer out of a node
  */
-STATIC char *
+STATIC const char *
 regnext(p)
-	register char *p;
+	register const char *p;
 {
 	register int offset;
 
@@ -1226,16 +1228,16 @@ void
 tregdump(r)
 	tregexp *r;
 {
-	register char *s;
-	register char op = EXACTLY;	/* Arbitrary non-END op. */
-	register char *next;
+	register const char *s;
+	register       char op = EXACTLY;  /* Arbitrary non-END op. */
+	register const char *next;
 
 	fprintf(stderr, "Compilation of %s:\n", r->pattern);
 	s = r->program + 1;
 	while (op != END) {	/* While that wasn't END last time... */
 		op = OP(s);
 		fprintf(stderr,
-			"%2d%s", s-r->program, regprop(s));/* Where, what. */
+			"%2d%s", s - r->program, regprop(s));/* Where, what. */
 		next = regnext(s);
 		if (next == NULL)		/* Next ptr. */
 			fprintf(stderr, "(0)");
@@ -1272,9 +1274,9 @@ tregdump(r)
  */
 STATIC char *
 regprop(op)
-	char *op;
+	const char *op;
 {
-	register char *p = NULL;
+	register const char *p = NULL;
 	static char buf[50];
 
 	strcpy(buf, ":");
@@ -1354,7 +1356,7 @@ regprop(op)
 
 void
 tregerror(s, prog)
-	char *s;
+	const char *s;
 	tregexp *prog;
 {
 	if (prog != NULL && prog->pattern != NULL)
@@ -1372,13 +1374,13 @@ tregerror(s, prog)
  * $ returns the number of characters in dest, or 0 if no expansion
  * took place.
  */
-u_char *
+const char *
 tregsub(prog, n)
 	tregexp *prog;
 	int n;
 {
 	register struct token *ts, *te;
-	u_char *cp, *ep;
+	char *cp;
 	int len;
 	int buflen;
 	char *buf;
@@ -1391,7 +1393,7 @@ tregsub(prog, n)
 		FAIL("invalid substitution parameter requested");
 
 	ts = prog->startp[n];
-	te = prog->endp[n];
+	te = prog->endp  [n];
 
 	if (ts == NULL)
 		/*FAIL("null substitution expansion");*/
@@ -1399,12 +1401,13 @@ tregsub(prog, n)
 
 	ts = prog->startp[n];
 
+	/* Must use malloc()ed buffer -- May resize while processing */
 	buflen = 4000;
-	buf = malloc(buflen);
+	buf = (char*)malloc(buflen);
 	len = printdToken(&buf, &buflen, ts, te, 0);
 	buf[len] = '\0';
 
-	cp = tmalloc(len+1);
+	cp = (char*) tmalloc(len+1);
 	memcpy(cp,buf,len+1);
 	free(buf);
 
