@@ -19,16 +19,17 @@ typedef struct tregexp {
 #endif
 	char regstart;		/* Internal use only. */
 	char reganch;		/* Internal use only. */
-	char *regmust;		/* Internal use only. */
+	const char *regmust;	/* Internal use only. */
 	int regmlen;		/* Internal use only. */
-	char *pattern;		/* Human version of the program */
+	const char *pattern;	/* Human version of the program */
 	char program[1];	/* Unwarranted chumminess with compiler. */
 } tregexp;
 
-tregexp *tregcomp();
-int tregexec();
-unsigned char *tregsub();
-void tregerror(), tregdump();
+   tregexp *tregcomp  __((const char *));
+       int  tregexec  __((tregexp *, struct token *));
+const char *tregsub   __((tregexp *, int));
+      void  tregerror __((const char *, tregexp *));
+      void  tregdump  __((tregexp *));
 
 /*
  * The first byte of the regexp internal "program" is actually this magic
