@@ -22,27 +22,7 @@
 #include "splay.h"
 #include "sh.h"
 
-
-extern RETSIGTYPE trap_handler();
-extern char *optarg;
-extern char *strerror();
-extern int errno;
-extern int interrupted;
-extern int leaux();
-extern int optind;
-extern int sprung;
-extern int xundefun();
-extern struct codedesc *interpret();
-extern struct sptree *spt_funclist;
-extern void mail_check();
-extern void prompt2_print();
-extern void prompt_print();
-extern void setfreefd();
-extern void sp_null();
-extern void trapped();
-extern void trapsnap(), v_envinit(), glob_init();
-extern void v_set(), path_flush(), mail_flush(), mail_intvl();
-
+#include "libsh.h"
 
 struct cmddef commands[] = {
 #include "sh-out.i"
@@ -55,22 +35,18 @@ struct sptree *spt_builtins = NULL;
 struct sptree *spt_searchpath;
 
 int saweof;
-
 FILE *runiofp = stdout;
-
-struct conscell *commandline = NULL;	/* argument to -c option */
-
-char *progname;
-
+conscell *commandline = NULL;	/* argument to -c option */
+const char *progname;
 struct osCmd avcmd;
 
-u_char shfl[CHARSETSIZE/NBBY];
+char shfl[CHARSETSIZE/NBBY];
 
 int
 zshtoplevel(errname)
-	char *errname;
+	const char *errname;
 {
-	unsigned char *table, *eotable;
+	void *table, *eotable;
 	int status = 0;
 	memtypes oval;
 
@@ -100,8 +76,7 @@ zshtoplevel(errname)
 			if (isset('V'))
 				table = optimize(1, table, &eotable);
 		}
-		interpret(table, eotable, (u_char *)NULL,
-			  &avcmd, &status,
+		interpret(table, eotable, NULL, &avcmd, &status,
 			  (struct codedesc *)NULL);
 	}
 
@@ -111,7 +86,7 @@ zshtoplevel(errname)
 
 void
 zshprofile(command)
-	char *command;
+	const char *command;
 {
 	memtypes oval = stickymem;
 
@@ -125,7 +100,7 @@ zshprofile(command)
 void
 zshinit(argc, argv)
 	int argc;
-	char **argv;
+	const char **argv;
 {
 	int c, errflag, io, uid, loadit;
 	register struct shCmd *shcmdp;
@@ -152,8 +127,7 @@ zshinit(argc, argv)
 		spt_builtins = sp_init();
 
 	for (shcmdp = &builtins[0]; shcmdp->name != NULL; ++shcmdp)
-		sp_install(symbol((u_char *)shcmdp->name), (u_char *)shcmdp,
-				 0, spt_builtins);
+	  sp_install(symbol(shcmdp->name), (void *)shcmdp, 0, spt_builtins);
 
 	TOKEN_NARGS(sBufferSet) = 1;
 	TOKEN_NARGS(sBufferAppend) = 1;
@@ -193,7 +167,11 @@ zshinit(argc, argv)
 	}
 	loadit = errflag = 0;
 	optind = 1;	/* not to be influenced by previous getopt()'s */
-	while ((c = getopt(argc, argv, "CILMOPRSYc:l:isaefhkntuvx")) != EOF) {
+	while (1) {
+		c = getopt(argc, (char*const*)argv,
+			   "CILMOPRSYc:l:isaefhkntuvx");
+		if (c == EOF)
+		  break;
 		switch (c) {
 		case 'O':	/* optimize */
 			if (isset(c))
@@ -284,8 +262,8 @@ zshinit(argc, argv)
 
 	avcmd.argv = s_listify(argc-optind, &argv[optind]);
 	if (isset('s')) {
-		struct conscell *d, *tmp;
-		d = conststring((u_char *)progname);
+		conscell *d, *tmp;
+		d = conststring(progname);
 		s_push(d, avcmd.argv);
 	}
 
@@ -304,7 +282,7 @@ zshinit(argc, argv)
 	v_set(IFS, DEFAULT_IFS);
 
 	if (isset('l') && argv[optind] != NULL)
-		exit(leaux(argv[optind], (struct stat *)NULL));
+	  exit(leaux(-1, argv[optind], (struct stat *)NULL));
 
 	stickymem = oval;
 }
