@@ -7,10 +7,10 @@
 #include "mailer.h"
 
 /* all users in this group are considered 'trusted'... i.e. From: is believed */
-char	*trusted_group = "zmailer";
+const char * trusted_group = "zmailer";
 
 /* these are the users to be trusted if trusted_group doesn't exist */
-char	*default_trusted[] = { "root", "daemon", "daemons", "uucp", 0 };
+const char * default_trusted[] = { "root", "daemon", "daemons", "uucp", 0 };
 
 /* this group will own control and message files after the router is done. */
 /* Users in this group can view the entire mail queue contents, if it doesn't
@@ -20,16 +20,16 @@ char	*files_group = NULL;		/* if this is NULL, we use files_gid */
 int	files_gid = 0;
 
 /* the list of message headers containing addresses used to identify sender */
-char	*prio_list[] = { "sender", "from", "reply-to", "errors-to", 0 };
+const char * prio_list[] = { "sender", "from", "reply-to", "errors-to", 0 };
 	
 /* the list of message headers containing addresses used for bouncing mail */
-char	*err_prio_list[] = { "sender", "errors-to", 0 };
+const char * err_prio_list[] = { "sender", "errors-to", 0 };
 
 /* stdout and stderr end up in logdir/progname when router runs as a daemon */
-char	*logdir = "/usr/spool/log";
+const char * logdir = "/usr/spool/log";
 
 /* suffix of the router configuration file, as in MAILSHARE/progname.suffix */
-char	*cf_suffix = "cf";
+const char * cf_suffix = "cf";
 
 /* how often (in seconds) should the router directory be scanned? */
 u_int	sweepintvl = 15;
