@@ -36,26 +36,31 @@ extern struct sptree *spt_databases; /* At conf.c */
 typedef enum { Nul, Boolean, Pathalias, Indirect, NonNull } postprocs;
 
 struct cache {
-	u_char		*key;
-	struct conscell	*value;
+	char		*key;
+	conscell	*value;
 	time_t		expiry;
 };
 
+#define DBFUNC(_fn_)  (* _fn_) __((search_info *))
+#define DBFUNCD(_fn_)  (* _fn_) __((conscell *DBFUNC(lookupfn), search_info *))
+#define DBFUNCV(_fn_) (* _fn_) __((search_info *, const char *))
+#define DBFUNCF(_fn_) (* _fn_) __((search_info *, FILE *))
+
 struct db_info {
-	char		*file;			/* a file parameter */
-	char		*subtype;		/* optional selector */
+	const char	*file;			/* a file parameter */
+	const char	*subtype;		/* optional selector */
 	int		flags;			/* miscellaneous options */
 	int		cache_size;		/* default cache size */
 	time_t		ttl;			/* default time to live */
-	struct conscell	*(*driver)();		/* completion routine */
-	struct conscell	*(*lookup)();		/* low-level lookup routine */
-	void		(*close)();		/* flush buffered data, close */
-	int		(*add)();		/* add key/value pair */
-	int		(*remove)();		/* remove key */
-	void		(*print)();		/* print database */
-	void		(*count)();		/* count database */
-	void		(*owner)();		/* print database owner */
-	int		(*modcheckp)();		/* should we reopen database? */
+	conscell	*DBFUNCD(driver);	/* completion routine */
+	conscell	*DBFUNC(lookup);	/* low-level lookup routine */
+	void		DBFUNC(close);		/* flush buffered data, close*/
+	int		DBFUNCV(add);		/* add key/value pair */
+	int		DBFUNC(remove);		/* remove key */
+	void		DBFUNCF(print);		/* print database */
+	void		DBFUNCF(count);		/* count database */
+	void		DBFUNCF(owner);		/* print database owner */
+	int		DBFUNC(modcheckp);	/* should we reopen database?*/
 	postprocs	postproc;		/* post-lookup applicator */
 	struct cache	*cache;			/* cache entry array */
 };
@@ -67,12 +72,17 @@ struct db_info {
 
 
 struct db_kind {
-	char		*name;		/* database type identification */
+	const char	*name;		/* database type identification */
 	struct db_info	config;		/* default configuration information */
 } db_kinds[] = {
 { "incore",	{ NULL, NULL, 0, 0, 0, NULL, search_core, close_core,
-		  add_core, remove_core, print_core, count_core, owner_core,
-		  NULL, Nul } },
+		  add_core,
+		  remove_core,
+		  print_core,
+		  count_core,
+		  owner_core,
+		  NULL,
+		  Nul } },
 { "header",	{ NULL, NULL, 0, 0, 0, NULL, search_header, close_header,
 		  add_header, remove_header, print_header, count_header,
 		  owner_header, NULL, Nul } },
@@ -89,10 +99,10 @@ struct db_kind {
 		  Nul } },
 #endif
 
-#ifdef	HAVE_HOSTS
-{ "hostsfile",	{ HOSTS_FILE, NULL, 0, 10, 0, NULL, search_hosts, NULL,
+#ifdef	HAVE_RESOLVER
+{ "hostsfile",	{ "/etc/hosts", NULL, 0, 10, 0, NULL, search_hosts, NULL,
 		  NULL, NULL, print_hosts, NULL, NULL, NULL, Nul } },
-#endif	/* HAVE_HOSTS */
+#endif	/* HAVE_RESOLVER */
 #ifdef	HAVE_RESOLVER
 #ifndef RESOLV_CONF
 # define RESOLV_CONF "/etc/resolv.conf"
@@ -130,23 +140,13 @@ struct db_kind {
 };
 
 
-/*
-extern char *pname();
-extern int D_db;
-extern int deferit;
-extern int stickymem;
-extern time_t now;
-extern void freeTokens();
-extern void v_set();
-*/
-
 /* drivers */
-static struct conscell	*find_domain __((struct conscell *(*lookupfn)(), struct search_info *sip));
-static struct conscell	*find_nodot_domain __((struct conscell *(*lookupfn)(), struct search_info *sip));
+static conscell	*find_domain       __((conscell *DBFUNC(lookupfn), search_info *sip));
+static conscell	*find_nodot_domain __((conscell *DBFUNC(lookupfn), search_info *sip));
 /* others.. */
-static void cacheflush __((struct db_info *dbip));
-extern struct conscell	*readchunk __((char *file, long offset));
-static int	iclistdbs __((struct spblk *spl));
+static void      cacheflush __((struct db_info *dbip));
+extern conscell	*readchunk  __((const char *file, long offset));
+static int	 iclistdbs  __((struct spblk *spl));
 
 
 
@@ -157,8 +157,8 @@ static int	iclistdbs __((struct spblk *spl));
 
 int
 run_relation(argc, argv)
-	int	argc;
-	char	**argv;
+	int		  argc;
+	const char	**argv;
 {
 	struct db_info proto_config, *dbip;
 	struct db_kind *dbkp;
@@ -166,15 +166,18 @@ run_relation(argc, argv)
 	int c, errflg, set_cache_size;
 	spkey_t symid;
 	memtypes oval;
-	char *cp, *dbtype;
+	char *cp, *dbtyp;
 
 	errflg = 0;
-	dbtype = NULL;
+	dbtyp = NULL;
 	set_cache_size = 0;
 	optind = 1;
 	proto_config
 		= db_kinds[sizeof db_kinds/(sizeof (struct db_kind))-1].config;
-	while ((c = getopt(argc, argv, "Cbilmnpud:f:s:t:e:")) != EOF)
+	while (1) {
+		c = getopt(argc, (char*const*)argv, "Cbilmnpud:f:s:t:e:");
+		if (c == EOF)
+			break;
 		switch (c) {
 		case 'b':	/* boolean postprocessor */
 			proto_config.postproc = Boolean;
@@ -213,7 +216,7 @@ run_relation(argc, argv)
 			set_cache_size = 1;
 			break;
 		case 't':	/* database type */
-			dbtype = optarg;
+			dbtyp = optarg;
 			if ((cp = strchr(optarg, ',')) != NULL
 			    || (cp = strchr(optarg, '/')) != NULL) {
 				*cp++ = '\0';
@@ -236,7 +239,8 @@ run_relation(argc, argv)
 			++errflg;
 			break;
 		}
-	if (errflg || optind != argc - 1 || dbtype == NULL) {
+	}
+	if (errflg || optind != argc - 1 || dbtyp == NULL) {
 		fprintf(stderr,
 		"Usage: %s -t dbtype[,subtype] [-f file -e# -s# -bilmnpu -d driver] name\n",
 			argv[0]);
@@ -263,18 +267,18 @@ run_relation(argc, argv)
 	for (dbkp = &db_kinds[0];
 	     dbkp < &db_kinds[(sizeof db_kinds)/sizeof (struct db_kind)];
 	     ++dbkp) {
-		if (dbkp->name == NULL || strcmp(dbkp->name, dbtype) == 0)
+		if (dbkp->name == NULL || strcmp(dbkp->name, dbtyp) == 0)
 			break;
 	}
 	/* the -1 in the following compensates for the terminating null entry */
 	if (dbkp >= &db_kinds[((sizeof db_kinds)/sizeof (struct db_kind))-1]) {
 		fprintf(stderr,
 			"relation: I don't know about the %s database type!\n",
-			dbtype);
+			dbtyp);
 		return 3;
 	}
 
-	symid = symbol((u_char *)argv[optind]);
+	symid = symbol(argv[optind]);
 	if (sp_lookup(symid, spt_databases) != NULL) {
 		fprintf(stderr, "%s: %s is already a defined database!\n",
 				argv[0], argv[optind]);
@@ -294,7 +298,7 @@ run_relation(argc, argv)
 	    dbkp->config.owner != owner_seq) {
 		fprintf(stderr,
 			"%s: Indirect (-i) postprocessor can be used only with {un,}ordered databases!\n%s  Tried to use with: -t %s\n",
-			argv[0], argv[0], dbtype);
+			argv[0], argv[0], dbtyp);
 		return 7;
 	}
 
@@ -307,7 +311,7 @@ run_relation(argc, argv)
 		proto_config.file = strsave(argv[optind]);
 		stickymem = oval;
 		/* and the subtype is used to stash the splay tree */
-		proto_config.subtype = (char *)sp_init();
+		proto_config.subtype = (char*) sp_init();
 	} else {
 		if (proto_config.file == NULL)
 			proto_config.file = dbkp->config.file;
@@ -349,8 +353,9 @@ run_relation(argc, argv)
 		dbip->cache->key = NULL;	/* search terminator */
 	} else
 		dbip->cache = NULL;	/* superfluous, but why not ... */
-	sp_install(symid, (u_char *)dbip, 0, spt_databases);
-	if ((spl = sp_lookup(symbol((u_char *)DBLOOKUPNAME), spt_builtins)) == NULL) {
+	sp_install(symid, dbip, 0, spt_databases);
+	spl = sp_lookup(symbol(DBLOOKUPNAME), spt_builtins);
+	if (spl == NULL) {
 		fprintf(stderr, "%s: '%s' isn't built in\n",
 				argv[0], DBLOOKUPNAME);
 		return 1;
@@ -363,12 +368,13 @@ run_relation(argc, argv)
 
 struct sptree *
 icdbspltree(name)
-	char *name;
+	const char *name;
 {
 	struct db_info *dbip;
 	struct spblk *spl;
 
-	if ((spl = sp_lookup(symbol((u_char *)name), spt_databases)) == NULL)
+	spl = sp_lookup(symbol(name), spt_databases);
+	if (spl == NULL)
 		return NULL;
 	dbip = (struct db_info *)spl->data;
 	return (struct sptree *)dbip->subtype;
@@ -393,7 +399,7 @@ iclistdbs(spl)
 	struct db_kind *dbkp;
 	struct db_info *dbip;
 	struct cache *cachep;
-	char *cp;
+	const char *cp;
 	int i;
 
 	printf("%-13s", pname(spl->key));
@@ -465,12 +471,11 @@ iclistdbs(spl)
 int
 run_db(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int errflag;
-	char *cp;
 	struct db_info *dbip;
-	struct search_info si;
+	search_info si;
 	struct spblk *spl;
 
 	if (argc == 2 && (argv[1][0] == 'i' || argv[1][0] == 't')) {
@@ -501,7 +506,8 @@ run_db(argc, argv)
 		return 1;
 	}
 
-	if ((spl = sp_lookup(symbol((u_char *)argv[2]), spt_databases)) == NULL) {
+	spl = sp_lookup(symbol(argv[2]), spt_databases);
+	if (spl == NULL) {
 		fprintf(stderr, "%s: unknown database \"%s\"!\n",
 				argv[0], argv[2]);
 		return 2;
@@ -515,20 +521,16 @@ run_db(argc, argv)
 
 	if (argv[3] != NULL) {
 		if (dbip->flags & DB_MAPTOLOWER) {
-			for (cp = argv[3]; *cp != '\0'; ++cp)
-				if (isupper(*cp))
-					*cp = tolower(*cp);
+			strlower((char*)argv[3]);
 		} else if (dbip->flags & DB_MAPTOUPPER) {
-			for (cp = argv[3]; *cp != '\0'; ++cp)
-				if (islower(*cp))
-					*cp = toupper(*cp);
+			strupper((char*)argv[3]);
 		}
 	}
 
 	si.file = dbip->file;
-	si.key = (u_char *)argv[3];
+	si.key  = argv[3];
 	si.subtype = dbip->subtype;
-	si.ttl = dbip->ttl;
+	si.ttl  = dbip->ttl;
 
 	switch (argv[1][0]) {
 	case 'a':	/* add db key value */
@@ -623,20 +625,19 @@ run_db(argc, argv)
  * implements caching, etc.
  */
 
-struct conscell *
+conscell *
 db(dbname, key)
-	u_char *dbname, *key;
+	const char *dbname, *key;
 {
 	register int i, j, k;
 	memtypes oval = stickymem;
-	struct conscell *l, *ll, *tmp;
+	conscell *l, *ll, *tmp;
 	struct spblk *spl;
 	struct db_info *dbip;
-	u_char *cp, *realkey;
-	struct search_info si;
+	char *realkey;
+	search_info si;
 	struct cache tce;
-	u_char kbuf[BUFSIZ];	/* XX: */
-	u_char *kbend = kbuf + sizeof(kbuf) -1;
+	char kbuf[BUFSIZ];	/* XX: */
 
 	if (key == NULL || *key == '\0') {
 		fprintf(stderr,
@@ -655,30 +656,20 @@ db(dbname, key)
 	/* apply flags */
 	realkey = NULL;
 	if (dbip->flags & DB_MAPTOLOWER) {
-		for (cp = kbuf, realkey = key; *realkey; ++realkey) {
-		  if (cp < kbend)
-		    if (isupper(*realkey))
-		      *cp++ = tolower(*realkey);
-		    else
-		      *cp++ = *realkey;
-		}
-		*cp = '\0';
+		strncpy(kbuf, key, sizeof(kbuf));
+		kbuf[sizeof(kbuf)-1] = 0;
+		strlower(kbuf);
 		key = kbuf;
 	} else if (dbip->flags & DB_MAPTOUPPER) {
-		for (cp = kbuf, realkey = key; *realkey; ++realkey) {
-		  if (cp < kbend)
-		    if (islower(*realkey))
-		      *cp++ = toupper(*realkey);
-		    else
-		      *cp++ = *realkey;
-		}
-		*cp = '\0';
+		strncpy(kbuf, key, sizeof(kbuf));
+		kbuf[sizeof(kbuf)-1] = 0;
+		strupper(kbuf);
 		key = kbuf;
 	}
 	si.file = dbip->file;
-	si.key = key;
+	si.key  = key;
 	si.subtype = dbip->subtype;
-	si.ttl = dbip->ttl;
+	si.ttl  = dbip->ttl;
 	if ((dbip->flags & DB_MODCHECK)
 	    && dbip->modcheckp != NULL && (*dbip->modcheckp)(&si)) {
 		if (dbip->close != NULL) {
@@ -753,7 +744,7 @@ db(dbname, key)
 		oval = stickymem;	/* cannot return before this is reset */
 		stickymem = MEM_MALLOC;	/* this is reset down below */
 		/* key gets clobbered somewhere, so save it here */
-		realkey = (u_char *)strsave((char *)key);
+		realkey = strsave(key);
 	}
 	l = NULL;
 	if ((dbip->driver == NULL &&
@@ -765,16 +756,15 @@ db(dbname, key)
 		case Boolean:
 			if (stickymem == MEM_MALLOC)
 				s_free_tree(l);
-			l = newstring((u_char *)strsave((char *)key));
+			l = newstring(strsave(key));
 			break;
 		case NonNull:
 			if (STRING(l) && *(l->string) == '\0') {
 				if (stickymem == MEM_MALLOC)
 					s_free_tree(l);
-				l = newstring((u_char *)strsave((char *)key));
+				l = newstring(strsave(key));
 			} else if (LIST(l) && car(l) == NULL) {
-				car(l) = newstring((u_char *)
-						   strsave((char *)key));
+				car(l) = newstring(strsave(key));
 			}
 			break;
 		case Indirect:
@@ -788,7 +778,7 @@ db(dbname, key)
 				l = NULL;
 				break;
 			}
-			i = atoi((char *)l->string);	/* file offset */
+			i = atoi(l->string);	/* file offset */
 			if (stickymem == MEM_MALLOC)
 				s_free_tree(l);
 			l = readchunk(dbip->subtype, (long)i);
@@ -807,11 +797,11 @@ db(dbname, key)
 	} else if (dbip->postproc == NonNull) {
 		if (D_db)
 			fprintf(stderr, "%s(%s) = %s\n", dbname, key, key);
-		l = newstring((u_char *)strsave((char *)key));
+		l = newstring(strsave(key));
 	} else {
 		if (dbip->cache_size > 0) {
 			stickymem = oval;
-			free((char *)realkey);
+			free(realkey);
 		}
 		return NULL;
 	}
@@ -854,7 +844,7 @@ db(dbname, key)
 	} else if (dbip->cache_size > 0) {
 		stickymem = oval;
 		ll = s_copy_tree(l);	/* scratch version returned to shell */
-		free((char *)realkey);
+		free(realkey);
 		s_free_tree(l);
 	} else
 		ll = l;
@@ -862,9 +852,9 @@ db(dbname, key)
 	return ll;
 }
 
-char *
+const char *
 dbfile(dbname)
-	u_char *dbname;
+	const char *dbname;
 {
 	struct db_info *dbip;
 	struct spblk *spl;
@@ -887,14 +877,15 @@ cacheflush(dbip)
 	struct db_info *dbip;
 {
 	int i;
-	struct conscell *ll;
+	conscell *ll;
 
 	if (dbip == NULL || dbip->cache_size == 0 || dbip->cache == NULL)
 		return;
 
 	/* flush cache */
 	for (i = 0; i < dbip->cache_size && CACHE(i,key) != NULL; ++i) {
-		if ((ll = CACHE(i,value)) != NULL)
+		ll = CACHE(i,value);
+		if (ll != NULL)
 			s_free_tree(ll);
 		if (CACHE(i,key) != NULL) /* always true */
 			free((char *)CACHE(i,key));
@@ -915,7 +906,9 @@ _sptdbreset(spl)
 {
 	char *av[4];
 
-	av[0] = "db"; av[1] = "flush"; av[2] = pname((u_int)(spl->key));
+	av[0] = "db";
+	av[1] = "flush";
+	av[2] = pname((u_int)(spl->key));
 	av[3] = NULL;
 	printf("flushing cache of %s\n", av[2]);
 	run_db(3, av);
@@ -928,9 +921,9 @@ dbfree()
 }
 #endif	/* MALLOC_TRACE */
 
-char *
+const char *
 dbtype(dbname)
-	u_char *dbname;
+	const char *dbname;
 {
 	struct db_info *dbip;
 	struct db_kind *dbkp;
@@ -968,32 +961,35 @@ dbtype(dbname)
  *	.
  */
 
-static struct conscell *
+static conscell *
 find_domain(lookupfn, sip)
-	struct conscell *(*lookupfn)();
-	struct search_info *sip;
+	conscell *DBFUNC(lookupfn);
+	search_info *sip;
 {
-	register u_char *cp;
-	struct conscell *l;
-	u_char *realkey, *buf;
+	register char *cp;
+	conscell *l;
+	const char *realkey;
+	char *buf;
 	int keylen;
 
 	/* check the key as given */
-	if ((l = (*lookupfn)(sip)) != NULL)
+	l = (*lookupfn)(sip);
+	if (l != NULL)
 		return l;
 
 	realkey = sip->key;
-	keylen = strlen((char*)realkey);
-#if 1
+	keylen  = strlen(realkey);
+#define PREDOT_TEST
+#ifdef PREDOT_TEST
 #ifdef HAVE_ALLOCA
-	buf = (u_char*)alloca(keylen+2);
+	buf = (char*) alloca(keylen+2);
 #else
-	buf = (u_char*)emalloc(keylen+2);
+	buf = (char*) emalloc(keylen+2);
 #endif
 	/* No exact match, see if you can find a  "." + keystring ? */
 	if (*realkey != '.') {
 		buf[0] = '.';
-		strcpy((char *)&buf[1], (char *)realkey);
+		strcpy(buf + 1, realkey);
 	}
 #else
 	buf = realkey;
@@ -1006,8 +1002,9 @@ find_domain(lookupfn, sip)
 			++cp;
 		if (*(cp-1) == '.') {
 			sip->key = cp-1;
-			if ((l = (*lookupfn)(sip)) != NULL) {
-#if 1
+			l = (*lookupfn)(sip);
+			if (l != NULL) {
+#ifdef PREDOT_TEST
 #ifndef HAVE_ALLOCA
 				free(buf);
 #endif
@@ -1016,21 +1013,31 @@ find_domain(lookupfn, sip)
 			}
 		}
 	}
-#if 0
+#ifndef PREDOT_TEST
 	/* if all else failed, try prepending a dot and look for subdomains */
-	if (*realkey != '.' && strlen((char *)realkey) < sizeof buf - 1) {
+	if (*realkey != '.') {
+#ifdef HAVE_ALLOCA
+		buf = (char*) alloca(keylen+2);
+#else
+		buf = (char*) emalloc(keylen+2);
+#endif
 		buf[0] = '.';
-		strcpy((char *)&buf[1], (char *)realkey);
+		memcpy(buf + 1, realkey, keylen+1);
 		sip->key = buf;
-		if ((l = (*lookupfn)(sip)) != NULL)
+		l = (*lookupfn)(sip);
+#ifndef HAVE_ALLOCA
+		free(buf);
+#endif
+		if (l != NULL)
 			return l;
 	}
 #endif
 	/* Still failed ?  Try to look for "." */
-	sip->key = (void*)".";
-	if ((l = (*lookupfn)(sip)) != NULL)
+	sip->key = ".";
+	l = (*lookupfn)(sip);
+	if (l != NULL)
 		return l;
-	
+
 	return NULL;
 }
 
@@ -1041,31 +1048,30 @@ find_domain(lookupfn, sip)
  *	edu
  */
 
-static struct conscell *
+static conscell *
 find_nodot_domain(lookupfn, sip)
-	struct conscell *(*lookupfn)();
-	struct search_info *sip;
+	conscell *DBFUNC(lookupfn);
+	search_info *sip;
 {
-	register u_char *cp;
-	struct conscell *l;
-	u_char *realkey;
+	register const char *cp;
+	conscell *l = NULL;
 
 	/* iterate over the subdomains of the key */
-	for (cp = realkey = sip->key; *cp;) {
+	for (cp = sip->key; *cp;) {
 		while (*cp && *cp != '.')
 			++cp;
 		while (*cp == '.')
 			++cp;
 		if (*(cp-1) == '.') {
 			sip->key = cp;
-			if ((l = (*lookupfn)(sip)) != NULL)
+			l = (*lookupfn)(sip);
+			if (l != NULL)
 				return l;
 		}
 	}
 #if 0
 	/* if all else failed, try the name itself */
-	if ((l = (*lookupfn)(sip)) != NULL)
-		return l;
+	l = (*lookupfn)(sip);
 #endif
-	return NULL;
+	return l;
 }
