@@ -41,108 +41,104 @@
 #include "mail.h"
 #include "interpret.h"
 #include "io.h"
+#include "libz.h"
+#include "libc.h"
 
 #include "prototypes.h"
 
-
 #ifndef	_IOFBF
 #define	_IOFBF	0
 #endif	/* !_IOFBF */
 
+extern conscell *s_value;
+
+
 /* The builtin functions are declared and initialized here.  */
 
-#define ARGCV __((int argc, char *argv[]))
-static int run_hostname ARGCV;
+#define ARGCV __((int argc, const char *argv[]))
+static int run_hostname    ARGCV;
 static int run_whataddress ARGCV;
-static int run_erraddrlog ARGCV;
+static int run_erraddrlog  ARGCV;
 
-static struct conscell *run_dblookup __((struct conscell *avl, struct conscell *il));
-extern struct conscell *sh_car();
-static struct conscell *run_cadr __((struct conscell *avl, struct conscell *il));
-static struct conscell *run_caddr __((struct conscell *avl, struct conscell *il));
-static struct conscell *run_cadddr __((struct conscell *avl, struct conscell *il));
-static struct conscell *run_listexpand __((struct conscell *avl, struct conscell *il));
+static conscell *run_dblookup __((conscell *avl, conscell *il));
+extern conscell *sh_car       __((conscell *avl, conscell *il));
+static conscell *run_cadr     __((conscell *avl, conscell *il));
+static conscell *run_caddr    __((conscell *avl, conscell *il));
+static conscell *run_cadddr   __((conscell *avl, conscell *il));
+static conscell *run_listexpand __((conscell *avl, conscell *il));
 
 static int run_stability ARGCV;
-static int run_process ARGCV;
-static int run_grpmems ARGCV;
+static int run_process   ARGCV;
+static int run_grpmems   ARGCV;
 static int run_praliases ARGCV;
 static int run_listaddrs ARGCV;
-static int run_homedir ARGCV;
-static int run_822date ARGCV;
-static int run_filepriv ARGCV;
-static int run_runas ARGCV;
-static int run_gensym ARGCV;
+static int run_homedir   ARGCV;
+static int run_822date   ARGCV;
+static int run_filepriv  ARGCV;
+static int run_runas     ARGCV;
+static int run_gensym    ARGCV;
 static void free_gensym __((void));
 static int run_uid2login ARGCV;
 static int run_login2uid ARGCV;
-static int run_basename ARGCV;
-static int run_recase ARGCV;
-static int run_squirrel ARGCV;
+static int run_basename  ARGCV;
+static int run_recase    ARGCV;
+static int run_squirrel  ARGCV;
 static int run_822syntax ARGCV;
-static int run_dequote ARGCV;
+static int run_dequote   ARGCV;
 static int run_condquote ARGCV;
 
 #if	defined(XMEM) && defined(CSRIMALLOC)
 static int run_malcontents ARGCV;
 #endif	/* CSRIMALLOC */
 
-extern char *traps[];
-extern int getline();
+extern const char *traps[];
 extern int nobody;
-extern int getmyhostname();
-extern struct group *getgrnam();
-extern struct passwd *getpwnam();
-extern time_t time();
-extern void initline();
-extern void setfreefd();
-extern void v_purge();
-
-extern char *newattribute_2 __((char *, char *, char*));
+extern struct group  *getgrnam __((const char *));
+extern struct passwd *getpwnam __((const char *));
+extern time_t time __((time_t *));
 
 #ifndef strchr
 extern char *strchr(), *strrchr();
 #endif
 
-
 struct shCmd fnctns[] = {
-{	"relation",	run_relation,		0,		0	},
-{	DBLOOKUPNAME,	0,			run_dblookup,	SH_ARGV	},
+{	"relation",	run_relation,	NULL,	NULL,	0	},
+{	DBLOOKUPNAME,	NULL,	run_dblookup,	NULL,	SH_ARGV	},
 /* The following are optional but are probably a good idea */
-{	"db",		run_db,			0,		0	},
-{	"trace",	run_trace,		0,		0	},
-{	"untrace",	run_trace,		0,		0	},
-{	"hostname",	run_hostname,		0,		0	},
-{	"sender",	run_whataddress,	0,		0	},
-{	"recipient",	run_whataddress,	0,		0	},
-{	"erraddron",	run_erraddrlog,		0,		0	},
-{	"channel",	0,			sh_car,		SH_ARGV	},
-{	"host",		0,			run_cadr,	SH_ARGV	},
-{	"user",		0,			run_caddr,	SH_ARGV	},
-{	"attributes",	0,			run_cadddr,	SH_ARGV	},
-{	"stability",	run_stability,		0,		0	},
-{	"daemon",	run_daemon,		0,		0	},
-{	"process",	run_process,		0,		0	},
-{	"rfc822",	run_rfc822,		0,		0	},
-{	"groupmembers",	run_grpmems,		0,		0	},
-{	"printaliases",	run_praliases,		0,		0	},
-{	"listaddresses",run_listaddrs,		0,		SH_ARGV	},
-{	"listexpand",	0,			run_listexpand,	SH_ARGV	},
-{	"homedirectory",run_homedir,		0,		0	},
-{	"rfc822date",	run_822date,		0,		0	},
-{	"filepriv",	run_filepriv,		0,		SH_ARGV	},
-{	"runas",	run_runas,		0,		SH_ARGV	},
-{	"gensym",	run_gensym,		0,		0	},
-{	"uid2login",	run_uid2login,		0,		0	},
-{	"login2uid",	run_login2uid,		0,		0	},
-{	"basename",	run_basename,		0,		0	},
-{	"recase",	run_recase,		0,		0	},
-{	"squirrel",	run_squirrel,		0,		0	},
-{	"rfc822syntax",	run_822syntax,		0,		0	},
-{	"dequote",	run_dequote,		0,		0,	},
-{	"condquote",	run_condquote,		0,		0,	},
+{	"db",		run_db,		NULL,	NULL,	0	},
+{	"trace",	run_trace,	NULL,	NULL,	0	},
+{	"untrace",	run_trace,	NULL,	NULL,	0	},
+{	"hostname",	run_hostname,	NULL,	NULL,	0	},
+{	"sender",	run_whataddress,NULL,	NULL,	0	},
+{	"recipient",	run_whataddress,NULL,	NULL,	0	},
+{	"erraddron",	run_erraddrlog,	NULL,	NULL,	0	},
+{	"channel",	NULL,		sh_car,	NULL,	SH_ARGV	},
+{	"host",		NULL,	run_cadr,	NULL,	SH_ARGV	},
+{	"user",		NULL,	run_caddr,	NULL,	SH_ARGV	},
+{	"attributes",	NULL,	run_cadddr,	NULL,	SH_ARGV	},
+{	"stability",	run_stability,	NULL,	NULL,	0	},
+{	"daemon",	run_daemon,	NULL,	NULL,	0	},
+{	"process",	run_process,	NULL,	NULL,	0	},
+{	"rfc822",	run_rfc822,	NULL,	NULL,	0	},
+{	"groupmembers",	run_grpmems,	NULL,	NULL,	0	},
+{	"printaliases",	run_praliases,	NULL,	NULL,	0	},
+{	"listaddresses",run_listaddrs,	NULL,	NULL,	SH_ARGV	},
+{	"listexpand",	NULL,	run_listexpand,	NULL,	SH_ARGV	},
+{	"homedirectory",run_homedir,	NULL,	NULL,	0	},
+{	"rfc822date",	run_822date,	NULL,	NULL,	0	},
+{	"filepriv",	run_filepriv,	NULL,	NULL,	SH_ARGV	},
+{	"runas",	run_runas,	NULL,	NULL,	SH_ARGV	},
+{	"gensym",	run_gensym,	NULL,	NULL,	0	},
+{	"uid2login",	run_uid2login,	NULL,	NULL,	0	},
+{	"login2uid",	run_login2uid,	NULL,	NULL,	0	},
+{	"basename",	run_basename,	NULL,	NULL,	0	},
+{	"recase",	run_recase,	NULL,	NULL,	0	},
+{	"squirrel",	run_squirrel,	NULL,	NULL,	0	},
+{	"rfc822syntax",	run_822syntax,	NULL,	NULL,	0	},
+{	"dequote",	run_dequote,	NULL,	NULL,	0	},
+{	"condquote",	run_condquote,	NULL,	NULL,	0	},
 #if	defined(XMEM) && defined(CSRIMALLOC)
-{	"malcontents",	run_malcontents,	0,		0	},
+{	"malcontents",	run_malcontents,NULL,	NULL,	0	},
 #endif	/* CSRIMALLOC */
 /* The rest have been added locally */
 { 0 }
@@ -167,8 +163,8 @@ int		D_regnarrate = 0;
 extern int	D_rfc822;
 
 static struct debugind {
-	char	*name;
-	int	*indicator;
+	const char	*name;
+	int		*indicator;
 } buggers[] = {
 	{	"rfc822",		&D_rfc822	},
 	{	"sequencer",		&D_sequencer	},
@@ -186,7 +182,7 @@ static struct debugind {
 	{	"resolv",		&D_resolv	},
 	{	"memory",		&D_alloc	},
 	{	"except",		0		},
-	{	0,			0		}
+	{	NULL,			0		}
 };
 
 /* The builtin trace function. This is also used by command line debug specs */
@@ -194,11 +190,11 @@ static struct debugind {
 int
 run_trace(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct debugind *dbi;
 	int debug;
-	char *prog;
+	const char *prog;
 
 	if (argc == 1) {
 		fprintf(stderr, "Usage: %s all", argv[0]);
@@ -239,7 +235,7 @@ run_trace(argc, argv)
 static int
 run_hostname(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	static char hostname[128];
 
@@ -264,7 +260,7 @@ run_hostname(argc, argv)
 static int
 run_whataddress(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	static int toggle = 0;
 
@@ -290,7 +286,7 @@ char *erraddrlog;
 static int
 run_erraddrlog(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	switch (argc) {
 	case 1:
@@ -317,11 +313,11 @@ run_erraddrlog(argc, argv)
  * and is expected to act like a normal function (i.e. print value on stdout).
  */
 
-static struct conscell *
+static conscell *
 run_dblookup(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *l;
+	conscell *l;
 
 	il = cdar(avl);
 	if (il == NULL || !STRING(il) || cdr(il) != NULL) {
@@ -333,11 +329,11 @@ run_dblookup(avl, il)
 	return l;
 }
 
-static struct conscell *
+static conscell *
 run_cadr(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *tmp;
+	conscell *tmp;
 
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
@@ -370,11 +366,11 @@ run_cadr(avl, il)
 	return car(il);
 }
 
-static struct conscell *
+static conscell *
 run_caddr(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *tmp;
+	conscell *tmp;
 
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
@@ -420,11 +416,11 @@ run_caddr(avl, il)
 	return car(il);
 }
 
-static struct conscell *
+static conscell *
 run_cadddr(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
-	struct conscell *tmp;
+	conscell *tmp;
 
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
@@ -483,6 +479,7 @@ run_cadddr(avl, il)
 	return car(il);
 }
 
+static RETSIGTYPE sig_exit __((int));
 static RETSIGTYPE
 sig_exit(sig)
 int sig;
@@ -508,12 +505,14 @@ int sigarg;
 	SIGNAL_HANDLE(SIGHUP, sig_hup);
 }
 
-void
-dohup()
+static void dohup __((int));
+static void
+dohup(sig)
+int sig;
 {
 	gothup = 0;
 	if (traps[SIGHUP] != NULL)
-		eval(traps[SIGHUP], "trap", NULL);
+		eval(traps[SIGHUP], "trap", NULL, NULL);
 }
 
 /*
@@ -542,11 +541,14 @@ struct de {
 	time_t		mtime;
 };
 
+static int decmp __((const void *, const void *));
 static int
 decmp(a, b)
      const void *a, *b;
 {
-	register const struct de *aa = (struct de *)a, *bb = (struct de *)b;
+	register const struct de *aa = (struct de *)a;
+	register const struct de *bb = (struct de *)b;
+
 	return bb->mtime - aa->mtime;
 }
 
@@ -574,22 +576,22 @@ rd_endstability()
 		free((char *)nbarray);
 }
 
-static int rd_doit __((char *filename, char *dirs));
+static int rd_doit __((const char *filename, const char *dirs));
 static int
 rd_doit(filename, dirs)
-	char *filename, *dirs;
+	const char *filename, *dirs;
 {
 	/* Do one file, return value is 0 or 1,
 	   depending on actually doing something
 	   on a file */
 
-	static u_int blen = 0;
 #ifdef	USE_ALLOCA
 	char *buf;
 #else
 	static char *buf = NULL;
+	static u_int blen = 0;
 #endif
-	char *av[3];
+	const char *av[3];
 	char *p;
 	int len;
 	char pathbuf[512];
@@ -607,7 +609,8 @@ rd_doit(filename, dirs)
 
 	len = strlen(filename);
 	thatpid = 0;
-	if ((p=strchr(filename, '-')) != NULL) {
+	p = strchr(filename, '-');
+	if (p != NULL) {
 	  /* message file is "inode-pid" */
 	  thatpid = atoi(p+1);
 	  if (thatpid < 10) {	/* old-style locking.. */
@@ -656,7 +659,7 @@ rd_doit(filename, dirs)
 
 	  sprintf(buf, "%ld-%d", (long)stbuf.st_ino, router_id);
 
-	  if (eqrename(pathbuf, buf, 1) < 0)
+	  if (eqrename(pathbuf, buf) < 0)
 	    return 0;		/* something is wrong, erename() complains.
 				   (some other process picked it ?) */
 	  filename = buf;
@@ -671,7 +674,7 @@ rd_doit(filename, dirs)
 	av[0] = "process"; /* I think this needs to be here */
 	av[1] = filename;
 	av[2] = NULL;
-	s_apply(2, &av[0]); /* "process" filename (within  rd_doit() ) */
+	s_apply(2, av); /* "process" filename (within  rd_doit() ) */
 	free_gensym();
 
 	setlevel(MEM_SHCMD,sh_memlevel);
@@ -683,13 +686,12 @@ rd_doit(filename, dirs)
 	return 1;
 }
 
-static int rd_stability __((DIR *dirp, char *dirs));
+static int rd_stability __((DIR *dirp, const char *dirs));
 static int
 rd_stability(dirp,dirs)
 	DIR *dirp;
-	char *dirs;
+	const char *dirs;
 {
-	register int i;
 	int deindex, nbindex, did_cnt;
 	int namelen;
 	struct dirent *dp;
@@ -749,14 +751,14 @@ rd_stability(dirp,dirs)
 		return deindex;
 	}
 
-	qsort((char *)dearray, deindex, sizeof dearray[0], decmp);
+	qsort((void *)dearray, deindex, sizeof dearray[0], decmp);
 
 	did_cnt = 0;
 	while (deindex-- > 0) {
 		if (mustexit)
 			break;
 		if (gothup) 
-			dohup();
+			dohup(0);
 		did_cnt += rd_doit(nbarray + dearray[deindex].f_name, dirs);
 	}
 	return did_cnt;
@@ -778,7 +780,7 @@ rd_instability(dirp, dirs)
 		if (mustexit)
 			break;
 		if (gothup)
-			dohup();
+			dohup(0);
 
 		/* Handle only files beginning with number -- plus "core"-
 		   files.. */
@@ -800,7 +802,7 @@ rd_instability(dirp, dirs)
 static int
 run_stability(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	switch (argc) {
 	case 1:
@@ -824,7 +826,7 @@ run_stability(argc, argv)
 int
 run_daemon(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 #define ROUTERDIR_CNT 30
 	DIR *dirp[ROUTERDIR_CNT];  /* Lets say we have max 30 router dirs.. */
@@ -978,8 +980,8 @@ run_daemon(argc, argv)
  */
 
 struct protosw {
-	char *pattern;
-	char *function;
+	const char *pattern;
+	const char *function;
 } psw[] = {
 /*{	"[0-9]*.x400",		"x400"		}, */
 /*{	"[0-9]*.fax",		"fax"		}, */
@@ -991,7 +993,7 @@ struct protosw {
 static int
 run_process(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct protosw *pswp;
 	char *file;
@@ -1037,7 +1039,7 @@ run_process(argc, argv)
 static int
 run_grpmems(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	char **cpp;
 	struct group *grp;
@@ -1046,7 +1048,8 @@ run_grpmems(argc, argv)
 		fprintf(stderr, "Usage: %s groupname\n", argv[0]);
 		return 1;
 	}
-	if ((grp = getgrnam(argv[1])) == NULL) {
+	grp = getgrnam(argv[1]);
+	if (grp == NULL) {
 		fprintf(stderr, "%s: no group '%s'\n", argv[0], argv[1]);
 		return 1;
 	}
@@ -1063,24 +1066,29 @@ static struct headerinfo aliashdr = {
 static int
 run_praliases(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	register struct header *h;
 	struct envelope *e;
-	char *cp;
+	const char *cp;
 	int errflg, count, status;
 	long offset, size, prevsize, maxsize;
 	FILE *indexfp;
 	int c, verbose;
-	char *indexfile, buf[8192], ibuf[BUFSIZ];
+	const char *indexfile;
+	char buf[8192], ibuf[BUFSIZ];
 	struct siobuf *osiop = NULL;
 	int tabsep = 0;
 
 
-	verbose = 0; indexfile = NULL;
+	verbose = 0;
+	indexfile = NULL;
 	optind = 1;
 	errflg = 0;
-	while ((c = getopt(argc, argv, "vo:t")) != EOF) {
+	while (1) {
+		c = getopt(argc, (char*const*)argv, "vo:t");
+		if (c == EOF)
+			break;
 		switch (c) {
 		case 'v':
 			++verbose;
@@ -1152,9 +1160,7 @@ run_praliases(argc, argv)
 
 	/* store all aliases in canonical lowercase */
 	for (h = e->e_headers, count = 0; h != NULL; h = h->h_next, ++count) {
-		for (cp = h->h_pname; cp && *cp; ++cp)
-			if (isascii(*cp) && isupper(*cp))
-				*cp = tolower(*cp);
+	  strlower((char*)h->h_pname);
 	}
 
 	if (count <= 0) {
@@ -1265,37 +1271,33 @@ FILE *fp;
 /* listexpand()  -- do a bunch of tasks usually done with a group of
    functions, starting with  "listaddresses" ... */
 
-static struct conscell *
+static conscell *
 run_listexpand(avl, il)
-	struct conscell *avl, *il;
+	conscell *avl, *il;
 {
 	struct header hs;
 	struct envelope *e;
 	struct address *ap, *aroot = NULL, **atail = &aroot;
 	struct token *t;
-	struct addr *pp;
-	struct conscell *al, *alp = NULL, *tmp;
+	conscell *al, *alp = NULL, *tmp;
 	char *localpart, *origaddr, *attributenam;
-	int c, n, len, errflag, stuff;
+	int c, n, errflag, stuff;
 	volatile int cnt;
-	char *comment, *erroraddress;
-	u_char *s, *s_end;
+	const char *comment, *erroraddress;
+	char *s, *s_end;
 	int privilege = -1;
 	struct siobuf *osiop = NULL;
-	FILE *mfp = NULL, *fp, f;
+	FILE *mfp = NULL, *fp;
 	char buf[4096];
 	char DSNbuf[4096+200];
 	int fd2;
 	char *olderrors = errors_to;
-	volatile FILE *errstr = stderr;
 	char *notary = NULL;
 	int no_dsn = 0;
 	int okaddresses = 0;
 	int errcount = 0;
 	int linecnt = 0;
 
-	extern struct conscell *s_value;
-
 	il = cdar(avl); /* The CDR (next) of the arg list.. */
 
 	errflag = 0;
@@ -1365,7 +1367,7 @@ run_listexpand(avl, il)
 	localpart    = (char*) cdr(il)->string;
 	origaddr     = (char*) cddr(il)->string;
 
-	/* We (memory-)leak this stuff for a moment.. */
+	/* We (memory-)leak this stuff for a moment.. (but it is tmalloc()ed)*/
 	e = (struct envelope *)tmalloc(sizeof (struct envelope));
 	e->e_nowtime = now;
 	e->e_file = (char*) car(avl)->string;
@@ -1447,7 +1449,7 @@ run_listexpand(avl, il)
 
 		/* fix up any trailing comma (more sendmail
 		   compatibility kluges) */
-		s = t->t_pname + TOKENLEN(t)-1;
+		s = (char*)t->t_pname + TOKENLEN(t)-1;
 		if (c && (*s == ',' || *s == '\n'))
 		  *s = '\0';
 
@@ -1528,7 +1530,7 @@ run_listexpand(avl, il)
 
 		/* fix up any trailing comma (more sendmail
 		   compatibility kluges) */
-		s = t->t_pname + TOKENLEN(t)-1;
+		s = (char*)t->t_pname + TOKENLEN(t)-1;
 		if (c && (*s == ',' || *s == '\n'))
 		  *s = '\0';
 
@@ -1549,11 +1551,10 @@ run_listexpand(avl, il)
 	cnt = 0;
 	al = NULL;
 	for (ap = aroot; ap != NULL; ap = ap->a_next) {
-		char *av[5];
 		int rc;
 		int omem;
-		struct conscell *l, *d, *lrc;
-		char *s, *s2, *se;
+		conscell *l, *lrc;
+		char *s2, *se;
 
 		buf[0] = 0;
 		pureAddressBuf(buf,sizeof(buf),ap->a_tokens);
@@ -1579,7 +1580,7 @@ run_listexpand(avl, il)
 		  se = DSNbuf + sizeof(DSNbuf)-1; /* BUF-end */
 		  s = buf;
 		  while (*s) {
-		    u_char c = *s;
+		    c = *s;
 		    if ('!' <= c && c <= '~' && c != '+' && c != '=') {
 		      if (s2 < se)
 			*s2++ = c;
@@ -1598,9 +1599,9 @@ run_listexpand(avl, il)
 		omem = stickymem;
 		/* stickymem = MEM_MALLOC; */
 
-		l       = newstring((u_char*)strsave(buf));
-		cdr(l)  = newstring((u_char*)strsave(origaddr));
-		cddr(l) = newstring((u_char*)strsave(s));
+		l       = newstring(strsave(buf));
+		cdr(l)  = newstring(strsave(origaddr));
+		cddr(l) = newstring(strsave(s));
 		l = ncons(l);
 
 		stickymem = omem;
@@ -1624,9 +1625,9 @@ run_listexpand(avl, il)
 		if (deferit && (d = v_find(DEFER))) {
 		  s_free_tree(lrc);
 		  lrc = NULL;
-		  l = conststring((u_char *)"hold");
+		  l = conststring("hold");
 		  cdr(l)   = copycell(cdr(d));
-		  cddr(l)  = newstring((u_char *)strsave(buf));
+		  cddr(l)  = newstring(strsave(buf));
 		  cdddr(l) = car(attributes);
 		  l = ncons(l);
 		  l = ncons(l);
@@ -1678,7 +1679,7 @@ run_listexpand(avl, il)
 		  lrc = NULL;
 		}
 
-		/* Now the  "(struct conscell *) l" contains a route,
+		/* Now the  "(conscell *) l" contains a route,
 		   it is a time to put them together.. */
 		/* l ->  ( ((chan param param2 attrs)) )  */
 		if (al == NULL) {
@@ -1709,9 +1710,9 @@ run_listexpand(avl, il)
 	}
 
 	if (al == NULL) { /* ERROR! NO ADDRESSES! */
-	  al = conststring((u_char *)"error");
-	  cdr(al)  = conststring((u_char *)"expansion");
-	  cddr(al) = newstring((u_char*)strsave(localpart));
+	  al = conststring("error");
+	  cdr(al)  = conststring("expansion");
+	  cddr(al) = newstring(strsave(localpart));
 	  al = ncons(al);
 	  al = ncons(al);
 	}
@@ -1727,7 +1728,7 @@ run_listexpand(avl, il)
 static int
 run_listaddrs(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct header hs;
 	struct envelope *e;
@@ -1735,10 +1736,10 @@ run_listaddrs(argc, argv)
 	struct token *t;
 	struct addr *pp;
 	int c, n, errflag, stuff;
-	char *comment, *erroraddress;
-	u_char *s, *s_end;
+	const char *comment, *erroraddress;
+	char *s, *s_end;
 	struct siobuf *osiop = NULL;
-	FILE *mfp = NULL, *fp, f;
+	FILE *mfp = NULL, *fp;
 	char buf[4096];
 	int fd2;
 	int okaddresses = 0;
@@ -1750,7 +1751,10 @@ run_listaddrs(argc, argv)
 	comment = "list";
 	optind = 1;
 
-	while ((c = getopt(argc, argv, "c:e:E:")) != EOF) {
+	while (1) {
+		c = getopt(argc, (char*const*)argv, "c:e:E:");
+		if (c == EOF)
+			break;
 		switch (c) {
 		case 'c':
 			comment = optarg;
@@ -1857,7 +1861,7 @@ run_listaddrs(argc, argv)
 
 		/* fix up any trailing comma (more sendmail
 		   compatibility kluges) */
-		s = t->t_pname + TOKENLEN(t)-1;
+		s = (char*)t->t_pname + TOKENLEN(t)-1;
 		if (c && (*s == ',' || *s == '\n'))
 		  *s = '\0';
 
@@ -1937,7 +1941,7 @@ run_listaddrs(argc, argv)
 
 		/* fix up any trailing comma (more sendmail
 		   compatibility kluges) */
-		s = t->t_pname + TOKENLEN(t)-1;
+		s = (char*)t->t_pname + TOKENLEN(t)-1;
 		if (c && (*s == ',' || *s == '\n'))
 		  *s = '\0';
 
@@ -1977,24 +1981,19 @@ run_listaddrs(argc, argv)
 static int
 run_homedir(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct passwd *pw;
-	int changed;
-	char *cp;
 
 	if (argc != 2) {
 		fprintf(stderr, "Usage: %s name\n", argv[0]);
 		return 1;
 	}
-	if ((pw = getpwnam(argv[1])) == NULL) {
-		for (cp = argv[1], changed = 0 ; *cp != '\0'; ++cp) {
-			if (isascii(*cp) && isupper(*cp)) {
-				*cp = tolower(*cp);
-				++changed;
-			}
-		}
-		if (!changed || (changed && (pw = getpwnam(argv[1])) == NULL))
+	pw = getpwnam(argv[1]);
+	if (pw == NULL) {
+		strlower((char*)argv[1]);
+		pw = getpwnam(argv[1]);
+		if (pw == NULL)
 			return 2;
 	}
 	printf("%s\n", pw->pw_dir);
@@ -2004,15 +2003,15 @@ run_homedir(argc, argv)
 static int
 run_822date(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	time_t now;
+	time_t dnow;
 
-	time(&now);
+	time(&dnow);
 	if (argc == 2 && strcmp(argv[1], "-s") == 0)
-		printf("%ld\n", now);
+		printf("%ld\n", dnow);
 	else
-		printf("%s", rfc822date(&now));
+		printf("%s", rfc822date(&dnow));
 	return 0;
 }
 
@@ -2020,12 +2019,13 @@ run_822date(argc, argv)
 static int
 run_filepriv(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	char *argv0 = argv[0];
+	const char *argv0 = argv[0];
 	int id;
 	struct stat stbuf;
-	char *file, *cp, *dir;
+	const char *file, *cp;
+	char *dir;
 	int maxperm = 0666 ^ filepriv_mask_reg; /* XOR.. */
 	FILE *fp;
 
@@ -2046,7 +2046,8 @@ run_filepriv(argc, argv)
 	if (argc == 3 && isascii(argv[2][0]) && isdigit(argv[2][0]))
 		id = atoi(argv[2]);
 	else {
-		if ((fp = fopen(file, "r")) == NULL) {
+		fp = fopen(file, "r");
+		if (fp == NULL) {
 			/* if we can't open it, don't trust it */
 			perror(file);
 			fprintf(stderr,
@@ -2071,7 +2072,8 @@ run_filepriv(argc, argv)
 		}
 		id = stbuf.st_uid;
 	}
-	if ((cp = strrchr(file, '/')) == NULL) {
+	cp = strrchr(file, '/');
+	if (cp == NULL) {
 		printf("%d\n", id);
 		return 0;
 	} else if (cp == file)	/* root path */
@@ -2106,10 +2108,10 @@ run_filepriv(argc, argv)
 static int
 run_runas(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int uid, r;
-	char *cp;
+	const char *cp;
 	static int initeduid = 0;
 	static short myuid = -1;
 
@@ -2125,7 +2127,7 @@ run_runas(argc, argv)
 	if (isdigit(*cp))	/* what if user id is "3com" ? */
 		uid *= atoi(cp);
 	else			/* look up login name and get uid */
-		uid = login_to_uid((u_char*)cp);
+		uid = login_to_uid(cp);
 
 	if (!initeduid)
 		myuid = geteuid();
@@ -2139,20 +2141,20 @@ run_runas(argc, argv)
 	/* must be builtin or defined function */
 	r = s_apply(argc - 2, argv + 2); /* within:  run_runas() */
 	if (myuid == 0 && uid != 0) {
-	  if (setreuid(0, 0) < 0)
-	    abort(); /* user-identity change failed! */
+		if (setreuid(0, 0) < 0)
+			abort(); /* user-identity change failed! */
 	}
 
 	return r;
 }
 
 int gensym;
-char *gs_name = "g%d";
+const char *gs_name = "g%d";
 
 static int
 run_gensym(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	printf(gs_name, gensym++);
 	putchar('\n');
@@ -2174,7 +2176,7 @@ free_gensym()
 static int
 run_uid2login(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	if (argc != 2 || !isdigit(argv[1][0])) {
 		fprintf(stderr, "Usage: %s uid\n", argv[0]);
@@ -2187,22 +2189,22 @@ run_uid2login(argc, argv)
 static int
 run_login2uid(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	if (argc != 2) {
 		fprintf(stderr, "Usage: %s login\n", argv[0]);
 		return 1;
 	}
-	printf("%d\n", login_to_uid((u_char*)argv[1]));
+	printf("%d\n", login_to_uid(argv[1]));
 	return 0;
 }
 
 static int
 run_basename(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
-	char *cp;
+	const char *cp;
 	int len;
 
 	if (argc == 1) {
@@ -2210,7 +2212,8 @@ run_basename(argc, argv)
 				argv[0]);
 		return 1;
 	}
-	if ((cp = strrchr(argv[1], '/')) == NULL)
+	cp = strrchr(argv[1], '/');
+	if (cp == NULL)
 		cp = argv[1];
 	else
 		++cp;
@@ -2229,7 +2232,7 @@ run_basename(argc, argv)
 static int
 run_recase(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	char *cp;
 	int c, flag, errflg, action = 0;
@@ -2237,7 +2240,10 @@ run_recase(argc, argv)
 	optind = 1;
 	errflg = 0;
 
-	while ((c = getopt(argc, argv, "ulp")) != EOF) {
+	while (1) {
+		c = getopt(argc, (char*const*)argv, "ulp");
+		if (c == EOF)
+			break;
 		switch (c) {
 		case 'u':	/* up-case */
 		case 'l':	/* low-case */
@@ -2257,18 +2263,14 @@ run_recase(argc, argv)
 
 	switch (action) {
 	case 'u':
-		for (cp = argv[optind]; *cp != '\0'; ++cp)
-			if (isascii(*cp) && islower(*cp))
-				*cp = toupper(*cp);
+		strupper((char*)argv[optind]);
 		break;
 	case 'l':
-		for (cp = argv[optind]; *cp != '\0'; ++cp)
-			if (isascii(*cp) && isupper(*cp))
-				*cp = tolower(*cp);
+		strlower((char*)argv[optind]);
 		break;
 	case 'p':
 		flag = 1;
-		for (cp = argv[optind]; *cp != '\0'; ++cp) {
+		for (cp = (char*)argv[optind]; *cp != '\0'; ++cp) {
 			if (isascii(*cp) && isalnum(*cp)) {
 				if (flag && islower(*cp))
 					*cp = toupper(*cp);
@@ -2288,7 +2290,7 @@ run_recase(argc, argv)
 static int
 run_malcontents(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	mal_contents(stdout);
 }
@@ -2296,10 +2298,10 @@ run_malcontents(argc, argv)
 
 
 static struct {
-	short	fyitype;
-	short	fyisave;
-	char	*fyiname;
-	char	*fyitext;
+	short		 fyitype;
+	short		 fyisave;
+	const char	*fyiname;
+	const char	*fyitext;
 } fyitable[] = {
 { FYI_BREAKIN, 0, "breakin",	"external message claims local origin!"	},
 { FYI_BADHEADER, 0, "badheader","message header syntax error!"		},
@@ -2331,7 +2333,7 @@ optsave(type, e)
 static int
 run_squirrel(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int i, j, errflag, flag;
 
@@ -2375,7 +2377,7 @@ static struct headerinfo addrhdr = {
 static int
 run_822syntax(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct header hs;
 	struct envelope es;
@@ -2404,22 +2406,23 @@ run_822syntax(argc, argv)
 static int
 run_dequote(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int len;
-	char *s = argv[1];
+	const char *s = argv[1];
 
 	if (argc != 2)
 	  return 2; /* Bad bad! Missing/extra arg! */
 
 	len = strlen(s);
 
-	if ((*s == '"'  && s[len-1] == '"' ) ||
+	if (len > 1 &&
+	    (*s == '"'  && s[len-1] == '"' ) ||
 	    (*s == '\'' && s[len-1] == '\'')) {
-	  s[len-1] = 0;
-	  ++s;
-	}
-	printf("%s\n",s);
+	  fwrite(s+1,1,len-2,stdout);
+	} else
+	  fwrite(s,1,len,stdout);
+	putc('\n',stdout);
 	return 0;
 }
 
@@ -2427,11 +2430,14 @@ run_dequote(argc, argv)
 static int
 run_condquote(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int len;
-	char *s = argv[1];
-	char *sp, *bs;
+	const char *s = argv[1];
+	const char *sp, *bs;
+	int mustquote = 0;
+	int candequote = 0;
+	int c;
 
 	/* We remove quotes when they are not needed, and add them when
 	   they really are needed! */
@@ -2439,24 +2445,43 @@ run_condquote(argc, argv)
 	if (argc != 2)
 	  return 2; /* Bad bad! Missing/extra arg! */
 
+	c = *s;
+	if (c == '"') {
+	  ++s; /* Starting quote */
+	  while (*s && *s != '"') {
+	    /* While within quoted string */
+	    if (*s == '\\')
+	      ++s;
+	    if (*s != 0)
+	      ++s;
+	  }
+	  if (*s != '"')
+	    mustquote = 1;
+	  else if (*s == 0)
+	    candequote = 1;
+	}
+	while (*s) {
+	  if (c == '\\') {
+	    ++s;
+	    c = *s;
+	  } else if (c == ' ' || c == '\t')
+	    mustquote = 1; /* Unquoted spaces! */
+	  if (c != 0)
+	    ++s;
+	}
+
+	s = argv[1];
 	len = strlen(s);
 
-	sp = strchr(s,' '); /* White space ? */
-	if (!sp) sp = strchr(s,'\t');
-
-	bs = strchr(s,'\\'); /* Backslashes ? */
-
 	/* Quoted, and without a need for quotes */
-	if ((*s == '"'  && s[len-1] == '"' ) && !sp && !bs) {
+	if (candequote) {
 	  /* XXX: THIS SHOULD REALLY USE SOME SYNTAX SCANNER -- LIKE THAT ONE
 	          FOR SMTP: RFC821SCN()    */
-	  s = argv[1];
-	  s[len-1] = 0;
-	  ++s;
-	  printf("%s\n",s); /* Dequoted! */
+	  fwrite(argv[1] + 1, 1, len -2, stdout); /* Dequoted! */
+	  putc('\n', stdout);
 	  return 0;
 	}
-	if (!(*s == '"'  && s[len-1] == '"' ) && (sp || bs)) {
+	if (mustquote) {
 	  /* We need to quote this */
 	  s = argv[1];
 	  putchar('"'); /* First quote */
