@@ -5,6 +5,9 @@
 
 /* NOTE: This can't co-exist with ndbm! */
 
+/* We close the db after EVERY operation -- every search, every
+   add, ... */
+
 /* LINTLIBRARY */
 
 #include "mailer.h"
@@ -30,45 +33,60 @@ extern int store();
 #ifndef dbmclose
  extern void dbmclose();
 #endif
-extern void v_set();
 
 
 int
 open_dbm(sip, comment)
-	struct search_info *sip;
+	search_info *sip;
 	char *comment;
 {
+	int i;
+
 	if (sip->file == NULL)
 		return -1;
-	if (inited == 0 && dbminit(sip->file) < 0) {
-		++deferit;
-		v_set(DEFER, DEFER_IO_ERROR);
-		fprintf(stderr, "%s: cannot open %s!\n", comment, sip->file);
-		return -1;
+	if (inited == 0) {
+		/* Retry it a few times */
+		for (i = 0; i < 3; ++i) {
+			if (dbminit(sip->file) < 0) {
+				sleep(1);
+				continue;
+			}
+			inited = 1;
+			return 0;
+		}
 	}
-	inited = 1;
-	return 0;
+
+	/* Didn't succeed to init the DBM database, we signal defer-IO.. */
+
+	++deferit;
+	v_set(DEFER, DEFER_IO_ERROR);
+	fprintf(stderr, "%s: cannot open %s!\n", comment, sip->file);
+	return -1;
 }
 
 /*
  * Search a DBM format database for a key pair.
  */
 
-struct conscell *
+conscell *
 search_dbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
-	struct conscell *tmp;
+	conscell *tmp;
 	datum val, key;
 
 	if (open_dbm(sip, "search_dbm") < 0)
 		return NULL;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
+	key.dptr  = sip->key;
+	key.dsize = strlen(sip->key) + 1;
 	val = fetch(key);
-	if (val.dptr == NULL)
+	if (val.dptr == NULL) {
+		close_dbm(sip);
 		return NULL;
-	return newstring((u_char *)strnsave(val.dptr, val.dsize));
+	}
+	tmp = newstring(strnsave(val.dptr, val.dsize));
+	close_dbm(sip);
+	return tmp;
 }
 
 /*
@@ -77,7 +95,7 @@ search_dbm(sip)
 
 void
 close_dbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	if (sip->file == NULL)
 		return;
@@ -95,24 +113,26 @@ close_dbm(sip)
 
 int
 add_dbm(sip, value)
-	struct search_info *sip;
+	search_info *sip;
 	char *value;
 {
 	datum val, key;
 
 	if (open_dbm(sip, "add_dbm") < 0)
 		return EOF;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
-	val.dptr = value;
+	key.dptr  = sip->key;
+	key.dsize = strlen(sip->key) + 1;
+	val.dptr  = value;
 	val.dsize = strlen(value)+1;
 	if (store(key, val) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
 		fprintf(stderr, "add_dbm: cannot store (\"%s\",\"%s\")\n",
 				sip->key, value);
+		close_dbm(sip);
 		return EOF;
 	}
+	close_dbm(sip);
 	return 0;
 }
 
@@ -122,20 +142,22 @@ add_dbm(sip, value)
 
 int
 remove_dbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	datum key;
 
 	if (open_dbm(sip, "remove_dbm") < 0)
 		return EOF;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
+	key.dptr  = sip->key;
+	key.dsize = strlen(sip->key) + 1;
 	if (delete(key) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
 		fprintf(stderr, "remove_dbm: cannot remove \"%s\"\n", sip->key);
+		close_dbm(sip);
 		return EOF;
 	}
+	close_dbm(sip);
 	return 0;
 }
 
@@ -145,7 +167,7 @@ remove_dbm(sip)
 
 void
 print_dbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	datum key, val;
@@ -162,6 +184,7 @@ print_dbm(sip, outfp)
 			fprintf(outfp, "%s\t%s\n", key.dptr, val.dptr);
 	}
 	fflush(outfp);
+	close_dbm(sip);
 }
 
 /*
@@ -170,7 +193,7 @@ print_dbm(sip, outfp)
 
 void
 count_dbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	datum key, val;
@@ -185,6 +208,7 @@ count_dbm(sip, outfp)
 	  }
 	fprintf(outfp,"%d\n",count);
 	fflush(outfp);
+	close_dbm(sip);
 }
 
 /*
@@ -195,7 +219,7 @@ count_dbm(sip, outfp)
 
 void
 owner_dbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	struct stat stbuf;
