@@ -23,7 +23,7 @@ struct __gdbmfoo {
 #ifdef GDBM_FASTMODE	/* This appeared somewhen in between 0.9 and 1.7.3 */
         int fast_write;
 #endif
-	void (*fatal_err)();
+	void (*fatal_err) __((void));
 	int desc;       /* this is what we do need! */
 	/* the rest is irrelevant for us.. */
 };
@@ -32,8 +32,12 @@ struct __gdbmfoo {
 
 #include <fcntl.h>
 #include <sys/file.h>
+#include "libsh.h"
 #include "search.h"
 #include "io.h"
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
 
 /* extern int gdbm_errno; */
 extern int deferit;
@@ -44,13 +48,13 @@ extern int nobody;	/* UID of NOBODY */
  * Search an GDBM format database for a key pair.
  */
 
-struct conscell *
+conscell *
 search_gdbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	GDBM_FILE db;
 	datum val, key;
-	struct conscell *tmp;
+	conscell *tmp;
 	struct spblk *spl;
 	int retry;
 	spkey_t symid;
@@ -58,11 +62,11 @@ search_gdbm(sip)
 	if (sip->file == NULL)
 		return NULL;
 	retry = 0;
-	symid = symbol((u_char *)sip->file);
-	spl = sp_lookup(symid, spt_files);
+	symid = symbol(sip->file);
+	spl   = sp_lookup(symid, spt_files);
 	if (spl == NULL || (db = (GDBM_FILE)spl->data) == NULL) {
 reopen:
-		db = gdbm_open(sip->file, 0, GDBM_READER, 0, NULL);
+		db = gdbm_open((char*)sip->file, 0, GDBM_READER, 0, NULL);
 		if (db == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
@@ -73,10 +77,10 @@ reopen:
 		if (spl == NULL)
 			sp_install(symid, (u_char *)db, O_RDONLY, spt_files);
 		else
-			spl->data = (u_char *)db;
+			spl->data = (void *)db;
 	}
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
+	key.dptr  = (char*)sip->key;
+	key.dsize = strlen(sip->key) + 1;
 	val = gdbm_fetch(db, key);
 	if (val.dptr == NULL) {
 		if (!retry && gdbm_errno) {
@@ -87,7 +91,7 @@ reopen:
 		}
 		return NULL;
 	}
-	tmp = newstring((u_char *)strnsave(val.dptr, val.dsize));
+	tmp = newstring(strnsave(val.dptr, val.dsize));
 	free(val.dptr);
 	return tmp;
 }
@@ -98,7 +102,7 @@ reopen:
 
 void
 close_gdbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	GDBM_FILE db;
 	struct spblk *spl;
@@ -106,7 +110,7 @@ close_gdbm(sip)
 
 	if (sip->file == NULL)
 		return;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL)
 		sp_delete(spl, spt_modcheck);
@@ -117,12 +121,13 @@ close_gdbm(sip)
 	sp_install(symid, (u_char *)NULL, 0, spt_files);
 }
 
+static GDBM_FILE open_gdbm __((search_info *, int, const char *));
 
-GDBM_FILE
+static GDBM_FILE
 open_gdbm(sip, flag, comment)
-	struct search_info *sip;
+	search_info *sip;
 	int flag;
-	char *comment;
+	const char *comment;
 {
 	GDBM_FILE db;
 	struct spblk *spl;
@@ -130,14 +135,14 @@ open_gdbm(sip, flag, comment)
 
 	if (sip->file == NULL)
 		return NULL;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_files);
 	if (spl != NULL && flag == O_RDWR && spl->mark != O_RDWR)
 		close_gdbm(sip);
 	if (flag == O_RDWR)	flag = GDBM_WRITER;
 	else			flag = GDBM_READER;
 	if (spl == NULL || (db = (GDBM_FILE)spl->data) == NULL) {
-		db = gdbm_open(sip->file, 0, flag, 0, NULL);
+		db = gdbm_open((char*)sip->file, 0, flag, 0, NULL);
 		if (db == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
@@ -146,9 +151,9 @@ open_gdbm(sip, flag, comment)
 			return NULL;
 		}
 		if (spl == NULL)
-			sp_install(symid, (u_char *)db, flag, spt_files);
+			sp_install(symid, (void *)db, flag, spt_files);
 		else
-			spl->data = (u_char *)db;
+			spl->data = (void *)db;
 	}
 	return db;
 }
@@ -159,18 +164,19 @@ open_gdbm(sip, flag, comment)
 
 int
 add_gdbm(sip, value)
-	struct search_info *sip;
-	char *value;
+	search_info *sip;
+	const char *value;
 {
 	GDBM_FILE db;
 	datum val, key;
 
-	if ((db = open_gdbm(sip, O_RDWR, "add_gdbm")) == NULL)
+	db = open_gdbm(sip, O_RDWR, "add_gdbm");
+	if (db == NULL)
 		return EOF;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
-	val.dptr = value;
-	val.dsize = strlen(value)+1;
+	key.dptr  = (char*)sip->key;
+	key.dsize = strlen(sip->key) + 1;
+	val.dptr  = (char*)value;
+	val.dsize = strlen(value) + 1;
 	if (gdbm_store(db, key, val, GDBM_REPLACE) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
@@ -187,20 +193,22 @@ add_gdbm(sip, value)
 
 int
 remove_gdbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	GDBM_FILE db;
 	datum key;
 
-	if ((db = open_gdbm(sip, O_RDWR, "remove_gdbm")) == NULL)
+	db = open_gdbm(sip, O_RDWR, "remove_gdbm");
+	if (db == NULL)
 		return EOF;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
+	key.dptr  = (char*)sip->key;
+	key.dsize = strlen(sip->key) + 1;
 	if (gdbm_delete(db, key) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
-		fprintf(stderr, "remove_gdbm: cannot remove \"%s\" from \"%s\"\n",
-				sip->key, sip->file);
+		fprintf(stderr,
+			"remove_gdbm: cannot remove \"%s\" from \"%s\"\n",
+			sip->key, sip->file);
 		return EOF;
 	}
 	return 0;
@@ -212,13 +220,14 @@ remove_gdbm(sip)
 
 void
 print_gdbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	GDBM_FILE db;
 	datum key, nextkey, val;
 
-	if ((db = open_gdbm(sip, O_RDONLY, "print_gdbm", NULL)) == NULL)
+	db = open_gdbm(sip, O_RDONLY, "print_gdbm");
+	if (db == NULL)
 		return;
 	for (key = gdbm_firstkey(db); key.dptr != NULL; nextkey = gdbm_nextkey(db, key)) {
 		val = gdbm_fetch(db, key);
@@ -244,14 +253,15 @@ print_gdbm(sip, outfp)
 
 void
 count_gdbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	GDBM_FILE db;
 	datum key, nextkey, val;
 	int count = 0;
 
-	if ((db = open_gdbm(sip, O_RDONLY, "count_gdbm", NULL)) != NULL)
+	db = open_gdbm(sip, O_RDONLY, "count_gdbm");
+	if (db != NULL)
 	  for (key = gdbm_firstkey(db); key.dptr != NULL; nextkey = gdbm_nextkey(db, key)) {
 	    val = gdbm_fetch(db, key);
 	    if (val.dptr == NULL)
@@ -277,7 +287,7 @@ count_gdbm(sip, outfp)
 
 void
 owner_gdbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	GDBM_FILE db;
@@ -299,7 +309,7 @@ owner_gdbm(sip, outfp)
 
 int
 modp_gdbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	GDBM_FILE db;
 	int	fno;
@@ -320,8 +330,8 @@ modp_gdbm(sip)
 	if (stbuf.st_nlink == 0)
 		return 1;	/* Unlinked underneath of us! */
 
-	symid = symbol((u_char *)sip->file);
-	spl = sp_lookup(symid, spt_modcheck);
+	symid = symbol(sip->file);
+	spl   = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
 		rval = stbuf.st_mtime != (time_t)spl->data
 			|| (long)stbuf.st_nlink != (long)spl->mark
