@@ -7,15 +7,16 @@
 
 #include "mailer.h"
 #include <ctype.h>
+#include <string.h>
 #include "search.h"
 #include "io.h"
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
+
+#include "../prototypes.h"
 
-#ifndef strchr
-extern char *strchr();
-#endif
-extern int cistrcmp();
 extern struct sptree *spt_headers, *spt_eheaders;
-extern void sp_null();
 
 /*
  * RFC822 header list maintenance.
@@ -63,6 +64,7 @@ struct headerinfo envelope_hdrs[] = {
 { "authinfo",	nilHeaderSemantics,	nilUserType,	eIdentinfo	},
 { "bodytype",	nilHeaderSemantics,	nilUserType,	eBodytype	},
 { "channel",		AnyWord,	nilUserType,	eChannel	},
+{ "comment",	nilHeaderSemantics,	nilUserType,	eComment	},
 { "env-end",	nilHeaderSemantics,	nilUserType,	eEnvEnd		},
 { "env-eof",	nilHeaderSemantics,	nilUserType,	eEnvEnd		},
 { "envid",	nilHeaderSemantics,	nilUserType,	eEnvid		},
@@ -82,7 +84,7 @@ struct headerinfo envelope_hdrs[] = {
 };
 
 struct semtrans {
-	char		*name;
+	const char	*name;
 	HeaderSemantics semantics;	/* S/SL rule name in rfc822.ssl */
 } hdrsemtable[] = {
 {	"AMailboxList",		AMailboxList		},
@@ -111,28 +113,31 @@ struct semtrans {
 {	"UserAtDomain",		UserAtDomain		},
 };
 
-HeaderSemantics
+static HeaderSemantics semname2enum __((const char *));
+
+static HeaderSemantics
 semname2enum(name)
-	char *name;
+	const char *name;
 {
 	int i;
 
 	for (i = 0; i < (sizeof hdrsemtable / sizeof hdrsemtable[0]); ++i) {
-		if (cistrcmp(name, hdrsemtable[i].name) == 0)
-			return hdrsemtable[i].semantics;
+	  if (cistrcmp(name, hdrsemtable[i].name) == 0)
+	    return hdrsemtable[i].semantics;
 	}
 	return nilHeaderSemantics;
 }
 
-char *
+static const char * semenum2name __((HeaderSemantics d));
+const char *
 semenum2name(d)
 	HeaderSemantics d;
 {
 	int i;
 
 	for (i = 0; i < (sizeof hdrsemtable / sizeof hdrsemtable[0]); ++i) {
-		if (d == hdrsemtable[i].semantics)
-			return hdrsemtable[i].name;
+	  if (d == hdrsemtable[i].semantics)
+	    return hdrsemtable[i].name;
 	}
 	return NULL;
 }
@@ -146,56 +151,55 @@ init_header()
 
 	oval = stickymem;
 	for (i = 0; i < (sizeof mandatory_hdrs / sizeof mandatory_hdrs[0]); ++i) {
-		rh.hdr_name = mandatory_hdrs[i].hdr_name;
-		rh.semantics = mandatory_hdrs[i].semantics;
-		rh.user_type = mandatory_hdrs[i].user_type;
-		rh.class = mandatory_hdrs[i].class;
-		stickymem = MEM_MALLOC;
-		rhp = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
-		stickymem = oval;
-		*rhp = rh;
-		sp_install(symbol((u_char *)rhp->hdr_name),
-			   (u_char *)rhp, 1, spt_headers);
+	  rh.hdr_name  = mandatory_hdrs[i].hdr_name;
+	  rh.semantics = mandatory_hdrs[i].semantics;
+	  rh.user_type = mandatory_hdrs[i].user_type;
+	  rh.class     = mandatory_hdrs[i].class;
+	  stickymem    = MEM_MALLOC;
+	  rhp  = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	  stickymem    = oval;
+	  *rhp = rh;
+	  sp_install(symbol(rhp->hdr_name), (void *)rhp, 1, spt_headers);
 	}
 	for (i = 0; i < (sizeof optional_hdrs / sizeof optional_hdrs[0]); ++i) {
-		rh.hdr_name = optional_hdrs[i].hdr_name;
-		rh.semantics = optional_hdrs[i].semantics;
-		rh.user_type = optional_hdrs[i].user_type;
-		rh.class = optional_hdrs[i].class;
-		stickymem = MEM_MALLOC;
-		rhp = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
-		*rhp = rh;
-		stickymem = oval;
-		sp_install(symbol((u_char *)rhp->hdr_name),
-			   (u_char *)rhp, 0, spt_headers);
+	  rh.hdr_name  = optional_hdrs[i].hdr_name;
+	  rh.semantics = optional_hdrs[i].semantics;
+	  rh.user_type = optional_hdrs[i].user_type;
+	  rh.class     = optional_hdrs[i].class;
+	  stickymem    = MEM_MALLOC;
+	  rhp  = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	  *rhp = rh;
+	  stickymem = oval;
+	  sp_install(symbol(rhp->hdr_name), (void *)rhp, 0, spt_headers);
 	}
 	for (i = 0; i < (sizeof envelope_hdrs / sizeof envelope_hdrs[0]); ++i) {
-		rh.hdr_name = envelope_hdrs[i].hdr_name;
-		rh.semantics = envelope_hdrs[i].semantics;
-		rh.user_type = envelope_hdrs[i].user_type;
-		rh.class = envelope_hdrs[i].class;
-		stickymem = MEM_MALLOC;
-		rhp = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
-		*rhp = rh;
-		stickymem = oval;
-		sp_install(symbol((u_char *)rhp->hdr_name),
-			   (u_char *)rhp, 0, spt_eheaders);
+	  rh.hdr_name  = envelope_hdrs[i].hdr_name;
+	  rh.semantics = envelope_hdrs[i].semantics;
+	  rh.user_type = envelope_hdrs[i].user_type;
+	  rh.class     = envelope_hdrs[i].class;
+	  stickymem = MEM_MALLOC;
+	  rhp = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	  *rhp = rh;
+	  stickymem = oval;
+	  sp_install(symbol(rhp->hdr_name), (void *)rhp, 0, spt_eheaders);
 	}
 }
 
-struct sptree *
+static struct sptree * open_header __((search_info *));
+
+static struct sptree *
 open_header(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	if (sip->subtype == NULL)
-		return NULL;
+	  return NULL;
 	return (struct sptree *)sip->subtype;
 }
 
 struct headerinfo *
-find_header(db, name)
-	struct sptree *db;
-	char *name;
+find_header(hdb, name)
+	struct sptree *hdb;
+	const char *name;
 {
 	struct spblk *spl;
 	register char *cp;
@@ -220,12 +224,14 @@ find_header(db, name)
 #endif
 	strcpy(buffer, name);
 	for (cp = buffer; *cp != '\0'; ++cp) {
-		if (isascii(*cp) && isupper(*cp))
-			*cp = tolower(*cp);
+	  int c = (*cp) & 0xFF;
+	  if (isascii(c) && isupper(c))
+	    *cp = tolower(c);
 	}
 
-	if ((spl = sp_lookup(symbol((u_char *)buffer), db)) == NULL)
-		return NULL;
+	spl = sp_lookup(symbol(buffer), hdb);
+	if (spl == NULL)
+	  return NULL;
 	return (struct headerinfo *)spl->data;
 }
 
@@ -233,56 +239,60 @@ find_header(db, name)
  * Search headers database for a key.
  */
 
-struct conscell *
+conscell *
 search_header(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
-	struct sptree *db;
-	struct conscell *tmp;
+	struct sptree *hdb;
 	struct headerinfo *rhp;
+	conscell *tmp;
 	char buf[1024];
 
-	if ((db = open_header(sip)) == NULL)
-		return NULL;
-	rhp = find_header(db, sip->key);
+	hdb = open_header(sip);
+	if (hdb == NULL)
+	  return NULL;
+	rhp = find_header(hdb, sip->key);
 	if (rhp == NULL)
-		return NULL;
+	  return NULL;
 	sprintf(buf, "%s:%s:%s",
-		     semenum2name(rhp->semantics),
-		     rhp->user_type == Sender ? "Sender" :
-		     rhp->user_type == Recipient ? "Recipient" : "",
-		     rhp->class == Resent ? "Resent" : "");
-	return newstring((u_char *)strsave(buf));
+		semenum2name(rhp->semantics),
+		rhp->user_type  == Sender    ? "Sender"    :
+		(rhp->user_type == Recipient ? "Recipient" : ""),
+		rhp->class      == Resent    ? "Resent"    : "");
+	return newstring(strsave(buf));
 }
 
 /*
  * Free any information stored in this database.
  */
 
-int
+static int hdfreedata __((struct spblk *));
+static int
 hdfreedata(spl)
 	struct spblk *spl;
 {
 	if (spl->data)
-		free((char *)spl->data);
+	  free((void *)spl->data);
 	return 0;
 }
 
 void
 close_header(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
-	struct sptree *db;
+	struct sptree *hdb;
 
-	if ((db = open_header(sip)) == NULL)
-		return;
-	sp_scan(hdfreedata, (struct spblk *)NULL, db);
-	sp_null(db);
+	hdb = open_header(sip);
+	if (hdb == NULL)
+	  return;
+	sp_scan(hdfreedata, (struct spblk *)NULL, hdb);
+	sp_null(hdb);
 }
 
-int
+static int enormal __((const char *));
+static int
 enormal(s)
-	char *s;
+	const char *s;
 {
 	return (s == NULL || *s == '\0' || cistrcmp(s, "none") == 0 ||
 		cistrcmp(s, "normal") == 0 || cistrcmp(s, "-") == 0);
@@ -293,107 +303,168 @@ enormal(s)
  */
 
 int
-add_header(sip, value)
-	struct search_info *sip;
-	char *value;
+add_header(sip, cvalue)
+	search_info *sip;
+	const char *cvalue;
 {
-	struct sptree *db;
+	struct sptree *hdb;
 	struct spblk *spl;
 	memtypes oval;
-	char *cp;
+	char *cp, *value, *vbuf;
+	char *wcp;
+	char *lcbuf;
 	struct headerinfo rh, *rhp;
+	int keylen;
 
-	if ((db = open_header(sip)) == NULL)
-		return EOF;
+	hdb = open_header(sip);
+	if (hdb == NULL)
+	  return EOF;
 
 	/* parse a line like:  rulename:sender/recipient/-:resent/normal */
-	if (*value == '\0') {
-		fprintf(stderr, "add_header: null header specification\n");
-		return EOF;
+	if (*cvalue == '\0') {
+	  fprintf(stderr, "add_header: null header specification\n");
+	  return EOF;
 	}
-	if ((cp = strchr(value, ':')) == NULL) {
-		fprintf(stderr,
-			"add_header: missing sender/recipient and resent/normal fields in value\n");
-		return EOF;
+
+	keylen = strlen(cvalue);
+#ifdef USE_ALLOCA
+	value  = (char*)alloca(keylen+1);
+#else
+	value  = (char*)emalloc(keylen+1);
+	vbuf   = value;
+#endif
+	memcpy(value, cvalue, keylen+1);
+
+	cp = strchr(value, ':');
+	if (cp == NULL) {
+	  fprintf(stderr,
+		  "add_header: missing sender/recipient and resent/normal fields in value\n");
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
 	*cp++ = '\0';
 	rh.semantics = semname2enum(value);
 	if (rh.semantics == nilHeaderSemantics) {
-		fprintf(stderr, "add_header: unknown parse rule '%s'\n", value);
-		return EOF;
+	  fprintf(stderr, "add_header: unknown parse rule '%s'\n", value);
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
 	value = cp;
-	if (*value == '\0')
-		return EOF;
-	if ((cp = strchr(value, ':')) == NULL) {
-		fprintf(stderr,
-			"add_header: missing resent/normal field in value\n");
-		return EOF;
+	if (*value == '\0') {
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
+	}
+	cp = strchr(value, ':');
+	if (cp == NULL) {
+	  fprintf(stderr,
+		  "add_header: missing resent/normal field in value\n");
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
 	*cp++ = '\0';
 	if (enormal(value))
-		rh.user_type = nilUserType;
+	  rh.user_type = nilUserType;
 	else if (cistrcmp(value, "Sender") == 0)
-		rh.user_type = Sender;
+	  rh.user_type = Sender;
 	else if (cistrcmp(value, "Recipient") == 0)
-		rh.user_type = Recipient;
+	  rh.user_type = Recipient;
 	else {
-		fprintf(stderr,
-			"add_header: sender/recipient misspecified as '%s'\n",
-			value);
-		return EOF;
+	  fprintf(stderr,
+		  "add_header: sender/recipient misspecified as '%s'\n",
+		  value);
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
 	value = cp;
-	if ((cp = strchr(value, ':')) != NULL) {
-		fprintf(stderr, "add_header: junk at end of value: '%s'\n",
-				value);
-		return EOF;
+	cp = strchr(value, ':');
+	if (cp != NULL) {
+	  fprintf(stderr, "add_header: junk at end of value: '%s'\n",
+		  value);
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
 	if (enormal(value))
-		rh.class = normal;
+	  rh.class = normal;
 	else if (cistrcmp(value, "Resent") == 0)
-		rh.class = Resent;
+	  rh.class = Resent;
 	else {
-		fprintf(stderr,
-			"add_header: resent/normal misspecified as '%s'\n",
-			value);
-		return EOF;
+	  fprintf(stderr,
+		  "add_header: resent/normal misspecified as '%s'\n",
+		  value);
+#ifndef USE_ALLOCA
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
 
-	/* make sure the key is lowercase and has only appropriate characters */
+	/* make sure the key is lowercase, and
+	   has only appropriate characters */
 
-	for (cp = (char *)sip->key; *cp != '\0'; ++cp) {
-		if (*cp != '-'
-		    && (!isalpha(*cp) || isspace(*cp) || *cp == ':')) {
-			fprintf(stderr,
-				"add_header: bad character in key '%s'\n",
-				sip->key);
-			return EOF;
-		}
-		if (isupper(*cp))
-			*cp = tolower(*cp);
+	keylen = strlen(sip->key);
+#ifdef USE_ALLOCA
+	lcbuf = alloca(keylen+1);
+#else
+	lcbuf = (char*) emalloc(keylen+1);
+#endif
+	memcpy(lcbuf, sip->key, keylen+1);
+	for (wcp = lcbuf; *wcp != '\0'; ++wcp) {
+	  int c = (*wcp) & 0xFF;
+	  if (c != '-' && (!isalpha(c) || isspace(c) || c == ':')) {
+	    fprintf(stderr,
+		    "add_header: bad character in key '%s'\n",
+		    sip->key);
+#ifndef USE_ALLOCA
+	    free(lcbuf);
+#endif
+	    return EOF;
+	  }
+	  if (isupper(c))
+	    *wcp = tolower(c);
+	  else
+	    *wcp = c;
 	}
 
-	rh.hdr_name = (char *)sip->key;
+	rh.hdr_name = sip->key;
 
 	oval = stickymem;
 	stickymem = MEM_MALLOC;
-	rhp = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
+	rhp  = (struct headerinfo *)tmalloc(sizeof (struct headerinfo));
 	*rhp = rh;
 	stickymem = oval;
 
-	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL)
-		sp_install(symbol(sip->key), (void*)rhp, 0, db);
+	spl = sp_lookup(symbol(sip->key), hdb);
+	if (spl == NULL)
+	  sp_install(symbol(sip->key), (void *)rhp, 0, hdb);
 	else if (spl->mark == 0) {
-		hdfreedata(spl);
-		spl->data = (u_char *)rhp;
+	  hdfreedata(spl);
+	  spl->data = (void *)rhp;
 	} else {
-		free((char *)rhp);
-		fprintf(stderr,
-			"add_header: cannot change permanent definition of '%s'\n",
-			sip->key);
-		return EOF;
+	  free((void *)rhp);
+	  fprintf(stderr,
+		  "add_header: cannot change permanent definition of '%s'\n",
+		  sip->key);
+#ifndef USE_ALLOCA
+	  free(lcbuf);
+	  free(vbuf);
+#endif
+	  return EOF;
 	}
+#ifndef USE_ALLOCA
+	free(lcbuf);
+	free(vbuf);
+#endif
 
 	return 0;
 }
@@ -404,23 +475,25 @@ add_header(sip, value)
 
 int
 remove_header(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
-	struct sptree *db;
+	struct sptree *hdb;
 	struct spblk *spl;
 
-	if ((db = open_header(sip)) == NULL)
-		return EOF;
-	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL) {
-		fprintf(stderr, "remove_header: no such key as \"%s\"!\n",
-				sip->key);
-		return EOF;
+	hdb = open_header(sip);
+	if (hdb == NULL)
+	  return EOF;
+	spl = sp_lookup(symbol(sip->key), hdb);
+	if (spl == NULL) {
+	  fprintf(stderr, "remove_header: no such key as \"%s\"!\n",
+		  sip->key);
+	  return EOF;
 	} else if (spl->mark == 1) {
-		fprintf(stderr, "remove_header: cannot remove permanent definition of '%s'\n", sip->key);
-		return EOF;
+	  fprintf(stderr, "remove_header: cannot remove permanent definition of '%s'\n", sip->key);
+	  return EOF;
 	}
 	hdfreedata(spl);
-	sp_delete(spl, db);
+	sp_delete(spl, hdb);
 	return 0;
 }
 
@@ -430,6 +503,7 @@ remove_header(sip)
 
 static FILE *pcfp;
 
+static int hdprintdata __((struct spblk *));
 static int
 hdprintdata(spl)
 	struct spblk *spl;
@@ -438,26 +512,27 @@ hdprintdata(spl)
 
 	rhp = (struct headerinfo *)spl->data;
 	fprintf(pcfp, "%-16s\t%s:%s:%s (%s)\n", pname(spl->key),
-		     semenum2name(rhp->semantics),
-		     rhp->user_type == Sender ? "Sender" :
-		     rhp->user_type == Recipient ? "Recipient" : "-",
-		     rhp->class == Resent ? "Resent" : "-",
-		     spl->mark == 0 ? "optional" : "permanent");
+		semenum2name(rhp->semantics),
+		rhp->user_type  == Sender    ? "Sender"    :
+		(rhp->user_type == Recipient ? "Recipient" : "-"),
+		rhp->class      == Resent    ? "Resent"    : "-",
+		spl->mark       == 0         ? "optional"  : "permanent");
 	return 0;
 }
 
 
 void
 print_header(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
-	struct sptree *db;
+	struct sptree *hdb;
 
-	if ((db = open_header(sip)) == NULL)
-		return;
+	hdb = open_header(sip);
+	if (hdb == NULL)
+	  return;
 	pcfp = outfp;
-	sp_scan(hdprintdata, (struct spblk *)NULL, db);
+	sp_scan(hdprintdata, (struct spblk *)NULL, hdb);
 	fflush(outfp);
 }
 
@@ -465,6 +540,7 @@ print_header(sip, outfp)
 
 static int   pc_cnt;
 
+static int hdcountdata __((struct spblk *));
 static int
 hdcountdata(spl)
 	struct spblk *spl;
@@ -475,21 +551,22 @@ hdcountdata(spl)
 
 void
 count_header(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
-	struct sptree *db;
+	struct sptree *hdb;
 	pc_cnt = 0;
 
-	if ((db = open_header(sip)) != NULL)
-	  sp_scan(hdcountdata, (struct spblk *)NULL, db);
+	hdb = open_header(sip);
+	if (hdb != NULL)
+	  sp_scan(hdcountdata, (struct spblk *)NULL, hdb);
 	fprintf(outfp,"%d\n",pc_cnt);
 	fflush(outfp);
 }
 
 void
 owner_header(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	fprintf(outfp, "%d\n", getuid());
