@@ -9,26 +9,26 @@
 #ifdef	HAVE_RESOLVER
 #include <netdb.h>
 #include "search.h"
-
-extern char *dottedquad(), **hp_getaddr();
-extern void hp_init();
-
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
 
 /*
  * Search the hosts database file for a hostname alias.
  */
 
-struct conscell *
+conscell *
 search_hosts(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	struct hostent *hp;
-	struct conscell *tmp;
+	conscell *tmp;
 
 	sethostent(1);
-	if ((hp = gethostbyname((char *)(sip->key))) == NULL)
+	hp = gethostbyname(sip->key);
+	if (hp == NULL)
 		return NULL;
-	return newstring((u_char *)strsave(hp->h_name));
+	return newstring(strsave(hp->h_name));
 }
 
 /*
@@ -37,7 +37,7 @@ search_hosts(sip)
 
 void
 print_hosts(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 #if	!defined(sgi) && !defined(HAVE_RESOLVER)
