@@ -8,10 +8,11 @@
 #include "mailer.h"
 #include "search.h"
 #include "io.h"
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
 
-extern struct sptree *spt_loginmap;
-extern struct sptree *spt_uidmap;
-extern void sp_null();
+extern struct sptree *spt_loginmap, *spt_uidmap;
 
 /*
  * In-core database maintenance.
@@ -23,9 +24,10 @@ extern void sp_null();
  */
 
 
-struct sptree *
+static struct sptree * open_core __((search_info *));
+static struct sptree *
 open_core(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	if (sip->subtype == NULL)
 		return NULL;
@@ -36,28 +38,31 @@ open_core(sip)
  * Search an incore database for a key.
  */
 
-struct conscell *
+conscell *
 search_core(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	struct sptree *db;
 	struct spblk *spl;
-	struct conscell *tmp;
+	conscell *tmp;
 
-	if ((db = open_core(sip)) == NULL)
+	db = open_core(sip);
+	if (db == NULL)
 		return NULL;
-	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL)
+	spl = sp_lookup(symbol(sip->key), db);
+	if (spl == NULL)
 		return NULL;
 	if (spl->data == NULL)
-		return newstring((u_char *)strnsave("", 1));
-	return newstring((u_char *)strsave((char *)spl->data));
+		return newstring(strnsave("", 1));
+	return newstring(strsave((char *)spl->data));
 }
 
 /*
  * Free any information stored in this database.
  */
 
-int
+static int icfreedata __((struct spblk *));
+static int
 icfreedata(spl)
 	struct spblk *spl;
 {
@@ -68,11 +73,12 @@ icfreedata(spl)
 
 void
 close_core(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	struct sptree *db;
 
-	if ((db = open_core(sip)) == NULL)
+	db = open_core(sip);
+	if (db == NULL)
 		return;
 	if (db != spt_loginmap)
 		sp_scan(icfreedata, (struct spblk *)NULL, db);
@@ -85,14 +91,15 @@ close_core(sip)
 
 int
 add_core(sip, value)
-	struct search_info *sip;
-	char *value;
+	search_info *sip;
+	const char *value;
 {
 	struct sptree *db;
 	struct spblk *spl;
 	memtypes oval;
 
-	if ((db = open_core(sip)) == NULL)
+	db = open_core(sip);
+	if (db == NULL)
 		return EOF;
 
 	oval = stickymem;
@@ -103,11 +110,12 @@ add_core(sip, value)
 		value = strsave(value);
 	stickymem = oval;
 
-	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL)
-		sp_install(symbol(sip->key), (void*)value, 0, db);
+	spl = sp_lookup(symbol(sip->key), db);
+	if (spl == NULL)
+		sp_install(symbol(sip->key), (const void*)value, 0, db);
 	else {
 		icfreedata(spl);
-		spl->data = (u_char *)value;
+		spl->data = (const void *) value;
 	}
 
 	return 0;
@@ -119,14 +127,16 @@ add_core(sip, value)
 
 int
 remove_core(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	struct sptree *db;
 	struct spblk *spl;
 
-	if ((db = open_core(sip)) == NULL)
+	db = open_core(sip);
+	if (db == NULL)
 		return EOF;
-	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL) {
+	spl = sp_lookup(symbol(sip->key), db);
+	if (spl == NULL) {
 		fprintf(stderr, "remove_core: no such key as \"%s\"!\n",
 				sip->key);
 		return EOF;
@@ -142,7 +152,8 @@ remove_core(sip)
 
 static FILE *pcfp;
 
-int
+static int icprintNS __((struct spblk *));
+static int
 icprintNS(spl)
 	struct spblk *spl;
 {
@@ -153,7 +164,8 @@ icprintNS(spl)
 	return 0;
 }
 
-int
+static int icprintSN __((struct spblk *));
+static int
 icprintSN(spl)
 	struct spblk *spl;
 {
@@ -161,7 +173,8 @@ icprintSN(spl)
 	return 0;
 }
 
-int
+static int icprintSS __((struct spblk *));
+static int
 icprintSS(spl)
 	struct spblk *spl;
 {
@@ -174,12 +187,13 @@ icprintSS(spl)
 
 void
 print_core(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	struct sptree *db;
 
-	if ((db = open_core(sip)) == NULL)
+	db = open_core(sip);
+	if (db == NULL)
 		return;
 	pcfp = outfp;
 	if (db == spt_loginmap)
@@ -197,6 +211,7 @@ print_core(sip, outfp)
 
 static int pc_cnt;
 
+static int iccount __((struct spblk *));
 static int iccount(spl)
 	struct spblk *spl;
 {
@@ -206,13 +221,14 @@ static int iccount(spl)
 
 void
 count_core(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	struct sptree *db;
 	pc_cnt = 0;
 
-	if ((db = open_core(sip)) != NULL) {
+	db = open_core(sip);
+	if (db != NULL) {
 	  sp_scan(iccount, (struct spblk *)NULL, db);
 	}
 	fprintf(outfp,"%d\n",pc_cnt);
@@ -221,7 +237,7 @@ count_core(sip, outfp)
 
 void
 owner_core(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	fprintf(outfp, "%d\n", getuid());
