@@ -14,36 +14,34 @@
 #include <sys/file.h>
 #include "search.h"
 #include "io.h"
-
-extern int errno;
-
-extern int deferit;
-extern void v_set();
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
 
 /*
  * Search an NDBM format database for a key pair.
  */
 
-struct conscell *
+conscell *
 search_ndbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	DBM *db;
 	datum val, key;
-	struct conscell *tmp;
+	conscell *tmp;
 	struct spblk *spl;
 	int retry;
 	spkey_t symid;
-	u_char *us;
+	char *us;
 
 	if (sip->file == NULL)
 		return NULL;
 	retry = 0;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_files);
 	if (spl == NULL || (db = (DBM *)spl->data) == NULL) {
 reopen:
-		db = dbm_open(sip->file, O_RDONLY, 0);
+		db = dbm_open((char* /* Sigh.. */)sip->file, O_RDONLY, 0); 
 		if (db == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
@@ -52,30 +50,30 @@ reopen:
 			return NULL;
 		}
 		if (spl == NULL)
-			sp_install(symid, (u_char *)db, O_RDONLY, spt_files);
+			sp_install(symid, (void *)db, O_RDONLY, spt_files);
 		else
-			spl->data = (u_char *)db;
+			spl->data = (void *)db;
 	}
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
+	key.dptr  = (char*) sip->key; /* Sigh.. the cast.. */
+	key.dsize = strlen(sip->key) + 1;
 	val = dbm_fetch(db, key);
 	if (val.dptr == NULL) {
 #ifdef HAVE_DBM_ERROR
-		if (!retry && dbm_error(db)) {
-			dbm_close(db);
-			++retry;
-			goto reopen;
-		}
+	  if (!retry && dbm_error(db)) {
+	    dbm_close(db);
+	    ++retry;
+	    goto reopen;
+	  }
 #else
-		if (!retry && errno != 0) {
-			dbm_close(db);
-			++retry;
-			goto reopen;
-		}
+	  if (!retry && errno != 0) {
+	    dbm_close(db);
+	    ++retry;
+	    goto reopen;
+	  }
 #endif
-		return NULL;
+	  return NULL;
 	}
-	us = (u_char *)strnsave(val.dptr, val.dsize);
+	us = strnsave(val.dptr, val.dsize);
 	return newstring(us);
 }
 
@@ -85,7 +83,7 @@ reopen:
 
 void
 close_ndbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	DBM *db;
 	struct spblk *spl;
@@ -93,7 +91,7 @@ close_ndbm(sip)
 
 	if (sip->file == NULL)
 		return;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL)
 		sp_delete(spl, spt_modcheck);
@@ -105,11 +103,12 @@ close_ndbm(sip)
 }
 
 
-DBM *
+static DBM * open_ndbm __((search_info *, int, const char *));
+static DBM *
 open_ndbm(sip, flag, comment)
-	struct search_info *sip;
+	search_info *sip;
 	int flag;
-	char *comment;
+	const char *comment;
 {
 	DBM *db;
 	struct spblk *spl;
@@ -117,12 +116,13 @@ open_ndbm(sip, flag, comment)
 
 	if (sip->file == NULL)
 		return NULL;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_files);
 	if (spl != NULL && flag == O_RDWR && spl->mark != O_RDWR)
 		close_ndbm(sip);
 	if (spl == NULL || (db = (DBM *)spl->data) == NULL) {
-		db = dbm_open(sip->file, flag, 0);
+		db = dbm_open((char * /* Sigh.. the cast.. */)sip->file,
+			      flag, 0);
 		if (db == NULL) {
 			++deferit;
 			v_set(DEFER, DEFER_IO_ERROR);
@@ -131,9 +131,9 @@ open_ndbm(sip, flag, comment)
 			return NULL;
 		}
 		if (spl == NULL)
-			sp_install(symid, (u_char *)db, flag, spt_files);
+			sp_install(symid, (void *)db, flag, spt_files);
 		else
-			spl->data = (u_char *)db;
+			spl->data = (void *)db;
 	}
 	return db;
 }
@@ -144,18 +144,19 @@ open_ndbm(sip, flag, comment)
 
 int
 add_ndbm(sip, value)
-	struct search_info *sip;
-	char *value;
+	search_info *sip;
+	const char *value;
 {
 	DBM *db;
 	datum val, key;
 
-	if ((db = open_ndbm(sip, O_RDWR, "add_ndbm")) == NULL)
+	db = open_ndbm(sip, O_RDWR, "add_ndbm");
+	if (db == NULL)
 		return EOF;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
-	val.dptr = value;
-	val.dsize = strlen(value)+1;
+	key.dptr  = (char*) sip->key;	/* Sigh.. the cast.. */
+	key.dsize = strlen(sip->key) + 1;
+	val.dptr  = (char*) value;	/* Sigh.. the cast.. */
+	val.dsize = strlen(value) + 1;
 	if (dbm_store(db, key, val, DBM_REPLACE) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
@@ -172,15 +173,16 @@ add_ndbm(sip, value)
 
 int
 remove_ndbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	DBM *db;
 	datum key;
 
-	if ((db = open_ndbm(sip, O_RDWR, "remove_ndbm")) == NULL)
+	db = open_ndbm(sip, O_RDWR, "remove_ndbm");
+	if (db == NULL)
 		return EOF;
-	key.dptr = (char *)(sip->key);
-	key.dsize = strlen((char *)(sip->key)) + 1;
+	key.dptr  = (char*) sip->key;	/* Sigh.. the cast.. */
+	key.dsize = strlen(sip->key) + 1;
 	if (dbm_delete(db, key) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
@@ -197,13 +199,14 @@ remove_ndbm(sip)
 
 void
 print_ndbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	DBM *db;
 	datum key, val;
 
-	if ((db = open_ndbm(sip, O_RDONLY, "print_ndbm")) == NULL)
+	db = open_ndbm(sip, O_RDONLY, "print_ndbm");
+	if (db == NULL)
 		return;
 	for (key = dbm_firstkey(db); key.dptr != NULL; key = dbm_nextkey(db)) {
 		val = dbm_fetch(db, key);
@@ -230,14 +233,15 @@ print_ndbm(sip, outfp)
 
 void
 count_ndbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	DBM *db;
 	datum key, val;
 	int cnt = 0;
 
-	if ((db = open_ndbm(sip, O_RDONLY, "count_ndbm")) != NULL)
+	db = open_ndbm(sip, O_RDONLY, "count_ndbm");
+	if (db != NULL)
 	  for (key = dbm_firstkey(db); key.dptr != NULL; key = dbm_nextkey(db)) {
 	    val = dbm_fetch(db, key);
 	    if (val.dptr == NULL)
@@ -263,13 +267,14 @@ count_ndbm(sip, outfp)
 
 void
 owner_ndbm(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	DBM *db;
 	struct stat stbuf;
 
-	if ((db = open_ndbm(sip, O_RDONLY, "owner_ndbm")) == NULL)
+	db = open_ndbm(sip, O_RDONLY, "owner_ndbm");
+	if (db == NULL)
 		return;
 	/* There are more timing hazards, when the internal fd is not
 	   available for probing.. */
@@ -284,7 +289,7 @@ owner_ndbm(sip, outfp)
 
 int
 modp_ndbm(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	DBM *db;
 	struct stat stbuf;
@@ -297,17 +302,17 @@ modp_ndbm(sip)
 		return 0;
 	if (fstat(dbm_dirfno(db), &stbuf) < 0) {
 		fprintf(stderr, "modp_ndbm: cannot fstat(\"%s\")!\n",
-				sip->file);
+			sip->file);
 		return 0;
 	}
 	if (stbuf.st_nlink == 0)
 		return 1;	/* Unlinked underneath of us! */
 
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
-		rval = ((long)stbuf.st_mtime != (long)spl->data
-			|| (long)stbuf.st_nlink != (long)spl->mark);
+		rval = ((long)stbuf.st_mtime != (long)spl->data ||
+			(long)stbuf.st_nlink != (long)spl->mark);
 	} else
 		rval = 0;
 	sp_install(symid, (u_char *)((long)stbuf.st_mtime),
