@@ -6,20 +6,23 @@
 /* LINTLIBRARY */
 
 #include "mailer.h"
+#include <stdio.h>
+#include <sys/file.h>
 #include <ctype.h>
 #include <fcntl.h>
-#include <sys/file.h>
 #include "search.h"
 #include "io.h"
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
+
 #ifdef	HAVE_MMAP
 #include <sys/mman.h>
 #endif
 #include <errno.h>
 
-extern int cistrcmp();
 extern int deferit;
-extern void v_set();
-extern char *skip821address();
+extern char *skip821address __((char *));
 
 #ifdef HAVE_MMAP
 extern void seq_remap __((struct file_map *, long newsize));
@@ -30,13 +33,13 @@ extern char *mfgets __((char *, int, struct file_map *));
  * Linear search of a file for keyword-value pairs.
  */
 
-struct conscell *
+conscell *
 search_seq(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	FILE *fp;
-	register unsigned char *cp, *s;
-	struct conscell *tmp;
+	register char *cp, *s;
+	conscell *tmp;
 	struct spblk *spl;
 	int retry;
 	spkey_t symid;
@@ -49,7 +52,7 @@ search_seq(sip)
 	if (sip->file == NULL)
 		return NULL;
 	retry = 0;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_files);
 	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
 reopen:
@@ -92,10 +95,9 @@ reopen:
 		fm->membuf = NULL;
 #endif
 		if (spl == NULL)
-			spl = sp_install(symid, (u_char *)fm,
-					 O_RDONLY, spt_files);
+		  spl = sp_install(symid, (void *)fm, O_RDONLY, spt_files);
 		else
-			spl->data = (u_char *)fm;
+		  spl->data = (void *)fm;
 	}
 	fp = fm->fp;
 #ifdef	HAVE_MMAP
@@ -115,10 +117,10 @@ reopen:
 	}
 #endif
 	fm->pos = 0;
-	while ((s = (u_char *)mfgets(buf, sizeof buf, fm)) != NULL)
+	while ((s = mfgets(buf, sizeof buf, fm)) != NULL)
 #else
 	fseek(fp, (off_t)0, 0);
-	while ((s = (u_char *)fgets(buf, sizeof buf, fp)) != NULL)
+	while ((s = fgets(buf, sizeof buf, fp)) != NULL)
 #endif
 	{
 		buf[sizeof buf - 1] = '\0';
@@ -129,21 +131,22 @@ reopen:
 			*(cp+1) = '\0';
 		else
 			*cp = '\0';
-		if (cistrcmp((char *)(sip->key), buf) == 0) {
+		if (cistrcmp(sip->key, buf) == 0) {
 			for (++cp; *cp; ++cp)
-				if (isascii(*cp) && !isspace(*cp))
-					break;
+			  if (isascii((*cp)&0xFF) && !isspace((*cp)&0xFF))
+			    break;
 			for (s = cp; *s != '\0'; ++s)
-				if (!isascii(*s) || isspace(*s))
-					break;
-			return newstring((u_char *)strnsave((char *)cp, s - cp));
+			  if (!isascii((*s)&0xFF) || isspace((*s)&0xFF))
+			    break;
+			return newstring(strnsave(cp, s - cp));
 		}
 	}
 	if (!retry && ferror(fp)) {
 		fclose(fp);
 #ifdef	HAVE_MMAP
+		/* We are throwing things away... */
 		if (fm->membuf != NULL)
-		  munmap(fm->membuf,fm->size);
+		  munmap((void*)fm->membuf,fm->size);
 #endif
 		if (fm->offsets != NULL)
 		  free(fm->offsets);
@@ -162,7 +165,7 @@ reopen:
 
 void
 close_seq(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	struct file_map *fm;
 	struct spblk *spl;
@@ -170,7 +173,7 @@ close_seq(sip)
 
 	if (sip->file == NULL)
 		return;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL)
 		sp_delete(spl, spt_modcheck);
@@ -180,18 +183,20 @@ close_seq(sip)
 	spl->data = NULL;	/* Delete the file-entry */
 	fclose(fm->fp);
 #ifdef	HAVE_MMAP
+	/* We are throwing things away... */
 	if (fm->membuf != NULL)
-	  munmap(fm->membuf,fm->size);
+	  munmap((void*)fm->membuf,fm->size);
 #endif
 	if (fm->offsets != NULL)
 	  free(fm->offsets);
 	free(fm);
 }
 
-FILE *
+static FILE * open_seq __((search_info *, const char *));
+static FILE *
 open_seq(sip, mode)
-	struct search_info *sip;
-	char *mode;
+	search_info *sip;
+	const char *mode;
 {
 	struct file_map *fm;
 	struct spblk *spl;
@@ -203,7 +208,7 @@ open_seq(sip, mode)
 
 	if (sip->file == NULL)
 		return NULL;
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_files);
 	if (spl != NULL
 	    && (*mode == 'w' || *mode == 'a'
@@ -245,9 +250,9 @@ open_seq(sip, mode)
 		fm->membuf = NULL;
 #endif
 		if (spl == NULL)
-			sp_install(symid, (u_char *)fm, imode, spt_files);
+			sp_install(symid, (void *)fm, imode, spt_files);
 		else
-			spl->data = (u_char *)fm;
+			spl->data = (void *)fm;
 	}
 	return fm->fp;
 }
@@ -258,12 +263,13 @@ open_seq(sip, mode)
 
 int
 add_seq(sip, value)
-	struct search_info *sip;
-	char *value;
+	search_info *sip;
+	const char *value;
 {
 	FILE *fp;
 
-	if ((fp = open_seq(sip, "r+")) == NULL)
+	fp = open_seq(sip, "r+");
+	if (fp == NULL)
 		return EOF;
 	fseek(fp, (off_t)0, 2);
 	if (value == NULL || *value == '\0')
@@ -280,14 +286,15 @@ add_seq(sip, value)
 
 void
 print_seq(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	FILE *fp;
 	int n;
 	char buf[BUFSIZ];
 
-	if ((fp = open_seq(sip, "r")) == NULL)
+	fp = open_seq(sip, "r");
+	if (fp == NULL)
 		return;
 
 	fseek(fp, (off_t)0, 0);
@@ -303,7 +310,7 @@ print_seq(sip, outfp)
 
 void
 count_seq(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	FILE *fp;
@@ -311,10 +318,11 @@ count_seq(sip, outfp)
 	char buf[BUFSIZ];
 	int cnt = 0;
 
-	if ((fp = open_seq(sip, "r")) != NULL) {
+	fp = open_seq(sip, "r");
+	if (fp != NULL) {
 #ifdef HAVE_MMAP_not
 	  struct file_map *fm;
-	  spkey_t symid = symbol((u_char *)sip->file);
+	  spkey_t symid     = symbol(sip->file);
 	  struct spblk *spl = sp_lookup(symid, spt_files);
 	  int cnt;
 
@@ -348,13 +356,16 @@ count_seq(sip, outfp)
 
 void
 owner_seq(sip, outfp)
-	struct search_info *sip;
+	search_info *sip;
 	FILE *outfp;
 {
 	FILE *fp;
 	struct stat stbuf;
 
-	if (sip->file == NULL || (fp = open_seq(sip, "r")) == NULL)
+	if (sip->file == NULL)
+		return;
+	fp = open_seq(sip, "r");
+	if (fp == NULL)
 		return;
 
 	if (fstat(FILENO(fp), &stbuf) < 0) {
@@ -368,34 +379,40 @@ owner_seq(sip, outfp)
 
 int
 modp_seq(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	FILE *fp;
-	struct stat stbuf;
+	struct stat stbuf, stbuf2;
 	struct spblk *spl;
 	spkey_t symid;
 	int rval;
 
-	if (sip->file == NULL || (fp = open_seq(sip, "r")) == NULL)
+	if (sip->file == NULL)
+		return 0;
+	fp = open_seq(sip, "r");
+	if (fp == NULL)
 		return 0;
 
-	if (fstat(FILENO(fp), &stbuf) < 0) {
+	if (stat(sip->file, &stbuf) < 0 ||
+	    fstat(FILENO(fp), &stbuf2) < 0) {
 		fprintf(stderr, "modp_seq: cannot fstat(\"%s\")!\n",
 				sip->file);
 		return 0;
 	}
 
-	if (stbuf.st_nlink == 0) /* Changed underneath of us! */
+	if (stbuf.st_ino != stbuf2.st_ino)
+		return 1; /* The name, and the FD point to different files! */
+	if (stbuf2.st_nlink == 0) /* Changed underneath of us! */
 		return 1;
 
-	symid = symbol((u_char *)sip->file);
+	symid = symbol(sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
-		rval = ((long)stbuf.st_mtime != (long)spl->data
-			|| (long)stbuf.st_nlink != (long)spl->mark);
+		rval = ((long)stbuf.st_mtime != (long)spl->data ||
+			(long)stbuf.st_nlink != (long)spl->mark);
 	} else
 		rval = 0;
-	sp_install(symid, (u_char *)((long)stbuf.st_mtime),
+	sp_install(symid, (void *)((long)stbuf.st_mtime),
 		   stbuf.st_nlink, spt_modcheck);
 	return rval;
 }
@@ -428,13 +445,13 @@ char *buf;
 int bufsize;
 struct file_map *fm;
 {
-	char *eof = fm->membuf + fm->size;
-	char *s   = fm->membuf + fm->pos;
+	const char *eof = fm->membuf + (int) fm->size;
+	const char *s   = fm->membuf + (int) fm->pos;
 	char *b   = buf;
 	int i;
 
 	if (fm->size == 0) return NULL; /* no buffer mapped.. */
-	if (s >= eof) return NULL; /* EOF.. */
+	if (s >= eof)      return NULL; /* EOF.. */
 
 	for (i = 0; *s != '\n' && s < eof && i < bufsize; ++i)
 		*b++ = *s++;
@@ -450,26 +467,27 @@ struct file_map *fm;
 
 /* Indirect mappings -- like  aliases -- use this */
 
-struct conscell *
-readchunk(file, offset)
-	char *file;
-	long offset;
+conscell * readchunk __((const char *, long));
+conscell *
+readchunk(file, foffset)
+	const char *file;
+	long foffset;
 {
 	FILE *fp;
-	register unsigned char *cp;
-	unsigned char *as;
-	struct conscell *tmp, *l;
+	register char *cp;
+	char *as;
+	conscell *tmp, *l;
 	struct spblk *spl;
 	int retry, flag, len;
 	spkey_t symid;
 	char buf[BUFSIZ];
 	struct file_map *fm;
 
-	if (file == NULL || offset < 0)
+	if (file == NULL || foffset < 0)
 		return NULL;
 	
 	retry = 0;
-	symid = symbol((u_char *)file);
+	symid = symbol(file);
 	spl = sp_lookup(symid, spt_files);
 	if (spl == NULL || (fm = (struct file_map *)spl->data) == NULL) {
 reopen:
@@ -489,19 +507,19 @@ reopen:
 		fm->offsets = NULL;
 		fm->membuf = NULL;
 		if (spl == NULL)
-		  spl = sp_install(symid, (u_char *)fm,
+		  spl = sp_install(symid, (void *)fm,
 				   O_RDONLY, spt_files);
 		else
-		  spl->data = (u_char *)fm;
+		  spl->data = (void *)fm;
 	}
 	fp = fm->fp;
 
-	if (fseek(fp, (off_t)offset, 0) != 0) {
+	if (fseek(fp, (off_t)foffset, 0) != 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
 		fprintf(stderr,
 			"indirect postprocessor: bad seek (%ld) on '%s'!\n",
-			(long)offset, file);
+			(long)foffset, file);
 		return NULL;
 	}
 
@@ -521,16 +539,16 @@ reopen:
 		flag = 1;
 	}
 
-	if (fseek(fp, (off_t)offset, 0) != 0) {
+	if (fseek(fp, (off_t)foffset, 0) != 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
 		fprintf(stderr,
 			"indirect postprocessor: bad seek (%ld) on '%s'!\n",
-			(long)offset, file);
+			(long)foffset, file);
 		return NULL;
 	}
 
-	cp = as = (u_char *)tmalloc(len+1);
+	cp = as = tmalloc(len+1);
 	l = NULL;
 	flag = 0;
 	buf[sizeof buf - 1] = '\0';
