@@ -6,9 +6,7 @@
 /* LINTLIBRARY */
 
 #include "hostenv.h"
-#include "mailer.h"
-#include "search.h"
-#include "io.h"
+
 #ifdef	HAVE_YP
 #include <rpc/rpc.h>
 #include <netdb.h>
@@ -17,22 +15,27 @@
 #include <rpcsvc/ypclnt.h>
 #include <ctype.h>
 
+#include "mailer.h"
+#include "search.h"
+#include "io.h"
+#include "libz.h"
+#include "libc.h"
+#include "libsh.h"
 
-extern void die();
 
 /*
  * Query a Yellow Pages database
  */
 
 
-struct conscell *
+conscell *
 search_yp(sip)
-	struct search_info *sip;
+	search_info *sip;
 {
 	int yperr;
 	char *valptr;
 	int vallen;
-	struct conscell *tmp;
+	conscell *tmp;
 	char *ypdomainptr = NULL;
 	char ypdomainname[YPMAXDOMAIN];
 
