@@ -10,6 +10,10 @@
 # endif
 #endif
 
+#include "libz.h"
+#include "libsh.h"
+#include "libc.h"
+
 #ifdef  USE_SETUIDX	/* IBM AIXism */
 # include <sys/id.h>
 # ifdef USE_SETREUID
@@ -21,50 +25,20 @@
 
 /* various libraries, standard libc included.. */
 
-extern char	*gs_name;
-extern char	*monthname[];
-extern char	*myhostname;
-extern char	*optarg;
+extern const char * gs_name;
+extern const char * monthname[];
 extern char	*prio_list[];
-extern char	*progname;
-extern char	*rfc822tz __(());
-extern char	*sb_retrieve __(());
 #ifndef HAVE_STRERROR /* System has it, and propably has prototype too..
 			 IRIX 6.2 */
 extern char	*strerror __((const int errno));
 #endif
 extern int	D_hdr_rewrite;
 extern int	D_router;
-extern int	cistrcmp();
-extern int	cistrncmp();
-#ifndef strchr
-extern char	*strchr();
-extern char	*strrchr();
-#endif
-extern int	strmatch();
-extern int	apply();
-extern int	lapply();
-extern int	deferit;
-extern int	fprintToken();
-extern int	gensym;
-extern int	opterr;
-extern int	optind;
-extern int	printToken();
 extern int	wanttabs;
-extern struct headerinfo *find_header();
-extern struct headerinfo *find_header();
+extern void                init_header __((void));
+extern struct headerinfo * find_header __((struct sptree *, const char *));
 extern struct headerinfo envelope_hdrs[];
 extern struct headerinfo mandatory_hdrs[];
-extern struct token * scan822 __((u_char **cpp, long n, int c1, int c2,
-				  int allowcomments, struct token **tlistp));
-extern char *rfc822date __((time_t *tim));
-extern union misc parse822();
-extern void	dumpHeader();
-extern void	hdr_print();
-extern void	sb_external();
-extern void	sb_external();
-extern void	v_set();
-extern void	v_setl();
 #if	!defined(tolower) && !defined(sgi) /* XX: should use string.h or some such .. */
 extern int	tolower __((const int));
 #endif
@@ -95,13 +69,13 @@ extern struct sptree_init splaytrees[];
 extern long	dateParse __((struct tm *localtmptr, struct token *t));
 
 /* File: db.c */
-extern int	run_relation __((int argc, char **argv));
-extern struct sptree *icdbspltree __((char *name));
-extern int	run_db __((int argc, char **argv));
-extern struct conscell	*db __((u_char *dbname, u_char *key));
-extern char    *dbfile __((u_char *dbname));
-extern void	dbfree __((void));
-extern char    *dbtype __((u_char *dbname));
+extern int	run_relation __((int argc, const char *argv[]));
+extern struct sptree *icdbspltree __((const char *name));
+extern int	run_db __((int argc, const char *argv[]));
+extern conscell	  *db    __((const char *dbname, const char *key));
+extern const char *dbfile __((const char *dbname));
+extern void	   dbfree __((void));
+extern const char *dbtype __((const char *dbname));
 
 /* File: functions.c */
 extern int	funclevel;
@@ -121,40 +95,40 @@ extern int	D_resolv;
 extern int	D_alloc;
 extern int	D_regnarrate;
 
-extern int run_trace __((int argc, char *argv[]));
+extern int run_trace __((int argc, const char *argv[]));
 extern char *erraddrlog;
 extern RETSIGTYPE sig_hup __((int));
-extern int	run_daemon __((int argc, char *argv[]));
+extern int	run_daemon __((int argc, const char *argv[]));
 extern int	gensym;
 extern void	optsave __((int type, struct envelope *e));
 
 /* File: rfc822.c */
 extern char	*errors_to;
 
-extern int	run_rfc822 __((int argc, char **argv));
+extern int	run_rfc822 __((int argc, const char *argv[]));
 extern int	makeLetter __((struct envelope *e, int octothrp));
 extern void	dumpInfo __((struct envelope *e));
 extern void	dumpHeaders __((struct header *h));
 extern void	dumpHeader __((struct header *h));
 extern int	isSenderAddr;
 extern int	isRecpntAddr;
-extern void	squirrel __((struct envelope *e, char *keyword, char *text));
+extern void	squirrel __((struct envelope *e, const char *keyword, const char *text));
 extern struct header	*erraddress __((struct envelope *e));
-extern void	defer __((struct envelope *e, char *why));
-extern struct header	*mkSender __((struct envelope *e, u_char *name, int flag));
+extern void	defer __((struct envelope *e, const char *why));
+extern struct header	*mkSender __((struct envelope *e, const char *name, int flag));
 extern struct header	*mkTrace __((struct envelope *e));
-extern int	nullhost __((u_char *s));
-extern struct conscell	*pickaddress __((struct conscell *l));
+extern int	nullhost __((const char *s));
+extern conscell	*pickaddress __((conscell *l));
 extern int	thesender __((struct envelope *e, struct address *a));
-extern struct conscell	*makequad __((void));
+extern conscell	*makequad __((void));
 extern int	iserrmessage __((void));
-extern int	sequencer __((struct envelope *e, char *file));
+extern int	sequencer __((struct envelope *e, const char *file));
 
 /* File: rfc822hdrs.c */
 extern union misc	 hdr_scanparse __((struct envelope *e, struct header *h, int commentflag, int no_line_crossing));
-extern struct header	*makeHeader __((struct sptree *db, u_char *s, int len));
+extern struct header	*makeHeader __((struct sptree *sb, const char *s, int len));
 extern struct headerinfo *senderDesc __((void));
-extern void	set_pname __((struct envelope *e, struct header *h, char *s));
+extern void   set_pname __((struct envelope *e, struct header *h, const char *s));
 extern struct header	*copySender __((struct envelope *e));
 extern struct header	*copyRecipient __((struct header *h));
 extern struct header	*mkMessageId __((struct envelope *e, time_t unixtime));
@@ -163,13 +137,14 @@ extern struct header	*mkDate __((int isresent, time_t unixtime));
 extern void	hdr_print __((struct header *h, FILE *fp));
 extern int	hdr_nilp __((struct header *h));
 extern void	pureAddress __((FILE *fp, struct addr *pp));
+extern int	pureAddressBuf __((char *buf, int len, struct addr *pp));
 extern int	printAddress __((FILE *fp, struct addr *pp, int onlylength));
 extern int	printLAddress __((FILE *fp, struct addr *pp, int col));
 extern char    *saveAddress __((struct addr *pp));
 extern void	errprint __((FILE *fp, struct addr *pp, int hdrlen));
 extern HeaderStamp hdr_type __((struct header *h));
 extern struct header	*hdr_warning __((struct header *h));
-extern void	hdr_errprint __((struct envelope *e, struct header *h, FILE *fp, char *msg));
+extern void	hdr_errprint __((struct envelope *e, struct header *h, FILE *fp, const char *msg));
 
 /* File: rfc822walk.c */
 extern struct address	*revaddress __((struct address *ap));
@@ -177,9 +152,10 @@ extern union misc	 parse822 __((HeaderSemantics entry, struct token **tlistp, st
 
 
 /* File: router.c */
-extern char *progname;
-extern char *mailshare;
-extern char *myhostname;
+extern int deferuid;
+extern const char *progname;
+extern const char *mailshare;
+extern const char *myhostname;
 extern time_t now;
 extern memtypes stickymem;
 extern int   mustexit;
@@ -188,22 +164,27 @@ extern int   deferit;
 extern int   router_id;
 extern int   savefile;
 
-extern int	main __((int argc, char *argv[]));
-extern int	login_to_uid __((u_char *name));
-extern u_char	*uidpwnam __((int uid));
+/* File: rtsyslog.c */
+extern void rtsyslog __(( time_t msgmtime, const char *msgfile, const char *from, const char *smtprelay, int size, int nrcpts, const char *msgid ));
+
+extern int	main __((int argc, const char *argv[]));
+extern int	login_to_uid __((const char *name));
+extern const char *uidpwnam __((int uid));
 extern int	isgoodguy __((int uid));
 extern void	logmessage __((struct envelope *e));
 
 /* File: shliase.c */
-extern int	s_apply __((int argc, char *argv[]));
-extern int	n_apply __((char **cpp, int argc, char *argv[]));
-extern struct header *hdr_rewrite __((u_char *name, struct header *h));
+extern int	l_apply __((const char *fname, conscell *l));
+extern int	s_apply __((int argc, const char *argv[]));
+extern int	n_apply __((char **cpp, int argc, const char *argv[]));
+extern struct header *hdr_rewrite __((const char *name, struct header *h));
 extern void	setenvinfo __((struct envelope *e));
-extern char     *build_gensym __((int uid, char *type, char *DSNstr, char *errors_to, char *sender));
-extern struct conscell *router __((struct address *a, int uid, char *type));
-extern struct conscell *crossbar __((struct conscell *from, struct conscell *to));
+extern char     *build_gensym __((int uid, const char *type, const char *DSNstr, const char *errorsto, const char *sender));
+extern conscell *router __((struct address *a, int uid, const char *type));
+extern conscell *crossbar __((conscell *from, conscell *to));
+extern char *newattribute_2 __((const char *, const char *, const char *));
 
-/* File: version.c */
-extern char *Version;
-extern char *CC_user;
-extern char *CC_pwd;
+extern int nobody;
+
+extern void prversion __((const char *));
+extern void setfreefd __((void));
