@@ -22,6 +22,8 @@
 #include "zsyslog.h"
 
 #include "prototypes.h"
+#include "libz.h"
+#include "libsh.h"
 
 #ifndef _IOFBF
 #define _IOFBF  0
@@ -46,36 +48,24 @@
 
 #endif	/* HAVE_LOCKF */
 
-extern int hdr_status();
-extern int efstat();
-extern int elink();
-extern int eunlink();
-extern int erename();
-extern int getline();
-extern int nobody;
-extern int deferuid;
-extern int s_equal();
-extern long offset();
-extern void initline();
+static void	reject __((struct envelope *e, const char *msgfile));
 
-static void	reject __((struct envelope *e, char *msgfile));
-
-static char *	prctladdr __((struct conscell *info, FILE *fp, int cfflag, char *comment));
-static void	prdsndata __((struct conscell *info, FILE *fp, int cfflag, char *comment));
-static struct conscell	*find_errto __((struct conscell *list));
+static const char * prctladdr __((conscell *info, FILE *fp, int cfflag, const char *comment));
+static void	prdsndata __((conscell *info, FILE *fp, int cfflag, const char *comment));
+static conscell	*find_errto __((conscell *list));
 
 #define dprintf	if (D_sequencer) printf
 
 #define	QCHANNEL(x)	(x)->string
-#define	QHOST(x)	(cdr(x))->string
-#define	QUSER(x)	(cddr(x))->string
+#define	QHOST(x)	(cdr(x))->cstring
+#define	QUSER(x)	(cddr(x))->cstring
 #define	QATTRIBUTES(x)	(cdr(cddr(x)))->string
 
 
-struct conscell *
+conscell *
 makequad()
 {
-	struct conscell *l, *tmp;
+	conscell *l, *tmp;
 
 	l = conststring(NULL);
 	cdr(l) = conststring(NULL);
@@ -104,25 +94,30 @@ iserrmessage()
 int
 run_rfc822(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct envelope *e;
-	char *file, buf[8196];
-	int status, c, errflg;
+	const char *file;
+	char buf[8196];
+	int status, errflg;
 	memtypes oval;
 
 	errflg = 0;
-/*
-	while ((c = getopt(argc, argv, "")) != EOF) {
-		switch (c) {
-		default:
-			++errflg;
-			break;
-		}
+#if 0
+	{
+	  int c;
+	  while ((c = getopt(argc, argv, "")) != EOF) {
+	    switch (c) {
+	    default:
+	      ++errflg;
+	      break;
+	    }
+	  }
 	}
-	if (errflg || optind != (argc - 1)) { 
-*/
-	if (argc != 2) {
+	if (errflg || optind != (argc - 1))
+#endif
+	if (argc != 2)
+	  {
 		fprintf(stderr, "Usage: %s messagefile\n", argv[0]);
 		return PERR_USAGE;
 	}
@@ -224,7 +219,7 @@ makeLetter(e, octothorp)
 	int octothorp;		/* does # at start of word start a comment? */
 {
 	register int	i;
-	register u_char	*cp;
+	register char	*cp;
 	struct header	*h;
 	int		n, inheader;
 	struct header	*ph, *nh;
@@ -242,7 +237,7 @@ makeLetter(e, octothorp)
 	}
 	e->e_localtime = *(localtime(&(e->e_statbuf.st_mtime)));
 #ifdef	HAVE_ST_BLKSIZE
-	initline(e->e_statbuf.st_blksize);
+	initline((int)e->e_statbuf.st_blksize);
 #else	/* !HAVE_ST_BLKSIZE */
 	initline(4096);
 #endif	/* !HAVE_ST_BLKSIZE */
@@ -265,7 +260,7 @@ makeLetter(e, octothorp)
 				continue;
 			/* record start of headers for posterity */
 			if (!inheader)
-				e->e_hdrOffset = offset(e->e_fp)-n;
+			  e->e_hdrOffset = lineoffset(e->e_fp)-n;
 			/* cons up a new one at top of header list */
 			h = makeHeader(spt_headers, linebuf, i);
 			h->h_next = e->e_headers;
@@ -277,8 +272,7 @@ makeLetter(e, octothorp)
 			if (inheader && linebuf[0] == ':') {
 				optsave(FYI_ILLHEADER, e);
 				/* cons up a new one at top of header list */
-				h = makeHeader(spt_headers,
-					       (u_char*)"X-Null-Field", 12);
+				h = makeHeader(spt_headers,"X-Null-Field",12);
 				h->h_next = e->e_headers;
 				e->e_headers = h;
 				h->h_lines = makeToken(linebuf+i+1, n-i-2);
@@ -289,7 +283,8 @@ makeLetter(e, octothorp)
 				if (e->e_headers == NULL) {
 				  /* Wow, continuation without previous header! */
 				  /* It must be body.. */
-				  repos_getline(e->e_fp, offset(e->e_fp) - n);
+				  repos_getline(e->e_fp,
+						lineoffset(e->e_fp) - n);
 				  break;
 				}
 				t = e->e_headers->h_lines;
@@ -303,7 +298,6 @@ makeLetter(e, octothorp)
 		} else if (!inheader		/* envelope information */
 			   && (*(cp=linebuf-i) == ' ' || *cp == '\t'
 				|| *cp == '\n')) {
-			struct headerinfo *hd;
 			HeaderSemantics osem;
 
 			if (octothorp && linebuf[0] == '#')
@@ -338,7 +332,7 @@ makeLetter(e, octothorp)
 			case eEnvEnd:		/* more elegance breaks */
 				inheader = 1;
 				/* record start of headers for posterity */
-				e->e_hdrOffset = offset(e->e_fp);
+				e->e_hdrOffset = lineoffset(e->e_fp);
 				e->e_eHeaders = h->h_next;
 				break;
 			default:		/* a real envelope header */
@@ -351,7 +345,7 @@ makeLetter(e, octothorp)
 			   We play as if it was the end of the headers, and
 			   start of the body.  Keep the first faulty input
 			   around by reseeking into its start.		    */
-			repos_getline(e->e_fp, offset(e->e_fp) - n);
+			repos_getline(e->e_fp, lineoffset(e->e_fp) - n);
 			break;
 		}
 	}
@@ -378,7 +372,7 @@ makeLetter(e, octothorp)
 	}
 	e->e_headers = ph;
 	/* record the start of the message body for posterity */
-	e->e_msgOffset = offset(e->e_fp);
+	e->e_msgOffset = lineoffset(e->e_fp);
 
 	return PERR_OK;
 }
@@ -418,7 +412,7 @@ dumpHeader(h)
 		return;
 	switch (h->h_descriptor->semantics) {
 	case DateTime:
-		printf("\tUNIX time %d, local time %s",
+		printf("\tUNIX time %ld, local time %s",
 			h->h_contents.d, ctime(&(h->h_contents.d)));
 		break;
 	case nilHeaderSemantics:
@@ -450,7 +444,7 @@ dumpHeader(h)
 			printf("\tWith: %s\n", formatToken(t));
 		if ((t = h->h_contents.r->r_convert) != NULL)
 			printf("\tConvert: %s\n", formatToken(t));
-		printf("\tUNIX time %d, local time %s",
+		printf("\tUNIX time %ld, local time %s",
 			h->h_contents.r->r_time,
 			ctime(&(h->h_contents.r->r_time)));
 		break;
@@ -516,7 +510,7 @@ char *errors_to = NULL;
 void
 squirrel(e, keyw, text)
 	struct envelope *e;
-	char *keyw, *text;
+	const char *keyw, *text;
 {
 	char *path;
 
@@ -525,7 +519,8 @@ squirrel(e, keyw, text)
 #else
 	path = alloca(5+sizeof(POSTMANDIR)+10+strlen(keyw));
 #endif
-	sprintf(path, "../%s/%d.%s", POSTMANDIR, e->e_statbuf.st_ino, keyw);
+	sprintf(path, "../%s/%d.%s",
+		POSTMANDIR, (int)e->e_statbuf.st_ino, keyw);
 
 	/* XX: We make a copy by link()ing file to two dirs, some
 	       systems (Andrew-FS in mind) can't do it! Copying is
@@ -534,16 +529,16 @@ squirrel(e, keyw, text)
 	/* If linking to  /postman/  fails, link it to router dir with
 	   two dots at the begin of the name + keyword at the end */
 	if (elink(e->e_file, path) < 0) {
-		sprintf(path, "..%d.%s", e->e_statbuf.st_ino, keyw);
+		sprintf(path, "..%d.%s", (int)e->e_statbuf.st_ino, keyw);
 		elink(e->e_file, path);
 	}
 
 	/* The successfull squirrel MUST NOT delete the original message! */
 
 	fprintf(stderr, "squirrel: %d.%s saved for inspection: %s\n",
-			e->e_statbuf.st_ino, keyw, text);
+		(int) e->e_statbuf.st_ino, keyw, text);
 	zsyslog((LOG_ERR, "%d.%s saved for inspection: %s\n",
-		 e->e_statbuf.st_ino, keyw, text));
+		 (int) e->e_statbuf.st_ino, keyw, text));
 #ifndef	USE_ALLOCA
 	free(path);
 #endif
@@ -556,7 +551,8 @@ erraddress(e)
 	register struct header *h;
 	struct header *best;
 	int minval;
-	char *cp, **cpp;
+	char **cpp;
+	const char *cp;
 
 	best = NULL;
 	minval = 10000;
@@ -590,7 +586,7 @@ erraddress(e)
 			h = mkSender(e, uidpwnam(e->e_statbuf.st_uid), 1);
 #if	0
 		else
-			h = mkSender(e, (u_char *)POSTMASTER, 1);
+			h = mkSender(e, POSTMASTER, 1);
 #endif
 	}
 	return h;
@@ -599,6 +595,7 @@ erraddress(e)
 
 /* Pick recipient address from the input line.
    EXTREMELY Simple minded parsing.. */
+static void pick_env_addr __((char *buf, FILE *mfp));
 static void pick_env_addr(buf,mfp)
 char *buf;
 FILE *mfp;
@@ -630,17 +627,19 @@ FILE *mfp;
 static void
 reject(e, msgfile)
 	struct envelope *e;
-	char *msgfile;
+	const char *msgfile;
 {
 	register struct header *h;
-	char *path, lastCh, nextToLastCh, *cp;
+	char *path, lastCh, nextToLastCh;
+	const char *c_cp;
 	int n;
 	FILE *mfp, *fp;
 	char buf[BUFSIZ], vbuf[BUFSIZ];
 
 	fprintf(stderr, "Rejecting because of %s\n", msgfile);
 	
-	if ((h = erraddress(e)) == NULL) {
+	h = erraddress(e);
+	if (h == NULL) {
 		squirrel(e, "noerraddr", "No one to return an error to!");
 		return;
 	}
@@ -654,7 +653,8 @@ reject(e, msgfile)
 	path = alloca(3+strlen(mailshare)+strlen(FORMSDIR)+strlen(msgfile));
 #endif
 	sprintf(path, "%s/%s/%s", mailshare, FORMSDIR, msgfile);
-	if ((fp = fopen(path, "r")) == NULL) {
+	fp = fopen(path, "r");
+	if (fp == NULL) {
 		perror(path);
 		squirrel(e, "norejform", "Couldn't open reject form file");
 #ifndef	USE_ALLOCA
@@ -665,13 +665,13 @@ reject(e, msgfile)
 #ifndef	USE_ALLOCA
 	free(path);
 #endif
-	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
-		squirrel(e, "mailcreatefail", "Couldn't open reject message file");
-		return;
+	mfp = mail_open(MSG_RFC822);
+	if (mfp == NULL) {
+	  squirrel(e, "mailcreatefail", "Couldn't open reject message file");
+	  return;
 	}
 	setvbuf(fp, vbuf, _IOFBF, sizeof vbuf);
 	while (fgets(buf,sizeof(buf),fp) != NULL) {
-	  int rc;
 	  if (strncmp("ADR",buf,3)==0) {
 	    pick_env_addr(buf+4,mfp);
 	  } else if (strncmp("HDR",buf,3)==0 ||
@@ -684,10 +684,10 @@ reject(e, msgfile)
 	fseek(fp,(off_t)0,0);
 
 	/* who is it from? */
-	cp = h->h_pname;
+	c_cp = h->h_pname;
 	h->h_pname = "To";
 	hdr_print(h, mfp);
-	h->h_pname = cp;
+	h->h_pname = c_cp;
 	lastCh = nextToLastCh = '\0';
 	while (fgets(buf,sizeof(buf),fp) != NULL) {
 	  int rc;
@@ -747,7 +747,7 @@ reject(e, msgfile)
 void
 defer(e, why)
 	struct envelope *e;
-	char *why;
+	const char *why;
 {
 	int i;
 	char *path, *s;
@@ -765,7 +765,7 @@ defer(e, why)
 #else
 	  path = emalloc(5+sizeof(DEFERREDDIR)+6+strlen(why));
 #endif
-	  sprintf(path,"../%s/%d.%s", DEFERREDDIR, stbuf.st_ino, why);
+	  sprintf(path,"../%s/%d.%s", DEFERREDDIR, (int)stbuf.st_ino, why);
 	} else {
 #ifdef	USE_ALLOCA
 	  path = alloca(5+sizeof(DEFERREDDIR)+strlen(e->e_file)+2+strlen(why));
@@ -803,12 +803,12 @@ defer(e, why)
 struct header *
 mkSender(e, name, flag)
 	struct envelope *e;
-	u_char *name;
+	const char *name;
 	int flag;
 {
 	register struct header	*sh, *h;
 	struct addr *pp, *qp, **ppp;
-	struct conscell *l;
+	conscell *l;
 	int didbracket;
 	struct spblk *spl;
 	
@@ -842,7 +842,7 @@ mkSender(e, name, flag)
 		*ppp = (struct addr *)tmalloc(sizeof (struct addr));
 		pp = *ppp;
 		ppp = &pp->p_next;
-		pp->p_tokens = makeToken((u_char *)spl->data,
+		pp->p_tokens = makeToken((char *)spl->data,
 					 strlen((char *)spl->data));
 		pp->p_tokens->t_type = Word;
 		pp->p_type = aPhrase;
@@ -851,7 +851,7 @@ mkSender(e, name, flag)
 		*ppp = (struct addr *)tmalloc(sizeof (struct addr));
 		pp = *ppp;
 		ppp = &pp->p_next;
-		pp->p_tokens = makeToken((u_char *)"<", 1);
+		pp->p_tokens = makeToken("<", 1);
 		didbracket = 1;
 		pp->p_tokens->t_type = Special;
 		pp->p_type = aSpecial;
@@ -893,7 +893,7 @@ mkSender(e, name, flag)
 		*ppp = (struct addr *)tmalloc(sizeof (struct addr));
 		pp = *ppp;
 		ppp = &pp->p_next;
-		pp->p_tokens = makeToken(name, strlen((char *)name));
+		pp->p_tokens = makeToken(name, strlen(name));
 		pp->p_tokens->t_type = Atom;
 		pp->p_type = anAddress;
 		l = NULL;
@@ -903,7 +903,7 @@ mkSender(e, name, flag)
 		*ppp = (struct addr *)tmalloc(sizeof (struct addr));
 		pp = *ppp;
 		ppp = &pp->p_next;
-		pp->p_tokens = makeToken((u_char *)">", 1);
+		pp->p_tokens = makeToken(">", 1);
 		pp->p_tokens->t_type = Special;
 		pp->p_type = aSpecial;
 	}
@@ -957,8 +957,7 @@ mkTrace(e)
 	if (myhostname != NULL) {
 		na = (struct addr *)tmalloc(sizeof (struct addr));
 		/* prepend now -- reverse later */
-		na->p_tokens = makeToken((u_char *)myhostname,
-					 strlen(myhostname));
+		na->p_tokens = makeToken(myhostname, strlen(myhostname));
 		na->p_next = NULL;
 		na->p_type = anAddress;
 		ap = (struct address *)tmalloc(sizeof (struct address));
@@ -985,7 +984,7 @@ mkTrace(e)
 		th->h_contents.r->r_with = NULL;
 	/* id */
 	na = (struct addr *)tmalloc(sizeof (struct addr));
-	na->p_tokens = makeToken((u_char *)e->e_file, strlen(e->e_file));
+	na->p_tokens = makeToken(e->e_file, strlen(e->e_file));
 	na->p_next = NULL;
 	na->p_type = anAddress;	/* really a message id */
 	ap = (struct address *)tmalloc(sizeof (struct address));
@@ -1006,15 +1005,15 @@ mkTrace(e)
 struct rwmatrix {
 	struct rwmatrix	*next;
 	struct rwmatrix	*down;
-	struct conscell *info;	/* rewrite, sender, or recipient list info */
-	struct conscell *errto;
+	conscell *info;	/* rewrite, sender, or recipient list info */
+	conscell *errto;
 	union {
 		int	number;	/* XOR address id number for recipients */
 		struct header *h;	/* for rewritings */
 	} urw;
 };
 
-
+static struct rwmatrix * rwalloc __((struct rwmatrix **));
 static struct rwmatrix *
 rwalloc(rwpp)
 	struct rwmatrix **rwpp;
@@ -1035,18 +1034,18 @@ rwalloc(rwpp)
 
 int
 nullhost(s)
-	u_char *s;
+	const char *s;
 {
-	return (s == NULL || *s == '\0' || strcmp((char *)s, "-") == 0);
+	return (s == NULL || *s == '\0' || strcmp(s, "-") == 0);
 	/* actually we should also check for localhostness, but lets not
 	   get carried away... */
 }
 
-struct conscell *
+conscell *
 pickaddress(l)
-	struct conscell *l;
+	conscell *l;
 {
-	struct conscell *la, *lx, *p;
+	conscell *la, *lx, *p;
 
 	/*
 	 * Given an AND-XOR tree returned from the router, pick one address
@@ -1074,7 +1073,7 @@ thesender(e, a)
 	struct envelope *e;
 	struct address *a;
 {
-	struct conscell *l;
+	conscell *l;
 
 	l = router(a, e->e_statbuf.st_uid, "sender");
 	if (l == NULL)
@@ -1108,25 +1107,25 @@ int ReceivedCount = 0;
 int
 sequencer(e, file)
 	struct envelope *e;
-	char *file;
+	const char *file;
 {
-	struct header *h, *ph, *nh, *oh, *msgidh, **hp;
+	struct header  *h, *ph, *nh, *oh, *msgidh, **hp;
 	struct address *a, *ap;
-	struct addr *p;
-	char *ofpname, *path, *qpath;
-	struct conscell *l, *routed_addresses, *sender, *to;
+	struct addr    *p = NULL;
+	char	       *ofpname, *path, *qpath;
+	conscell       *l, *routed_addresses, *sender, *to;
 	struct rwmatrix *rwhead, *nsp, *rwp = NULL, *rcp = NULL;
-	struct token *t = NULL;
-	int idnumber, nxor, i;
-	int def_uid, header_error, perr;
-	FILE *ofp, *vfp;
-	char vbuf[2048], verbosefile[1024];
-	char *envid;
-	char *notaryret;
-	int nrcpts = 0;
-	char *fromaddr = "?from?";
-	char *msgidstr = "?msgid?";
-	char *smtprelay = NULL;
+	struct token   *t = NULL;
+	int   idnumber, nxor, i;
+	int   def_uid, header_error, perr;
+	FILE	       *ofp, *vfp;
+	char  vbuf[2048], verbosefile[1024];
+	const char     *envid;
+	const char     *notaryret;
+	int   nrcpts = 0;
+	const char     *fromaddr = "?from?";
+	const char     *msgidstr = "?msgid?";
+	const char     *smtprelay = NULL;
 
 	deferuid = 0;
 
@@ -1149,37 +1148,40 @@ sequencer(e, file)
 	  if (h != NULL)
 	    smtprelay = h->h_lines->t_pname;
 	  else {
-	    char *s = uidpwnam(e->e_statbuf.st_uid);
-	    smtprelay = strnsave("",strlen(s)+20);
-	    sprintf(smtprelay, "%s@localhost", s);
+	    const char *s  = uidpwnam(e->e_statbuf.st_uid);
+	    char *sr = strnsave("",strlen(s)+20);
+	    sprintf(sr, "%s@localhost", s);
+	    smtprelay = sr;
 	  }
 	} else {
 	  /* Ok, we don't trust it!  In fact we might overrule the data
 	     that the message writer coded in... */
-	  char *s, *p, *t;
+	  const char *s, *ps;
+	  char *ts;
 	  int totlen;
 
 	  dprintf("Message has 'rcvdfrom' envelope header, but we don't trust it!\n");
 	  s = uidpwnam(e->e_statbuf.st_uid);
-	  p = (h ? h->h_lines->t_pname : NULL);
-	  totlen = 10 + strlen(s) + 20 + (p ? strlen(p) + 10 : 0);
-	  /* t = strnsave("", totlen); */
-	  t = tmalloc(totlen); /* Alloc the space */
+	  ps = "";
+	  if (h) ps = h->h_lines->t_pname;
+	  totlen = 10 + strlen(s) + 20 + (h ? strlen(ps) + 10 : 0);
+	  /* ts = strnsave("", totlen); */
+	  ts = tmalloc(totlen); /* Alloc the space */
 	  if (h) {
-	    sprintf(t, "rcvdfrom %s@localhost (fake: %s)", s, h->h_lines->t_pname);
+	    sprintf(ts, "rcvdfrom %s@localhost (fake: %s)", s, h->h_lines->t_pname);
 	  } else
-	    sprintf(t, "rcvdfrom %s@localhost", s);
+	    sprintf(ts, "rcvdfrom %s@localhost", s);
 
-	  h = makeHeader(spt_eheaders, t, 8);
+	  h = makeHeader(spt_eheaders, ts, 8);
 	  h->h_next = e->e_eHeaders;
 	  e->e_eHeaders = h;
 
-	  h->h_lines = makeToken(t+9, strlen(t+9));
+	  h->h_lines    = makeToken(ts+9, strlen(ts+9));
 	  h->h_lines->t_type = Line;
 	  h->h_contents = hdr_scanparse(e, h, 0, 0);
-	  h->h_stamp = hdr_type(h);
+	  h->h_stamp    = hdr_type(h);
 
-	  smtprelay = t+9;
+	  smtprelay     = ts+9;
 	}
 
 	if (deferuid)
@@ -1190,7 +1192,8 @@ sequencer(e, file)
 	perr = 0;
 	if (myhostname) {	/* appease honeyman et al groupies */
 		dprintf("Stamp it with a trace header\n");
-		if ((h = mkTrace(e)) != NULL) {
+		h = mkTrace(e);
+		if (h != NULL) {
 			h->h_next = e->e_headers;
 			e->e_headers = h;
 		}
@@ -1239,7 +1242,7 @@ sequencer(e, file)
 		if (perr == 0)
 			perr = PERR_ENVELOPE;
 	}
-	e->e_from_trusted = makequad();
+	e->e_from_trusted  = makequad();
 	e->e_from_resolved = makequad();
 
 	if (myhostname) {	/* we care about message id's */
@@ -1272,13 +1275,13 @@ sequencer(e, file)
 	envid = NULL;
 	if (h != NULL && h->h_contents.a != NULL
 	    && h->h_contents.a->a_pname != NULL) {
-		envid = (char*)h->h_contents.a->a_pname;
+		envid = h->h_contents.a->a_pname;
 	}
 	FindEnvelope(eNotaryRet);
 	notaryret = NULL;
 	if (h != NULL && h->h_contents.a != NULL
 	    && h->h_contents.a->a_pname != NULL) {
-		notaryret = (char*)h->h_contents.a->a_pname;
+		notaryret = h->h_contents.a->a_pname;
 	}
 	FindEnvelope(eFrom);
 	if (h == NULL && e->e_trusted) {
@@ -1291,8 +1294,7 @@ sequencer(e, file)
 			    && p->p_tokens != NULL) {
 				t = p->p_tokens;
 				QCHANNEL(e->e_from_trusted) =
-					(u_char *)strnsave((char *)t->t_pname,
-							   TOKENLEN(t));
+					strnsave(t->t_pname, TOKENLEN(t));
 			} else {
 				/*
 				 * No origination channel, or channel
@@ -1321,7 +1323,7 @@ sequencer(e, file)
 			h = mkSender(e, uidpwnam(e->e_statbuf.st_uid), 0);
 		}
 		if (h == NULL)
-			h = mkSender(e, (u_char *)POSTMASTER, 1);
+			h = mkSender(e, POSTMASTER, 1);
 		/* assert h != NULL */
 		h->h_next = e->e_eHeaders;
 		e->e_eHeaders = h;
@@ -1379,8 +1381,7 @@ sequencer(e, file)
 			    && p->p_tokens != NULL) {
 				t = p->p_tokens;
 				QCHANNEL(e->e_from_trusted) =
-					(u_char *)strnsave((char *)t->t_pname,
-							   TOKENLEN(t));
+					strnsave(t->t_pname, TOKENLEN(t));
 			}
 			if (QCHANNEL(e->e_from_trusted) == NULL) {
 				/*
@@ -1495,7 +1496,7 @@ sequencer(e, file)
 	if (QUSER(e->e_from_trusted) == NULL)
 		QUSER(e->e_from_trusted) = QUSER(e->e_from_resolved);
 	if (QATTRIBUTES(e->e_from_trusted) == NULL)
-		QATTRIBUTES(e->e_from_trusted) =QATTRIBUTES(e->e_from_resolved);
+		QATTRIBUTES(e->e_from_trusted)=QATTRIBUTES(e->e_from_resolved);
 
 	if (deferuid)
 	  return PERR_DEFERRED;
@@ -1671,7 +1672,7 @@ sequencer(e, file)
 			}
 			DSN = (struct notary *)tmalloc(sizeof(struct notary));
 			DSN->envid = envid;
-			DSN->dsn = (char*)h->h_contents.a->a_pname;
+			DSN->dsn = h->h_contents.a->a_pname;
 			continue;
 		}
 		if (DSN != NULL) {
@@ -1684,7 +1685,7 @@ sequencer(e, file)
 				continue;
 
 			if (routed_addresses == NULL) {
-				struct conscell *tmp;
+				conscell *tmp;
 				routed_addresses = ncons(car(l));
 			} else {
 				cdr(s_last(car(l))) = car(routed_addresses);
@@ -1699,7 +1700,7 @@ sequencer(e, file)
 	dprintf("Crossbar applied to all (sender,routed-address) pairs\n");
 
 	{
-	  struct conscell *tmp;
+	  conscell *tmp;
 	  sender = ncons(e->e_from_trusted);
 	}
 #if 0
@@ -1717,8 +1718,8 @@ sequencer(e, file)
 		/* first level is ANDs */
 		++idnumber, nxor = 0;
 		for (to = car(l); to != NULL; to = cdr(to)) {
-			struct conscell *x, *tmp, *errto, *gg;
-			struct conscell *sender1;
+			conscell *x, *tmp, *errto, *gg;
+			conscell *sender1;
 
 			/* secondlevel is XORs */
 #if 0
@@ -1744,7 +1745,7 @@ sequencer(e, file)
 			gg = cdr(cddr(car(sender1))); /* recipient attrib! */
 			errto = find_errto(gg);   /* Possible 'ERR' value */
 			if (!errto)
-			  errto = cddar(sender1);  /* Sender 'user' */
+				errto = cddar(sender1);  /* Sender 'user' */
 
 			/* Rewriter level: */
 			for (rwp = rwhead; rwp != NULL; rwp = rwp->next)
@@ -1882,7 +1883,8 @@ sequencer(e, file)
 		 */
 		setreuid(0, e->e_statbuf.st_uid);
 
-		if ((vfp = fopen(verbosefile, "a")) != NULL) {
+		vfp = fopen(verbosefile, "a");
+		if (vfp != NULL) {
 			fseek(vfp, (off_t)0, 2);
 			setvbuf(vfp, NULL, _IOLBF, 0);
 			fprintf(vfp, "router processed message %s\n", file);
@@ -2121,13 +2123,13 @@ sequencer(e, file)
 	fsync(FILENO(ofp));
 #endif
 	if ((fclose(ofp) != 0) || (erename(file, qpath) != 0)) {
-		unlink(qpath);
-		unlink(ofpname);
+	  unlink(qpath);
+	  unlink(ofpname);
 #ifndef	USE_ALLOCA
-		free(ofpname);
-		free(qpath);
+	  free(ofpname);
+	  free(qpath);
 #endif
-		return PERR_CTRLFILE;
+	  return PERR_CTRLFILE;
 	}
 
 #ifndef USE_ALLOCA
@@ -2140,18 +2142,18 @@ sequencer(e, file)
 	sprintf(path, "../%s/%s", TRANSPORTDIR, file);
 	unlink(path);	/* Should actually always fail.. */
 	if (erename(ofpname, path) < 0) {
-		unlink(qpath);
-		unlink(path);
+	  unlink(qpath);
+	  unlink(path);
 #ifndef	USE_ALLOCA
-		free(ofpname);
-		free(qpath);
-		free(path);
+	  free(ofpname);
+	  free(qpath);
+	  free(path);
 #endif
-		return PERR_CTRLFILE;
+	  return PERR_CTRLFILE;
 	}
 
 	rtsyslog(e->e_statbuf.st_mtime, path, fromaddr, smtprelay,
-		 e->e_statbuf.st_size, nrcpts, msgidstr);
+		 (int) e->e_statbuf.st_size, nrcpts, msgidstr);
 
 #ifndef	USE_ALLOCA
 	free(ofpname);
@@ -2163,75 +2165,84 @@ sequencer(e, file)
 	return PERR_OK;
 }
 
-static char *
+static const char *
 prctladdr(info, fp, cfflag, comment)
-	struct conscell *info;
+	conscell *info;
 	FILE *fp;
 	int cfflag;
-	char *comment;
+	const char *comment;
 {
 	int i = 0;
-	register struct conscell *l, *x;
-	char *user = "?user?";
+	register conscell *l, *x;
+	const char *user = "?user?";
+	const char *channel = NULL;
+
+	/* We print the quad of  channel/host/user/privilege  information
+	   with this routine, and we return pointer to the user info.
+	   For an "error" channel we return an empty string for the user. */
 
 	for (l = car(info); l != NULL; l = cdr(l)) {
-		++i;
-		if (STRING(l)) {
-			if (cdr(l) == NULL /* No next == this is the last */
-			    && (x = v_find(l->string)) != NULL /* can find ? */
-			    && (x = cdr(x)) != NULL /* found, ... */
-			    && LIST(x)) {	    /* ... and it is a list */
-				/* Find the 'privilege' item */
-				for (x = car(x); x != NULL; x = cddr(x)) {
-					if (STRING(x)
-					    && strcmp((char *)x->string,
-						      "privilege") == 0) {
-						x = cdr(x);
-						break;
-					}
-				}
-				/* if x == NULL, no privilege was specified */
-			} else
-				x = l;
-			if (x != NULL) {
-				if (x->string == NULL || *x->string == '\0')
-					putc('-', fp);
-				else {
-				  char *s = strchr(x->string,' ');
-				  if (!s) s = strchr(x->string,'\t');
-				  if (s && x->string[0] != '"') {
-				    s = x->string;
-				    putc('"',fp);
-				    while (*s) {
-				      if (*s == '"' || *s == '\\')
-					putc('\\',fp);
-				      putc(*s,fp);
-				      ++s;
-				    }
-				    putc('"',fp);
-				  } else
-				    fprintf(fp, "%s", x->string);
-				}
-				if (i == 3 && x->string != NULL)
-				  user = x->string;
-			}
-			if (cdr(l))
-				putc(' ', fp);
+	  ++i;
+	  if (STRING(l)) {
+	    if (cdr(l) == NULL /* No next == this is the last */
+		&& (x = v_find(l->string)) != NULL /* can find ? */
+		&& (x = cdr(x)) != NULL /* found, ... */
+		&& LIST(x)) {	    /* ... and it is a list */
+	      /* Find the 'privilege' item */
+	      for (x = car(x); x != NULL; x = cddr(x)) {
+		if (STRING(x)
+		    && strcmp((char *)x->string,
+			      "privilege") == 0) {
+		  x = cdr(x);
+		  break;
+		}
+	      }
+	      /* if x == NULL, no privilege was specified */
+	    } else
+	      x = l;
+	    if (x != NULL) {
+	      if (x->string == NULL || *x->string == '\0')
+		putc('-', fp);
+	      else {
+		char *s = strchr(x->string,' ');
+		if (!s) s = strchr(x->string,'\t');
+		if (s && x->string[0] != '"') {
+		  s = x->string;
+		  putc('"',fp);
+		  while (*s) {
+		    if (*s == '"' || *s == '\\')
+		      putc('\\',fp);
+		    putc(*s,fp);
+		    ++s;
+		  }
+		  putc('"',fp);
 		} else
-			fprintf(stderr, "Malformed %s\n", comment);
+		  fprintf(fp, "%s", x->string);
+	      }
+	      if (i == 3 && x->string != NULL)
+		user = x->string;
+	      if (i == 1 && x->string != NULL)
+		channel = x->string;
+	    }
+	    if (cdr(l))
+	      putc(' ', fp);
+	  } else
+	    fprintf(stderr, "Malformed %s\n", comment);
 	}
+	if ((cfflag == _CF_SENDER) && channel && strcmp(channel,"error") == 0)
+		user = ""; /* error channel source address -> no user! */
 	return user;
 }
 
 static void
 prdsndata(info, fp, cfflag, comment)
-	struct conscell *info;
-	FILE *fp;
-	int cfflag;
-	char *comment;
+	conscell   *info;
+	FILE       *fp;
+	int         cfflag;
+	const char *comment;
 {
 	int i = 0;
-	register struct conscell *l, *x = NULL;
+	register conscell *l, *x = NULL;
 
 	for (l = car(info); l != NULL; l = cdr(l)) {
 	  ++i;
@@ -2257,12 +2268,11 @@ prdsndata(info, fp, cfflag, comment)
 }
 
 
-static struct conscell *
+static conscell *
 find_errto(info)
-struct conscell *info;
+conscell *info;
 {
-	int i = 0;
-	register struct conscell *x = NULL;
+	register conscell *x = NULL;
 
 	if (STRING(info))
 	  info = v_find(info->string);
