@@ -39,10 +39,11 @@ hdr_scanparse(e, h, commentflag, no_line_crossing)
 	int commentflag, no_line_crossing;
 {
 	register struct token *t;
-	long	len;
-	u_char	*cp, *ocp, c1, c2;
-	union misc retval;
-	struct token *tlist, **prev_tp, *scan_t, *nt;
+	long		 len;
+	const char	*cp, *ocp;
+	char		 c1, c2;
+	union misc	 retval;
+	struct token	*tlist, **prev_tp, *scan_t, *nt;
 	struct headerinfo *hd;
 
 	/* If we don't recognize the header, we can't parse it */
@@ -133,32 +134,32 @@ hdr_scanparse(e, h, commentflag, no_line_crossing)
  */
 
 struct header *
-makeHeader(db, s, len)
-	struct sptree *db;
-	register u_char	*s;
+makeHeader(hdb, s, len)
+	struct sptree *hdb;
+	register const char	*s;
 	int	len;
 {
 	struct headerinfo	*hlp;
 	struct header	*h;
 	char c;
-	u_char *p = s;
+	const char *p = s;
 	int testlen = len;
 
 	h = (struct header *)tmalloc(sizeof (struct header));
-	h->h_pname = strnsave((char *)s, len);
+	h->h_pname = strnsave(s, len);
 
 	/* Using strlen() may go beyond the original buffer,
 	   as is the case with partial reading of the email
 	   headers to a limited size buffer.. */
 	while (*p != 0 && --testlen >= 0) ++p;
 
-	if (*p != 0 || testlen > 0) {
+	if (testlen > 0 || *p != 0) {
 		c = *(s+len);
-		*(s+len) = '\0';
-		hlp = find_header(db, s);
-		*(s+len) = c;
+		*(char *)(s+len) = '\0';
+		hlp = find_header(hdb, s);
+		*(char *)(s+len) = c;
 	} else
-		hlp = find_header(db, s);
+		hlp = find_header(hdb, s);
 
 	if (hlp == NULL)
 		hlp = &nullhdr;
@@ -186,7 +187,7 @@ void
 set_pname(e, h, s)
 	struct envelope *e;
 	struct header *h;
-	char *s;
+	const char *s;
 {
 	struct headerinfo *sd;
 	char buf[BUFSIZ];
@@ -214,7 +215,8 @@ copySender(e)
 	struct envelope *e;
 {
 	register struct header	*h;
-	register char		**cpp, *cp;
+	register char		**cpp;
+	const char		*cp;
 	struct header		*best;
 	int			minval;
 	
@@ -285,8 +287,7 @@ mkMessageId(e, unixtime)
 	register struct header *h;
 	static struct headerinfo *msgidDesc;
 	static int genseq = 0;
-	char *cp, *s, *p, tzbuf[20];
-	u_char buf[1024];	/* XX: tsk tsk */
+	char buf[1024];	/* XX: tsk tsk */
 	struct tm *ts;
 	
 	if (msgidDesc == NULL) {
@@ -302,31 +303,34 @@ mkMessageId(e, unixtime)
 	h->h_stamp = newHeader;
 	h->h_next = 0;
 #if 0
-	cp = rfc822tz(&unixtime, &ts, 1);
-	for (s = cp, p = tzbuf; *s != '\0'; ++s) {
-		if (isascii(*s) && isupper(*s))
-			*p++ = tolower(*s);
-		else if (*s == '(' || *s == ')')
-			continue; /* Skip them.. */
-		else if (isascii(*s) && isspace(*s))
-			*p++ = '_';
-		else
-			*p++ = *s;
+	{
+	  char *cp, *s, *p, tzbuf[20];
+	  cp = rfc822tz(&unixtime, &ts, 1);
+	  for (s = cp, p = tzbuf; *s != '\0'; ++s) {
+	    if (isascii(*s) && isupper(*s))
+	      *p++ = tolower(*s);
+	    else if (*s == '(' || *s == ')')
+	      continue; /* Skip them.. */
+	    else if (isascii(*s) && isspace(*s))
+	      *p++ = '_';
+	    else
+	      *p++ = *s;
+	  }
+	  *p = '\0';
+	  cp = tzbuf;
+	  sprintf(buf, "<%02d%s%d.%02d%02d%02d%s.%s+%d@%s>",
+		  ts->tm_year % 100, monthname[ts->tm_mon], ts->tm_mday,
+		  ts->tm_hour, ts->tm_min, ts->tm_sec,
+		  cp, e->e_file, ++genseq, myhostname);
 	}
-	*p = '\0';
-	cp = tzbuf;
-	sprintf((char *)buf, "<%02d%s%d.%02d%02d%02d%s.%s+%d@%s>",
-		     ts->tm_year % 100, monthname[ts->tm_mon], ts->tm_mday,
-		     ts->tm_hour, ts->tm_min, ts->tm_sec,
-		     cp, e->e_file, ++genseq, myhostname);
 #else /* New way to do this .. more compressed one.. */
 	ts = gmtime(&unixtime);
-	sprintf((char *)buf, "<%04d%02d%02d%02d%02d%02dZ%s+%d@%s>",
+	sprintf(buf, "<%04d%02d%02d%02d%02d%02dZ%s+%d@%s>",
 		ts->tm_year + 1900, ts->tm_mon + 1, ts->tm_mday,
 		ts->tm_hour,        ts->tm_min,     ts->tm_sec,
 		e->e_file,          ++genseq,       myhostname);
 #endif
-	h->h_lines = makeToken(buf, strlen((char *)buf));
+	h->h_lines = makeToken(buf, strlen(buf));
 	h->h_lines->t_type = Line;
 	h->h_contents = hdr_scanparse(e, h, 0, 0);
 	/* OPTIMIZE: save this in a hashtable somewhere */
@@ -353,7 +357,7 @@ mkToHeader(e, buf)
 	h->h_descriptor = toDesc;
 	h->h_stamp = newHeader;
 	h->h_next = 0;
-	h->h_lines = makeToken(buf, strlen((char *)buf));
+	h->h_lines = makeToken(buf, strlen(buf));
 	h->h_lines->t_type = Line;
 	h->h_contents = hdr_scanparse(e, h, 0, 0);
 	/* OPTIMIZE: save this in a hashtable somewhere */
@@ -370,7 +374,7 @@ mkDate(isresent, unixtime)
 	time_t unixtime;
 {
 	register struct header	*h;
-	u_char *cp, *s;
+	char *cp, *s;
 	static struct headerinfo *dateDesc;
 	
 	/* There is really only one kind of dateDesc.. */
@@ -386,10 +390,10 @@ mkDate(isresent, unixtime)
 	h->h_stamp = newHeader;
 	h->h_next = 0;
 	h->h_contents.d = unixtime;
-	cp = (u_char*)rfc822date(&(h->h_contents.d));
-	s = cp + strlen((char*)cp) -1;
+	cp = rfc822date(&(h->h_contents.d));
+	s = cp + strlen(cp) -1;
 	if (*cp != 0 && *s == '\n') *s = 0;
-	h->h_lines = makeToken(cp, strlen((char *)cp));
+	h->h_lines = makeToken(cp, strlen(cp));
 	h->h_lines->t_type = Line;
 	/* OPTIMIZE: save this in a hashtable somewhere */
 	return h;
@@ -410,7 +414,6 @@ hdr_print(h, fp)
 	struct token *t;
 	struct address *ap;
 	struct addr *pp;
-	/* char *cp, *s; */
 
 	if (h == NULL)
 		return;
@@ -446,7 +449,7 @@ hdr_print(h, fp)
 		if (h->h_lines != NULL) {
 			/* Write out the original lines if possible */
 			for (t = h->h_lines; t != NULL; t = t->t_next) {
-				fwrite((char *)t->t_pname, sizeof (char),
+				fwrite(t->t_pname, sizeof (char),
 				       (int)(TOKENLEN(t)), fp);
 				putc('\n', fp);
 			}
@@ -481,7 +484,7 @@ hdr_print(h, fp)
 		putc(' ', fp);
 		{
 		  char *cp, *s;
-		  cp = (char*)rfc822date(&(h->h_contents.r->r_time));
+		  cp = rfc822date(&(h->h_contents.r->r_time));
 		  s = cp + strlen(cp) -1;
 		  if (*cp != 0 && *s == '\n') *s = 0;
 		  fputs(cp, fp);
@@ -490,7 +493,7 @@ hdr_print(h, fp)
 		break;
 	case nilHeaderSemantics:
 		for (t = h->h_lines; t != NULL; t = t->t_next) {
-			fwrite((char *)(t->t_pname), sizeof (char),
+			fwrite(t->t_pname, sizeof (char),
 			       (int)TOKENLEN(t), fp);
 			putc('\n', fp);
 		}
@@ -548,7 +551,7 @@ hdr_print(h, fp)
 		/* Use this ONLY if no original lines are available! */
 		if (h->h_contents.d > 0L && h->h_lines == NULL) {
 			char *cp, *s;
-			cp = (char *)rfc822date(&(h->h_contents.d));
+			cp = rfc822date(&(h->h_contents.d));
 			s = cp + strlen(cp) -1;
 			if (*cp != 0 && *s == '\n') *s = 0;
 			fputs(cp, fp);
@@ -556,8 +559,7 @@ hdr_print(h, fp)
 			break;
 		} /* else we have the original line ... */
 		for (t = h->h_lines; t != NULL; t = t->t_next) {
-			char *cp, *s;
-			cp = (char *)t->t_pname;
+			const char *cp = t->t_pname;
 			/* Skip over the leading white-space */
 			while (*cp == ' '||*cp == '\t') ++cp;
 			fputs(cp, fp);
@@ -566,7 +568,7 @@ hdr_print(h, fp)
 		break;
 	default:
 		for (t = h->h_lines; t != NULL; t = t->t_next) {
-			fwrite((char *)t->t_pname, sizeof (char),
+			fwrite(t->t_pname, sizeof (char),
 			       (int)(TOKENLEN(t)), fp);
 			putc('\n', fp);
 		}
@@ -627,7 +629,6 @@ pureAddressBuf(buf, len, pp)
 	int len;
 	register struct addr *pp;
 {
-	register struct token *t;
 	int tlen;
 
 	for (; pp != NULL ; pp = pp->p_next) {
@@ -844,7 +845,7 @@ saveAddress(pp)
 					*cp++ = '[';
 				else if (t->t_type == String)
 					*cp++ = '"';
-				memcpy(cp, (char *)t->t_pname, len);
+				memcpy(cp, t->t_pname, len);
 				cp += len;
 				if (t->t_type == DomainLiteral)
 					*cp++ = ']';
@@ -858,7 +859,7 @@ saveAddress(pp)
 			case aSpecial:
 				if (t != pp->p_tokens && *(t->t_pname) == '<')
 					*cp++ = ' ';
-				*cp++ = (char)(*t->t_pname);
+				*cp++ = *t->t_pname;
 				break;
 			}
 		}
@@ -1092,7 +1093,7 @@ hdr_warning(h)
 			       myhostname);
 	}
 
-	wh->h_lines = makeToken((u_char *)cp, strlen(cp));
+	wh->h_lines = makeToken(cp, strlen(cp));
 
 	sb_external(FILENO(stdout));
 	printf("\t%s:", h->h_pname);
@@ -1101,7 +1102,7 @@ hdr_warning(h)
 	len = strlen(cp);
 	if (*(cp + len - 1) == '\n')
 		--len;
-	wh->h_lines->t_next = makeToken((u_char *)cp, len);
+	wh->h_lines->t_next = makeToken(cp, len);
 	if (h->h_contents.a == NULL)
 		return wh;
 	return h;
@@ -1117,7 +1118,7 @@ hdr_errprint(e, h, fp, msg)
 	struct envelope *e;
 	struct header *h;
 	FILE *fp;
-	char *msg;
+	const char *msg;
 {
 	int i;
 	struct address *ap;
