@@ -16,7 +16,7 @@ STATIC		/* This will apply to the array in the .sst file */
 #include "rfc822.sst.c"
 
 extern void	EmitToken __((struct token *t, ComponentClass ac, struct address *ap));
-STATIC char	*Input822TokenName __(());
+STATIC const char * Input822TokenName __((int));
 
 /* private semantics mechanism variables: */
 STATIC ComponentClass currentTokenType;
@@ -33,7 +33,7 @@ STATIC TableOperation	operation;	/* the current operation */
 /* convenience */
 STATIC struct address nullAddr = { NULL, NULL, newAddress, 0, NULL, NULL };
 #ifdef ATHACK
-STATIC struct token atsigntoken = { (u_char *)"@", 1, Special, NULL };
+STATIC struct token atsigntoken = { "@", 1, Special, NULL };
 #endif
 
 /* Tracing Control */
@@ -89,9 +89,9 @@ STATIC InputTokens	nextInputToken = tNewLine;
 STATIC InputTokens	acceptedToken;
 
 struct receivedtag {
-	char		*name;
-	int		len;
-	InputTokens	value;
+	const char	*name;
+	int		 len;
+	InputTokens	 value;
 };
 
 STATIC struct receivedtag rt[] = {
@@ -109,7 +109,10 @@ STATIC struct receivedtag rt[] = {
 STATIC InputTokens	savedInputToken;
 STATIC int		RunningSslRecovery = 0;
 
-STATIC struct { ErrorCodes ec; char *msg; } ear[] = {
+STATIC struct {
+	ErrorCodes ec;
+	const char *msg;
+} ear[] = {
 { eSyntaxError,			"Syntax error"				},
 { ePrematureEndOfFile,		"Unexpected end of file"		},
 { eExtraneousProgramText,	"Extraneous program text"		},
@@ -120,7 +123,7 @@ STATIC struct { ErrorCodes ec; char *msg; } ear[] = {
 { eMissingSemicolonInReceived,	"missing semicolon before timestamp"	},
 { eMissingEndOfAddress,		"missing end of address"		},
 { eMissingEndOfMailbox,		"missing end of mailbox"		},
-{ eIllegalWordInPhrase,		"illegal word in phrase"	},
+{ eIllegalWordInPhrase,		"illegal word in phrase"		},
 { eIllegalSpecialInPhrase,	"illegal special character in phrase"	},
 { eIllegalPeriodInPhrase,	"illegal period in phrase"		},
 { eIllegalPhraseMustBeQuoted,	"phrases containing '.' must be quoted"	},
@@ -142,6 +145,7 @@ STATIC struct { ErrorCodes ec; char *msg; } ear[] = {
 
 /* This procedure emits the error message associated with errCode */
 
+STATIC void SslError __((ErrorCodes, struct address *));
 STATIC void
 SslError(errCode, ap)
 	ErrorCodes errCode;
@@ -151,16 +155,16 @@ SslError(errCode, ap)
 	int i;
 
 	if (errCode == eNoError)
-		abort();
+	  abort();
 
 	for (t = NULL, i = 0; i < (sizeof ear/sizeof ear[0]); ++i) {
-		if (ear[i].ec == errCode) {
-			t = makeToken((u_char *)ear[i].msg, strlen(ear[i].msg));
-			break;
-		}
+	  if (ear[i].ec == errCode) {
+	    t = makeToken(ear[i].msg, strlen(ear[i].msg));
+	    break;
+	  }
 	}
 	if (t == NULL)
-		t = makeToken((u_char *)"Unknown", sizeof("Unknown")-1);
+	  t = makeToken("Unknown", sizeof("Unknown")-1);
 	t->t_type = Error;
 	EmitToken(t, cError, ap);
 
@@ -168,8 +172,8 @@ SslError(errCode, ap)
 
 	if ((int)(errCode) >= (int)(firstFatalErrorCode)
 	    || noErrors == maxErrors) {
-		aborted = 1;
-		processing = 0;
+	  aborted = 1;
+	  processing = 0;
 	}
 }
 
@@ -179,6 +183,7 @@ SslError(errCode, ap)
  * indicators and the values and text associated with input tokens.  
  */
 
+STATIC void AcceptInputToken __((struct token **, struct address *));
 STATIC void
 AcceptInputToken(tlistp, ap)
 	struct token **tlistp;
@@ -186,121 +191,121 @@ AcceptInputToken(tlistp, ap)
 {
 
 	if (acceptedToken == tEndOfHeader && nextInputToken == tEndOfHeader)
-		abort();
+	  abort();
 
 	if (nextInputToken == tSyntaxError) {
-		acceptedToken = tSyntaxError;
-		/* ptrAcceptedToken = NULL; */
-		nextInputToken = savedInputToken;
-		goto ok;
+	  acceptedToken = tSyntaxError;
+	  /* ptrAcceptedToken = NULL; */
+	  nextInputToken = savedInputToken;
+	  goto ok;
 	} else {
-		/* Accept Token */
-		acceptedToken = nextInputToken;
-		ptrAcceptedToken = ptrNextInputToken;
+	  /* Accept Token */
+	  acceptedToken = nextInputToken;
+	  ptrAcceptedToken = ptrNextInputToken;
 	}
 
 	/* Read Next Input Token */
 	do {
-next:
-		if (*tlistp == NULL) {
-			nextInputToken = tEndOfHeader;
-			ptrNextInputToken = NULL;
-		} else {
-			ptrNextInputToken = *tlistp;
-			switch ((*tlistp)->t_type) {
-			case Error:
-			case Comment:
-				if (ptrAcceptedToken == NULL && ap != NULL) {
-					EmitToken(*tlistp,
-						(*tlistp)->t_type == Error ?
-							cError : cComment, ap);
-				} else {
-					(*pending) = copyToken(*tlistp);
-					pending = &((*pending)->t_next);
-				}
-				(*tlistp) = (*tlistp)->t_next;
-				goto next;
-			case Space:
+	next:
+	  if (*tlistp == NULL) {
+	    nextInputToken = tEndOfHeader;
+	    ptrNextInputToken = NULL;
+	  } else {
+	    ptrNextInputToken = *tlistp;
+	    switch ((*tlistp)->t_type) {
+	    case Error:
+	    case Comment:
+	      if (ptrAcceptedToken == NULL && ap != NULL) {
+		EmitToken(*tlistp,
+			  (*tlistp)->t_type == Error ?
+			  cError : cComment, ap);
+	      } else {
+		(*pending) = copyToken(*tlistp);
+		pending = &((*pending)->t_next);
+	      }
+	      (*tlistp) = (*tlistp)->t_next;
+	      goto next;
+	    case Space:
 #if 0
 # if 0
-				if (RunningSslRecovery) {
-				  /* Copy it only during SslRecovery */
-				  if (ptrAcceptedToken == NULL && ap != NULL) {
-				    EmitToken(*tlistp, cSpace, ap);
-				  } else {
-				    (*pending) = copyToken(*tlistp);
-				    pending = &((*pending)->t_next);
-				  }
-				}
+	      if (RunningSslRecovery) {
+		/* Copy it only during SslRecovery */
+		if (ptrAcceptedToken == NULL && ap != NULL) {
+		  EmitToken(*tlistp, cSpace, ap);
+		} else {
+		  (*pending) = copyToken(*tlistp);
+		  pending = &((*pending)->t_next);
+		}
+	      }
 # else
-				nextInputToken = tSpace;
-				(*tlistp) = (*tlistp)->t_next;
-				break;
+	      nextInputToken = tSpace;
+	      (*tlistp) = (*tlistp)->t_next;
+	      break;
 # endif
 #endif
-				/* fall through */
-			case Fold:
-				(*tlistp) = (*tlistp)->t_next;
-				goto next;
-			case Special:
-				switch ((*tlistp)->t_pname[0]) {
-				case ';':
-					nextInputToken = tSemicolon;
-					break;
-				case ',':
-					nextInputToken = tComma;
-					break;
-				case '.':
-					nextInputToken = tPeriod;
-					break;
-				case ':':
-					if ((*tlistp)->t_pname[1] == ':')
-					  nextInputToken = tDoubleColon;
-					else
-					  nextInputToken = tColon;
-					break;
-				case '<':
-					nextInputToken = tLeftAngle;
-					break;
-				case '>':
-					nextInputToken = tRightAngle;
-					break;
-				case '@':
-					nextInputToken = tAtSign;
-					break;
-				default:
-					nextInputToken = tOtherSpecial;
-					break;
-				}
-				*tlistp = (*tlistp)->t_next;
-				break;
-			case String:
-				nextInputToken = tQuotedString;
-				*tlistp = (*tlistp)->t_next;
-				break;
-			case Atom:
-				nextInputToken = tAtom;
-				*tlistp = (*tlistp)->t_next;
-				break;
-			case DomainLiteral:
-				nextInputToken = tDomainLiteral;
-				*tlistp = (*tlistp)->t_next;
-				break;
-			case Word:
-			case Empty:
-				abort();
-			default:
-				break;
-			}
-		}
+	      /* fall through */
+	    case Fold:
+	      (*tlistp) = (*tlistp)->t_next;
+	      goto next;
+	    case Special:
+	      switch ((*tlistp)->t_pname[0]) {
+	      case ';':
+		nextInputToken = tSemicolon;
+		break;
+	      case ',':
+		nextInputToken = tComma;
+		break;
+	      case '.':
+		nextInputToken = tPeriod;
+		break;
+	      case ':':
+		if ((*tlistp)->t_pname[1] == ':')
+		  nextInputToken = tDoubleColon;
+		else
+		  nextInputToken = tColon;
+		break;
+	      case '<':
+		nextInputToken = tLeftAngle;
+		break;
+	      case '>':
+		nextInputToken = tRightAngle;
+		break;
+	      case '@':
+		nextInputToken = tAtSign;
+		break;
+	      default:
+		nextInputToken = tOtherSpecial;
+		break;
+	      }
+	      *tlistp = (*tlistp)->t_next;
+	      break;
+	    case String:
+	      nextInputToken = tQuotedString;
+	      *tlistp = (*tlistp)->t_next;
+	      break;
+	    case Atom:
+	      nextInputToken = tAtom;
+	      *tlistp = (*tlistp)->t_next;
+	      break;
+	    case DomainLiteral:
+	      nextInputToken = tDomainLiteral;
+	      *tlistp = (*tlistp)->t_next;
+	      break;
+	    case Word:
+	    case Empty:
+	      abort();
+	    default:
+	      break;
+	    }
+	  }
 	} while (nextInputToken == tNewLine);
 
 ok:
 	/* Trace Input */
 	if (tracefp)
-	    printf("Input token accepted %s (%d)  Next input token %s (%d)\n",
-		   Input822TokenName((int)acceptedToken), acceptedToken,
-		   Input822TokenName((int)nextInputToken), nextInputToken);
+	  printf("Input token accepted %s (%d)  Next input token %s (%d)\n",
+		 Input822TokenName((int)acceptedToken), acceptedToken,
+		 Input822TokenName((int)nextInputToken), nextInputToken);
 }
 
 /*
@@ -340,6 +345,7 @@ ok:
  * been matched.
  */
 
+STATIC void SslSyntaxError __((struct token **, struct address *));
 STATIC void
 SslSyntaxError(tlistp, ap)
 	struct token **tlistp;
@@ -349,35 +355,35 @@ SslSyntaxError(tlistp, ap)
 		abort();
 
 	if (nextInputToken == tSyntaxError) {
-		/* Currently recovering from syntax error */
-		/* Complete recovery by synchronizing input to a new address */
-		nextInputToken = savedInputToken;
-		RunningSslRecovery = 1;
-		while (nextInputToken != tSemicolon
-			&& nextInputToken != tComma
-			&& nextInputToken != tEndOfHeader) {
-			AcceptInputToken(tlistp, ap);
-			EmitToken(ptrAcceptedToken, cResync, ap);
-		}
-		RunningSslRecovery = 0;
-		if (sslTop == 0)
-			/* Return from main S/SL procedure */
-			processing = 0;
-		else
-			sslPointer = sslStack[sslTop--];
+	  /* Currently recovering from syntax error */
+	  /* Complete recovery by synchronizing input to a new address */
+	  nextInputToken = savedInputToken;
+	  RunningSslRecovery = 1;
+	  while (nextInputToken != tSemicolon
+		 && nextInputToken != tComma
+		 && nextInputToken != tEndOfHeader) {
+	    AcceptInputToken(tlistp, ap);
+	    EmitToken(ptrAcceptedToken, cResync, ap);
+	  }
+	  RunningSslRecovery = 0;
+	  if (sslTop == 0)
+	    /* Return from main S/SL procedure */
+	    processing = 0;
+	  else
+	    sslPointer = sslStack[sslTop--];
 	} else {
-		/* First syntax error on the line */
-		if (sslTable[sslPointer] == (int)(tEndOfHeader)) {
-			/* Ignore missing logical newlines */
-		} else {
-			/* mark syntax error, gobble tokens, jump to resync */
-			savedInputToken = nextInputToken;
-			nextInputToken = tSyntaxError;
-		}
+	  /* First syntax error on the line */
+	  if (sslTable[sslPointer] == (int)(tEndOfHeader)) {
+	    /* Ignore missing logical newlines */
+	  } else {
+	    /* mark syntax error, gobble tokens, jump to resync */
+	    savedInputToken = nextInputToken;
+	    nextInputToken = tSyntaxError;
+	  }
 	}
 }
 
-STATIC char *
+STATIC const char *
 Input822TokenName(in)
 	int in;
 {
@@ -411,7 +417,8 @@ Input822TokenName(in)
 	return "unknown input token";
 }
 
-STATIC char *
+STATIC const char * TableOperationName __((TableOperation));
+STATIC const char *
 TableOperationName(op)
 	TableOperation	op;
 {
@@ -446,6 +453,7 @@ TableOperationName(op)
 	return "unknown operation";
 }
 
+STATIC void SslTrace __((void));
 STATIC void
 SslTrace()
 {
@@ -454,7 +462,7 @@ SslTrace()
 	       sslTable[sslPointer]);
 }
 
-
+STATIC void SslFailure __((FailureCodes));
 STATIC void
 SslFailure(failCode)
 	FailureCodes failCode;
@@ -485,6 +493,7 @@ SslFailure(failCode)
  * if a match is found and false otherwise.
  */
 
+STATIC void SslChoice __((int));
 STATIC void
 SslChoice(choiceTag)
 	int choiceTag;
@@ -497,33 +506,33 @@ SslChoice(choiceTag)
 	for (numberOfChoices = sslTable[choicePointer++];
 	     numberOfChoices > 0;
 	     choicePointer += 2, --numberOfChoices) {
-		if (tracefp)
-			printf("Matching %s (%d) with %s (%d)\n",
-				Input822TokenName(choiceTag), choiceTag,
-				Input822TokenName(sslTable[choicePointer]),
-				sslTable[choicePointer]);
-		if (sslTable[choicePointer] == choiceTag
+	  if (tracefp)
+	    printf("Matching %s (%d) with %s (%d)\n",
+		   Input822TokenName(choiceTag), choiceTag,
+		   Input822TokenName(sslTable[choicePointer]),
+		   sslTable[choicePointer]);
+	  if (sslTable[choicePointer] == choiceTag
 #ifdef	ATHACK
-		    || (choiceTag == (int)tAtom
-				  && sslTable[choicePointer] == (int)tAtSign
-				  && ptrNextInputToken != NULL
-				  && ptrNextInputToken->t_len == 2
-				  && (ptrNextInputToken->t_pname[0] == 'a'
-				      || ptrNextInputToken->t_pname[0] == 'A')
-				  && (ptrNextInputToken->t_pname[1] == 't'
-				      || ptrNextInputToken->t_pname[1] == 'T')
-				  && (ptrNextInputToken = &atsigntoken))
+	      || (choiceTag == (int)tAtom
+		  && sslTable[choicePointer] == (int)tAtSign
+		  && ptrNextInputToken != NULL
+		  && ptrNextInputToken->t_len == 2
+		  && (ptrNextInputToken->t_pname[0] == 'a'
+		      || ptrNextInputToken->t_pname[0] == 'A')
+		  && (ptrNextInputToken->t_pname[1] == 't'
+		      || ptrNextInputToken->t_pname[1] == 'T')
+		  && (ptrNextInputToken = &atsigntoken))
 #endif	/* ATHACK */
-		    ) {
-			sslPointer = sslTable[choicePointer+1];
-			choiceTagMatched = 1;
-			if (tracefp)
-				printf("Matched!\n");
-			return;
-		}
+	      ) {
+	    sslPointer = sslTable[choicePointer+1];
+	    choiceTagMatched = 1;
+	    if (tracefp)
+	      printf("Matched!\n");
+	    return;
+	  }
 	}
 	if (tracefp)
-		printf("No match!\n");
+	  printf("No match!\n");
 	sslPointer = choicePointer;
 }
 
@@ -538,29 +547,29 @@ EmitToken(t, ac, ap)
 	AddrComponent type;
 
 	if (deferred != NULL) {
-		if (ac == cError)  {	/* prepend errors to deferred tokens */
-			nt = copyToken(t);
-			ptp = &headDeferred;
-			for(t = headDeferred; t ; ptp = &t->t_next, t = *ptp)
-				if (t->t_type != Error)
-					break;
-			nt->t_next = *ptp;
-			*ptp = nt;
-			if (deferred == &headDeferred)
-				deferred = &nt->t_next;
-		} else {
-			(*deferred) = copyToken(t);
-			deferred = &((*deferred)->t_next);
-		}
-		if (headPending) {
-			/* printf("append pending\n"); */
-			(*deferred) = headPending;
-			deferred = pending;
-			(*pending) = NULL;
-			pending = &headPending;
-			headPending = 0;
-		}
-		return;
+	  if (ac == cError)  {	/* prepend errors to deferred tokens */
+	    nt = copyToken(t);
+	    ptp = &headDeferred;
+	    for(t = headDeferred; t ; ptp = &t->t_next, t = *ptp)
+	      if (t->t_type != Error)
+		break;
+	    nt->t_next = *ptp;
+	    *ptp = nt;
+	    if (deferred == &headDeferred)
+	      deferred = &nt->t_next;
+	  } else {
+	    (*deferred) = copyToken(t);
+	    deferred = &((*deferred)->t_next);
+	  }
+	  if (headPending) {
+	    /* printf("append pending\n"); */
+	    (*deferred) = headPending;
+	    deferred = pending;
+	    (*pending) = NULL;
+	    pending = &headPending;
+	    headPending = 0;
+	  }
+	  return;
 	}
 	switch (ac) {
 	case cPhrase:	type = aPhrase; break;
@@ -582,26 +591,26 @@ EmitToken(t, ac, ap)
 	 * reverseComponent() function will eventually do the reversal.
 	 */
 	if (ap->a_tokens != NULL && ap->a_tokens->p_type == type) {
-		/* prepend now -- reverse later */
-		nt = copyToken(t);
-		nt->t_next = ap->a_tokens->p_tokens;
-		ap->a_tokens->p_tokens = nt;
+	  /* prepend now -- reverse later */
+	  nt = copyToken(t);
+	  nt->t_next = ap->a_tokens->p_tokens;
+	  ap->a_tokens->p_tokens = nt;
 	} else {
-		na = (struct addr *)tmalloc(sizeof (struct addr));
-		/* prepend now -- reverse later */
-		nt = copyToken(t);
-		nt->t_next = NULL;
-		na->p_tokens = nt;
-		na->p_next = ap->a_tokens;
-		na->p_type = type;
-		ap->a_tokens = na;
+	  na = (struct addr *)tmalloc(sizeof (struct addr));
+	  /* prepend now -- reverse later */
+	  nt = copyToken(t);
+	  nt->t_next = NULL;
+	  na->p_tokens = nt;
+	  na->p_next = ap->a_tokens;
+	  na->p_type = type;
+	  ap->a_tokens = na;
 	}
 	if (headPending) {
-		(*pending) = NULL;
-		for (t = headPending, headPending = NULL;
-		     t != NULL; t = t->t_next)
-			EmitToken(t, t->t_type==Error ? cError : cComment, ap);
-		pending = &headPending;
+	  (*pending) = NULL;
+	  for (t = headPending, headPending = NULL;
+	       t != NULL; t = t->t_next)
+	    EmitToken(t, t->t_type==Error ? cError : cComment, ap);
+	  pending = &headPending;
 	}
 }
 
@@ -614,26 +623,27 @@ revaddress(ap)
 
 	/* reverse order of address components and tokens */
 	if (ap == NULL)
-		return NULL;
+	  return NULL;
 	/* reverse the various linked lists */
 	pprev = NULL;
 	for (p = ap->a_tokens; p != NULL; p = pnext) {
-		pnext = p->p_next;
-		p->p_next = pprev;
-		pprev = p;
-		rprev = NULL;
-		for (r = p->p_tokens; r != NULL; r = rnext) {
-			rnext = r->t_next;
-			r->t_next = rprev;
-			rprev = r;
-		}
-		p->p_tokens = rprev;
+	  pnext = p->p_next;
+	  p->p_next = pprev;
+	  pprev = p;
+	  rprev = NULL;
+	  for (r = p->p_tokens; r != NULL; r = rnext) {
+	    rnext = r->t_next;
+	    r->t_next = rprev;
+	    rprev = r;
+	  }
+	  p->p_tokens = rprev;
 	}
 	ap->a_tokens = pprev;
 	return ap;
 }
 
-void
+STATIC void revappend __((struct token **, struct address *));
+STATIC void
 revappend(tprev,ap)
 struct token **tprev;
 struct address *ap;
@@ -645,11 +655,11 @@ struct address *ap;
 	/* append all the tokens together */
 	if (ap == NULL || ap->a_tokens == NULL
 	    || ap->a_stamp == BadAddress)
-		return;
+	  return;
 	for (p = ap->a_tokens; p ; p = p->p_next) {
-		*tprev = p->p_tokens;
-		for (t = p->p_tokens; t ; t = t->t_next)
-			tprev = &(t->t_next);
+	  *tprev = p->p_tokens;
+	  for (t = p->p_tokens; t ; t = t->t_next)
+	    tprev = &(t->t_next);
 	}
 	*tprev = NULL;
 }
@@ -681,7 +691,6 @@ parse822(entry, tlistp, ltmp, tfp)
 {
 	int		inReceived, i;
 	struct token	*t, *torig;
-	struct addr	*p;
 	struct address	*ap, *na, *pap;
 	struct received	rcvd;
 	union misc	retval;
@@ -712,318 +721,318 @@ parse822(entry, tlistp, ltmp, tfp)
 	/* Initialize Input/Output */
 	AcceptInputToken(tlistp, ap);
 	if (nextInputToken == tEndOfHeader) {
-		retval.a = NULL;
-		return retval;
+	  retval.a = NULL;
+	  return retval;
 	}
 
 	/* Walk the S/SL Table */
 
 	sslPointer = (int)entry;
 	while (processing || inAddress) {
-		if (processing)
-			operation = (TableOperation)(sslTable[sslPointer++]);
-		else
-			operation = oCloseAddress;
+	  if (processing)
+	    operation = (TableOperation)(sslTable[sslPointer++]);
+	  else
+	    operation = oCloseAddress;
 
-		/* Trace Execution */
-		if (tracefp)
-			SslTrace();
+	  /* Trace Execution */
+	  if (tracefp)
+	    SslTrace();
 
-		switch (operation) {
-		case oCall:
-			if (sslTop < sslStackSize) {
-				sslStack[++sslTop] = sslPointer + 1;
-				sslPointer = sslTable[sslPointer];
-			} else {
-				if (ap == NULL) {
-					ap = (struct address *)
-					    tmalloc(sizeof (struct address));
-					*ap = nullAddr;
-					inAddress = 1;
-				}
-				SslError(eSslStackOverflow, ap);
-				processing = 0;
-			}
-			break;
-		case oReturn:
-			if (sslTop == 0)
-				/* Return from main S/SL procedure */
-				processing = 0;
-			else
-				sslPointer = sslStack[sslTop--];
-			break;
-		case oRuleEnd:
-			SslFailure(fChoiceRuleFailed);
-			break;
-		case oJump:
-			sslPointer = sslTable[sslPointer];
-			break;
-		case oInput:
-			if (sslTable[sslPointer] == (int)(nextInputToken)
+	  switch (operation) {
+	  case oCall:
+	    if (sslTop < sslStackSize) {
+	      sslStack[++sslTop] = sslPointer + 1;
+	      sslPointer = sslTable[sslPointer];
+	    } else {
+	      if (ap == NULL) {
+		ap = (struct address *)
+		  tmalloc(sizeof (struct address));
+		*ap = nullAddr;
+		inAddress = 1;
+	      }
+	      SslError(eSslStackOverflow, ap);
+	      processing = 0;
+	    }
+	    break;
+	  case oReturn:
+	    if (sslTop == 0)
+	      /* Return from main S/SL procedure */
+	      processing = 0;
+	    else
+	      sslPointer = sslStack[sslTop--];
+	    break;
+	  case oRuleEnd:
+	    SslFailure(fChoiceRuleFailed);
+	    break;
+	  case oJump:
+	    sslPointer = sslTable[sslPointer];
+	    break;
+	  case oInput:
+	    if (sslTable[sslPointer] == (int)(nextInputToken)
 #ifdef	ATHACK
-			    || (nextInputToken == tAtom
-				&& sslTable[sslPointer] == (int)tAtSign
-				&& ptrNextInputToken != NULL
-				&& ptrNextInputToken->t_len == 2
-				&& (ptrNextInputToken->t_pname[0] == 'a'
-				    || ptrNextInputToken->t_pname[0] == 'A')
-				&& (ptrNextInputToken->t_pname[1] == 't'
-				    || ptrNextInputToken->t_pname[1] == 'T')
-				&& (ptrNextInputToken = &atsigntoken))
+		|| (nextInputToken == tAtom
+		    && sslTable[sslPointer] == (int)tAtSign
+		    && ptrNextInputToken != NULL
+		    && ptrNextInputToken->t_len == 2
+		    && (ptrNextInputToken->t_pname[0] == 'a'
+			|| ptrNextInputToken->t_pname[0] == 'A')
+		    && (ptrNextInputToken->t_pname[1] == 't'
+			|| ptrNextInputToken->t_pname[1] == 'T')
+		    && (ptrNextInputToken = &atsigntoken))
 #endif	/* ATHACK */
-				) {
-				AcceptInputToken(tlistp, ap);
-				++sslPointer;
-			} else {
-				++sslPointer;
-				/* Syntax error in input */
-				if (ap == NULL) {
-					ap = (struct address *)
-					    tmalloc(sizeof (struct address));
-					*ap = nullAddr;
-					inAddress = 1;
-				}
-				SslSyntaxError(tlistp, ap);
-			}
-			break;
-		case oInputAny:
-			if (nextInputToken != tEndOfHeader)
-				AcceptInputToken(tlistp, ap);
-			else {
-				/* Premature end of file */
-				if (ap == NULL) {
-					ap = (struct address *)
-					    tmalloc(sizeof (struct address));
-					*ap = nullAddr;
-					inAddress = 1;
-				}
-				SslSyntaxError(tlistp, ap);
-			}
-			break;
-		case oInputChoice:
-			if (inReceived && nextInputToken == tAtom) {
-				t = ptrNextInputToken;
-				for (i = 0; rt[i].name != NULL; ++i) {
-					if (TOKENLEN(t) == strlen(rt[i].name)
-					    && CISTREQN(t->t_pname,
-						    rt[i].name, rt[i].len)) {
-						nextInputToken = rt[i].value;
-						break;
-					}
-				}
-			}
-			SslChoice((int)nextInputToken);
-
-			if (choiceTagMatched)
-				AcceptInputToken(tlistp, ap);
-			break;
-		case oEmit:
-			++sslPointer;
-			break;
-		case oError:
-			++sslPointer;
-			if (ap == NULL) {
-				ap = (struct address *)
-					tmalloc(sizeof (struct address));
-				*ap = nullAddr;
-				inAddress = 1;
-			}
-			SslError((ErrorCodes)sslTable[sslPointer-1], ap);
-			break;
-		case oChoice:
-			if (inReceived && resultValue == (int)tAtom) {
-				t = ptrNextInputToken;
-				for (i = 0; rt[i].name != NULL; ++i) {
-					if (TOKENLEN(t) == strlen(rt[i].name)
-					    && CISTREQN(t->t_pname,
-						    rt[i].name, rt[i].len)) {
-						resultValue = (int)rt[i].value;
-						break;
-					}
-				}
-			}
-			SslChoice(resultValue);
-			break;
-		case oChoiceEnd:
-			SslFailure(fSemanticChoiceFailed);
-			break;
-		case oSetParameter:
-			parameterValue = sslTable[sslPointer++];
-			break;
-		case oSetResult:
-			resultValue = sslTable[sslPointer++];
-			break;
-		case oSetResultFromInput:
-			resultValue = (int)nextInputToken;
-			break;
-
-		/* The Following Are Pass Dependent Semantic Mechanisms */
-
-		case oSetComponentType:
-			currentTokenType = (ComponentClass)parameterValue;
-			break;
-		case oEmitTokenCurType:
-			parameterValue = (int)currentTokenType;
-			/* fall through */
-		case oEmitToken:
-			if (ap == NULL) {
-				ap = (struct address *)
-					tmalloc(sizeof (struct address));
-				*ap = nullAddr;
-				inAddress = 1;
-			}
-			EmitToken(ptrAcceptedToken,
-				  (ComponentClass)parameterValue, ap);
-			break;
-		case oDeferEmitToken:
-			if (deferred != NULL)
-				abort();
-			deferred = &headDeferred;
-			if (headPending) {
-				if (ap == NULL) {
-					ap = (struct address *)
-					    tmalloc(sizeof (struct address));
-					*ap = nullAddr;
-					inAddress = 1;
-				}
-				(*pending) = NULL;
-				for (t = headPending, headPending = NULL;
-				     t != NULL; t = t->t_next)
-					EmitToken(t, t->t_type == Error ?
-						     cError : cComment, ap);
-				pending = &headPending;
-			}
-			break;
-		case oReleaseEmitToken:	/* comments... */
-			if (ap == NULL) {
-				ap = (struct address *)
-					tmalloc(sizeof (struct address));
-				*ap = nullAddr;
-				inAddress = 1;
-			}
-			t = headDeferred;
-			headDeferred = NULL;
-			(*deferred) = NULL;
-			deferred = NULL;
-			for (; t != NULL; t = t->t_next)
-				if (t->t_type == Comment)
-					EmitToken(t, cComment, ap);
-				else if (t->t_type == Error)
-					EmitToken(t, cError, ap);
-				else
-					EmitToken(t,
-						(ComponentClass)parameterValue,
-						ap);
-			break;
-		case oOpenAddress:
-			if (inAddress)
-				break;
-			na = (struct address *)tmalloc(sizeof (struct address));
-			*na = nullAddr;
-			na->a_next = ap;
-			ap = na;
-			inAddress = 1;
-			break;
-		case oAppendAddress:
-			if (inAddress)
-				break;
-			ap = revaddress(ap);
-			inAddress = 1;
-			break;
-		case oCloseAddress:
-			if (!inAddress)
-				break;
-			ap = revaddress(ap);
-			inAddress = 0;
-			break;
-		case oDateTime:
-			if (ptrNextInputToken != NULL) {
-				retval.d = dateParse(ltmp, ptrNextInputToken);
-			} else
-				retval.d = 0;
-			break;
-		case oEnterReceived:
-			inReceived = 1;
-			rcvd.r_from = rcvd.r_by = rcvd.r_id = rcvd.r_for = NULL;
-			rcvd.r_via = rcvd.r_with = rcvd.r_convert = NULL;
-			rcvd.r_time = 0L;
-			break;
-		case oExitReceived:
-			inReceived = 0;
-			break;
-		case oSaveReceivedComponent:
-			switch ((ReceivedComponent)parameterValue) {
-			case rcFrom:
-				rcvd.r_from = revaddress(ap);
-				break;
-			case rcBy:
-				rcvd.r_by = revaddress(ap);
-				break;
-			case rcVia:
-				/* grab a single token */
-				if (ap != NULL
-				    && ap->a_stamp != BadAddress
-				    && ap->a_tokens != NULL
-				    && ap->a_tokens->p_tokens != NULL) {
-					rcvd.r_via = ap->a_tokens->p_tokens;
-					rcvd.r_via->t_next = NULL; /* in case */
-				}
-				break;
-			case rcWith:
-				revappend(&rcvd.r_with,ap);
-				break;
-			case rcConvert:
-				revappend(&rcvd.r_convert,ap);
-				break;
-			case rcId:
-				rcvd.r_id = revaddress(ap);
-				break;
-			case rcFor:
-				rcvd.r_for = revaddress(ap);
-				break;
-			case rcDate:
-				rcvd.r_time = retval.d;
-				break;
-			default:
-				break;
-			}
-			ap = NULL;
-			break;
-		case oSetReturnType:
-			returnType = (ReturnValue)parameterValue;
-			break;
-		case oRewind:
-			if (ap == NULL
-			    || (ap->a_tokens == NULL && ap->a_next == NULL)) {
-				*tlistp = torig;
-				pending = &headPending;
-				deferred = NULL;
-				nextInputToken = tNewLine;
-				acceptedToken = tNewLine;
-				AcceptInputToken(tlistp, ap);
-				resultValue = (int)Success;
-			} else
-				resultValue = (int)Failure;
-			break;
-		    
-		default:
-			printf("Unknown operation %d\n", (int)operation);
-			abort();
+		) {
+	      AcceptInputToken(tlistp, ap);
+	      ++sslPointer;
+	    } else {
+	      ++sslPointer;
+	      /* Syntax error in input */
+	      if (ap == NULL) {
+		ap = (struct address *)
+		  tmalloc(sizeof (struct address));
+		*ap = nullAddr;
+		inAddress = 1;
+	      }
+	      SslSyntaxError(tlistp, ap);
+	    }
+	    break;
+	  case oInputAny:
+	    if (nextInputToken != tEndOfHeader)
+	      AcceptInputToken(tlistp, ap);
+	    else {
+	      /* Premature end of file */
+	      if (ap == NULL) {
+		ap = (struct address *)
+		  tmalloc(sizeof (struct address));
+		*ap = nullAddr;
+		inAddress = 1;
+	      }
+	      SslSyntaxError(tlistp, ap);
+	    }
+	    break;
+	  case oInputChoice:
+	    if (inReceived && nextInputToken == tAtom) {
+	      t = ptrNextInputToken;
+	      for (i = 0; rt[i].name != NULL; ++i) {
+		if (TOKENLEN(t) == strlen(rt[i].name)
+		    && CISTREQN(t->t_pname,
+				rt[i].name, rt[i].len)) {
+		  nextInputToken = rt[i].value;
+		  break;
 		}
+	      }
+	    }
+	    SslChoice((int)nextInputToken);
+
+	    if (choiceTagMatched)
+	      AcceptInputToken(tlistp, ap);
+	    break;
+	  case oEmit:
+	    ++sslPointer;
+	    break;
+	  case oError:
+	    ++sslPointer;
+	    if (ap == NULL) {
+	      ap = (struct address *)
+		tmalloc(sizeof (struct address));
+	      *ap = nullAddr;
+	      inAddress = 1;
+	    }
+	    SslError((ErrorCodes)sslTable[sslPointer-1], ap);
+	    break;
+	  case oChoice:
+	    if (inReceived && resultValue == (int)tAtom) {
+	      t = ptrNextInputToken;
+	      for (i = 0; rt[i].name != NULL; ++i) {
+		if (TOKENLEN(t) == strlen(rt[i].name)
+		    && CISTREQN(t->t_pname,
+				rt[i].name, rt[i].len)) {
+		  resultValue = (int)rt[i].value;
+		  break;
+		}
+	      }
+	    }
+	    SslChoice(resultValue);
+	    break;
+	  case oChoiceEnd:
+	    SslFailure(fSemanticChoiceFailed);
+	    break;
+	  case oSetParameter:
+	    parameterValue = sslTable[sslPointer++];
+	    break;
+	  case oSetResult:
+	    resultValue = sslTable[sslPointer++];
+	    break;
+	  case oSetResultFromInput:
+	    resultValue = (int)nextInputToken;
+	    break;
+
+	    /* The Following Are Pass Dependent Semantic Mechanisms */
+
+	  case oSetComponentType:
+	    currentTokenType = (ComponentClass)parameterValue;
+	    break;
+	  case oEmitTokenCurType:
+	    parameterValue = (int)currentTokenType;
+	    /* fall through */
+	  case oEmitToken:
+	    if (ap == NULL) {
+	      ap = (struct address *)
+		tmalloc(sizeof (struct address));
+	      *ap = nullAddr;
+	      inAddress = 1;
+	    }
+	    EmitToken(ptrAcceptedToken,
+		      (ComponentClass)parameterValue, ap);
+	    break;
+	  case oDeferEmitToken:
+	    if (deferred != NULL)
+	      abort();
+	    deferred = &headDeferred;
+	    if (headPending) {
+	      if (ap == NULL) {
+		ap = (struct address *)
+		  tmalloc(sizeof (struct address));
+		*ap = nullAddr;
+		inAddress = 1;
+	      }
+	      (*pending) = NULL;
+	      for (t = headPending, headPending = NULL;
+		   t != NULL; t = t->t_next)
+		EmitToken(t, t->t_type == Error ?
+			  cError : cComment, ap);
+	      pending = &headPending;
+	    }
+	    break;
+	  case oReleaseEmitToken:	/* comments... */
+	    if (ap == NULL) {
+	      ap = (struct address *)
+		tmalloc(sizeof (struct address));
+	      *ap = nullAddr;
+	      inAddress = 1;
+	    }
+	    t = headDeferred;
+	    headDeferred = NULL;
+	    (*deferred) = NULL;
+	    deferred = NULL;
+	    for (; t != NULL; t = t->t_next)
+	      if (t->t_type == Comment)
+		EmitToken(t, cComment, ap);
+	      else if (t->t_type == Error)
+		EmitToken(t, cError, ap);
+	      else
+		EmitToken(t,
+			  (ComponentClass)parameterValue,
+			  ap);
+	    break;
+	  case oOpenAddress:
+	    if (inAddress)
+	      break;
+	    na = (struct address *)tmalloc(sizeof (struct address));
+	    *na = nullAddr;
+	    na->a_next = ap;
+	    ap = na;
+	    inAddress = 1;
+	    break;
+	  case oAppendAddress:
+	    if (inAddress)
+	      break;
+	    ap = revaddress(ap);
+	    inAddress = 1;
+	    break;
+	  case oCloseAddress:
+	    if (!inAddress)
+	      break;
+	    ap = revaddress(ap);
+	    inAddress = 0;
+	    break;
+	  case oDateTime:
+	    if (ptrNextInputToken != NULL) {
+	      retval.d = dateParse(ltmp, ptrNextInputToken);
+	    } else
+	      retval.d = 0;
+	    break;
+	  case oEnterReceived:
+	    inReceived = 1;
+	    rcvd.r_from = rcvd.r_by = rcvd.r_id = rcvd.r_for = NULL;
+	    rcvd.r_via = rcvd.r_with = rcvd.r_convert = NULL;
+	    rcvd.r_time = 0L;
+	    break;
+	  case oExitReceived:
+	    inReceived = 0;
+	    break;
+	  case oSaveReceivedComponent:
+	    switch ((ReceivedComponent)parameterValue) {
+	    case rcFrom:
+	      rcvd.r_from = revaddress(ap);
+	      break;
+	    case rcBy:
+	      rcvd.r_by = revaddress(ap);
+	      break;
+	    case rcVia:
+	      /* grab a single token */
+	      if (ap != NULL
+		  && ap->a_stamp != BadAddress
+		  && ap->a_tokens != NULL
+		  && ap->a_tokens->p_tokens != NULL) {
+		rcvd.r_via = ap->a_tokens->p_tokens;
+		rcvd.r_via->t_next = NULL; /* in case */
+	      }
+	      break;
+	    case rcWith:
+	      revappend(&rcvd.r_with,ap);
+	      break;
+	    case rcConvert:
+	      revappend(&rcvd.r_convert,ap);
+	      break;
+	    case rcId:
+	      rcvd.r_id = revaddress(ap);
+	      break;
+	    case rcFor:
+	      rcvd.r_for = revaddress(ap);
+	      break;
+	    case rcDate:
+	      rcvd.r_time = retval.d;
+	      break;
+	    default:
+	      break;
+	    }
+	    ap = NULL;
+	    break;
+	  case oSetReturnType:
+	    returnType = (ReturnValue)parameterValue;
+	    break;
+	  case oRewind:
+	    if (ap == NULL
+		|| (ap->a_tokens == NULL && ap->a_next == NULL)) {
+	      *tlistp = torig;
+	      pending = &headPending;
+	      deferred = NULL;
+	      nextInputToken = tNewLine;
+	      acceptedToken = tNewLine;
+	      AcceptInputToken(tlistp, ap);
+	      resultValue = (int)Success;
+	    } else
+	      resultValue = (int)Failure;
+	    break;
+		    
+	  default:
+	    printf("Unknown operation %d\n", (int)operation);
+	    abort();
+	  }
 	}
 
 	if (returnType == rAddress) {
-		if (nextInputToken != tEndOfHeader && !aborted && ap != NULL)
-			SslError(eExtraneousProgramText, ap);
-		/* reverse order of addresses */
-		for (na = NULL; ap != NULL; na = ap, ap = pap) {
-			pap = ap->a_next;
-			ap->a_next = na;
-		}
-		retval.a = na;
+	  if (nextInputToken != tEndOfHeader && !aborted && ap != NULL)
+	    SslError(eExtraneousProgramText, ap);
+	  /* reverse order of addresses */
+	  for (na = NULL; ap != NULL; na = ap, ap = pap) {
+	    pap = ap->a_next;
+	    ap->a_next = na;
+	  }
+	  retval.a = na;
 	} else if (returnType == rReceived) {
-		retval.r = (struct received *)tmalloc(sizeof (struct received));
-		*(retval.r) = rcvd;
+	  retval.r = (struct received *)tmalloc(sizeof (struct received));
+	  *(retval.r) = rcvd;
 	}
 	return retval;
 }
