@@ -25,12 +25,8 @@
 #include "splay.h"
 
 #include "prototypes.h"
-#include "libz.h"
 
-extern int loginit();/* should be RETSIGTYPE but we're playing games */
-extern int nobody;
-extern int sh_include();
-extern int zshtoplevel();
+const char *postoffice = NULL;
 
 #if 0
 extern struct group *getgrnam __();
@@ -39,25 +35,17 @@ extern struct passwd *getpwuid __();
 #endif
 
 extern struct shCmd fnctns[];
-extern time_t time();
-extern void prversion();
-extern void tfree();
-extern void trapexit();
-extern void detach();
-extern void zshinit();
-extern void setfreefd();
-extern void init_header();
+extern time_t time __((time_t *));
 
-static void initialize __((char *configfile, int argc, char *argv[]));
-static void logit __((char *file, char *id, char *from, char *to));
+static void initialize __((const char *configfile, int argc, const char *argv[]));
+static void logit __((const char *file, const char *id, const char *from, const char *to));
 
 
-extern char *progname; /* At zmsh_init() et.friends */
-char	*mailshare;
-char	*myhostname = 0;
-extern char *postoffice; /* at mail_open() */
-char	*pidfile = PID_ROUTER;
-char	*logfn;
+extern const char *progname; /* At zmsh_init() et.friends */
+const char * mailshare;
+const char * myhostname = 0;
+const char * pidfile = PID_ROUTER;
+const char * logfn;
 time_t	now;
 
 extern memtypes stickymem;
@@ -69,24 +57,30 @@ int	deferit;
 int	deferuid;
 int	origoptind;
 int	savefile = 0;
-char	*zshopts = "-O";
+const char * zshopts = "-O";
 int	nosyslog = 1;
 
 int
 main(argc, argv)
-	int	argc;
-	char	*argv[];
+	int	    argc;
+	const char *argv[];
 {
 	int c, errflg, daemonflg, killflg, interactiveflg, tac, nrouters;
 	int version;
 	long offout, offerr;
-	char *config, *cp, *tav[20], *av[3];
+	char *config, *cp;
+	const char *tav[20], *av[3];
 #ifdef	XMEM
 	FILE *fp;
 #endif	/* XMEM */
 
+#ifdef HAVE_SETGROUPS
+	/* We null supplementary groups list entirely */
+	setgroups(0, NULL);
+#endif
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
+	progname = strrchr(argv[0], '/');
+	if (progname == NULL)
 		progname = argv[0];
 	else
 		++progname;
@@ -97,7 +91,11 @@ main(argc, argv)
 	nrouters = 1;
 
 
-	while ((c = getopt(argc, argv, "m:n:dikf:o:t:L:P:SV")) != EOF)
+	while (1) {
+		c = getopt(argc, (char*const*)argv, "m:n:dikf:o:t:L:P:sSV");
+		if (c == EOF)
+			break;
+	  
 		switch (c) {
 		case 'd':	/* become a daemon */
 			daemonflg = 1;
@@ -145,6 +143,9 @@ main(argc, argv)
 		case 'k':	/* kill the previous daemon upon startup */
 			killflg = 1;
 			break;
+		case 's':
+			stability = !stability;
+			break;
 		case 'S':	/* Logging also always to SYSLOG,
 				   not only at serious stuff */
 			nosyslog = 0;
@@ -163,6 +164,7 @@ main(argc, argv)
 			errflg++;
 			break;
 		}
+	}
 	if (errflg || (interactiveflg && daemonflg)) {
 		fprintf(stderr,
 			"Usage: %s [ -dikV -n #routers -t traceflag -f configfile -L logfile -P postoffice]\n",
@@ -170,7 +172,8 @@ main(argc, argv)
 		exit(128+errflg);
 	}
 	time(&now);
-	if ((mailshare = getzenv("MAILSHARE")) == NULL)
+	mailshare = getzenv("MAILSHARE");
+	if (mailshare == NULL)
 		mailshare = MAILSHARE;
 	if (config == NULL) {
 		config = cf_suffix; /* XX: ???? delete this ? */
@@ -199,15 +202,16 @@ main(argc, argv)
 			logdir = cp;
 		logfn = smalloc(MEM_PERM, 2 + (u_int)(strlen(logdir)
 					  + strlen(progname)));
-		sprintf(logfn, "%s/%s", logdir, progname);
+		sprintf((char*)logfn, "%s/%s", logdir, progname);
 	}
 	/* setvbuf(stdout, (char *)NULL, _IOLBF, 0);
 	   setvbuf(stderr, (char *)NULL, _IOLBF, 0); */
 	setvbuf(stdout, (char *)NULL, _IONBF, 0);
 	setvbuf(stderr, (char *)NULL, _IONBF, 0);
+
 	if (logfn != NULL) {
 		/* loginit is a signal handler, so can't pass log */
-		if (loginit() < 0)	/* do setlinebuf() there */
+		if (loginit(0) < 0)	/* do setlinebuf() there */
 			die(1, "log initialization failure");
 		signal(SIGHUP, sig_hup); /* close and reopen log files */
 	} else
@@ -242,14 +246,15 @@ main(argc, argv)
 		/* XX: check if another daemon is running already */
 		if (offout < ftell(stdout) || offerr < ftell(stderr)) {
 		  fprintf(stderr, "%d %d %d %d\n",
-			  offout, ftell(stdout), offerr, ftell(stderr));
+			  (int) offout, (int) ftell(stdout),
+			  (int) offerr, (int) ftell(stderr));
 		  fprintf(stderr, "%s: daemon not started.\n", progname);
 		  die(1, "errors during startup");
 		}
 		if (tac == 0)		/* leave worldy matters behind */
 		  detach();
 		printf("%s: router daemon (%s)\n\tstarted at %s\n",
-		       progname, Version, (char *)rfc822date(&now));
+		       progname, Version, rfc822date(&now));
 		if (killflg)
 		  if (killprevious(-SIGTERM, pidfile) != 0) {
 		    /* Indicates failure at pidfile creation! */
@@ -305,9 +310,9 @@ write(30, "\n", 1);
 	  /* NOTREACHED */
 	} else if (interactiveflg) {
 #ifdef	MALLOC_TRACE
-	  zshtoplevel((char *)NULL);
+	  zshtoplevel(NULL);
 #else	/* !MALLOC_TRACE */
-	  trapexit(zshtoplevel((char *)NULL));
+	  trapexit(zshtoplevel(NULL));
 #endif	/* MALLOC_TRACE */
 	  /* NOTREACHED */
 	}
@@ -329,16 +334,17 @@ write(30, "\n", 1);
 
 static void
 initialize(configfile, argc, argv)
-	char *configfile;
+	const char *configfile;
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct group *grp;
 	struct sptree_init *sptip;
 	int ac;
-	char **cpp, **av;
+	char **cpp;
+	const char **av;
 
-	av = (char **)emalloc((5+argc)*(sizeof (char *)));
+	av = (const char **)emalloc((5+argc)*(sizeof (char *)));
 	/* initialize shell */
 	ac = 0;
 	av[ac++] = progname;
@@ -354,8 +360,8 @@ initialize(configfile, argc, argv)
 		register struct shCmd *shcmdp;
 
 		for (shcmdp = &fnctns[0]; shcmdp->name != NULL; ++shcmdp)
-			sp_install(symbol((u_char *)shcmdp->name),
-				   (u_char *)shcmdp, 0, spt_builtins);
+			sp_install(symbol(shcmdp->name),
+				   (void*)shcmdp, 0, spt_builtins);
 	}
 
 	/* initialize splay trees in router */
@@ -364,25 +370,23 @@ initialize(configfile, argc, argv)
 	av[4] = NULL;
 
 	for (sptip = &splaytrees[0]; sptip->spta != NULL; ++sptip) {
-		if (sptip->incore_name != NULL) {
-
-			if (sptip->spta == &spt_headers)
-				av[2] = "header";
-			else
-				av[2] = "incore";
-			av[3] = sptip->incore_name;
-			if (run_relation(4, av) == 0)
-				*(sptip->spta) =
-					icdbspltree(sptip->incore_name);
-		} else
-			*(sptip->spta) = sp_init();
+	  if (sptip->incore_name != NULL) {
+	    if (sptip->spta == &spt_headers)
+	      av[2] = "header";
+	    else
+	      av[2] = "incore";
+	    av[3] = sptip->incore_name;
+	    if (run_relation(4, av) == 0)
+	      *(sptip->spta) = icdbspltree(sptip->incore_name);
+	  } else
+	    *(sptip->spta) = sp_init();
 	}
 
 	init_header();
 
 	/* trusted users */
 	for (cpp = default_trusted; cpp != NULL && *cpp != NULL; ++cpp)
-		sp_install(symbol((u_char *)*cpp), NULL, 0, spt_goodguys);
+		sp_install(symbol(*cpp), NULL, 0, spt_goodguys);
 
 	if (files_group != NULL) {
 		if ((grp = getgrnam(files_group)) == NULL)
@@ -396,25 +400,25 @@ initialize(configfile, argc, argv)
 	av[ac++] = ".";
 	if (strchr(configfile, '/') == NULL) {
 		av[ac] = emalloc(strlen(configfile)+sizeof "./"+1);
-		sprintf(av[ac++], "./%s", configfile);
+		sprintf((char*)av[ac++], "./%s", configfile);
 	} else
 		av[ac++] = configfile;
 	av[ac] = NULL;
 	setfreefd();
 	sh_include(ac, av);
 	if (av[1] != configfile)
-		free(av[1]);
+		free((void*)av[1]);
 	free((char *)av);
 }
 
 int
 login_to_uid(name)
-	u_char	*name;
+	const char	*name;
 {
 	struct passwd *pw;
 	uid_t uid;
 	char buf[BUFSIZ];
-	u_char *cp, *fn;
+	char *cp, *fn;
 	struct spblk *spl;
 
 	spl = sp_lookup(symbol(name), spt_loginmap);
@@ -422,49 +426,51 @@ login_to_uid(name)
 		memtypes oval = stickymem;
 
 		stickymem = MEM_MALLOC;
-		if ((pw = getpwnam((char *)name)) == NULL) {
+		pw = getpwnam(name);
+		if (pw == NULL) {
 			uid = nobody;
 		} else {
 			uid = pw->pw_uid;
-			cp = (u_char *)strsave(pw->pw_name);
-			sp_install((u_int)uid, cp, 0, spt_uidmap);
+			cp = strsave(pw->pw_name);
+			sp_install(uid, cp, 0, spt_uidmap);
 			fullname(pw->pw_gecos,buf,sizeof buf,pw->pw_name);
-			fn = (u_char *)strsave(buf);
+			fn = strsave(buf);
 			sp_install(symbol(cp), fn, 0, spt_fullnamemap);
 		}
-		sp_install(symbol(name), (u_char *)((long)uid), 0, spt_loginmap);
+		sp_install(symbol(name), (void *)((long)uid), 0, spt_loginmap);
 		stickymem = oval;
 	} else
-		uid = (int)((long)spl->data);
+		uid = (int)(spl->data);
 	return uid;
 }
 
-u_char *
+const char *
 uidpwnam(uid)
 	int	uid;
 {
 	struct passwd *pw;
-	register u_char *cp;
-	u_char *fn;
+	register const char *cp;
+	char *fn;
 	struct spblk *spl;
 	char buf[BUFSIZ];
 
 	spl = sp_lookup((u_int)uid, spt_uidmap);
 	if (spl == NULL) {
-		if ((pw = getpwuid(uid)) == NULL) {
+		pw = getpwuid(uid);
+		if (pw == NULL) {
 			/* memory shall be temporary in
 			   its nature for this data! */
 			sprintf(buf, "uid#%d", uid);
-			cp = (u_char *)strsave(buf);
+			cp = strsave(buf);
 			deferuid = 1;
 		} else {
 			memtypes oval = stickymem;
 			stickymem = MEM_MALLOC;
-			cp = (u_char *)strsave(pw->pw_name);
-			sp_install(symbol(cp), (u_char *)((long)pw->pw_uid), 0,
+			cp = strsave(pw->pw_name);
+			sp_install(symbol(cp), (void*)((long)pw->pw_uid), 0,
 				   spt_loginmap);
 			fullname(pw->pw_gecos,buf,sizeof buf,pw->pw_name);
-			fn = (u_char *)strsave(buf);
+			fn = strsave(buf);
 			sp_install(symbol(cp), fn, 0, spt_fullnamemap);
 			sp_install((u_int)uid, cp, 0, spt_uidmap);
 			stickymem = oval;
@@ -487,7 +493,7 @@ isgoodguy(uid)
 	 * If you're wondering about this comparison... I had to store
 	 * *something* in the splay tree; I'm really using it as a boolean.
 	 */
-	spl = sp_lookup(symbol((u_char *)uidpwnam(uid)), spt_goodguys);
+	spl = sp_lookup(symbol(uidpwnam(uid)), spt_goodguys);
 	return spl != NULL;
 }
 
@@ -498,7 +504,8 @@ logmessage(e)
 	struct envelope *e;
 {
 	int n, len;
-	char *from, *to, *cp;
+	const char *from;
+	char *to, *cp;
 	char buf[MAXSAFESIZE];
 	struct header *h;
 	struct addr *p;
@@ -560,7 +567,7 @@ logmessage(e)
 
 static void
 logit(file, id, from, to)
-	char *file, *id, *from, *to;
+	const char *file, *id, *from, *to;
 {
 	int flen, baselen;
 #if 0
@@ -632,12 +639,13 @@ logit(file, id, from, to)
 #endif
 }
 
-char *
+const char *
 mail_host()
 {
 	return myhostname;
 }
 
+extern void printfds __((void));
 void
 printfds()
 {
