@@ -8,22 +8,21 @@
 
 #include "prototypes.h"
 
-struct conscell **return_valuep = 0;
-struct conscell *s_value = 0;
+conscell **return_valuep = NULL;
+conscell *s_value        = NULL ;
 
-extern int l_apply __((u_char *fname, struct conscell *l));
 int
 l_apply(fname, l)
-	u_char *fname;
-	struct conscell *l;
+	const char *fname;
+	conscell *l;
 {
 	int retval;
-	struct conscell *retvp = NULL;
-	struct conscell **oretvp = return_valuep;
+	conscell *retvp = NULL;
+	conscell **oretvp = return_valuep;
 
 	return_valuep = &retvp;
 
-	retval = lapply((char *)fname, l);
+	retval = lapply(fname, l);
 	s_value = retvp;
 
 	return_valuep = oretvp;
@@ -34,11 +33,11 @@ l_apply(fname, l)
 int
 s_apply(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	int retval;
-	struct conscell *retvp = NULL;
-	struct conscell **oretvp = return_valuep;
+	conscell *retvp = NULL;
+	conscell **oretvp = return_valuep;
 
 	return_valuep = &retvp;
 
@@ -51,7 +50,8 @@ s_apply(argc, argv)
 int
 n_apply(cpp, argc, argv)
 	int argc;
-	char **cpp, *argv[];
+	char **cpp;
+	const char *argv[];
 {
 	int retval;
 
@@ -66,15 +66,16 @@ n_apply(cpp, argc, argv)
  * This is used to rewrite an address.
  */
 
-static int s_rewrite __((u_char *func, struct token *t, u_char *argx));
+static int s_rewrite __((const char *func, struct token *t, const char *argx));
+
 static int
 s_rewrite(func, t, argx)
-	u_char *func, *argx;
+	const char *func, *argx;
 	struct token *t;
 {
-	register u_char *cp, *bp;
-	u_char *av[4];
-	u_char *buf = malloc(4000);
+	register char *cp, *bp;
+	const char *av[4];
+	char *buf = malloc(4000);
 	int bufspc = 4000;
 	int rc;
 
@@ -102,7 +103,7 @@ s_rewrite(func, t, argx)
 	av[1] = buf;
 	av[2] = argx;
 	av[3] = NULL;
-	rc = s_apply(argx == NULL ? 2 : 3, (char **)&av[0]);
+	rc = s_apply(argx == NULL ? 2 : 3, av);
 	free(buf);
 	return rc;
 }
@@ -120,6 +121,7 @@ s_rewrite(func, t, argx)
  */
 
 static struct addr *mergeAddress __((struct addr *pp, struct token *t));
+
 static struct addr *
 mergeAddress(pp, t)
 	struct addr *pp;
@@ -162,21 +164,22 @@ mergeAddress(pp, t)
 
 struct header *
 hdr_rewrite(name, h)
-	u_char *name;
+	const char *name;
 	struct header *h;
 {
 	register struct address *ap;
 	register struct addr *pp;
 	register struct token *t;
-	struct address *nap, *pap;
+	struct address *nap = NULL, *pap;
 	struct token *nt, *pt, *addrtokens;
 	struct header *nh;
-	struct conscell *l;
-	u_char *cp, *eocp, *s, buf[4096]; 	/* XX */
+	conscell *l;
+	const char *cp, *eocp;
+	char *s, buf[4096], *eobuf; 	/* XX */
 
 	if (D_hdr_rewrite) {
 		printf("---------------------------\n");
-		printf("Sending this through %s:\n", (char *)name);
+		printf("Sending this through %s:\n", name);
 		dumpHeader(h);
 	}
 	nh = (struct header *)tmalloc(sizeof (struct header));
@@ -209,7 +212,7 @@ hdr_rewrite(name, h)
 				*s++ = *cp;
 			}
 			*s = 0;
-			addrtokens->t_pname = (u_char *)strsave((char *)buf);
+			addrtokens->t_pname = strsave(buf);
 		}
 		nap = (struct address *)tmalloc(sizeof (struct address));
 		nap->a_pname = NULL;
@@ -252,10 +255,9 @@ hdr_rewrite(name, h)
 			l = s_copy_tree(s_value);
 			s_free_tree(s_value);
 			s_value = NULL;
-			/* t = HDR_SCANNER(l->string); */
-			t = scan822(&l->string, strlen((char*)(l->string)),
-				    (u_char)'!', (u_char)'%',
-				    0, &ap->a_tokens->p_tokens);
+			/* t = HDR_SCANNER(l->cstring); */
+			t = scan822(&l->cstring, strlen(l->cstring),
+				    '!', '%', 0, &ap->a_tokens->p_tokens);
 			/* X: check for errors! */
 			nap->a_tokens = mergeAddress(ap->a_tokens, t);
 		}
@@ -278,7 +280,7 @@ setenvinfo(e)
 	struct envelope *e;
 {
 	struct header *h;
-	struct conscell *pl, *plhead, *tmp;
+	conscell *pl, *plhead, *tmp;
 	char buf[20];
 
 	/* include header size ("headersize"), message size ("size"),
@@ -289,43 +291,43 @@ setenvinfo(e)
 #define	CONSTSTR(s)	cdr(pl) = conststring(s); pl = cdr(pl)
 #define	NEWSTR(s)	cdr(pl) = newstring(s); pl = cdr(pl)
 #define	NEWSTRD(d)	sprintf(buf, "%ld", (long)(d)); \
-			cdr(pl) = newstring((u_char *)strsave(buf)); \
+			cdr(pl) = newstring(strsave(buf)); \
 			pl = cdr(pl)
 
-	pl = plhead = conststring((u_char *)"file");
-	CONSTSTR((u_char *)e->e_file);
+	pl = plhead = conststring("file");
+	CONSTSTR(e->e_file);
 
 	if (e->e_messageid != NULL) {
-		CONSTSTR((u_char *)"message-id");
-		CONSTSTR((u_char *)e->e_messageid);
+		CONSTSTR("message-id");
+		CONSTSTR(e->e_messageid);
 	}
 
-	CONSTSTR((u_char *)"uid");
+	CONSTSTR("uid");
 	NEWSTRD(e->e_statbuf.st_uid);
 
-	CONSTSTR((u_char *)"gid");
+	CONSTSTR("gid");
 	NEWSTRD(e->e_statbuf.st_gid);
 
-	CONSTSTR((u_char *)"size");
+	CONSTSTR("size");
 	NEWSTRD(e->e_statbuf.st_size - e->e_hdrOffset);
 
-	CONSTSTR((u_char *)"headersize");
+	CONSTSTR("headersize");
 	NEWSTRD(e->e_msgOffset - e->e_hdrOffset);
 
-	CONSTSTR((u_char *)"bodysize");
+	CONSTSTR("bodysize");
 	NEWSTRD(e->e_statbuf.st_size - e->e_msgOffset);
 
-	CONSTSTR((u_char *)"now");
+	CONSTSTR("now");
 	NEWSTRD(e->e_nowtime);
 
-	CONSTSTR((u_char *)"delay");
+	CONSTSTR("delay");
 	NEWSTRD(e->e_nowtime - e->e_statbuf.st_mtime);
 
-	CONSTSTR((u_char *)"resent");
-	CONSTSTR(e->e_resent ? (u_char *)"yes" : (u_char *)"no");
+	CONSTSTR("resent");
+	CONSTSTR(e->e_resent ? "yes" : "no");
 
-	CONSTSTR((u_char *)"trusted");
-	CONSTSTR(e->e_trusted ? (u_char *)"yes" : (u_char *)"no");
+	CONSTSTR("trusted");
+	CONSTSTR(e->e_trusted ? "yes" : "no");
 
 	/* for every non-address envelope header, include
 	   header-name header-value pair in property list */
@@ -333,9 +335,9 @@ setenvinfo(e)
 	for (h = e->e_eHeaders; h != NULL; h = h->h_next) {
 		if (h->h_descriptor->user_type != nilUserType)
 			continue;
-		CONSTSTR((u_char *)h->h_descriptor->hdr_name);
+		CONSTSTR(h->h_descriptor->hdr_name);
 		if (h->h_lines == NULL || *h->h_lines->t_pname == '\0') {
-			CONSTSTR((u_char *)h->h_descriptor->hdr_name);
+			CONSTSTR(h->h_descriptor->hdr_name);
 		} else {
 			CONSTSTR(h->h_lines->t_pname);
 		}
@@ -350,10 +352,10 @@ static char gsbuf[30];
  *  newattribute_2()
  */
 char *newattribute_2(onam,nam,val)
-char *onam, *nam, *val;
+const char *onam, *nam, *val;
 {
-	struct conscell *l, *lc, *tmp, **pl;
-	struct conscell	*l1;
+	conscell *l, *lc, *tmp, **pl;
+	conscell	*l1;
 
 	l1 = v_find(onam);
 	if (!l1)
@@ -374,10 +376,10 @@ char *onam, *nam, *val;
 	}
 
 	/* Prepend in reverse order */
-	tmp = newstring((u_char*)strsave(val));
+	tmp = newstring(strsave(val));
 	cdr(tmp) = car(l);
 	car(l) = tmp;
-	tmp = newstring((u_char*)strsave(nam));
+	tmp = newstring(strsave(nam));
 	cdr(tmp) = car(l);
 	car(l) = tmp;
 
@@ -391,42 +393,42 @@ char *onam, *nam, *val;
  * Build gensym
  */
 char *
-build_gensym(uid,type,DSNstr,errors_to,sender)
+build_gensym(uid,type,DSNstr,errorsto,sender)
 int uid;
-char *type, *DSNstr, *errors_to, *sender;
+const char *type, *DSNstr, *errorsto, *sender;
 {
 	char buf[20];
-	struct conscell *l, *lc, *tmp;
+	conscell *l, *lc, *tmp;
 
 	/* assemble the default attribute list: (privilege <uid>) */
-	l = conststring((u_char *)"privilege");
+	l = conststring("privilege");
 	sprintf(buf, "%d", uid);
-	cdr(l) = newstring((u_char *)strsave(buf));
+	cdr(l) = newstring(strsave(buf));
 	lc = cdr(l);
 	if (type != NULL) {
-		cdr(lc) = conststring((u_char *)"type");
+		cdr(lc) = conststring("type");
 		lc = cdr(lc);
-		cdr(lc) = newstring((u_char *)strsave(type));
+		cdr(lc) = newstring(strsave(type));
 		lc = cdr(lc);
 	}
 	if (DSNstr != NULL) {
-		cdr(lc) = conststring((u_char *)"DSN");
+		cdr(lc) = conststring("DSN");
 		lc = cdr(lc);
-		cdr(lc) = newstring((u_char *)strsave(DSNstr));
+		cdr(lc) = newstring(strsave(DSNstr));
 		lc = cdr(lc);
 	}
 	/* See if some "errorsto" definition is available.. */
-	if (errors_to != NULL) {
-		cdr(lc) = conststring((u_char *)"ERR");
+	if (errorsto != NULL) {
+		cdr(lc) = conststring("ERR");
 		lc = cdr(lc);
-		cdr(lc) = newstring((u_char *)strsave(errors_to));
+		cdr(lc) = newstring(strsave(errorsto));
 		lc = cdr(lc);
 	}
 	/* See if some "sender" definition is available.. */
 	if (sender != NULL) {
-		cdr(lc) = conststring((u_char *)"sender");
+		cdr(lc) = conststring("sender");
 		lc = cdr(lc);
-		cdr(lc) = newstring((u_char *)strsave(sender));
+		cdr(lc) = newstring(strsave(sender));
 		lc = cdr(lc);
 	}
 	cdr(lc) = NULL;
@@ -445,20 +447,20 @@ char *type, *DSNstr, *errors_to, *sender;
  * function to deal with it.
  */
 
-struct conscell *
+conscell *
 router(a, uid, type)
 	struct address *a;
 	int uid;
-	char *type;
+	const char *type;
 {
 	register struct token *t, *tt;
 	int r;
 	struct token *last;
 	struct addr *p;
-	struct conscell *l, *lc, *tmp;
-	char buf[30], *gsym;
+	conscell *l, *tmp;
+	const char *gsym;
 	struct notary *DSN = NULL;
-	char *DSNstr;
+	const char *DSNstr;
 
 	if (a == NULL)
 		return NULL;
@@ -493,12 +495,12 @@ router(a, uid, type)
 
 	deferit = 0;
 	v_set(DEFER, "");
-	r = s_rewrite(ROUTER, t, (u_char *)gsym);
+	r = s_rewrite(ROUTER, t, gsym);
 #if 0
 	if (deferit) {
 		s_free_tree(s_value);
 		s_value = NULL;
-		r = s_rewrite(DEFERENV, t, (u_char *)gsym);
+		r = s_rewrite(DEFERENV, t, gsym);
 	}
 #endif
 	if (r != 0 || s_value == NULL || !LIST(s_value)) {
@@ -548,11 +550,11 @@ router(a, uid, type)
  * The munged parameters will eventually find their way onto the envelopes.
  */
 
-struct conscell *
+conscell *
 crossbar(from, to)
-	struct conscell *from, *to;
+	conscell *from, *to;
 {
-	struct conscell *l, *tmp;
+	conscell *l, *tmp;
 
 	l = ncons(from);
 	cdar(l) = to;
