@@ -66,7 +66,7 @@ doagenda()
 /* Do immediate scheduling of given host-indentifier. */
 void
 turnme(turnarg)
-char *turnarg;
+const char *turnarg;
 {
 	struct spblk *spl;
 	struct web *wp;
