@@ -25,6 +25,7 @@
 #include "mail.h"
 #include "scheduler.h"
 #include "malloc.h"
+#include "mailer.h"
 
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
@@ -56,24 +57,21 @@ struct conshell *envarlist = NULL;
 int	D_alloc = 0;
 
 
-extern char **hp_getaddr(), **hp_nextaddr();
+#include "prototypes.h"
+#include "memtypes.h"
+#include "token.h"
+#include "libz.h"
+#include "libc.h"
+
 extern char *qoutputfile;
-extern char *strerror();
-extern char *whathost(), *dottedquad();
 extern int errno, pokedaemon();
-extern int isalive();
-extern void docat();
-extern void hp_init();
-extern void printaddrs();
-extern void prversion(), checkrouter(), checkscheduler();
-extern void report();
 
 #ifndef strchr
 extern char *strchr(), *strrchr();
 #endif
 
-char	*progname;
-char	*postoffice;
+const char	*progname;
+const char	*postoffice;
 
 int	debug, verbose, summary, user, status, onlyuser, nonlocal, schedq;
 int	sawcore, othern;
@@ -107,7 +105,10 @@ main(argc, argv)
 
 	progname = argv[0];
 	verbose = debug = errflg = status = user = onlyuser = summary = 0;
-	while ((c = getopt(argc, argv, "dip:r:stu:vVSQ")) != EOF) {
+	while (1) {
+	  c = getopt(argc, argv, "dip:r:stu:vVSQ");
+	  if (c == EOF)
+	    break;
 	  switch (c) {
 	  case 'd':
 	    ++debug;
@@ -208,7 +209,8 @@ main(argc, argv)
 	    port = serv->s_port;
 
 	  if (host == NULL) {
-	    if (((host = getzenv("MAILSERVER")) == NULL || *host == '\n')
+	    host = getzenv("MAILSERVER");
+	    if ((host == NULL || *host == '\n')
 		&& (host = whathost(path)) == NULL) {
 	      if (status > 0) {
 		host = "127.0.0.1"; /* "localhost" */
@@ -223,10 +225,13 @@ main(argc, argv)
 	      }
 	    }
 	  }
-	  if ((hp = gethostbyname(host)) == NULL) {
-	    if ((naddr.s_addr = inet_addr(host)) != -1) {
+	  hp = gethostbyname(host);
+	  if (hp == NULL) {
+	    naddr.s_addr = inet_addr(host);
+	    if (naddr.s_addr != -1) {
 	      naddr.s_addr = ntohl(naddr.s_addr);
-	      if ((hp = gethostbyaddr((void*)&naddr, sizeof(naddr), AF_INET)) == NULL){
+	      hp = gethostbyaddr((void*)&naddr, sizeof(naddr), AF_INET);
+	      if (hp == NULL){
 		hp = &he;
 		he.h_name = host;
 		he.h_length = 4;
@@ -250,7 +255,8 @@ main(argc, argv)
 	    exit(0);
 	}
 	/* try grabbing a port */
-	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	fd = socket(AF_INET, SOCK_STREAM, 0);
+	if (fd < 0) {
 	  fprintf(stderr, "%s: ", progname);
 	  perror("socket");
 	  exit(EX_UNAVAILABLE);
@@ -367,7 +373,7 @@ main(argc, argv)
 
 void
 docat(file, fd)
-	char *file;
+	const char *file;
 	int fd;
 {
 	char buf[BUFSIZ];
@@ -405,9 +411,10 @@ checkrouter()
 	if (postoffice == NULL)
 	  return;
 	sprintf(path, "%s/%s", postoffice, ROUTERDIR);
-	if ((dirp = opendir(path)) == NULL) {
-	  fprintf(stderr, "%s: opendir(%s):", progname, path);
-	  perror(" ");
+	dirp = opendir(path);
+	if (dirp == NULL) {
+	  fprintf(stderr, "%s: opendir(%s): %s\n",
+		  progname, path, strerror(errno));
 	  return;
 	}
 	for (dp = readdir(dirp), n = 0; dp != NULL; dp = readdir(dirp)) {
@@ -435,8 +442,7 @@ checkrouter()
 	     then the router dumped core... so let'em know about it. */
 	  sprintf(path, "%s/%s/core",postoffice,ROUTERDIR);
 	  if (fstat(FILENO(fp), &pidbuf) < 0) {
-	    fprintf(stderr, "\n%s: ", progname);
-	    perror("fstat: ");
+	    fprintf(stderr, "\n%s: fstat: %s", progname, strerror(errno));
 	  } else if (stat(path, &corebuf) == 0 && pidbuf.st_mtime < corebuf.st_mtime)
 	    printf("core dumped\n");
 	  else
@@ -459,9 +465,10 @@ checkrouter()
 	}
 
 	sprintf(path, "%s/%s", postoffice, DEFERREDDIR);
-	if ((dirp = opendir(path)) == NULL) {
-	  fprintf(stderr, "%s: opendir(%s):", progname, path);
-	  perror(" ");
+	dirp = opendir(path);
+	if (dirp == NULL) {
+	  fprintf(stderr, "%s: opendir(%s): %s",
+		  progname, path, strerror(errno));
 	  return;
 	}
 	for (dp = readdir(dirp), n = 0; dp != NULL; dp = readdir(dirp)) {
@@ -481,20 +488,21 @@ checkrouter()
 	  printf("%d message%s deferred\n", n, n != 1 ? "s" : "");
 }
 
-
+int countfiles __((const char *));
 int countfiles(dirpath)
-char *dirpath;
+const char *dirpath;
 {
-	char path[512];
+	char dpath[512];
 
 	struct dirent *dp;
 	DIR *dirp;
 	int n = 0;
 
-	if ((dirp = opendir(dirpath)) == NULL) {
-	  fprintf(stderr, "%s: opendir(%s):", progname, path);
-	  perror(" ");
-	  return;
+	dirp = opendir(dirpath);
+	if (dirp == NULL) {
+	  fprintf(stderr, "%s: opendir(%s): %s\n",
+		  progname, dpath, strerror(errno));
+	  return -1;
 	}
 	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
 	  if (dp->d_name[0] == '.' &&
@@ -509,12 +517,12 @@ char *dirpath;
 	    if (dp->d_name[0] >= 'A' && dp->d_name[0] <= 'Z' &&
 		dp->d_name[1] == 0) {
 	      struct stat stbuf;
-	      sprintf(path, "%s/%s", dirpath, dp->d_name);
-	      if (lstat(path,&stbuf) != 0 ||
+	      sprintf(dpath, "%s/%s", dirpath, dp->d_name);
+	      if (lstat(dpath,&stbuf) != 0 ||
 		  !S_ISDIR(stbuf.st_mode)) {
 		++othern;
 	      } else {
-		n += countfiles(path);
+		n += countfiles(dpath);
 	      }
 	    } else {
 	      ++othern;
@@ -539,8 +547,6 @@ checkscheduler()
 {
 	int pid, n, r;
 	FILE *fp;
-	struct dirent *dp;
-	DIR *dirp;
 
 	if (postoffice == NULL)
 	  return;
@@ -587,14 +593,14 @@ checkscheduler()
 }
 
 int
-isalive(pidfile, pidp, fpp)
-	char *pidfile;
+isalive(pidfil, pidp, fpp)
+	const char *pidfil;
 	int *pidp;
 	FILE **fpp;
 {
 	if (postoffice == NULL)
 		return 0;
-	sprintf(path, "%s/%s", postoffice, pidfile);
+	sprintf(path, "%s/%s", postoffice, pidfil);
 	
 	if ((*fpp = fopen(path, "r")) == NULL) {
 	  /* fprintf(stderr, "%s: cannot open %s (%s)\n",
@@ -625,7 +631,7 @@ isalive(pidfile, pidp, fpp)
 	{ char *s; if ((s = strchr(buf,'\n'))) *s = '\0'; }
 				
 
-char	*names[SIZE_L+2];
+const char *names[SIZE_L+2];
 
 #define	L_VERTEX	SIZE_L
 #define L_END		SIZE_L+1
@@ -633,6 +639,7 @@ struct sptree *spt_ids[SIZE_L+2];
 
 #define	EQNSTR(a,b)	(!strncmp(a,b,strlen(b)))
 
+extern int parse __((FILE *));
 int
 parse(fp)
 	FILE *fp;
@@ -679,7 +686,7 @@ parse(fp)
 	  if (fgets(buf, bufsize-1, fp) == NULL)
 	    break;
 
-	  switch (list) {
+	  switch ((int)list) {
 	  case L_CTLFILE:
 	    /* decid:\tfile\tnaddr; off1[,off2,...][\t#message] */
 	    if (!isdigit(buf[0])) {
@@ -704,7 +711,7 @@ parse(fp)
 	    while (isascii(*cp) && !isspace(*cp))
 	      ++cp;
 	    *cp++ = '\0';
-	    spl = sp_lookup(symbol((u_char *)ocp), spt_ids[L_CTLFILE]);
+	    spl = sp_lookup(symbol(ocp), spt_ids[L_CTLFILE]);
 	    if (spl == NULL || (cfp = (struct ctlfile *)spl->data) == NULL) {
 	      cfp = (struct ctlfile *)emalloc(sizeof (struct ctlfile));
 	      memset((void*)cfp,0,sizeof(struct ctlfile));
@@ -717,13 +724,12 @@ parse(fp)
 	      cfp->id = 0;
 	      cfp->mid = strsave(ocp);
 	      cfp->mark = 0;
-	      sp_install(symbol((u_char *)ocp),
-			 (u_char *)cfp, 0, spt_ids[L_CTLFILE]);
+	      sp_install(symbol(ocp), (void *)cfp, 0, spt_ids[L_CTLFILE]);
 	    }
-	    while (isascii(*cp) && isspace(*cp))
+	    while (*cp == ' ' || *cp == '\t')
 	      ++cp;
 	    ocp = cp;
-	    while (isascii(*cp) && isdigit(*cp))
+	    while ('0' <= *cp && *cp <= '9')
 	      ++cp;
 	    *cp++ = '\0';
 	    if ((i = atoi(ocp)) < 1) {
@@ -763,7 +769,7 @@ parse(fp)
 	    v->orig[L_CTLFILE] = v->orig[L_CHANNEL] = v->orig[L_HOST] = NULL;
 	    v->next[L_CHANNEL] = v->next[L_HOST] = NULL;
 	    v->prev[L_CHANNEL] = v->prev[L_HOST] = NULL;
-	    sp_install(key, (u_char *)v, 0, spt_ids[L_VERTEX]);
+	    sp_install(key, (void *)v, 0, spt_ids[L_VERTEX]);
 	    break;
 	  case L_CHANNEL:
 	    /* (channel|host):\tdecid[>decid...] */
@@ -791,16 +797,16 @@ parse(fp)
 	    *cp++ = '\0';
 
 	    /* Look for channel/host identifier splay-tree */
-	    spl = sp_lookup(symbol((u_char *)buf), spt_ids[list]);
+	    spl = sp_lookup(symbol(buf), spt_ids[list]);
 	    if (spl == NULL || (w = (struct web *)spl->data) == NULL) {
 	      w = (struct web *)emalloc(sizeof (struct web));
 	      memset((void*)w,0,sizeof(struct web));
 	      w->name = strsave(buf);
 	      w->kids = 0;
 	      w->link = w->lastlink = NULL;
-	      sp_install(symbol((u_char *)buf), (u_char *)w, 0, spt_ids[list]);
+	      sp_install(symbol(buf), (void *)w, 0, spt_ids[list]);
 	    }
-	    while (isascii(*cp) && isspace(*cp))
+	    while (*cp == ' ' || *cp == '\t')
 	      ++cp;
 
 	    /* Pick each vertex reference */
@@ -835,6 +841,7 @@ parse(fp)
 
 static int r_i;
 
+extern int repscan __((struct spblk *));
 int
 repscan(spl)
 	struct spblk *spl;
@@ -923,10 +930,11 @@ repscan(spl)
 	return 0;
 }
 
+void query2 __((FILE *));
 void query2(fp)
 	FILE *fp;
 {
-	int rc, len;
+	int  len;
 	char buf[512];
 
 	if (schedq) {
