@@ -19,6 +19,7 @@
 #include "ta.h"
 
 #include "prototypes.h"
+#include "libz.h"
 
 extern char *rfc822date __((time_t *));
 
@@ -41,18 +42,20 @@ extern int never_full_content; /* at conf.c */
  */
 
 struct not {
-	char  *not;
-	char  *orcpt;
-	char  *notify;
-	int    notifyflgs;
-	char  *rcpntp;
-	time_t tstamp;
+	char	    *not;
+	const char  *message;
+	const char  *orcpt;
+	const char  *notify;
+	int          notifyflgs;
+	char	    *rcpntp;
+	time_t       tstamp;
 };
 
+static void decodeXtext __((FILE *, const char *));
 static void
 decodeXtext(fp,xtext)
 	FILE *fp;
-	char *xtext;
+	const char *xtext;
 {
 	for (;*xtext;++xtext) {
 	  if (*xtext == '+') {
@@ -66,43 +69,55 @@ decodeXtext(fp,xtext)
 	}
 }
 
+static void scnotaryreport __((FILE *, struct not *, int *, int));
+
 static void	/* There is notaryreport() on transporters also.. */
-scnotaryreport(errfp,notary,haserrsp)
+scnotaryreport(errfp,notary,haserrsp,notifyrespectflg)
 	FILE *errfp;
 	struct not *notary;
 	int *haserrsp;
+	int notifyrespectflg;
 {
-	char *rcpt, *action, *status, *diagnostic, *wtt;
-	char const *typetag;
-	char *channel;
+	char *rcpt, *action, *status, *diagstr, *wtt;
+	char *cp;
+	const char *typetag;
 
 	static char const *type_rfc   = "RFC822";
 	static char const *type_local = "X-LOCAL";
 
-	/* NOTARY: addres / action / status / diagnostic / wtt */
+	/* NOTARY: addres / action / status / diagstr / wtt */
 
 	if (notary == NULL) {
 	  return; /* XX: ?? call into here should not happen.. */
 	}
 
+	if ((notary->notifyflgs & NOT_NEVER) && (notifyrespectflg >= 0))
+	  return; /* We ignore this recipient, because so has been
+		     asked, and this is not a double-fault... */
+
 	rcpt     = notary->not;
 	if (*rcpt == 0)
 	  return;	/* XX: ?? eh, well.. we should not be called.. */
 
-	action   = strchr(rcpt,'\001');
-	*action++ = 0;
-	status   = strchr(action,'\001');
-	*status++ = 0;
-	diagnostic = strchr(status,'\001');
-	if (diagnostic == NULL) {
-	  diagnostic = status;
+	cp     = strchr(rcpt,'\001');
+	*cp++  = 0;
+	action = cp;
+	cp     = strchr(action,'\001');
+	*cp++  = 0;
+	status = cp;
+	cp     = strchr(status,'\001');
+	diagstr = cp;
+	if (cp == NULL) {
+	  diagstr = status;
 	  status = NULL;
-	  wtt = NULL;
+	  wtt    = NULL;
 	} else {
-	  *diagnostic++ = 0;
-	  wtt      = strchr(diagnostic,'\001');
-	  if (wtt) /* Uhh... Mal-formed input from the transporters ? */
-	    *wtt++    = 0;
+	  *cp++  = 0;
+	  diagstr = cp;
+	  cp    = strchr(diagstr,'\001');
+	  if (cp) /* Uhh... Mal-formed input from the transporters ? */
+	    *cp++    = 0;
+	  wtt = cp;
 	}
 
 	if (*rcpt == '\003' /* XX: MAGIC! */) {
@@ -141,7 +156,7 @@ scnotaryreport(errfp,notary,haserrsp)
 	    *haserrsp = 1;
 	  fprintf(errfp, "Status: %s\n", status);
 	}
-	fprintf(errfp, "Diagnostic-Code: %s\n", diagnostic);
+	fprintf(errfp, "Diagnostic-Code: %s\n", diagstr);
 	if (wtt && wtt[0] != 0)
 	  fprintf(errfp, "Remote-MTA: %s\n", wtt);
 	if (notary->tstamp != 0)
@@ -150,7 +165,7 @@ scnotaryreport(errfp,notary,haserrsp)
 	action[-1] = '\001';
 	if (status)
 	  status[-1] = '\001';
-	diagnostic[-1] = '\001';
+	diagstr[-1] = '\001';
 	if (wtt)
 	  wtt[-1]    = '\001';
 }
@@ -162,10 +177,10 @@ void
 msgerror(vp, offset, message)
 	struct vertex *vp;
 	long offset;
-	char *message;
+	const char *message;
 {
 	FILE *fp;
-	char *notary = "";
+	const char *notary = "";
 	char path[128];
 
 	if (vp->notary) notary = vp->notary;
@@ -177,7 +192,8 @@ msgerror(vp, offset, message)
 	}
 
 	/* exclusive access required, but we're the only scheduler... */
-	if ((fp = fopen(path, "a")) == NULL) {
+	fp = fopen(path, "a");
+	if (fp == NULL) {
 	  fprintf(stderr,
 		  "Cannot open control file %s to deposit", vp->cfp->mid);
 	  fprintf(stderr,
@@ -197,15 +213,17 @@ msgerror(vp, offset, message)
 	fclose(fp);
 }
 
+void printenvaddr __((FILE *, const char *));
 void printenvaddr(fp,addr)
 FILE *fp;
-char *addr;
+const char *addr;
 {
-	char *s;
+	const char *s;
+
 	fprintf(fp, "todsn ORCPT=rfc822;");
 	s = addr;
-	for  (;*s;++s) {
-	  u_char c = *s;
+	for  (; *s; ++s) {
+	  char c = (*s) & 0xFF;
 	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
 	    putc(c,fp);
 	  else
@@ -217,6 +235,7 @@ char *addr;
 
 /* Pick recipient address from the input line.
    EXTREMELY Simple minded parsing.. */
+static void pick_env_addr __((char *, FILE *));
 static void pick_env_addr(buf,mfp)
 char *buf;
 FILE *mfp;
@@ -244,16 +263,18 @@ FILE *mfp;
 }
 
 
+static void writeheader __((FILE *, const char *, int *, const char *, const char *));
+
 static void
 writeheader(errfp, eaddr, no_error_reportp, deliveryform, boundary)
-     FILE *errfp;
-     char *eaddr;
-     int *no_error_reportp;
-     char *deliveryform;
-     char *boundary;
+	FILE *errfp;
+	const char *eaddr;
+	int *no_error_reportp;
+	const char *deliveryform;
+	const char *boundary;
 {
 	FILE *fp;
-	char path[MAXPATHLEN], mpath[MAXPATHLEN];
+	char path[MAXPATHLEN];
 	char buf[BUFSIZ];
 
 	if (*eaddr == '|' /* Nobody's address starts with a pipe ? */ ||
@@ -267,7 +288,8 @@ writeheader(errfp, eaddr, no_error_reportp, deliveryform, boundary)
 	  printenvaddr(errfp,eaddr);
 	sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
 		deliveryform ? deliveryform : "delivery");
-	if ((fp = fopen(path, "r")) != NULL) {
+	fp = fopen(path, "r");
+	if (fp != NULL) {
 	  int inhdr = 1, hadsubj =0;
 	  buf[sizeof(buf)-1] = 0;
 	  while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
@@ -334,10 +356,9 @@ reporterrs(cfpi)
 	int i, n, wroteheader, byteidx, headeridx = -1, drptidx, fd;
 	long *lp;
 	time_t tstamp;
-	char *midbuf, *cp, *s, *eaddr;
+	char *midbuf, *cp, *eaddr;
 	char *deliveryform;
 	FILE *errfp, *fp;
-	char *boundarystr;
 	char *notary;
 	struct not *notaries = NULL;
 	char *envid;
@@ -350,6 +371,7 @@ reporterrs(cfpi)
 	int lastoffset;
 	char path[MAXPATHLEN], mpath[MAXPATHLEN];
 	int mypid;
+	char boundarystr[400];
 
 	if (cfpi->haderror == 0)
 		return;
@@ -357,13 +379,11 @@ reporterrs(cfpi)
 		return;
 	no_error_report = cfpi->iserrmesg;
 
-	boundarystr = (char*)buildboundary("dlvryreport");
-
-	eaddr = cfpi->erroraddr;
+	eaddr        = cfpi->erroraddr;
 	deliveryform = cfpi->deliveryform;
-	envid = cfpi->envid;
+	envid        = cfpi->envid;
 	/* re-read the control file to get some unstashed information */
-	midbuf = cfpi->mid;
+	midbuf       = cfpi->mid;
 
 	/* exclusive access required, but we're the only scheduler... */
 	if (cfpi->dirind > 0)
@@ -371,7 +391,8 @@ reporterrs(cfpi)
 	else
 	  strcpy(mpath,cfpi->mid);
 
-	if ((fd = open(mpath, O_RDWR, 0)) < 0 ||
+	fd = open(mpath, O_RDWR, 0);
+	if (fd < 0 ||
 	    (cfp = slurp(fd, cfpi->id)) == NULL) {
 	  fprintf(stderr,
 		  "%s: unexpected absence of control file %s for error processing!\n",
@@ -381,14 +402,6 @@ reporterrs(cfpi)
 	  return;
 	}
 	lp = &cfp->offset[0];
-	if ((errfp = mail_open(MSG_RFC822)) == NULL) {
-	  fprintf(stderr,
-		  "%s: cannot open output mail file to return diagnostics! errno=%d, errstr='%s'\n",
-		  progname, errno, strerror(errno));
-	  close(cfp->fd);
-	  free_cfp_memory(cfp);
-	  return;
-	}
 	mypid = getpid();
 	if (cfp->mid) free(cfp->mid);
 	cfp->mid = midbuf; /* This is the original one! */
@@ -465,12 +478,13 @@ reporterrs(cfpi)
 	      notaries = (void*)erealloc((void*)notaries,
 					 sizeof(struct not)*(notaryspc+1));
 	    }
-	    notaries[notarycnt].not    = notary;
-	    notaries[notarycnt].orcpt  = NULL;
-	    notaries[notarycnt].notify = NULL;
+	    notaries[notarycnt].not     = notary;
+	    notaries[notarycnt].orcpt   = NULL;
+	    notaries[notarycnt].notify  = NULL;
+	    notaries[notarycnt].message = NULL;
 	    notaries[notarycnt].notifyflgs = 0;
-	    notaries[notarycnt].rcpntp = rcpntpointer;
-	    notaries[notarycnt].tstamp = tstamp;
+	    notaries[notarycnt].rcpntp  = rcpntpointer;
+	    notaries[notarycnt].tstamp  = tstamp;
 	    if (drptidx > 0) {
 	      char *d = cfp->contents + drptidx;
 	      while (*d) {
@@ -513,37 +527,71 @@ reporterrs(cfpi)
 		if (*d) *d++ = 0;
 		continue;
 	      }
-	      if (notaries[notarycnt].notifyflgs & NOT_NEVER)
-		continue; /* Don't report on this recipient! */
+	      /* Report conditional testing in the latter phases */
 	    }
+	    while (isascii(*cp) && isspace(*cp))
+	      ++cp;
+	    notaries[notarycnt].message = cp;
 	    ++notarycnt;
 	    notaries[notarycnt].not = NULL;
+	  } else {
+	    /* No notaries ?!  What ?  Store the CP pointer anyway.. */
+	    /* End of failure list processing */
 	  }
 
-	  if (!wroteheader) {
-	    writeheader(errfp, eaddr, &no_error_report, deliveryform, boundarystr);
-	    /* fprintf(errfp, "\n"); -- not needed ?? */
-	    wroteheader = 1;
+	  lockaddr(cfp->fd, NULL, *lp + 1, _CFTAG_NORMAL, _CFTAG_OK, mpath, "diagmsg", mypid);
+	}
+	/* End of all diagnostics message lines */
 
-	  } /* End of failure list processing */
+	if (notaries == NULL) {
+	  /* Oops!  No recipients on which to report anything ?! */
+	  close(cfp->fd);
+	  cfp->mid = NULL; /* we don't want to loose the original one! */
+	  free_cfp_memory(cfp);
+	  return;
+	}
 
-	  if (notaries == NULL) {
-	    /* Oops!  No recipients on which to report anything ?! */
-	    mail_abort(errfp);
-	    close(cfp->fd);
-	    cfp->mid = NULL; /* we don't want to loose the original one! */
-	    free_cfp_memory(cfp);
-	    return;
+	errfp = mail_open(MSG_RFC822);
+	if (errfp == NULL) {
+	  fprintf(stderr,
+		  "%s: cannot open output mail file to return diagnostics! errno=%d, errstr='%s'\n",
+		  progname, errno, strerror(errno));
+	  /* Release 'notaries' datasets */
+	  for (i = 0; i < notarycnt; ++i) {
+	    if (notaries[i].not != NULL)
+	      free(notaries[i].not);
 	  }
+	  free((void*)notaries);
+	  close(cfp->fd);
+	  free_cfp_memory(cfp);
+	  return;
+	}
 
-	  
+	{
+	  char *dom = mydomain(); /* transports/libta/buildbndry.c */
+	  char fname[20];
+	  struct stat stbuf;
+
+	  fstat(fileno(errfp),&stbuf);
+	  sprintf(fname,"%d",stbuf.st_ino);
+	  taspoolid(boundarystr, sizeof(boundarystr), stbuf.st_ctime, fname);
+	  strcat(boundarystr, "=_/");
+	  strcat(boundarystr, dom);
+	}
+
+	writeheader(errfp, eaddr, &no_error_report, deliveryform, boundarystr);
+
+	for (i = 0; i < notarycnt; ++i) {
 	  /* Scan to the start of the message text */
-	  while (isascii(*cp) && isspace(*cp))
-	    ++cp;
-	  fprintf(errfp, "<%s>: ", rcpntpointer);
-	  if (strchr(cp, '\r')) {
+	  const char *ccp, *s;
+	  if ((notaries[i].notifyflgs & NOT_NEVER) && (no_error_report >= 0))
+	    continue;
+	  /* Report is not outright rejected, or this is double fault */
+	  fprintf(errfp, "<%s>: ", notaries[i].rcpntp);
+	  ccp = notaries[i].message;
+	  if (strchr(ccp, '\r')) {
 	    fprintf(errfp, "...\\\n\t");
-	    for (s = cp; *s != '\0'; ++s) {
+	    for (s = ccp; *s != '\0'; ++s) {
 	      if (*s == '\r')
 		putc('\n', errfp), putc('\t', errfp);
 	      else
@@ -552,18 +600,8 @@ reporterrs(cfpi)
 	    putc('\n', errfp);
 	  } else
 	    fprintf(errfp, "%s\n", cp);
-	  lockaddr(cfp->fd, NULL, *lp + 1, _CFTAG_NORMAL, _CFTAG_OK, mpath, "diagmsg", mypid);
 	}
-	/* End of all diagnostics message lines */
 
-	if (!wroteheader) {
-	  writeheader(errfp, eaddr, &no_error_report, deliveryform, boundarystr);
-	  /* fprintf(errfp, "\n"); -- not needed ?? */
-	  wroteheader = 1;
-
-	} /* End of failure list processing */
-
-	
 	fprintf(errfp, "\n");
 	fprintf(errfp, "--%s\n", boundarystr);
 	fprintf(errfp, "Content-Type: message/delivery-status\n\n");
@@ -584,7 +622,7 @@ reporterrs(cfpi)
 
 	/* Now scan 'em all again for IETF-NOTARY */
 	for (i = 0; i < notarycnt; ++i) {
-	  scnotaryreport(errfp, &notaries[i],&has_errors);
+	  scnotaryreport(errfp, &notaries[i],&has_errors,no_error_report);
 	  if (notaries[i].not != NULL)
 	    free(notaries[i].not);
 	}
@@ -601,7 +639,8 @@ reporterrs(cfpi)
 	  sprintf(path, "../%s/%s",
 		  QUEUEDIR, cfpi->mid);
 
-	if ((fp = fopen(path, "r")) != NULL) {
+	fp = fopen(path, "r");
+	if (fp != NULL) {
 
 	  char buf[BUFSIZ];
 
@@ -617,7 +656,7 @@ reporterrs(cfpi)
 	    /* Scan the input, and drop off the Zmailer
 	       envelope headers */
 	    while (fgets(buf,sizeof(buf),fp) != NULL) {
-	      char *s = buf;
+	      const char *s = buf;
 	      while (*s && *s != ':' && *s != ' ' && *s != '\t') ++s;
 	      if (*s == ':') break;
 	      *buf = 0;
@@ -632,8 +671,10 @@ reporterrs(cfpi)
 	      fseek(fp, (off_t)0, SEEK_SET);
 	    }
 	  }
-	  if (has_errors && !never_full_content &&
-	      (!cfp->dsnretmode || CISTREQN(cfp->dsnretmode,"FULL",4))) {
+	  if (has_errors &&
+	      ((no_error_report < 0) ||
+	       (!never_full_content &&
+		(!cfp->dsnretmode || CISTREQN(cfp->dsnretmode,"FULL",4))))) {
 	    /* Copy out the rest (=body) with somewhat more efficient method */
 
 	    while ((n = fread(buf, sizeof buf[0], sizeof buf, fp)) > 0)
