@@ -28,17 +28,20 @@ static void __free(ptr,file,line) void *ptr; char*file;int line; { fprintf(stder
 #endif
 
 
+#if 0
 #ifndef strchr /* IBM AIX is a bit peculiar.. these are macroes! */
 extern char *strcpy(), *strncpy(), *strcat();
 extern char *strchr(), *strrchr();
 #endif
+#endif
 
-
+/* lib/ranny.c */
+extern u_int ranny __((u_int m));
 
 /* agenda.c */
 extern time_t qipcretry;
 extern int doagenda __((void));
-extern void turnme __((char *));
+extern void turnme __((const char *));
 
 /* conf.c */
 extern char *qlogdir;
@@ -51,10 +54,8 @@ extern int  nobody;
 extern int  sweepinterval;
 
 /* msgerror.c */
-extern void msgerror __((struct vertex *vp, long offset, char *message));
+extern void msgerror __((struct vertex *vp, long offset, const char *message));
 extern void reporterrs __((struct ctlfile *cfpi));
-extern char *mail_alloc __((u_int n));
-extern int  mail_free __((char *s));
 
 /* pipes.c */
 extern int  pipes_create         __((int *tochild, int *fromchild));
@@ -70,8 +71,8 @@ extern struct config_entry *rrcf_head;
 extern struct config_entry *default_entry;
 extern void   defaultconfigentry __((struct config_entry *ce, struct config_entry *defaults));
 extern void   vtxprint __((struct vertex *vp));
-extern struct config_entry *readconfig __((char *file));
-extern struct config_entry *rereadconfig __((struct config_entry *head, char *file));
+extern struct config_entry *readconfig __((const char *file));
+extern struct config_entry *rereadconfig __((struct config_entry *head, const char *file));
 
 /* resources.c */
 extern int  resources_query_nofiles  __((void));
@@ -80,13 +81,12 @@ extern void resources_limit_nofiles __((int nfiles));
 extern int  resources_query_pipesize __((int fildes));
 
 /* scheduler.c */
-extern int  transportmaxnofiles;
-extern char *progname;
-extern char *postoffice;
-extern char *rendezvous;
-extern char *pidfile;
-extern char *mailshare;
-extern char *log;
+extern int         transportmaxnofiles;
+extern const char *progname;
+extern const char *rendezvous;
+extern const char *pidfile;
+extern const char *mailshare;
+extern const char *log;
 extern int  hungry_childs;
 extern int  global_maxkids;
 extern int  verbose;
@@ -98,20 +98,19 @@ extern int  vtxredo __((struct spblk *spl));
 extern char *timestring __((void));
 extern time_t now;
 extern FILE *statuslog;
-extern void resync_file __((char *filename));
+extern void resync_file __((const char *filename));
 extern int thread_count_recipients __((void));
 extern time_t mytime __((time_t *));
-extern int dq_insert __((void*, long, char*, int));
+extern int dq_insert __((void*, long, const char*, int));
 extern int in_dirscanqueue __((void *, long));
-extern char *cfpdirname __((int));
+extern const char *cfpdirname __((int));
 
 /* threads.c */
 extern struct thread *thread_head, *thread_tail;
 extern struct threadgroup *create_threadgroup __((struct config_entry *cep, struct web *wc, struct web *wh, int withhost, void (*ce_fillin)__((struct threadgroup *, struct config_entry *)) ));
 extern void  delete_threadgroup __((struct threadgroup *thgp));
 extern void  thread_linkin __((struct vertex *cp, struct config_entry *cep, int cfgid, void (*ce_fillin)__((struct threadgroup *, struct config_entry *)) ));
-extern char  *strlower __((char *s));
-extern struct web *web_findcreate __((int flag, u_char *s));
+extern struct web *web_findcreate __((int flag, const char *s));
 extern void        unweb __((int flag, struct web *wp));
 extern int   thread_start __((struct thread *thr));
 extern int   idle_cleanup __((void));
@@ -129,6 +128,7 @@ extern void feed_child      __((struct procinfo *cpidp));
 extern int flush_child      __((struct procinfo *cpidp));
 extern void idle_child      __((struct procinfo *cpidp));
 extern int  start_child     __((struct vertex *vhead, struct web *channel, struct web *host));
+extern void shutdown_kids   __(( void ));
 extern RETSIGTYPE sig_chld __((int sig));
 extern int mux __((time_t timeout));
 extern void queryipccheck __((void));
@@ -141,9 +141,19 @@ extern int system __((char*));
 extern void update __((int, char *));
 extern void unctlfile __((struct ctlfile *cfp, int no_unlink));
 extern void unvertex __((struct vertex *, int justfree, int ok));
-extern void deletemsg __((char *, struct ctlfile *));
+extern void deletemsg __((const char *, struct ctlfile *));
 extern char *saytime __((long, char *, int));
 extern void expire __((struct vertex *, int));
 
+/* wantconn.c */
+extern int wantconn __(( int sock, const char *prgname ));
+
+extern int isalive __(( const char *pidfil, int *pidp, FILE **fpp ));
+extern void docat __(( const char *file, int fd ));
+extern void printaddrs __(( struct vertex *v ));
+extern void checkrouter __(( void ));
+extern void checkscheduler __(( void ));
+extern void report __(( FILE *fp ));
+
 /* Transport library */
 /* extern int lockaddr __((int, char *, long, int, int)); */
