@@ -12,8 +12,10 @@
 #include "scheduler.h"
 #include <fcntl.h>
 #include <sys/file.h>
-#include "prototypes.h"
+#include <sys/stat.h>
 
+#include "prototypes.h"
+#include "libz.h"
 
 /* define NO_VERBOSE_MAILQ, if you don't want to output potentially HUGE
    mailq report -- necessary at some sites, while definitely undesirable
@@ -23,10 +25,9 @@ extern int mailq_Q_mode; /* Argument 'Q' means: output only 'mailq -Q' -data.. *
 
 extern int errno;
 
-static int qpctlfile();
-static int qpchannel();
-static int qphost();
-extern char *saytime();
+static int qpctlfile __((struct spblk *spl));
+static int qpchannel __((struct spblk *spl));
+static int qphost __((struct spblk *spl));
 
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
@@ -102,7 +103,7 @@ qprint(fd)
 
 	/* XX: I suppose we don't really need to do this if we use TCP. */
 	if (rendezvous != NULL) {
-	  time_t now = mytime(NULL) - 1;
+	  now = mytime(NULL) - 1;
 #ifdef	HAVE_UTIME
 	  tvp.actime = tvp.modtime = now;
 	  utime(rendezvous, &tvp);
@@ -114,7 +115,6 @@ qprint(fd)
 	}
 }
 
-static int qpctlfile __((struct spblk *spl));
 static int qpctlfile(spl)
 	struct spblk *spl;
 {
@@ -194,7 +194,6 @@ static int qpctlfile(spl)
 	  return 0;
 	}
 
-static int qpchannel __((struct spblk *spl));
 static int qpchannel(spl)
 	struct spblk *spl;
 {
@@ -207,13 +206,12 @@ static int qpchannel(spl)
 	    if (vp->qid)
 	      fprintf(qpfp, ">%d", vp->qid);
 	    else
-	      fprintf(qpfp, ">999%04d", vp->cfp->id);
+	      fprintf(qpfp, ">999%04d", (int)vp->cfp->id);
 	  fprintf(qpfp, "\n");
 	}
 	return 0;
 }
 
-static int qphost __((struct spblk *spl));
 static int qphost(spl)
 	struct spblk *spl;
 {
@@ -226,7 +224,7 @@ static int qphost(spl)
 	    if (vp->qid)
 	      fprintf(qpfp, ">%d", vp->qid);
 	    else
-	      fprintf(qpfp, ">999%04d", vp->cfp->id);
+	      fprintf(qpfp, ">999%04d", (int)vp->cfp->id);
 	  fprintf(qpfp, "\n");
 	}
 	return 0;
