@@ -16,6 +16,8 @@
 #include "prototypes.h"
 #include "mail.h"
 
+#include "libz.h"
+
 static void celink __((struct config_entry *, struct config_entry **, struct config_entry **));
 static int readtoken __((FILE *fp, char *buf, int buflen, int *linenump));
 static u_int parse_intvl __((char *string));
@@ -34,25 +36,23 @@ static int rc_user     __((char *key, char *arg, struct config_entry *ce));
 static int rc_skew     __((char *key, char *arg, struct config_entry *ce));
 static int rc_bychannel __((char *key, char *arg, struct config_entry *ce));
 static int rc_ageorder __((char *key, char *arg, struct config_entry *ce));
+static int rc_queueonly __((char *key, char *arg, struct config_entry *ce));
 static int rc_deliveryform __((char *key, char *arg, struct config_entry *ce));
 static int rc_overfeed __((char *key, char *arg, struct config_entry *ce));
 
-extern char *strerror();
 extern int errno;
 
-static char *mailbin = NULL;
+static const char *mailbin = NULL;
 
 extern struct group *getgrnam();
 extern struct passwd *getpwnam();
 
-extern void die();
-
 struct config_entry *default_entry = NULL;
 struct config_entry *rrcf_head     = NULL;
 
 static struct rckeyword {
-	char	*name;
-	int	(*parsef)();
+	const char	*name;
+	int		(*parsef)();
 } rckeys[] = {
 {	"interval",		rc_interval	},	/* time */
 {	"idlemax",		rc_idlemax	},	/* time */
@@ -73,6 +73,7 @@ static struct rckeyword {
 {	"skew",			rc_skew		},	/* number */
 {	"bychannel",		rc_bychannel	},	/* boolean */
 {	"ageorder",		rc_ageorder	},	/* boolean */
+{	"queueonly",		rc_queueonly	},	/* boolean */
 {	"overfeed",		rc_overfeed	},	/* number */
 {	NULL,			0		}
 };
@@ -219,14 +220,13 @@ celink(ce, headp, tailp)
 
 struct config_entry *
 readconfig(file)
-	char *file;
+	const char *file;
 {
 	char *cp, *s, *a, line[BUFSIZ];
 	int errflag, n;
 	struct config_entry *ce, *head, *tail;
 	struct rckeyword *rckp;
 	struct vertex v;
-	struct threadgroup tg;
 	FILE *fp;
 	int linenum = 0;
 
@@ -256,8 +256,7 @@ readconfig(file)
 	      ce->channel = strsave(line);
 	      ce->host    = strsave("*");
 	    }
-	    if (line[0]=='*' && (strcmp(line,"*/*") == 0 ||
-				 line[1] == 0)) {
+	    if (strcmp(line,"*/*") == 0 || strcmp(line,"*") == 0) {
 	      /* The default entry.. */
 	      if (default_entry != NULL) {
 		if (ce->channel) free (ce->channel);
@@ -312,6 +311,7 @@ readconfig(file)
 	  celink(ce, &head, &tail);
 	fclose(fp);
 	if (verbose) {
+	  struct threadgroup tg;
 	  v.orig[L_CHANNEL] = v.orig[L_HOST] = NULL;
 	  v.thgrp = &tg;
 	  for (ce = head; ce != NULL; ce = ce->next) {
@@ -435,7 +435,7 @@ parse_intvl(string)
 struct config_entry *
 rereadconfig(head, file)
 	struct config_entry *head;
-	char *file;
+	const char *file;
 {
 	struct config_entry *ce, *nce, *head2;
 
@@ -499,8 +499,9 @@ static int rc_command(key, arg, ce)
 	if (*arg != '/') {
 
 	  if (mailbin == NULL)
-	    if ((mailbin = getzenv("MAILBIN")) == NULL)
-	      mailbin = MAILBIN;
+	    mailbin = getzenv("MAILBIN");
+	  if (mailbin == NULL)
+	    mailbin = MAILBIN;
 
 	  ce->command = emalloc(strlen(mailbin)+1+strlen(qdefaultdir)
 				+1+strlen(arg)+1);
@@ -730,3 +731,11 @@ static int rc_deliveryform(key, arg, ce)
 	ce->deliveryform = strsave(arg);
 	return 0;
 }
+
+static int rc_queueonly(key, arg, ce)
+	char *key, *arg;
+	struct config_entry *ce;
+{
+	ce->flags |= CFG_QUEUEONLY;
+	return 0;
+}
