@@ -23,6 +23,10 @@
 #include <string.h>
 #include "ta.h"
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
 #else /* not HAVE_DIRENT_H */
@@ -45,6 +49,10 @@
 
 #include "prototypes.h"
 #include "zsyslog.h"
+#include "libz.h"
+
+extern int optind;
+extern char *optarg;
 
 #ifndef	MAXNAMLEN /* POSIX.1 ... */
 #define MAXNAMLEN NAME_MAX
@@ -69,14 +77,14 @@ struct conshell *envarlist = NULL;
 #define TRANSPORTMAXNOFILES 32 /* Number of files a transporter may
 				  need open -- or any of its children.. */
 int	transportmaxnofiles = TRANSPORTMAXNOFILES; /* Default value */
-char	*progname;
-extern char *postoffice;	/* At  mail_open()  */
-char	*rendezvous;
-char	*pidfile = PID_SCHEDULER;
-char	*mailshare;
-char	*logfn;
-char	*statusfn;
-FILE    *statuslog = NULL;
+const char * progname;
+const char * postoffice;
+const char * rendezvous;
+const char * pidfile = PID_SCHEDULER;
+const char * mailshare;
+const char * logfn;
+const char * statusfn;
+FILE    * statuslog = NULL;
 int	slow_shutdown = 0;
 extern int readsockcnt; /* from transport.c: mux() */
  
@@ -102,13 +110,13 @@ int	hashlevels = 0;		/* How many levels of hashes are supported
 				   at the transport directory ?  (To speed
 				   up the directory processing, file opens,
 				   etc.. ) */
-char	*procselect = NULL;	/* Non-null defines  channel/host specifier
+char * procselect = NULL;	/* Non-null defines  channel/host specifier
 				   that is ALLOWED TO RUN, specifying this
 				   (by means of '-P chan/host' -option) will
 				   prevent running anything else, and also
 				   prevent running error processing, or
 				   job-specifier deletions. */
-char	*procselhost = NULL;	/* Just spliced out 'host'-part of the above */
+char * procselhost = NULL;	/* Just spliced out 'host'-part of the above */
 extern int forkrate_limit;	/* How many forks per second ? */
 
 static int vtxprep_skip      = 0;
@@ -122,16 +130,16 @@ static int newents_limit = 200;
 #include "memtypes.h"
 extern memtypes stickymem;
 
-static struct ctlfile *schedule __((int fd, char *file, struct config_entry *ce, long ino));
-static struct ctlfile *vtxprep __((struct ctlfile *, char *));
+static struct ctlfile *schedule __((int fd, const char *file, struct config_entry *ce, long ino));
+static struct ctlfile *vtxprep __((struct ctlfile *, const char *));
 static int  vtxmatch __((struct vertex *, struct config_entry *));
-static void link_in __((int flag, struct vertex *vp, u_char *s));
-static int  lockverify __((struct ctlfile *, char *));
-static int  globmatch   __((char *, char*));
-static void vtxdo   __((struct vertex *, struct config_entry *, char *));
+static void link_in __((int flag, struct vertex *vp, const char *s));
+static int  lockverify __((struct ctlfile *, const char *));
+static int  globmatch   __((const char *, const char*));
+static void vtxdo   __((struct vertex *, struct config_entry *, const char *));
 
-extern void  cfp_mksubdirs __((char *, char*));
-extern char *cfpdirname __((int));
+extern void  cfp_mksubdirs __((const char *, const char*));
+extern const char *cfpdirname __((int));
 
 static RETSIGTYPE sig_exit   __((int sig));
 static RETSIGTYPE sig_quit   __((int sig));
@@ -139,25 +147,19 @@ static RETSIGTYPE sig_alarm  __((int sig));
 static RETSIGTYPE sig_iot    __((int sig));
 static RETSIGTYPE sig_readcf __((int sig));
 
-extern char *Version;
-extern char *optarg;
-extern char *strerror __((int errno));
+extern char *strerror __((int err));
 
-static int timeserver_pid = 0;
-extern time_t mytime __((time_t *));
-static void init_timeserver __((void));
+static int    timeserver_pid = 0;
+extern time_t mytime          __((time_t *));
+static void   init_timeserver __((void));
 
 #ifdef HAVE_SELECT /* Well, not exactly kosher assumption.. */
 /* extern int gettimeofday __((struct timeval *,)); */
 #else
 extern time_t time __((time_t *));
 #endif
-extern u_char *rfc822date __((time_t *));
-extern void detach __((void));
-extern void die __((int rc, char *reason));
-extern int  killprevious __((int signal, char *pidfile));
-extern void prversion __((char *name));
 
+static int loginitsched __((int));
 static int loginitsched(sig)
 int sig;
 {
@@ -196,7 +198,7 @@ int sig;
 #endif
 	  setvbuf(statuslog, (char *)NULL, _IOLBF, 0);
 	}
-	SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE (*)()) loginitsched);
+	SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE(*)__((int))) loginitsched);
 	return 0;
 }
 
@@ -223,19 +225,20 @@ static int syncweb __((struct dirqueue *dq, struct config_entry *ce));
 int global_maxkids = 1000;
 time_t now;
 
+FILE * vfp_open __((struct ctlfile *));
 FILE *vfp_open(cfp)
 struct ctlfile *cfp;
 {
-  FILE *vfp;
+	FILE *vfp;
 
-  setreuid(0, cfp->uid);
-  vfp = fopen(cfp->vfpfn, "a+");
-  setreuid(0, 0);
-  if (!vfp) return NULL; /* Failure to open */
+	setreuid(0, cfp->uid);
+	vfp = fopen(cfp->vfpfn, "a+");
+	setreuid(0, 0);
+	if (!vfp) return NULL; /* Failure to open */
 
-  fseek(vfp, (off_t)0, SEEK_END);
-  setvbuf(vfp, (char *)NULL, _IOLBF, 0);   /* Set LINEBUFFERING */
-  return vfp;
+	fseek(vfp, (off_t)0, SEEK_END);
+	setvbuf(vfp, (char *)NULL, _IOLBF, 0);   /* Set LINEBUFFERING */
+	return vfp;
 }
 
 
@@ -245,7 +248,6 @@ struct ctlfile *cfp;
 struct spblk   *spl;
 {
 	struct vertex *vp, *nvp;
-	int resyncing = (spl != NULL);
 
 	/* Delete from the  spt_mesh[]  */
 
@@ -307,12 +309,14 @@ struct ctlfile *cfp;
 struct dirqueue dirqb;
 struct dirqueue *dirq = &dirqb;
 
-char **ArgvSave;
+const char **ArgvSave;
+
+extern int main __((int, const char **));
 
 int
 main(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	struct ctlfile *cfp;
 	char *config, *cp;
@@ -320,6 +324,11 @@ main(argc, argv)
 	long offout, offerr;
 	struct config_entry *tp;
 
+#ifdef HAVE_SETGROUPS
+	/* We null supplementary groups list entirely */
+	setgroups(0, NULL);
+#endif
+
 	freeze = 0;
 	/* setlinebuf(stderr);  -- no need for this ? */
 
@@ -349,7 +358,11 @@ main(argc, argv)
 	dlyverbose = 0;
 	syncstart = 0;
 	verbose = errflg = version = 0;
-	while ((c = getopt(argc, argv, "divE:f:Fl:HL:N:p:P:q:QR:SVW")) != EOF)
+	for (;;) {
+		c = getopt(argc, (char*const*)argv,
+			   "divE:f:Fl:HL:N:p:P:q:QR:SVW");
+		if (c == EOF)
+		  break;
 		switch (c) {
 		case 'f':	/* override default config file */
 			config = optarg;
@@ -387,7 +400,7 @@ main(argc, argv)
 				transportmaxnofiles = TRANSPORTMAXNOFILES;
 			break;
 		case 'p':
-			procselect = optarg;
+			procselect  = optarg;
 			procselhost = strchr(procselect,'/');
 			if (procselhost)
 			  *procselhost++ = 0;
@@ -436,26 +449,30 @@ main(argc, argv)
 			errflg++;
 			break;
 		}
+	}
+
 	if (errflg) {
 	  fprintf(stderr,
 		  "Usage: %s [-dHisvV -f configfile -L logfile -P postoffice -Q rendezvous]\n",
 		  progname);
 	  exit(128+errflg);
 	}
-	if ((mailshare = getzenv("MAILSHARE")) == NULL)
+	mailshare = getzenv("MAILSHARE");
+	if (mailshare == NULL)
 	  mailshare = MAILSHARE;
-	if ((cp = getzenv("LOGDIR")) != NULL)
+	cp = getzenv("LOGDIR");
+	if (cp != NULL)
 	  qlogdir = cp;
 	if (daemonflg && logfn == NULL) {
 	  logfn = emalloc(2 + (u_int)(strlen(qlogdir) + strlen(progname)));
-	  sprintf(logfn, "%s/%s", qlogdir, progname);
+	  sprintf((char*)logfn, "%s/%s", qlogdir, progname);
 	}
 	if (logfn != NULL) {
 	  /* loginit is a signal handler, so can't pass log */
 	  if (loginitsched(SIGHUP) < 0) /* do setlinebuf() there */
 	    die(1, "log initialization failure");
 	  /* close and reopen log files */
-	  SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE(*)())loginitsched);
+	  SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE(*)__((int))) loginitsched);
 	} else {
 	  SIGNAL_IGNORE(SIGHUP); /* no surprises please */
 	  setvbuf(stdout, (char *)NULL, _IOLBF, 0);
@@ -492,7 +509,8 @@ main(argc, argv)
 				       + strlen(qcf_suffix)));
 	  sprintf(config, "%s/%s.%s", mailshare, progname, qcf_suffix);
 	}
-	if ((tp = readconfig(config)) == NULL) {
+	tp = readconfig(config);
+	if (tp == NULL) {
 	  cp = emalloc(strlen(config)+50);
 	  sprintf(cp, "null control file, propably errors in it: %s", config);
 	  die(1, cp);
@@ -624,7 +642,6 @@ main(argc, argv)
 	  /* Submit one item from pre-scheduler-queue into the scheduler */
 	  if (dirq->wrksum > 100 ) {
 	    /* If more than 100 in queue, submit 20 in a burst.. */
-	    int i;
 	    for (i=0; i < 20; ++i)
 	      syncweb(dirq, tp);
 	  }
@@ -739,7 +756,7 @@ int
 dq_insert(DQ, ino, file, delay)
 	void *DQ;
 	long ino;
-	char *file;
+	const char *file;
 	int delay;
 {
 	struct stat stbuf;
@@ -777,7 +794,7 @@ dq_insert(DQ, ino, file, delay)
 	dsn->not_before = now + delay;
 	strcpy(dsn->name,file);
 
-	sp_install(ino,(u_char*)dsn,0,dirscan_mesh);
+	sp_install(ino, (void *)dsn, 0, dirscan_mesh);
 
 	/* Put the new entry into the normal queue, unless there
 	   is stuff at the delayed queue, OR the delay is positive! */
@@ -823,7 +840,6 @@ int in_dirscanqueue(DQ,ino)
 	void *DQ;
 	long ino;
 {
-	int i;
 	struct dirqueue *dq = DQ;
 
 	if (dq == NULL)
@@ -836,6 +852,7 @@ int in_dirscanqueue(DQ,ino)
 	return 0;
 }
 
+static int dq_ctimecompare __((const void *, const void *));
 static int dq_ctimecompare(b,a) /* we want oldest entry LAST */
 const void *a; const void *b;
 {
@@ -1045,12 +1062,12 @@ static int syncweb(dq, cehead)
 }
 
 void resync_file(file)
-	char *file;
+	const char *file;
 {
 	struct stat stbuf;
 	struct spblk *spl;
-	long ino, fd;
-	char *s;
+	long ino;
+	const char *s;
 
 	lstat(file,&stbuf);
 
@@ -1065,13 +1082,13 @@ void resync_file(file)
 	spl = sp_lookup((u_long)ino, spt_mesh[L_CTLFILE]);
 	if (spl == NULL) {
 	  if (!in_dirscanqueue(NULL,ino)) {
-	    printf("Resyncing file \"%s\" (ino=%d)",file,ino);
+	    printf("Resyncing file \"%s\" (ino=%ld)", file, ino);
 	    printf(" .. not in processing database\n");
 	  }
 	  /* Not (anymore) in processing! */
 	  return;
 	}
-	printf("Resyncing file \"%s\" (ino=%d)",file,ino);
+	printf("Resyncing file \"%s\" (ino=%d)", file, (int) ino);
 	printf(" .. in processing db\n");
 
 	/* cfp_free()->unvertex()->unctlfile() will do reinsertion */
@@ -1093,7 +1110,7 @@ void resync_file(file)
  */
 static struct ctlfile *schedule(fd, file, cehead, ino)
 	int fd;
-	char *file;
+	const char *file;
 	struct config_entry *cehead;
 	long ino;
 {
@@ -1125,7 +1142,7 @@ static struct ctlfile *schedule(fd, file, cehead, ino)
 	  cfp->contents = NULL;
 	}
 
-	sp_install(cfp->id, (u_char *)cfp, 0, spt_mesh[L_CTLFILE]);
+	sp_install(cfp->id, (void *)cfp, 0, spt_mesh[L_CTLFILE]);
 	++global_wrkcnt;
 	return cfp;
 }
@@ -1158,7 +1175,7 @@ slurp(fd, ino)
 	  close(fd);
 	  return NULL;	/* XX: give error */
 	}
-	contents = emalloc(stbuf.st_size+1);
+	contents = emalloc((int)stbuf.st_size + 1);
 	if (eread(fd, contents, stbuf.st_size) != stbuf.st_size) { /* slurp */
 	  close(fd);
 	  free(contents); /* Failed to read ?!?! */
@@ -1261,6 +1278,7 @@ struct offsort {
 
 static char *bcp;
 
+static int bcfcn __((const void *, const void *));
 static int bcfcn(a, b)
      const void *a, *b;
 {
@@ -1272,7 +1290,7 @@ static int bcfcn(a, b)
 static int
 lockverify(cfp,cp)		/* Return 1 when lock process does not exist */
 	struct ctlfile *cfp;	/* Call only when the lock is marked active! */
-	char *cp;
+	const char *cp;
 {
 	char	lockbuf[1+_CFTAG_RCPTPIDSIZE];
 	int	lockpid;
@@ -1311,16 +1329,15 @@ lockverify(cfp,cp)		/* Return 1 when lock process does not exist */
  */
 static struct ctlfile *vtxprep(cfp, file)
 	struct ctlfile *cfp;
-	char *file;
+	const char *file;
 {
 	register int i, opcnt;
 	register long *lp;
 	int svn;
-	char *cp, *cp2, *channel, *host, *l_channel, *l_host;
+	char *cp, *channel, *host, *l_channel, *l_host;
 	char *echannel, *ehost, *l_echannel, *l_ehost, mfpath[100], flagstr[2];
 	char *latest_sender = NULL;
 	char *senderchannel = NULL;
-	struct spblk *spl;
 	struct vertex *vp, *pvp, **pvpp, *head;
 	struct stat stbuf;
 	struct offsort *offarr;
@@ -1652,7 +1669,8 @@ static struct ctlfile *vtxprep(cfp, file)
 	for (i = 0; i < opcnt; ++i) {
 	  channel = bcp + offarr[i].offset /* - 2 */;
 	  while (*channel == ' ') ++channel; /* Skip possible space(s) */
-	  if ((echannel = strchr(channel, ' ')) == NULL) /* error! */
+	  echannel = strchr(channel, ' ');
+	  if (echannel == NULL) /* error! */
 	    continue;
 	  *echannel = '\0';
 	  host = echannel + 1;
@@ -1727,8 +1745,8 @@ static struct ctlfile *vtxprep(cfp, file)
 	      *pvpp = vp;
 	      pvpp = &vp->next[L_CTLFILE];
 	      pvp = vp;
-	      link_in(L_HOST,    vp, (u_char *)l_host);
-	      link_in(L_CHANNEL, vp, (u_char *)l_channel);
+	      link_in(L_HOST,    vp, l_host);
+	      link_in(L_CHANNEL, vp, l_channel);
 	    }
 	    /* create a new vertex node */
 	    svn = i;
@@ -1773,8 +1791,8 @@ static struct ctlfile *vtxprep(cfp, file)
 	  *pvpp = vp;
 	  pvpp = &vp->next[L_CTLFILE];
 	  pvp = vp;
-	  link_in(L_HOST, vp, (u_char *)host);
-	  link_in(L_CHANNEL, vp, (u_char *)channel);
+	  link_in(L_HOST, vp, host);
+	  link_in(L_CHANNEL, vp, channel);
 	}
 
 	*l_echannel = *l_ehost = ' ';
@@ -1827,7 +1845,7 @@ static int vtxmatch(vp, tp)
 	return 1;
 }
 
-
+static void ce_fillin __((struct threadgroup *, struct config_entry *));
 static void ce_fillin(thg,cep)
 	struct threadgroup *thg;
 	struct config_entry *cep;
@@ -1882,7 +1900,7 @@ static void ce_fillin(thg,cep)
 static void vtxdo(vp, cehead, path)
 	struct vertex *vp;
 	struct config_entry *cehead;
-	char *path;
+	const char *path;
 {
 	struct config_entry *tp;
 	int n;
@@ -1940,11 +1958,46 @@ int vtxredo(spl)
 
 /* Shell-GLOB-style matching */
 static int globmatch(pattern, string)
-	register char	*pattern;
-	register char	*string;
+	register const char	*pattern;
+	register const char	*string;
 {
 	while (1) {
 	  switch (*pattern) {
+	  case '{':
+	    {
+	      const char *p = pattern+1;
+	      const char *s = string;
+
+	      /* This matches at the END of the pattern:  '*.{fii,foo,faa}' */
+
+	      for ( ; *p != 0 && *p != '}'; ++p) {
+		if (*p == ',') {
+		  if (*s == '\0')
+		    return 1; /* We have MATCH! */
+		  s = string;
+		  continue;
+		}
+		if (*s != *p) {
+		  /* Not the same .. */
+		  s = string;
+		  /* Ok, perhaps next pattern segment ? */
+		  while (*p != '\0' && *p != '}' && *p != ',')
+		    ++p;
+		  if (*p != ',')
+		    return 0; /* No next pattern ?
+				 We definitely have no match! */
+		  continue;
+		}
+		if (*s != 0)
+		  ++s;
+	      }
+	      if (*p == '\0' || *p == '}')
+		if (*s == 0)
+		  return 1;
+	      return 0;
+
+	    }
+	    break;
 	  case '*':
 	    ++pattern;
 	    if (*pattern == 0) {
@@ -1954,11 +2007,11 @@ static int globmatch(pattern, string)
 	    /* We do 'common case' optimization here, but will loose some
 	       performance, if somebody gives '*foo*' as a pattern.. */
 	    {
-	      char *p = pattern;
+	      const char *p = pattern;
 	      int i = 0, c;
-	      while ((c = *p++)) {
+	      while ((c = *p++) != 0) {
 		/* Scan for special chars in pattern.. */
-		if (c == '*' || c == '[' || c == '\\' || c == '?') {
+		if (c == '*' || c == '[' || c == '{' || c == '\\' || c == '?') {
 		  i = 1; /* Found! */
 		  break;
 		}
@@ -2030,7 +2083,7 @@ static int globmatch(pattern, string)
 static void link_in(flag, vp, s)
 	int flag;
 	struct vertex *vp;
-	u_char *s;
+	const char *s;
 {
 	struct web *wp = web_findcreate(flag,s);
 
@@ -2129,7 +2182,7 @@ static void init_timeserver()
 	if (ppid < 0) return; /* Error ?? brr.. */
 
 	ArgvSave[1] = NULL;
-	strcpy(ArgvSave[0],"TimeServer");
+	strcpy((char*)ArgvSave[0],"TimeServer");
 
 	ppid = getppid(); /* who is our parent ? */
 
@@ -2181,7 +2234,7 @@ time_t *timep;
 	return time(timep); /* The classical old version.. */
 }
 
-char *
+const char *
 cfpdirname(hash)
 int hash;
 {
@@ -2202,7 +2255,7 @@ int hash;
 
 void
 cfp_mksubdirs(topdir,newpath)
-char *topdir, *newpath;
+const char *topdir, *newpath;
 {
 	char path[256];
 	int omask = umask(022);
