@@ -33,6 +33,7 @@ extern int freeze;
 extern int slow_shutdown;
 extern time_t now;
 extern char *procselect, *procselhost;
+extern time_t sched_starttime; /* From main() */
 
 struct threadgroup *
 create_threadgroup(cep, wc, wh, withhost, ce_fillin)
@@ -116,7 +117,7 @@ if (verbose) printf("delete_threadgroup(%s/%d/%s)\n",
 	free(thgp);
 }
 
-
+static void _thread_timechain_unlink __((struct thread *));
 static void _thread_timechain_unlink(thr)
 struct thread *thr;
 {
@@ -134,6 +135,7 @@ struct thread *thr;
 	thr->prevtr = NULL;
 }
 
+static void _thread_timechain_append __((struct thread *));
 static void _thread_timechain_append(thr)
 struct thread *thr;
 {
@@ -201,6 +203,7 @@ struct config_entry *cep;
 	return thr;
 }
 
+static void delete_thread __((struct thread *, int));
 static void
 delete_thread(thr, ok)
 struct thread *thr;
@@ -321,6 +324,7 @@ int ok;
 	free(thr);
 }
 
+static void _thread_linkfront __((struct thread *, struct vertex *, struct vertex *));
 static void _thread_linkfront(thr,ap,vp)
 struct thread *thr;
 struct vertex *ap, *vp;
@@ -338,6 +342,7 @@ struct vertex *ap, *vp;
 
 
 /* the  _thread_linktail()  links a vertex into thread */
+static void _thread_linktail __((struct thread *, struct vertex *));
 static void _thread_linktail(thr,vp)
 struct thread *thr;
 struct vertex *vp;
@@ -466,23 +471,12 @@ if (verbose) printf("thread_linkin() to thg=0x%p[%s/%d/%s]; created a new thread
 	thread_start(thr);
 }
 
-char *strlower(s)
-char *s;
-{
-	char *cp;
-	for (cp = s; *cp != '\0'; ++cp)
-	  if (isascii(*cp) && isupper(*cp))
-	    *cp = tolower(*cp);
-	return s;
-}
-
 struct web *
 web_findcreate(flag, s)
 int flag;
-u_char *s;
+const char *s;
 {
 	struct spblk *spl;
-	register u_char *cp;
 	struct web *wp;
 
 	/* caller has done 'strlower()' to our input.. */
@@ -492,8 +486,8 @@ u_char *s;
 	  /* Not found, create it */
 	  wp = (struct web *)emalloc(sizeof (struct web));
 	  memset((void*)wp, 0, sizeof (struct web));
-	  sp_install(symbol(s), (u_char *)wp, 0, spt_mesh[flag]);
-	  wp->name     = strsave((char *)s);
+	  sp_install(symbol(s), (void *)wp, 0, spt_mesh[flag]);
+	  wp->name     = strsave(s);
 	  wp->kids     = 0;
 	  wp->link     = NULL;
 	  wp->linkcnt  = 0;
@@ -587,6 +581,7 @@ web_disentangle(vp, ok)
 	  delete_thread(vp->thread, ok);
 }
 
+static int vtx_ctime_cmp __((const void *, const void *));
 static int vtx_ctime_cmp(ap, bp)
      const void *ap, *bp;
 {
@@ -1361,6 +1356,7 @@ if (verbose) printf("idle_cleanup()\n");
 	return freecount;
 }
 
+static time_t oldest_age_on_thread __((struct thread *));
 static time_t oldest_age_on_thread(th) /* returns the AGE in seconds.. */
 struct thread *th;
 {
@@ -1385,8 +1381,6 @@ FILE *fp;
 	int threadsum = 0;
 	char timebuf[20];
 
-	extern time_t sched_starttime; /* From main() */
-
 	mytime(&now);
 
 	if (thrg_root == NULL) {
@@ -1403,7 +1397,6 @@ FILE *fp;
 	  struct procinfo *p;
 	  struct thread *thr;
 	  int thr_once = 1;
-	  time_t oldest;
 
 	  thg_once = 0;
 	  fprintf(fp,"%s/%s/%d\n",
