@@ -65,7 +65,6 @@ int	numkids = 0;
 int	readsockcnt = 0; /* Count how many childs to read there are;
 			    this for the SLOW Shutdown */
 
-extern char *getzenv();
 extern int errno;
 
 extern int slow_shutdown;
@@ -118,7 +117,7 @@ int
 flush_child(proc)
 struct procinfo *proc;
 {
-	int rc, len;
+	int rc;
 
 	if (proc->tofd < 0) {
 	  if (proc->vertex)
@@ -201,7 +200,7 @@ struct procinfo *proc;
 	if (slow_shutdown)
 	  strcpy(cmdbuf,"\n");
 	else if (vtx->cfp->dirind > 0) {
-	  char *d = cfpdirname(vtx->cfp->dirind);
+	  const char *d = cfpdirname(vtx->cfp->dirind);
 	  if (proc->thg->withhost)
 	    sprintf(cmdbuf, "%s/%s\n", d, vtx->cfp->mid);
 	  else
@@ -290,6 +289,7 @@ start_child(vhead, chwp, howp)
 	}
 	/*
 	 * Replace the $host and $channel strings in the command line.
+	 * (also $LOGDIR)
 	 */
 	os = buf;
 	for (i = 0; vhead->thgrp->ce.argv[i] != NULL; ++i) {
@@ -307,11 +307,15 @@ start_child(vhead, chwp, howp)
 		  ++cp;
 		if (*cp == '}') {
 		  *cp = '\0';
-		  if (strcmp(ocp,"host")==0)
+		  if (strcmp(ocp,"host")==0) {
 		    strcpy(s,howp->name);
-		  else if (strcmp(ocp,"channel")==0)
+		  } else if (strcmp(ocp,"channel")==0) {
 		    strcpy(s,chwp->name);
-		  else
+		  } else if (strcmp(ocp,"LOGDIR")==0) {
+		    char *e = getzenv("LOGDIR");
+		    if (e == NULL) e = "/var/log/mail";
+		    strcpy(s, e);
+		  } else
 		    strcpy(s, getzenv(ocp));
 		  s += strlen(s);
 		  *cp = '}';
@@ -478,7 +482,6 @@ shutdown_kids()
 static void reclaim(fromfd, tofd)
 	int fromfd, tofd;
 {
-	struct vertex *vp, *nvp = NULL;
 	struct procinfo *proc = &cpids[fromfd];
 
 if (verbose)
