@@ -14,20 +14,26 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include "zsyslog.h"
+#include "splay.h"
 #include "mail.h"
 #include "scheduler.h"
 
 #include "prototypes.h"
+#include "libz.h"
 
 static struct vertex *findvertex __((long, long, int*));
 static int ctlowner __((struct ctlfile *));
 static void vtxupdate __((struct vertex *, int, int));
-static void expaux __((struct vertex *, int, char *));
+static void expaux __((struct vertex *, int, const char *));
 
+extern FILE *vfp_open __((struct ctlfile *));
+
+#if 0
 extern int ermdir();
 extern int rmdir();
 extern int eunlink();
 extern struct spblk *sp_fhead(), *sp_fnext();
+#endif
 extern time_t now;
 extern int global_wrkcnt;
 extern char *procselect;
@@ -47,19 +53,23 @@ extern char *procselect;
 
 /* dispatch table for diagnostic types */
 
-static int u_ok       __((struct vertex *, long, long, long, char*, char*));
-static int u_ok2      __((struct vertex *, long, long, long, char*, char*));
-static int u_deferred __((struct vertex *, long, long, long, char*, char*));
-static int u_error    __((struct vertex *, long, long, long, char*, char*));
-static int u_error2   __((struct vertex *, long, long, long, char*, char*));
-static int u_retryat  __((struct vertex *, long, long, long, char*, char*));
+#define DARGS __((struct vertex *, long, long, long, const char*, const char*))
+
+static int u_ok       DARGS ;
+static int u_ok2      DARGS ;
+static int u_ok3      DARGS ;
+static int u_deferred DARGS ;
+static int u_error    DARGS ;
+static int u_error2   DARGS ;
+static int u_retryat  DARGS ;
 
 static struct diagcodes {
-	char	*name;
-	int	(*fcn)();
+	const char	*name;
+	int		(*fcn) DARGS ;
 } diags[] = {
 		{	"ok",		u_ok		},
 		{	"ok2",		u_ok2		},
+		{	"ok3",		u_ok3		},
 		{	"deferred",	u_deferred	},
 		{	"error",	u_error		},
 		{	"error2",	u_error2	},
@@ -73,7 +83,7 @@ update(fd, diagnostic)
 	char	*diagnostic;
 {
 	register char	*cp;
-	char	*type, *message, *notary;
+	const char	*type, *message, *notary;
 	long	offset;
 	long	inum;
 	int	index;
@@ -259,7 +269,6 @@ unctlfile(cfp, no_unlink)
 	    printf("   and %s/\n", path);
 
 	  if (cfp->vfpfn != NULL) {
-	    extern FILE *vfp_open __((struct ctlfile *));
 	    FILE *vfp = vfp_open(cfp);
 	    if (vfp) {
 	      fprintf(vfp, "scheduler done processing %s\n", cfp->mid);
@@ -283,8 +292,7 @@ void unvertex(vp, justfree, ok)
 	register struct vertex *vp;
 	int justfree, ok;
 {
-	int	i, remove;
-	int	reset_proc = 0;
+	int	i, removeme;
 
 	if (vp->ngroup > 0)
 	  return;
@@ -330,14 +338,14 @@ void unvertex(vp, justfree, ok)
 	    vp->prev[i]->next[i] = vp->next[i];
 	  if (i == L_CTLFILE)
 	    continue;
-	  remove = 0;
+	  removeme = 0;
 	  if (vp->orig[i]->link == vp)
 	    if ((vp->orig[i]->link = vp->next[i]) == NULL)
-	      remove = 1;
+	      removeme = 1;
 	  if (vp->orig[i]->lastlink == vp)
 	    if ((vp->orig[i]->lastlink = vp->prev[i]) == NULL)
-	      remove = 1;
-	  if (remove) {
+	      removeme = 1;
+	  if (removeme) {
 	    vp->orig[i]->linkcnt -= 1;
 	    unweb(i, vp->orig[i]);
 	    vp->orig[i] = NULL;
@@ -436,7 +444,7 @@ static int ctlowner(cfp)
 
 void
 deletemsg(msgid, curcfp)
-	char *msgid;
+	const char *msgid;
 	struct ctlfile *curcfp;
 {
 	struct ctlfile *cfp = NULL;
@@ -587,11 +595,12 @@ static void vtxupdate(vp, index, ok)
 	  }
 }
 
+static void logstat __((FILE *, struct vertex *, const char *));
 
 static void logstat(fp,vp,reason)
 	FILE *fp;
 	struct vertex *vp;
-	char *reason;
+	const char *reason;
 {
 	mytime(&now);
 	fprintf(fp,"%ld %s %ld %ld %s %s/%s\n",
@@ -606,7 +615,7 @@ static void logstat(fp,vp,reason)
 static void expaux(vp, index, buf)
 	struct vertex *vp;
 	int index;
-	char *buf;
+	const char *buf;
 {
 	int i;
 
@@ -636,7 +645,7 @@ expire(vp, index)
 {
 	int i;
 	char *emsg, buf[BUFSIZ];
-	static char *fmt = "\r%s, problem was:\r%s";
+	const char *fmt = "\r%s, problem was:\r%s";
 
 	if (vp->notary == NULL) {
 	  /* addres / action / status / diagnostic / wtt */
@@ -677,11 +686,11 @@ expire(vp, index)
 static int u_ok(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
 	long	index, inum, offset;
-	char	*notary;
-	char	*message;
+	const char	*notary;
+	const char	*message;
 {
 	if (verbose)
-	  fprintf(stderr,"%s: %d/%d/%s/ok %s\n", vp->cfp->logident, inum,
+	  fprintf(stderr,"%s: %ld/%ld/%s/ok %s\n", vp->cfp->logident, inum,
 		  offset, notary, message ? message : "-");
 #if 0
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
@@ -708,14 +717,14 @@ static int u_ok(vp, index, inum, offset, notary, message)
 static int u_ok2(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
 	long	index, inum, offset;
-	char	*notary;
-	char	*message;
+	const char	*notary;
+	const char	*message;
 {
 	if (vp->notaryflg & NOT_SUCCESS) {
 	  vp->cfp->haderror = 1; /* The transporter logged it for us! */
 	}
 	if (verbose)
-	  fprintf(stderr,"%s: %d/%d/%s/ok2 %s\n", vp->cfp->logident, inum,
+	  fprintf(stderr,"%s: %ld/%ld/%s/ok2 %s\n", vp->cfp->logident, inum,
 		  offset, notary, message ? message : "-");
 #if 0
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
@@ -726,7 +735,35 @@ static int u_ok2(vp, index, inum, offset, notary, message)
 	}
 #endif
 	if (statuslog)
-	  logstat(statuslog,vp,"ok");
+	  logstat(statuslog,vp,"ok2");
+
+	/* Delete this vertex from scheduling datasets */
+	vtxupdate(vp, index, 1);
+	return 1;
+}
+
+/*ARGSUSED*/
+static int u_ok3(vp, index, inum, offset, notary, message)
+	struct vertex *vp;
+	long	index, inum, offset;
+	const char	*notary;
+	const char	*message;
+{
+	/* Success, but the transporter was able to relay the DSN info
+	   to another system, thus no diagnostics here! */
+	if (verbose)
+	  fprintf(stderr,"%s: %ld/%ld/%s/ok3 %s\n", vp->cfp->logident, inum,
+		  offset, notary, message ? message : "-");
+#if 0
+	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
+	  fprintf(vp->cfp->vfp, "%s: ok %s\n",
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
+		  message == NULL ? "(sent)" : message);
+	}
+#endif
+	if (statuslog)
+	  logstat(statuslog,vp,"ok3");
 
 	/* Delete this vertex from scheduling datasets */
 	vtxupdate(vp, index, 1);
@@ -737,10 +774,10 @@ static int u_ok2(vp, index, inum, offset, notary, message)
 static int u_deferred(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
 	long	index, inum, offset;
-	char	*notary;
-	char	*message;
+	const char	*notary;
+	const char	*message;
 {
-	/* printf("%s: %d/%d/%s/deferred %s\n", vp->cfp->logident,
+	/* fprintf(stderr,"%s: %ld/%ld/%s/deferred %s\n", vp->cfp->logident,
 	   inum, offset, notary, message ? message : "-"); */
 	if (message != NULL) {
 	  if (vp->message != NULL)
@@ -768,10 +805,10 @@ static int u_deferred(vp, index, inum, offset, notary, message)
 }
 
 static int u_error(vp, index, inum, offset, notary, message)
-	struct vertex *vp;
-	long	index, inum, offset;
-	char	*notary;
-	char	*message;
+	struct vertex  *vp;
+	long		index, inum, offset;
+	const char	*notary;
+	const char	*message;
 {
 	if (message == NULL)
 	  message = "(unknown)";
@@ -802,8 +839,8 @@ static int u_error(vp, index, inum, offset, notary, message)
 static int u_error2(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
 	long	index, inum, offset;
-	char	*notary;
-	char	*message;
+	const char	*notary;
+	const char	*message;
 {
 	if (message == NULL)
 	  message = "(unknown)";
@@ -837,19 +874,22 @@ static int u_error2(vp, index, inum, offset, notary, message)
 static int u_retryat(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
 	long	index, inum, offset;
-	char	*notary;
-	char	*message;
+	const char	*notary;
+	const char	*message;
 {
 	time_t	retrytime;
-	char	*cp;
+	long    dtvalue;
+	const char * cp;
 
 	if (*message == '+')
 	  ++message;
+	dtvalue = 0;
+	sscanf(message, "%ld", &dtvalue);
 	for (cp = message; *cp != '\0' && isdigit(*cp); ++cp)
 	  continue;
 	if (*cp != '\0')
-	  *cp++ = '\0';
-	retrytime = atoi(message);
+	  ++cp;
+	retrytime = dtvalue;
 	message = cp;
 	if (*message == '\0')
 	  message = NULL;
