@@ -18,33 +18,36 @@ extern int errno;
 #include	<fcntl.h>
 int loadavg_current()
 {
-	/*
-	   $ cat /proc/loadavg 
-	   0.01 0.01 0.00
-	 */
+    /*
+       $ cat /proc/loadavg 
+       0.01 0.01 0.00
+     */
 
-	int rc, loadavg = 0;
-	char buf[20];
-	int fd = open("/proc/loadavg",O_RDONLY,0);
+    int rc, loadavg = 0;
+    char buf[20];
+    int fd = open("/proc/loadavg", O_RDONLY, 0);
 
-	if (fd < 0) return 0;	/* Not available.. */
+    if (fd < 0)
+	return 0;		/* Not available.. */
 
-	do {
-	  rc = read(fd, buf, sizeof(buf)-1);
-	  if (rc < 0) {
-	    if (errno == EINTR) continue;
+    do {
+	rc = read(fd, buf, sizeof(buf) - 1);
+	if (rc < 0) {
+	    if (errno == EINTR)
+		continue;
 	    close(fd);
-	    return 0; /* Some other error */
-	  } else
+	    return 0;		/* Some other error */
+	} else
 	    break;
-	} while (1);
+    } while (1);
 
-	close(fd);
-	buf[rc] = 0;
+    close(fd);
+    buf[rc] = 0;
 
-	loadavg = atoi(buf);
-	if (loadavg < 0) return 0; /* Error ?? */
-	return loadavg;
+    loadavg = atoi(buf);
+    if (loadavg < 0)
+	return 0;		/* Error ?? */
+    return loadavg;
 }
 #else
 /* ================================================================ */
@@ -52,8 +55,8 @@ int loadavg_current()
 	/* These routines are pulled from  TOP ... */
 
 	/* Linkage:
-		SVR4:      -lkvm -lelf
-		SunOS4.x:  -lkvm
+	   SVR4:      -lkvm -lelf
+	   SunOS4.x:  -lkvm
 	 */
 #include <fcntl.h>
 extern char *sys_errlist[];
@@ -64,11 +67,11 @@ extern char *sys_errlist[];
 static struct nlist nlst[] =
 {
 #if	defined(i386) || defined(__svr4__)
-  {"avenrun"},
+    {"avenrun"},
 #else
-  {"_avenrun"},	/* Sparc and 68k SunOS 4.x have it this way.. */
+    {"_avenrun"},		/* Sparc and 68k SunOS 4.x have it this way.. */
 #endif
-  {NULL}
+    {NULL}
 };
 static unsigned long avenrun_offset = 0;
 static int kd_inited = 0;
@@ -84,28 +87,28 @@ static int kd_inited = 0;
  */
 
 
-static int
-machine_init()
+static int machine_init()
 {
-	kvm_t *kd;
+    kvm_t *kd;
 
-	kd_inited = 1;
-	kd = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL);
-	if (kd == NULL) return 0;
+    kd_inited = 1;
+    kd = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL);
+    if (kd == NULL)
+	return 0;
 
-	if (kvm_nlist (kd, nlst) < 0) {
-	  kvm_close(kd);
-	  kd = NULL;
-	  return 0;
-	}
-	if (nlst[0].n_type == 0) { /* It wasn't found.. Our only! */
-	  kvm_close(kd);
-	  kd = NULL;
-	  return;
-	}
-	avenrun_offset = nlst[0].n_value; /* 0:th offset */
+    if (kvm_nlist(kd, nlst) < 0) {
 	kvm_close(kd);
-	return 1;
+	kd = NULL;
+	return 0;
+    }
+    if (nlst[0].n_type == 0) {	/* It wasn't found.. Our only! */
+	kvm_close(kd);
+	kd = NULL;
+	return;
+    }
+    avenrun_offset = nlst[0].n_value;	/* 0:th offset */
+    kvm_close(kd);
+    return 1;
 }
 
 /*
@@ -114,40 +117,39 @@ machine_init()
  *      "ptr" points to a buffer into which the value is retrieved,
  *      "size" is the size of the buffer (and the object to retrieve),
  */
-static int
-getkval (offset, ptr, size)
-	unsigned long offset;
-	int *ptr;
-	int size;
+static int getkval(offset, ptr, size)
+unsigned long offset;
+int *ptr;
+int size;
 {
-	kvm_t *kd;
-	int ret;
+    kvm_t *kd;
+    int ret;
 
-	kd = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL);
-	ret = kvm_read (kd, offset, (char *) ptr, size);
-	kvm_close(kd);
-	return ret;
+    kd = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL);
+    ret = kvm_read(kd, offset, (char *) ptr, size);
+    kvm_close(kd);
+    return ret;
 }
 
 int loadavg_current()
 {
-	long avenrun[3];
+    long avenrun[3];
 
-	if (!kd_inited)
-	  if (machine_init() == 0)
+    if (!kd_inited)
+	if (machine_init() == 0)
 	    return 0;
 
-	/* get load average array */
-	if (!getkval(avenrun_offset, (int *) avenrun, sizeof(avenrun), NULL))
-	  return 0; /* Failed.. */
-	return (int)(avenrun[0] >> 8); /* a fixed-point decimal number */
+    /* get load average array */
+    if (!getkval(avenrun_offset, (int *) avenrun, sizeof(avenrun), NULL))
+	return 0;		/* Failed.. */
+    return (int) (avenrun[0] >> 8);	/* a fixed-point decimal number */
 }
 
 #else
 /* ================================================================ */
 int loadavg_current()
 {
-	return 0;	/* Dummy.. */
+    return 0;			/* Dummy.. */
 }
 /* ================================================================ */
 #endif
