@@ -9,18 +9,15 @@
 
 #include "smtpserver.h"
 
-int
-childsameip(void*addr)
+int childsameip(void *addr)
 {
-  return 0;
+    return 0;
 }
 
-void
-childregister(int pid, void *addr)
+void childregister(int cpid, void *addr)
 {
 }
 
-void
-childreap(int pid)
+void childreap(int cpid)
 {
 }
