@@ -71,13 +71,22 @@
  */
 
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
+#include <ctype.h>
+#include "hostenv.h"
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 
 extern int optind;
 extern char *optarg;
 
-extern char *malloc();
+/* extern char *malloc(); */
+
 
 #ifndef __STDC__
 extern FILE *fopen();
@@ -120,15 +129,17 @@ extern void exit();
 #define	BeginRule()		/*(void)0*/
 #define	SaveRule()		/*(void)0*/
 
-char *machineOpNames[200] = { "oCall", "oReturn", "oRuleEnd", "oJump",
-				"oInput", "oInputAny", "oInputChoice",
-				"oEmit", "oError",  "oChoice", "oChoiceEnd",
-				"oSetParameter", "oSetResult",
-				"oSetResultFromInput",
-				0 };
+char const * machineOpNames[200] = {
+  "oCall", "oReturn", "oRuleEnd", "oJump",
+  "oInput", "oInputAny", "oInputChoice",
+  "oEmit", "oError",  "oChoice", "oChoiceEnd",
+  "oSetParameter", "oSetResult",
+  "oSetResultFromInput",
+  NULL };
 
-char *errorCodeNames[100] = { "eNoError", "eSyntaxError",
-	"ePrematureEndOfFile", "eExtraneousProgramText", 0 };
+const char * errorCodeNames[100] = { "eNoError", "eSyntaxError",
+				     "ePrematureEndOfFile",
+				     "eExtraneousProgramText", NULL };
 
 #ifdef	SELFCONTAINED
 /*
@@ -563,12 +574,13 @@ InputTokens savedToken;
 int	nextChar;
 
 /* Keyword Token Table */
-char *keywordText[] = { "mechanism", "output", "input", "error", "rules",
-		"type", "end", "do", "od", "if", "fi"
+const char *keywordText[] = { "mechanism", "output", "input", "error", "rules",
+			      "type", "end", "do", "od", "if", "fi"
 };
 
 InputTokens keywordToken[] = { tMechanism, tOutput, tInput, tError, tRules,
-		tType, tEnd, tCycle, tCycleEnd, tChoice, tChoiceEnd
+			       tType, tEnd, tCycle, tCycleEnd, tChoice,
+			       tChoiceEnd
 };
 
 /* Special Symbol Token Table */
@@ -580,14 +592,14 @@ boolean firstChar = true;
 
 /* The Assembled Table */
 int   outputTable[maxOutputTableSize];
-char *outputNameTable[maxOutputTableSize];
+const char *outputNameTable[maxOutputTableSize];
 OutputAddress outputPointer = 0;
 
 /* The Semantic Mechanism Data Structures of the S/SL Processor */
 
 /* The Symbol Table */
 SymbolClass symClass[maxSymbols];
-char *symText[maxSymbols];
+const char *symText[maxSymbols];
 int symHash[maxSymbols];
 int symValue[maxSymbols];
 SymbolClass symParmClass[maxSymbols];
@@ -705,11 +717,11 @@ extern Validity VerifyChoiceLabel __((int));
 extern void AcceptInputToken __((void));
 extern void ChangeChoiceClass __((SymbolClass));
 extern void ChangeSymbolClass __((void));
-extern void Emit __((int, char *));
+static void Emit __((int value, const char *name));
 extern void EmitChoiceTable __((void));
 extern void EmitCycleAddress __((void));
 extern void EmitFirstChoiceAddress __((void));
-extern void EmitFixup __((OutputAddress, int, char *));
+extern void EmitFixup __((OutputAddress, int, const char *));
 extern void EmitJumpAddress __((OutputAddress));
 extern void EmitJumpFixup __((OutputAddress));
 extern void EmitNullAddress __((void));
@@ -722,9 +734,9 @@ extern void EnterSecondNewSymbolValue __((void));
 extern void EnterSymbolValueFromAddress __((void));
 extern void Error __((ErrorCodes));
 extern void EvaluateNumber __((void));
-extern void GenerateClass __((SymbolClass, char *));
-extern void GenerateEntryPoints __((char *));
-extern void GenerateOutputTable __((char *));
+extern void GenerateClass __((SymbolClass, const char *));
+extern void GenerateEntryPoints __((const char *));
+extern void GenerateOutputTable __((const char *));
 extern void GenerateSymbolDefinitions __((void));
 extern void IncrementCurrentClassValue __((void));
 extern void InitCallTable __((void));
@@ -752,6 +764,7 @@ extern void SslFailure __((FailureCodes));
 extern void SslGenerateCompoundInputToken __((InputTokens));
 extern void SslSyntaxError __((void));
 extern void SslTrace __((void));
+extern int  hash __((const char *));
 extern void xEmitFirstChoiceValue __((void));
 extern void xEnterChoiceLabel __((int));
 extern void xEnterNewSymbolValue __((void));
@@ -763,9 +776,9 @@ extern void yEnterSymbolResultClass __((void));
 boolean tracing;
 boolean summarize;
 
+extern int main __((int argc, char *argv[]));
 
-
-void
+int
 main(argc, argv)
 	int argc;
 	char *argv[];
@@ -773,9 +786,10 @@ main(argc, argv)
 	SymbolClass c, d;
 	int ch, errflag;
 	int v;
-	char *n;
+	const char *n;
 	boolean listing;
-	char buf[1024 /* PATH_MAX */], *useEnum, *typeName;
+	char buf[1024 /* PATH_MAX */];
+	const char *useEnum, *typeName;
 	char *inFile, *tableFile, *headerFile, *listFile, *entryFile;
 
 	progname = argv[0];
@@ -1258,7 +1272,7 @@ strsave(s)
 
 int
 hash(s)
-	register char *s;
+	register const char *s;
 {
 	int i;
 
@@ -1267,18 +1281,21 @@ hash(s)
 	return i;
 }
 
+extern char *LowerCase __((const char *s));
+
 char *
 LowerCase(s)
-	register char *s;
+	register const char *s;
 {
 	static char buf[BUFSIZ];
 	char *cp;
 
 	for (cp = buf; *s != '\0'; ++s, ++cp) {
-		if (isascii(*s) && isupper(*s))
-			*cp = tolower(*s);
+		int c = (*s) & 0xFF;
+		if (isascii(c) && isupper(c))
+			*cp = tolower(c);
 		else
-			*cp = *s;
+			*cp = c;
 	}
 	*cp = '\0';
 	return buf;
@@ -1419,7 +1436,7 @@ Error(errCode)
 	}
 }
 
-
+extern void ReadNextChar __((void));
 void
 ReadNextChar()
 {
@@ -1478,7 +1495,7 @@ LookupKeyword()
 	}
 }
 
-
+extern void GetNextInputToken __((void));
 void
 GetNextInputToken()
 {
@@ -1689,10 +1706,10 @@ AcceptInputToken()
 
 /* Emit an output table element to the assembled table */
 
-void
+static void
 Emit(value,name)
-	int value;
-	char *name;
+const int value;
+const char *name;
 {
 	if (name && *name == '?')
 	  name = NULL;
@@ -1716,7 +1733,7 @@ void
 EmitFixup(fixupAddress, value, name)
 	OutputAddress fixupAddress;
 	int value;
-	char *name;
+	const char *name;
 {
 	if (fixupAddress >= outputPointer
 	    || outputTable[fixupAddress] != nullAddress)
@@ -1727,12 +1744,12 @@ EmitFixup(fixupAddress, value, name)
 
 	/* Trace Table Assembly */
 	if (tracing)
-		printf("Output fixup at %d: %d (%s)\n", fixupAddress, value,name);
+	  printf("Output fixup at %d: %d (%s)\n", fixupAddress, value,name);
 }
 
 
 /* Reserve a table location to be fixed up */
-
+extern void EmitNullAddress __((void));
 void
 EmitNullAddress()
 {
@@ -1746,7 +1763,7 @@ void
 EmitJumpAddress(targetAddress)
 	OutputAddress targetAddress;
 {
-	Emit(targetAddress,NULL);
+	Emit((int)targetAddress,NULL);
 }
 
 
@@ -1756,7 +1773,7 @@ void
 EmitJumpFixup(jumpAddress)
 	OutputAddress jumpAddress;
 {
-	EmitFixup(jumpAddress, outputPointer, NULL);
+	EmitFixup(jumpAddress, (int)outputPointer, NULL);
 }
 
 
@@ -1917,6 +1934,7 @@ IncrementCurrentClassValue()
 }
 
 
+extern void CopySymbolTextToTokenBuffer __((void));
 void
 CopySymbolTextToTokenBuffer()
 {
@@ -1986,7 +2004,7 @@ void
 InitSymbolTable()
 {
 	SymbolClass i;
-	char **cpp;
+	const char **cpp;
 
 	/* Initialize Symbol Table */
 	if (notFound != 0)
@@ -2064,7 +2082,7 @@ ResolveCalls()
 {
 	u_int i;
 	int v;
-	char *n;
+	const char *n;
 
 	i = 0;
 	while (i != callTop) {
@@ -2284,7 +2302,7 @@ EmitChoiceTable()
 
 
 	/* Emit choice table */
-	Emit(labelTop - choiceLabelIndex[choiceTop],NULL); /* Number of entries */
+	Emit((int)(labelTop - choiceLabelIndex[choiceTop]), NULL); /* Number of entries */
 	for (i = choiceLabelIndex[choiceTop]; i != labelTop;) {
 		++i;
 		Emit(labelValue[i],NULL);
@@ -2338,7 +2356,7 @@ InitChoiceStack()
 void
 GenerateClass(class, name)
 	SymbolClass class;
-	char *name;
+	const char *name;
 {
 	SymbolIndex s;
 	SymbolClass c;
@@ -2416,7 +2434,7 @@ GenerateSymbolDefinitions()
 
 void
 GenerateOutputTable(typeName)
-	char *typeName;
+	const char *typeName;
 {
 	OutputAddress i;
 	int col = 16, coladd;
@@ -2451,7 +2469,7 @@ if (outputNameTable[i] && outputNameTable[i][0] == '"')
 
 void
 GenerateEntryPoints(enumName)
-	char *enumName;
+	const char *enumName;
 {
 	SymbolIndex s;
 
