@@ -29,6 +29,16 @@
 /* #include "useful.h"  */
 /* #include "userdbm.h" */
 
+#include "malloc.h"
+#include "libz.h"
+#include "libc.h"
+
+extern void   sendmessage __((const char *, const char *));
+extern char * newstr __((const char *));
+extern void   setinterval __((time_t));
+extern void   setreply __((void));
+extern void   usage __((void));
+
 #ifndef LONG_MAX
 # define LONG_MAX 1000000000 /* 100 million seconds - 3.2 years */
 #endif
@@ -97,10 +107,8 @@ char from[MAXLINE];
 char *subject_str = NULL;	/* Glob subject from input */
 int dblog = 1;
 
-extern void purge_input();
-extern char *strerror();
+extern void purge_input __((void));
 extern struct passwd *getpwnam();
-extern char *newstr();
 extern int optind, opterr;
 extern char *optarg;
 extern FILE *freopen(), *tmpfile();
@@ -108,6 +116,9 @@ extern char *getenv();
 #ifndef HAVE_STDLIB_H
 extern char *malloc();
 #endif
+extern void usrerr __((char *));
+extern void syserr __((char *));
+extern void readheaders __((void));
 
 int
 main(argc, argv)
@@ -118,10 +129,7 @@ main(argc, argv)
 	struct passwd *pw;
 	ALIAS *cur;
 	time_t interval;
-	char *homedir;
-	char *myname;
 	char *msgfile = NULL;
-	char buf[MAXLINE];
 	int ch, iflag;
 
 	/* process arguments */
@@ -281,15 +289,15 @@ main(argc, argv)
 **		sends mail to 'user' using /usr/lib/sendmail.
 */
 
+void
 sendmessage(msgf, myname)
-	char *msgf;
-	char *myname;
+	const char *msgf;
+	const char *myname;
 {
 	FILE *f;
 	FILE *tmpf;
 	char linebuf[512];
 	char *s;
-	int  pp[2];
 
 #ifdef VDEBUG
 	fprintf(stderr, "sendmessage(%s, %s)\n", msgf, myname);
@@ -348,6 +356,7 @@ sendmessage(msgf, myname)
 **		none.
 */
 
+void
 usrerr(msg)
 	char *msg;
 {
@@ -368,6 +377,7 @@ usrerr(msg)
 */
 
 /* VARARGS 1 */
+void
 syserr(msg)
 	char *msg;
 {
@@ -389,7 +399,7 @@ syserr(msg)
 
 char *
 newstr(s)
-	char *s;
+	const char *s;
 {
 	char *p;
 
@@ -406,6 +416,7 @@ newstr(s)
  * readheaders --
  *	read mail headers
  */
+void
 readheaders()
 {
 	register ALIAS *cur;
@@ -444,7 +455,8 @@ readheaders()
 				for (p = buf + 5; *p && *p != ' '; ++p);
 				*p = '\0';
 				strcpy(from, buf + 5);
-				if (p = strchr(from, '\n'))
+				p = strchr(from, '\n');
+				if (p != NULL)
 					*p = '\0';
 				if (junkmail()) {
 					purge_input();
@@ -509,6 +521,7 @@ findme:			for (cur = names; !tome && cur; cur = cur->next)
  * nsearch --
  *	do a nice, slow, search of a string for a substring.
  */
+int
 nsearch(name, str)
 	register char *name, *str;
 {
@@ -524,11 +537,12 @@ nsearch(name, str)
  * junkmail --
  *	read the header and return if automagic/junk/bulk mail
  */
+int
 junkmail()
 {
 	static struct ignore {
-		char	*name;
-		int	len;
+		const char	*name;
+		int		len;
 	} ignore[] = {
 		"-request", 8,		"postmaster", 10,	"uucp", 4,
 		"mailer-daemon", 13,	"mailer", 6,		"-relay", 6,
@@ -547,7 +561,8 @@ junkmail()
 	 */
 	if (!(p = strchr(from, '%')))
 		if (!(p = strchr(from, '@'))) {
-			if (p = strrchr(from, '!'))
+			p = strrchr(from, '!');
+			if (p != NULL)
 				++p;
 			else
 				p = from;
@@ -586,7 +601,7 @@ purge_input()
  *	find out if user has gotten a vacation message recently.
  *	use memcpy for machines with alignment restrictions
  */
-recent()
+int recent()
 {
 	DBT key, data;
 	time_t then, next;
@@ -634,6 +649,7 @@ recent()
  * setinterval --
  *	store the reply interval
  */
+void
 setinterval(interval)
 	time_t interval;
 {
@@ -659,6 +675,7 @@ setinterval(interval)
  * setreply --
  *	store that this user knows about the vacation.
  */
+void
 setreply()
 {
 	DBT key, data;
@@ -681,6 +698,7 @@ setreply()
 #endif
 }
 
+void
 usage()
 {
 	fprintf(stderr,"vacation: [-i] [-d] [-a alias] [-m msgfile] [-r interval] login\n");
