@@ -15,13 +15,14 @@
 #include "mail.h"
 #include "zmsignal.h"
 #include "zsyslog.h"
-#include "ta.h"
 
+#include "ta.h"
+#include "libz.h"
 
 #define	PROGNAME	"errormail"
 #define	CHANNEL		"error"	/* the default channel name we deliver for */
 
-char *dfltform[] = {
+const char *dfltform[] = {
 		"From: The Post Office <postmaster>",
 		"Subject: email delivery error",
 		"Cc: The Postmaster <postmaster>",
@@ -36,18 +37,13 @@ char *dfltform[] = {
 char *progname;
 
 extern char *optarg;
-extern int optind;
-extern int emptyline();
-extern void prversion();
-extern void process();
+extern int  optind;
+extern void process __((struct ctldesc *));
 
 #ifndef strchr
-extern char *strrchr();
-extern char *strchr();
+extern char *strrchr(), *strchr();
 #endif
 
-extern char *mydomain();
-
 #ifndef	MAXPATHLEN
 #define	MAXPATHLEN 1024
 #endif	/* MAXPATHLEN */
@@ -60,10 +56,11 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char *channel, msgfilename[MAXPATHLEN+1];
+	const char *channel;
+	char msgfilename[MAXPATHLEN+1];
 	int errflg, c;
 	struct ctldesc *dp;
-	RETSIGTYPE (*oldsig)();
+	RETSIGTYPE (*oldsig) __((int));
 
 	SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
@@ -85,7 +82,10 @@ main(argc, argv)
 	  ++progname;
 	errflg = 0;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "c:V")) != EOF) {
+	while (1) {
+	  c = getopt(argc, argv, "c:V");
+	  if (c == EOF)
+	    break;
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -109,7 +109,7 @@ main(argc, argv)
 	zopenlog("errormail", LOG_PID, LOG_MAIL);
 
 	while (!getout) {
-	  char *s, *host;
+	  char *host;
 
 	  printf("#hungry\n");
 	  fflush(stdout);
@@ -155,24 +155,28 @@ tmalloc(n)
 #endif
 */
 
-void encodeXtext(mfp,s)
-     FILE *mfp;
-     char *s;
+void encodeXtext __((FILE *, const char *, int));
+void encodeXtext(mfp,s,len)
+	FILE *mfp;
+	const char *s;
+	int len;
 {
-	while (*s) {
-	  u_char c = *s;
+	while (*s && len > 0) {
+	  int c = (*s) & 0xFF;
 	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
 	    putc(c,mfp);
 	  else
 	    fprintf(mfp,"+%02X",c);
 	  ++s;
+	  --len;
 	}
 }
 
+static void decodeXtext __((FILE *, const char *));
 static void
 decodeXtext(fp,xtext)
 	FILE *fp;
-	char *xtext;
+	const char *xtext;
 {
 	for (;*xtext;++xtext) {
 	  if (*xtext == '+') {
@@ -189,32 +193,32 @@ decodeXtext(fp,xtext)
 
 /* Pick recipient address from the input line.
    EXTREMELY Simple minded parsing.. */
+static void pick_env_addr __((const char *buf, FILE *mfp));
 static void pick_env_addr(buf,mfp)
-char *buf;
+const char *buf;
 FILE *mfp;
 {
-	char *s = buf;
+	const char *s = buf;
 
 	while (*s != 0 && *s != ' ' && *s != '\t' && *s != ':') ++s;
 	if (*s != ':') return; /* BAD INPUT! */
 	buf = ++s; /* We have skipped the initial header.. */
 
-	s = strchr(buf,'\n');
-	if (s) *s = 0;
 	s = strchr(buf,'<');
 	if (s != NULL) {
 	  /*  Cc:  The Postoffice managers <postoffice> */
 	  buf = ++s;
 	  s = strrchr(buf,'>');
-	  if (s) *s = 0;
-	  else return; /* No trailing '>' ? BAD BAD! */
+	  if (s == NULL)
+	    return; /* No trailing '>' ? BAD BAD! */
 	  fprintf(mfp,"todsn NOTIFY=NEVER ORCPT=rfc822;");
-	  encodeXtext(mfp,buf);
-	  fprintf(mfp,"\nto <%s>\n",buf);
+	  encodeXtext(mfp, buf, (int)(s - 1 - buf));
+	  fprintf(mfp,"\nto <%.*s>\n",(int)(s - 1 - buf), buf);
 	} else {
 	  /*  Cc: some-address  */
 	  fprintf(mfp,"todsn NOTIFY=NEVER ORCPT=rfc822;");
-	  encodeXtext(mfp,buf);
+	  s = buf + strlen(buf) - 1;
+	  encodeXtext(mfp,buf,strlen(buf));
 	  fprintf(mfp,"\nto <%s>\n",buf);
 	}
 }
@@ -224,11 +228,11 @@ process(dp)
 	struct ctldesc *dp;
 {
 	char buf[BUFSIZ];
-	char **cpp, *mailshare, *mfpath;
+	const char **cpp, *mailshare, *mfpath;
 	FILE *mfp, *efp;
 	int n;
 	struct rcpt *rp;
-	char *boundarystr = buildboundary("errmail");
+	const char *boundarystr = buildboundary("errmail");
 	char lastchar;
 	int reportcnt = 0;
 	struct stat stbuf;
@@ -263,18 +267,20 @@ process(dp)
 	rp = dp->recipients;
 
 	fprintf(mfp, "todsn NOTIFY=NEVER ORCPT=RFC822;");
-	encodeXtext(mfp, rp->addr->link->user);
+	encodeXtext(mfp, rp->addr->link->user, strlen(rp->addr->link->user));
 	fprintf(mfp, "\nto <%s>\n",rp->addr->link->user);
 
 	/* copy error message file itself */
-	if ((mailshare = getzenv("MAILSHARE")) == NULL)
+	mailshare = getzenv("MAILSHARE");
+	if (mailshare == NULL)
 	  mailshare = MAILSHARE;
 
 	mfpath = emalloc(3 + strlen(mailshare) + strlen(FORMSDIR) +
 			 strlen(rp->addr->host));
-	sprintf(mfpath, "%s/%s/%s", mailshare, FORMSDIR, rp->addr->host);
+	sprintf((char*)mfpath, "%s/%s/%s", mailshare, FORMSDIR, rp->addr->host);
 
-	if ((efp = fopen(mfpath, "r")) != NULL) {
+	efp = fopen(mfpath, "r");
+	if (efp != NULL) {
 	  int inhdr = 1;
 	  buf[sizeof(buf)-1] = 0;
 	  while (fgets(buf,sizeof(buf)-1,efp) != NULL) {
@@ -367,9 +373,9 @@ process(dp)
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 	  /* If not prohibited, print it! */
 	  const char *typetag;
-	  char *rcpt;
-	  static char const *type_rfc   = "RFC822";
-	  static char const *type_local = "X-LOCAL";
+	  const char *rcpt;
+	  static const char *type_rfc   = "RFC822";
+	  static const char *type_local = "X-LOCAL";
 
 	  if (rp->notify != NULL && CISTREQN(rp->notify,"NEVER",5))
 	    continue;
@@ -399,8 +405,10 @@ process(dp)
 
 	rp = dp->recipients;
 	/* copy original message file */
+
 	/* seek to message body -- try it anyway */
-	lseek(dp->msgfd, (off_t)(dp->msgbodyoffset), 0 /* L_SET -- usually */);
+	lseek(dp->msgfd, (off_t)(dp->msgbodyoffset), SEEK_SET);
+
 	/* write the (new) headers with local "Received:"-line.. */
 	writeheaders(rp,mfp,"\n",0,0);
 
