@@ -18,6 +18,10 @@
 #include "fuzzy.h"
 
 
+#ifndef SEEK_SET
+#define SEEK_SET  0
+#endif
+
 #define	PROGNAME	"fuzzyalias"
 #define	CHANNEL		"fuzzy"	/* the default channel name we deliver for */
 
@@ -220,8 +224,10 @@ FILE *mfp;
 	  /*  Cc:  The Postoffice managers <postoffice> */
 	  buf = ++s;
 	  s = strrchr(buf,'>');
-	  if (s) *s = 0;
-	  else return; /* No trailing '>' ? BAD BAD! */
+	  if (s)
+	    *s = 0;
+	  else
+	    return; /* No trailing '>' ? BAD BAD! */
 	  fprintf(mfp,"to <%s>\n",buf);
 	} else {
 	  /*  Cc: some-address  */
@@ -439,8 +445,10 @@ process(dp, answer)
 
 	rp = dp->recipients;
 	/* copy original message file */
+
 	/* seek to message body -- try it anyway */
-	lseek(dp->msgfd, dp->msgbodyoffset, 0 /* L_SET -- usually */);
+	lseek(dp->msgfd, dp->msgbodyoffset, SEEK_SET);
+
 	/* write the (new) headers with local "Received:"-line.. */
 	writeheaders(rp,mfp,"\n",0,0);
 
