@@ -100,9 +100,9 @@ typedef char msgdata;
 # define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif
 
-#ifndef	L_SET
-#define	L_SET	0
-#endif	/* L_SET */
+#ifndef	SEEK_SET
+#define	SEEK_SET 0
+#endif	/* SEEK_SET */
 
 #define	PROGNAME	"hold"	/* for logging */
 #define	CHANNEL		"hold"	/* the default channel name we look at */
@@ -112,9 +112,7 @@ char *progname;
 char *cmdline, *eocmdline;
 int pid;
 
-#ifndef MALLOC_TRACE
-extern univptr_t emalloc();
-#else
+#ifdef MALLOC_TRACE
 #define	MEM_MALLOC 1000
 int	stickymem = MEM_MALLOC;	/* for strnsave() */
 struct conshell *envarlist = NULL;
@@ -123,16 +121,13 @@ int	D_alloc = 0;
 
 extern char *optarg;
 extern int optind;
-extern void prversion();
-extern void process();
-extern int  hold();
-extern time_t time();
+extern void process __((struct ctldesc *));
+extern int  hold __((const char *, char **));
 extern char **environ;
 
 #ifndef strchr
 extern char *strchr(), *strrchr();
 #endif
-extern struct hostent *gethostbyname(), *gethostbyaddr();
 extern int   ranny();
 
 #ifdef	lint
@@ -145,10 +140,11 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char *channel, *host, file[MAXPATHLEN+1];
+	const char *channel, *host;
+	char file[MAXPATHLEN+1];
 	int errflg, c, i;
 	struct ctldesc *dp;
-	RETSIGTYPE (*oldsig)();
+	RETSIGTYPE (*oldsig) __((int));
 
 	pid = getpid();
 	cmdline = &argv[0][0];
@@ -175,7 +171,10 @@ main(argc, argv)
 	  ++progname;
 	errflg = 0;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "c:V")) != EOF) {
+	while (1) {
+	  c = getopt(argc, argv, "c:V");
+	  if (c == EOF)
+	    break;
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -248,19 +247,20 @@ process(dp)
 	FILE *mfp;
 	int n, sawok, code;
 	struct rcpt *rp;
-	char *cp, buf[BUFSIZ];
+	const char *cp;
+	char buf[BUFSIZ];
 
 	sawok = 0;
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 	  cp = rp->addr->user;
-	  rp->status = hold(rp->addr->host, &(rp->addr->user));
+	  rp->status = hold(rp->addr->host, (char**)&(rp->addr->user));
 	  if (rp->status == EX_OK) {
 	    rp->addr->user = cp;
 	    sawok = 1;
 	  } else {
-	    char *action = NULL;
-	    char *status = NULL;
-	    char *diagnostics = NULL;
+	    const char *action = NULL;
+	    const char *status = NULL;
+	    const char *diagnostics = NULL;
 	    switch (rp->status) {
 	    case EX_PROTOCOL:
 	    case EX_SOFTWARE:
@@ -287,12 +287,13 @@ process(dp)
 	if (!sawok)
 	  return;
 
-	if (lseek(dp->msgfd, (off_t)(dp->msgbodyoffset), L_SET) < 0L)
+	if (lseek(dp->msgfd, (off_t)(dp->msgbodyoffset), SEEK_SET) < 0L)
 	  warning("Cannot seek to message body! (%m)", (char *)NULL);
 
 	setreuid(-1, atoi(dp->senders->misc));
 
-	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
+	mfp = mail_open(MSG_RFC822);
+	if (mfp == NULL) {
 	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,"delayed",
@@ -366,11 +367,15 @@ process(dp)
  * string is canonicalized and parsed hierarchically.
  */
 
-extern int hold_ns(), hold_timeout(), hold_io(), hold_script(), hold_home();
+extern int hold_ns	__(( const char* ));
+extern int hold_timeout	__(( const char* ));
+extern int hold_io	__(( const char* ));
+extern int hold_script	__(( const char* ));
+extern int hold_home	__(( const char* ));
 
 struct holds_info {
-	char	*type;
-	int	(*f)();
+	const char	*type;
+	int	(*f) __(( const char * ));
 } holds[] = {
 	{	"ns",		hold_ns		},
 	{	"timeout",	hold_timeout	},
@@ -383,7 +388,8 @@ struct holds_info {
 
 int
 hold(s, errmsgp)
-	char *s, **errmsgp;
+	const char *s;
+	char **errmsgp;
 {
 	char *cp, *colon;
 	struct holds_info *hip;
@@ -393,7 +399,7 @@ hold(s, errmsgp)
 	spkey_t symid;
 
 	colon = NULL;
-	for (cp = s; *cp != '\0'; ++cp) {
+	for (cp = (char*)s; *cp != '\0'; ++cp) {
 	  if (isascii(*cp) && isupper(*cp))
 	    *cp = tolower(*cp);
 	  else if (*cp == ':')
@@ -430,7 +436,7 @@ hold(s, errmsgp)
 	  strcpy(cp, errormsg);
 	} else
 	  cp = "deferred";
-	sp_install(symid, (u_char *)cp, v, spt_hash);
+	sp_install(symid, (const void *)cp, v, spt_hash);
 	*errmsgp = cp;
 
 	return v;
@@ -442,8 +448,8 @@ hold(s, errmsgp)
 extern int	h_errno;
 
 struct qtypes {
-	char	*typename;
-	u_short	value;
+	const char	*typename;
+	int		 value;
 } qt[] = {
 	{	"cname",	T_CNAME		},
 	{	"mx",		T_MX		},
@@ -468,7 +474,7 @@ struct qtypes {
 
 int
 hold_ns(s)
-	char *s;
+	const char *s;
 {
 	struct qtypes *qtp;
 	char *cp, host[1024]; /* 256 should be enough .. */
@@ -550,6 +556,7 @@ hold_ns(s)
 	hp = NULL;
 	switch (qtp->value) {
 	case 1:
+	  /* XXX: getaddrinfo() */
 	  hp = gethostbyname(host);
 	  break;
 	case 2:
@@ -566,7 +573,7 @@ hold_ns(s)
 
 int
 hold_timeout(s)
-	char *s;
+	const char *s;
 {
 	time_t now, then;
 
@@ -579,7 +586,7 @@ hold_timeout(s)
 
 int
 hold_io(s)
-	char *s;
+	const char *s;
 {
 	return ranny(9) == 0;	/* 10% of the time */
 }
@@ -589,21 +596,24 @@ hold_io(s)
 
 int
 hold_script(command)
-	char *command;
+	const char *command;
 {
 	int i, in[2];
-	char *env[20], buf[8192], *cp, *s, *arg;
+	const char *env[20], *s;
+	char buf[8192], *cp, *arg;
 	FILE *errfp;
 	int status;
 
-	if ((arg = strchr(command, '/')) != NULL)
+	arg = strchr(command, '/');
+	if (arg != NULL)
 	  *arg++ = '\0';
 	else
 	  arg = NULL;
 	i = 0;
 	env[i++] = "SHELL=/bin/sh";
 	cp = buf;
-	if ((s = getzenv("PATH")) == NULL)
+	s = getzenv("PATH");
+	if (s == NULL)
 	  env[i++] = "PATH=/usr/ucb:/usr/bin:/bin";
 	else {
 	  sprintf(cp, "PATH=%s", s);
@@ -612,16 +622,19 @@ hold_script(command)
 	}
 	env[i++] = "HOME=/tmp";
 	env[i++] = "USER=anonymous";
-	if ((s = getzenv("ZCONFIG")) == NULL)
+	s = getzenv("ZCONFIG");
+	if (s == NULL)
 	  s = ZMAILER_ENV_FILE;
 	sprintf(cp, "ZCONFIG=%s", s);
 	env[i++] = cp;
-	if ((s = getzenv("MAILSHARE")) == NULL)
+	s = getzenv("MAILSHARE");
+	if (s == NULL)
 	  s = MAILBIN;
 	cp += strlen(cp) + 1;
 	sprintf(cp, "MAILSHARE=%s", s);
 	env[i++] = cp;
-	if ((s = getzenv("MAILBIN")) == NULL)
+	s = getzenv("MAILBIN");
+	if (s == NULL)
 	  s = MAILSHARE;
 	cp += strlen(cp) + 1;
 	sprintf(cp, "MAILBIN=%s", s);
@@ -638,7 +651,7 @@ hold_script(command)
 	sprintf(cp, "%s/bin/%s", s, command);
 
 	if ((pid = fork()) == 0) { /* child */
-	  environ = env;
+	  environ = (char**) env;
 	  dup2(in[1],1);
 	  dup2(in[1],2);
 	  close(0);
@@ -700,7 +713,7 @@ hold_script(command)
 
 int
 hold_home(user)
-char *user;
+const char *user;
 {
 	struct passwd *pw;
 	struct stat st;
