@@ -39,13 +39,14 @@
 #include <sysexits.h>
 #include "mail.h"
 #include "malloc.h"
+#include "libz.h"
 #include "ta.h"
+
 #ifdef	HAVE_MMAP
 #include <sys/mman.h>
 #endif
 
-extern int cistrcmp();
-extern char *routermxes();
+extern char *routermxes __((char *, struct address *));
 extern int errno;
 
 #ifndef strrchr
@@ -108,16 +109,18 @@ ctlclose(dp)
 	}
 #ifdef	HAVE_MMAP
 	if (dp->let_buffer != NULL)
-	  munmap(dp->let_buffer, dp->let_end - dp->let_buffer);
+	  munmap((void*)dp->let_buffer, dp->let_end - dp->let_buffer);
+	dp->let_buffer = NULL;
 	if (dp->ctlmap != NULL)
-	  munmap(dp->ctlmap, dp->contentsize);
+	  munmap((void*)dp->ctlmap, dp->contentsize);
+	dp->ctlmap = NULL;
 #endif
 	if (dp->ctlfd >= 0)
 	  close(dp->ctlfd);
 	if (dp->msgfd >= 0)
 	  close(dp->msgfd);
 	if (dp->contents != NULL)
-	  free(dp->contents);
+	  free((void*)dp->contents);
 	if (dp->offset != NULL)
 	  free((char *)dp->offset);
 	for (ap = dp->senders; ap != NULL; ap = nextap) {
@@ -194,16 +197,28 @@ ctladdr(cp)
 	return ap;
 }
 
+#ifdef __STDC__
+
+struct ctldesc *
+ctlopen(const char *file, const char *channel, const char *host,
+	int *exitflagp, int (*selectaddr)(const char *, const char *, void *),
+	void *saparam,  int (*matchrouter)(const char *, struct address *, void *),
+	void *mrparam)
+
+#else
+
 struct ctldesc *
 ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrparam)
-	char *file, *channel, *host;
+	const char *file, *channel, *host;
 	int *exitflagp;
-	int (*selectaddr)();
+	int (*selectaddr)  __((const char *, const char *, void *));
 	void *saparam;
-	int (*matchrouter)();
+	int (*matchrouter) __((const char *, struct address *, void *));
 	void *mrparam;
+
+#endif
 {
-	register char *s;
+	register char *s, *contents;
 	char *mfpath;
 	int i, n;
 	struct address *ap;
@@ -224,18 +239,20 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 
 	memset(&d,0,sizeof(d));
 	if (*file >= 'A') {
+	  char *p;
 	  /* Has some hash subdirectory in front of itself */
 	  strncpy(dirprefix,file,sizeof(dirprefix));
 	  dirprefix[sizeof(dirprefix)-1] = 0;
-	  s = strrchr(dirprefix,'/');
-	  if (s) *++s = 0;
+	  p = strrchr(dirprefix,'/');
+	  if (p) *++p = 0;
 	  /*  "A/B/"  */
 	} else
 	    dirprefix[0] = 0;
 
 
 	d.msgfd = -1; /* The zero is not always good for your health .. */
-	if ((d.ctlfd = open(file, O_RDWR, 0)) < 0) {
+	d.ctlfd = open(file, O_RDWR, 0);
+	if (d.ctlfd < 0) {
 	  char cwd[MAXPATHLEN], buf[MAXPATHLEN+MAXPATHLEN+100];
 	  int e = errno;	/* Save it over the getwd() */
 
@@ -265,11 +282,12 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 	}
 	/* 4 is the minimum number of characters per line */
 	n = sizeof (long) * (stbuf.st_size / 4);
-	if ((d.contents = emalloc((u_int)stbuf.st_size+1)) == NULL
+	if ((d.contents = (s = emalloc((u_int)stbuf.st_size+1))) == NULL
 	    || (d.offset = (long *)emalloc((u_int)n)) == NULL) {
 	  warning("Out of virtual memory!", (char *)NULL);
 	  exit(EX_SOFTWARE);
 	}
+	contents = s;
 
 	fcntl(d.ctlfd, F_SETFD, 1); /* Close-on-exec */
 
@@ -290,28 +308,28 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 #else
 	d.ctlmap = NULL;
 #endif
-	d.contentsize = stbuf.st_size;
-	d.contents[d.contentsize] = 0; /* Treat it as a long string.. */
-	if (read(d.ctlfd, d.contents, d.contentsize) != d.contentsize) {
+	d.contentsize = (int) stbuf.st_size;
+	s[ d.contentsize ] = 0; /* Treat it as a long string.. */
+	if (read(d.ctlfd, s, d.contentsize) != d.contentsize) {
 	  warning("Wrong size read from control file \"%s\"! (%m)",
 		  file);
-	  free(d.contents);
-	  free((char *)d.offset);
+	  free((void*)d.contents);
+	  free((void*)d.offset);
 	  close(d.ctlfd);
 	  return NULL;
 	}
-	n = markoff(d.contents, d.contentsize, d.offset, file);
+	n = markoff(contents, d.contentsize, d.offset, file);
 	if (n < 4) {
 	  /*
 	   * If it is less than the minimum possible number of control
 	   * lines, then there is something wrong...
 	   */
-	  free(d.contents);
+	  free(contents);
 	  free((char *)d.offset);
 	  close(d.ctlfd);
 #ifdef HAVE_MMAP
 	  if (d.ctlmap != NULL)
-	    munmap(d.ctlmap, d.contentsize);
+	    munmap((void*)d.ctlmap, d.contentsize);
 #endif	  
 	  warning("Truncated or illegal control file \"%s\"!", file);
 	  /* exit(EX_PROTOCOL); */
@@ -338,7 +356,8 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 	for (i = 0; i < n; ++i) {
 	  if (*exitflagp && d.recipients == NULL)
 	    break;
-	  s = d.contents + d.offset[i];
+	  /* Shudder... we trash the memory block here.. */
+	  s = contents + d.offset[i];
 	  switch (*s) {
 	  case _CF_SENDER:
 	    ap = ctladdr(s+2);
@@ -386,7 +405,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 		    && !(*matchrouter)(host, ap, mrparam))
 #endif
 		|| (ap->routermxes == NULL && selectaddr != NULL
-		    && !(*selectaddr)(host, ap->host, saparam))
+		    && !(*selectaddr)(host, (const char *)ap->host, saparam))
 		|| (ap->routermxes == NULL && selectaddr == NULL
 		    && host != NULL && cistrcmp(host,ap->host) !=0)
 		|| !lockaddr(d.ctlfd, d.ctlmap, d.offset[i]+1,
@@ -645,10 +664,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 
 int
 ctlsticky(spec_host, addr_host, cbparam)
-	char *spec_host, *addr_host;
+	const char *spec_host, *addr_host;
 	void *cbparam;
 {
-	static char *hostref = NULL;
+	static const char *hostref = NULL;
 
 	if (hostref == NULL) {
 	  if (spec_host != NULL)
