@@ -22,9 +22,13 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <sys/stat.h>
-#include "mail.h"
-#include "ta.h"
 
+#include "mail.h"
+#include "malloc.h"
+#include "libz.h"
+#include "libc.h"
+
+#include "ta.h"
 
 /*
 
@@ -44,19 +48,14 @@
 
 int ta_logs_diagnostics = 1;
 
-
-extern char *strdup();
-extern void *emalloc();
-extern void *erealloc();
-
 char *notarybuf = NULL;
 time_t retryat_time = 0; /* Used at SMTP to avoid trying same host too soon.. */
 
 
-char *
+const char *
 notaryacct(rc,okstr)
 int rc;
-char *okstr;
+const char *okstr;
 {
 	static char msgbuf[20];
 	switch (rc) {
@@ -85,7 +84,7 @@ static char *wtthost = NULL; /* NOTARY wtthost: 'While-Talking-To -host' */
 static char *wttip   = NULL;
 
 void notary_setwtt(host)
-char *host;
+const char *host;
 {
 	if (wtthost) free(wtthost);
 	if (host)
@@ -95,7 +94,7 @@ char *host;
 }
 
 void notary_setwttip(ip)
-char *ip;
+const char *ip;
 {
 	if (wttip) free(wttip);
 	if (ip)
@@ -115,7 +114,7 @@ void
 notaryreport(arg1,arg2,arg3,arg4)
      const char *arg1, *arg2, *arg3, *arg4;
 {
-	char *fmt = "%s\001%s\001%s\001%s\001%s";
+	const char *fmt = "%s\001%s\001%s\001%s\001%s";
 	int len;
 	static char *A1 = NULL, *A2 = NULL, *A3 = NULL, *A4 = NULL;
 
@@ -144,13 +143,13 @@ notaryreport(arg1,arg2,arg3,arg4)
 #ifdef HAVE_STDARG_H
 #ifdef __STDC__
 void
-diagnostic(struct rcpt *rp, int rc, int timeout, char *fmt, ...)
+diagnostic(struct rcpt *rp, int rc, int timeout, const char *fmt, ...)
 #else /* Not ANSI-C */
 void
 diagnostic(rp, rc, timeout, fmt) /* (rp, rc, timeout, "fmtstr", remotemsg) */
 	struct rcpt *rp;
 	int rc, timeout;
-	char *fmt;
+	const char *fmt;
 #endif
 #else
 /*VARARGS*/
@@ -158,13 +157,13 @@ void
 diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remotemsg) */
 	struct rcpt *rp;
 	int rc, timeout;
-	char *fmt;
+	const char *fmt;
 	va_dcl
 #endif
 {
 	char	message[8192];
 	char	statmsgbuf[32+16];
-	char	*statmsg;
+	const char * statmsg;
 	char	mark;
 	register char *s, *es, *s2;
 	va_list	ap;
@@ -223,9 +222,12 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 
 	switch (rp->status) {
 	case EX_OK:
-		if (ta_logs_diagnostics)
-		  statmsg = "ok2";
-		else
+		if (ta_logs_diagnostics) {
+		  if (!(rp->notifyflgs & _DSN_NOTIFY_SUCCESS))
+		    statmsg = "ok3";
+		  else
+		    statmsg = "ok2";
+		} else
 		  statmsg = "ok";
 		mark = _CFTAG_OK;
 		logreport = ta_logs_diagnostics && report_notary;
@@ -279,7 +281,7 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 	     transport specification file ourselves */
 	  int oldfl = fcntl(rp->desc->ctlfd, F_GETFL);
 	  int len = 80 + strlen(notarybuf) + strlen(message);
-	  int rc;
+	  int rc2;
 	  long oldalarm;
 	  char *sbuf;
 
@@ -295,22 +297,24 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 #endif
 
 	  /* Log the diagnostic string to the file */
+	  len = -1;
 #ifndef SPRINTF_CHAR
 	  len = 
 #endif
-	    sprintf(sbuf, "%c%c%ld:%ld:%ld::%ld\t%s\t%s\n",
+	    sprintf(sbuf, "%c%c%d:%d:%d::%ld\t%s\t%s\n",
 		    _CF_DIAGNOSTIC, _CFTAG_NORMAL,
 		    rp->id, rp->headeroffset, rp->drptoffset,
-		    time(NULL), notarybuf, message);
+		    (long)time(NULL), notarybuf, message);
 #ifdef SPRINTF_CHAR
 	  len = strlen(sbuf);
 #endif
 	  oldalarm = alarm(0);	/* We do NOT want to be alarmed while
 				   writing to the log! */
 
-	  rc = write(rp->desc->ctlfd, sbuf, len);
-	  if (rc != len) {
+	  rc2 = write(rp->desc->ctlfd, sbuf, len);
+	  if (rc2 != len || rc2 < 0 || len < 0) {
 	    /* UAARGH! -- write failed, must have disk full! */
+	    printf("#HELP! diagnostic writeout with bad results!: len=%d, rc=%d\n", len, rc2);
 	    exit(EX_DATAERR);
 	  }
 	  fsync(rp->desc->ctlfd);
@@ -329,14 +333,14 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 	  *notarybuf = 0;
 	}
 
-	printf("%d/%ld\t%s\t%s %s\n",
+	printf("%d/%d\t%s\t%s %s\n",
 	       rp->desc->ctlid, rp->id,
 	       (notarybuf && report_notary) ? notarybuf : "",
 	       statmsg, message);
 
 	if (!lockaddr(rp->desc->ctlfd, rp->desc->ctlmap,
 		      rp->lockoffset, _CFTAG_LOCK, mark,
-		      rp->desc->msgfile, "?host?", getpid())) {
+		      (char*)rp->desc->msgfile, "?host?", getpid())) {
 	  /* something went wrong in unlocking it, concurrency problem? */
 	}
 	rp->lockoffset = 0;	/* mark this recipient unlocked */
