@@ -52,6 +52,8 @@
 #include <arpa/nameser.h>
 #include <resolv.h>
 
+#include "libc.h"
+
 #ifndef HFIXEDSZ	/* An ancient resolver (arpa/nameser.h) file ? */
 # define HFIXEDSZ 12
 #endif
@@ -170,29 +172,29 @@ typedef union {
     u_char buf[MAXPACKET];
 } querybuf;
 
-#define ALIGN      4 /* IPv4 works ok with 32-bit alignment */
+#define ALIGN_A    4 /* IPv4 works ok with 32-bit alignment */
 #define ALIGN_AAAA 8 /* IPv6 can use 64-bit machines more efficiently.. */
 
 extern int h_errno;
 extern FILE *verboselog;
 
 int
-getrr(host, ttlp, hbsize, rrtype, cnamelevel, verboselog)	/* getrrtype with completion */
+getrr(host, ttlp, hbsize, rrtype, cnamelevel, vlog)	/* getrrtype with completion */
 	char *host;
 	int *ttlp;
 	int hbsize;
 	int rrtype;
 	int cnamelevel;
-	FILE *verboselog;
+	FILE *vlog;
 {
 	int rval;
 	char buf[BUFSIZ], **domain;
 
-	if ((rval = getrrtype(host, ttlp, hbsize, rrtype, cnamelevel, verboselog)) > 0)
+	if ((rval = getrrtype(host, ttlp, hbsize, rrtype, cnamelevel, vlog)) > 0)
 	  return rval;
 	for (domain = _res.dnsrch; *domain != NULL; domain++) {
 	  sprintf(buf, "%s.%s", host, *domain);
-	  if ((rval = getrrtype(buf, ttlp, BUFSIZ, rrtype, cnamelevel, verboselog)) > 0) {
+	  if ((rval = getrrtype(buf, ttlp, BUFSIZ, rrtype, cnamelevel, vlog)) > 0) {
 	    strncpy(host, buf, hbsize<BUFSIZ?hbsize:BUFSIZ);
 	    host[hbsize - 1] = '\0';
 	    return rval;
@@ -203,13 +205,13 @@ getrr(host, ttlp, hbsize, rrtype, cnamelevel, verboselog)	/* getrrtype with comp
 
 
 int
-getrrtype(host, ttlp, hbsize, rrtype, cnamelevel, verboselog)
+getrrtype(host, ttlp, hbsize, rrtype, cnamelevel, vlog)
 	char *host;
 	int *ttlp;
 	int hbsize;
 	int rrtype;
 	int cnamelevel;
-	FILE *verboselog;
+	FILE *vlog;
 {
 
 	HEADER *hp;
@@ -251,8 +253,7 @@ getrrtype(host, ttlp, hbsize, rrtype, cnamelevel, verboselog)
 	  if (rrtype == T_CNAME && hp->rcode == NOERROR) {
 	    if (qdcount > 0 && strchr(host, '.') == NULL) {
 	      cp = (msgdata *)&answer + sizeof(HEADER);
-	      if (dn_expand((msgdata *)&answer,
-			    eom, cp, (u_char*)host, hbsize) >= 0) {
+	      if (dn_expand((msgdata *)&answer, eom, cp, host, hbsize) >= 0) {
 		if (host[0] == '\0') {
 		  host[0] = '.';
 		  host[1] = '\0';
@@ -313,15 +314,15 @@ getrrtype(host, ttlp, hbsize, rrtype, cnamelevel, verboselog)
 	      if (cp + (IPPORT_SMTP/8) < nextcp
 		  && (*(cp+(IPPORT_SMTP/8))
 		      & (0x80>>IPPORT_SMTP%8))) {
-		if (verboselog)
-		  fprintf(verboselog,"WKS: Found TCP/SMTP\n");
+		if (vlog)
+		  fprintf(vlog,"WKS: Found TCP/SMTP\n");
 		return 1;
 	      }
-	      if (verboselog)
-		fprintf(verboselog,"WKS: Found WKS of TCP, but not SMTP\n");
+	      if (vlog)
+		fprintf(vlog,"WKS: Found WKS of TCP, but not SMTP\n");
 	    } else
-	      if (verboselog)
-		fprintf(verboselog,"WKS: Found WKS other protocol, than TCP\n");
+	      if (vlog)
+		fprintf(vlog,"WKS: Found WKS other protocol, than TCP\n");
 	    cp = nextcp;
 	    continue;
 	  } else {
@@ -616,7 +617,7 @@ getanswer_r(answer, anslen, qname, qtype, result)
 			}
 
 			if (type == T_A)
-			  bp += ALIGN      - ((u_long)bp % ALIGN     );
+			  bp += ALIGN_A    - ((u_long)bp % ALIGN_A   );
 			else
 			  bp += ALIGN_AAAA - ((u_long)bp % ALIGN_AAAA);
 
