@@ -21,9 +21,9 @@
 #include "mail.h"
 #include "ta.h"
 
-#ifndef	L_SET
-#define	L_SET	0
-#endif	/* L_SET */
+#ifndef	SEEK_SET
+#define	SEEK_SET  0
+#endif	/* SEEK_SET */
 
 int
 lockaddr(fd, map, offset, was, new, file, host, mypid)
@@ -68,7 +68,7 @@ lockaddr(fd, map, offset, was, new, file, host, mypid)
 	  /* MMAP()ed block helps.. */
 	  memcpy(lockbuf,map+offset,sizeof(lockbuf));
 	} else {
-	  if (lseek(fd, offset, L_SET) < 0L) {
+	  if (lseek(fd, offset, SEEK_SET) < 0L) {
 	    warning("lockaddr: lseek() failure");
 	    return 0;
 	  }
@@ -80,7 +80,7 @@ lockaddr(fd, map, offset, was, new, file, host, mypid)
 	newlock = ((lockbuf[1] == ' ') ||
 		   (lockbuf[1] >= '0' && lockbuf[1] <= '9'));
 	if (lockbuf[0] == was) {
-	  if (!map && lseek(fd, offset, L_SET) < 0L) {
+	  if (!map && lseek(fd, offset, SEEK_SET) < 0L) {
 	    warning("lockaddr: lseek() failure 2");
 	    return 0;
 	  }
