@@ -4,9 +4,9 @@
  *
  */
 
+#include "hostenv.h"
 #include <stdio.h>
 #include <sys/param.h>
-#include "hostenv.h"
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/file.h>
@@ -14,12 +14,12 @@
 #include <unistd.h>
 
 
-void
+int
 fd_nonblockingmode(fd)
 	int fd;
 {
-	int i;
-	i = fcntl(fd, F_GETFL, 0);
+	int i, i2;
+	i2 = i = fcntl(fd, F_GETFL, 0);
 	if (i >= 0) {
 #ifdef O_NONBLOCK /* POSIXy thing */
 	  /* set up non-blocking I/O */
@@ -33,14 +33,15 @@ fd_nonblockingmode(fd)
 #endif
 	  i = fcntl(fd, F_SETFL, i);
 	}
+	return i2;
 }
 
-void
+int
 fd_blockingmode(fd)
 	int fd;
 {
-	int i;
-	i = fcntl(fd, F_GETFL, 0);
+	int i, i2;
+	i2 = i = fcntl(fd, F_GETFL, 0);
 	if (i >= 0) {
 #ifdef O_NONBLOCK /* POSIXy thing */
 	  /* set up blocking I/O */
@@ -54,5 +55,13 @@ fd_blockingmode(fd)
 #endif
 	  i = fcntl(fd, F_SETFL, i);
 	}
+	return i2;
 }
 
+
+void
+fd_restoremode(fd,mode)
+int fd, mode;
+{
+	fcntl(fd, F_SETFL, mode);
+}
