@@ -13,7 +13,7 @@
 #include "zsyslog.h"
 
 extern int errno;
-extern char *progname;
+extern const char *progname;
 
 #ifdef HAVE_VPRINTF
 
@@ -23,20 +23,22 @@ extern char *progname;
 # include <varargs.h>
 #endif
 
+#include "ta.h"
+
 #ifdef HAVE_STDARG_H
 #ifdef __STDC__
 void
-warning(char *fmt, ...)
+warning(const char *fmt, ...)
 #else /* Not ANSI-C */
 void
 warning(fmt)
-	char *fmt;
+	const char *fmt;
 #endif
 #else
 /*VARARGS*/
 void
 warning(fmt, va_alist) /* ("fmtstr", remotemsg) */
-	char *fmt;
+	const char *fmt;
 	va_dcl
 #endif
 {
