@@ -22,7 +22,7 @@ int
 writeheaders(rp,fp,newline,convertmode,maxwidth)
 	struct rcpt *rp;
 	FILE *fp;
-	char *newline;
+	const char *newline;
 	int convertmode, maxwidth;
 {
 	char **msgheaders = *(rp->newmsgheader);
