@@ -43,12 +43,15 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h> /* F_LOCK is there at some systems.. */
 #endif
+#include <string.h>
 #include "mail.h"
 #include "zsyslog.h"
 #include "zmsignal.h"
+
+#include "malloc.h"
+#include "libz.h"
 #include "ta.h"
 #include "splay.h"
-/* #include "malloc.h" */
 
 #ifdef  HAVE_WAITPID
 # include <sys/wait.h>
@@ -110,23 +113,23 @@ struct utimbuf {
 };
 #endif
 
-#ifndef	L_SET
-#define	L_SET	0
-#endif	/* !L_SET */
-#ifndef	L_XTND
-#define	L_XTND	2
-#endif	/* !L_XTND */
+#ifndef	SEEK_SET
+#define	SEEK_SET 0
+#endif	/* !SEEK_SET */
+#ifndef	SEEK_END
+#define	SEEK_END	2
+#endif	/* !SEEK_END */
 #ifndef	F_OK
 #define	F_OK	0
 #endif	/* !F_OK */
 
 #ifdef HAVE_LOCKF
-char *MBOXLOCKS_default = ".L:L";
+const char *MBOXLOCKS_default = ".L:L";
 #else
 #ifdef HAVE_FLOCK
-char *MBOXLOCKS_default = ".F:F";
+const char *MBOXLOCKS_default = ".F:F";
 #else
-char *MBOXLOCKS_default = ".:"; /* Huh ?? No lockf() nor flock() ??? */
+const char *MBOXLOCKS_default = ".:"; /* Huh ?? No lockf() nor flock() ??? */
 #endif
 #endif
 
@@ -183,7 +186,7 @@ char *MBOXLOCKS_default = ".:"; /* Huh ?? No lockf() nor flock() ??? */
  * order, first to last) but if the MAILBOX zenvariable exists it will
  * override the entire list.
  */
-char *maildirs[] = {
+const char *maildirs[] = {
 	"/var/mail",
 	"/usr/mail",
 	"/var/spool/mail",
@@ -263,21 +266,23 @@ struct writestate {
 struct sptree *spt_users = NULL;
 #endif /* HAVE_SOCKET -- no use to biff, if no socket available .. */
 
-char *defcharset;
-char *progname;
-char *channel;
-char *logfile;
+const char *defcharset;
+const char *progname;
+const char *channel;
+const char *logfile;
 FILE *logfp = NULL;
 FILE *verboselog = NULL;
-int readalready = 0;		/* does buffer contain valid message data? */
+#ifndef HAVE_MMAP
+int   readalready = 0;		/* does buffer contain valid message data? */
+#endif
 uid_t currenteuid;		/* the current euid */
 extern int nobody;		/* safe uid for file/program delivery */
-int dobiff = 1;			/* enable biff notification */
-int dorbiff = 0;		/*    "   rbiff   " */
-int dorbiff_always = 0;
-int keepatime = 1;		/* preserve mbox st_atime, for login etc. */
-int creatflag = 1;		/* attempt to create files that don't exist */
-int mime8bit = 0;		/* Has code which translates  MIME text/plain
+int  dobiff = 1;		/* enable biff notification */
+int  dorbiff = 0;		/*    "   rbiff   " */
+int  dorbiff_always = 0;
+int  keepatime = 1;		/* preserve mbox st_atime, for login etc. */
+int  creatflag = 1;		/* attempt to create files that don't exist */
+int  mime8bit = 0;		/* Has code which translates  MIME text/plain
 				   Quoted-Printable  to 8BIT.. */
 int keep_header8 = 0;		/* Headers can have 8-bit stuff ? */
 int convert_qp = 0;		/* Flag: We have a job.. */
@@ -287,52 +292,48 @@ char *mime_boundary = NULL;
 int   mime_boundary_len = 0;
 int mmdf_mode = 0;		/* Write out MMDF-style mail folder
 				   ("\001\001\001\001" as the separator..)  */
-long eofindex = -1;		/* When negative, putmail() can't truncate() */
-int dirhashes = 0;
+long eofindex  = -1;		/* When negative, putmail() can't truncate() */
+int  dirhashes = 0;
 
 extern RETSIGTYPE wantout __((int));
-extern time_t time();
 extern int optind;
 extern char *optarg;
-extern int emptyline();
-extern void biff __((char *, char *, long));
+extern void biff __((const char *, const char *, long));
 extern void rbiff __((struct biffer *));
-extern void prversion __((const char *)), getnobody();
+extern void prversion __((const char *));
 #if 0
 extern struct passwd *getpwnam __((const char *));
 extern struct passwd *getpwuid __((uid_t));
 #endif
 extern int setupuidgid __((struct rcpt *, int, int));
-extern int createfile __((struct rcpt *, char *, uid_t));
-extern int exstat __((struct rcpt *, char *, struct stat *, int (*)() ));
-extern int creatembox __((struct rcpt *, char *, char **, uid_t*, gid_t*, struct passwd *));
-extern char *exists __((char *, char *, struct rcpt *));
-extern void setrootuid();
+extern int createfile __((struct rcpt *, const char *, int));
+extern int exstat __((struct rcpt *, const char *, struct stat *, int (*)(const char*, struct stat*) ));
+extern int creatembox __((struct rcpt *, const char *, char **, uid_t*, gid_t*, struct passwd *));
+extern char *exists __((const char *, const char *, struct rcpt *));
+extern void setrootuid __((struct rcpt *));
 extern void process __((struct ctldesc *dp));
-extern void deliver __((struct ctldesc *dp, struct rcpt *rp, char *userbuf, char *timestring));
-extern void program __((struct ctldesc *dp, struct rcpt *rp, char *userbuf, char *timestring));
-extern FILE *putmail __((struct ctldesc *dp, struct rcpt *rp, int fdmail, char *fdopmode, char *timestring, char *file));
-extern int appendlet __((struct ctldesc *dp, struct rcpt *rp, FILE *fp, char *file, int ismime));
+extern void deliver __((struct ctldesc *dp, struct rcpt *rp, const char *userbuf, const char *timestring));
+extern void program __((struct ctldesc *dp, struct rcpt *rp, const char *userbuf, const char *timestring));
+extern FILE *putmail __((struct ctldesc *dp, struct rcpt *rp, int fdmail, const char *fdopmode, const char *timestring, const char *file));
+extern int appendlet __((struct ctldesc *dp, struct rcpt *rp, FILE *fp, const char *file, int ismime));
 extern char **environ;
-extern int writebuf __((struct writestate *, char *buf, int len));
-extern int writemimeline __((struct writestate *, char *buf, int len));
+extern int writebuf __((struct writestate *, const char *buf, int len));
+extern int writemimeline __((struct writestate *, const char *buf, int len));
 
 static int do_return_receipt_to = 0;
-static void  return_receipt __((struct ctldesc *dp, char *retrecpaddr, char *uidstr));
-static char *find_return_receipt_hdr __((struct rcpt *rp));
+static void  return_receipt __((struct ctldesc *dp, const char *retrecpaddr, const char *uidstr));
+static const char *find_return_receipt_hdr __((struct rcpt *rp));
 
 extern int  qp_to_8bit   __((struct rcpt *rp));
 extern int  qptext_check __((struct rcpt *rp));
 
-extern char *rfc822date __((time_t *));
-
 extern void hp_init();
 extern char **hp_getaddr();
 extern int errno;
 #ifndef strchr
 extern char *strchr(), *strrchr();
 #endif
-extern int  lstat(); /* remaps to  stat() if USE_LSTAT is not defined.. */
+extern int  lstat __((const char *, struct stat *)); /* remaps to  stat() if USE_LSTAT is not defined.. */
 extern FILE *fdopen();
 #ifndef MALLOC_TRACE
 extern void * emalloc __((size_t));
@@ -346,12 +347,14 @@ int	D_alloc = 0;
 static int readrwho __((void));
 #endif
 
+static void sig_alrm __((int));
 static void sig_alrm(sig)
 int sig;
 {
 	/* Sigh, actually dummy routine.. */
 }
 
+static int free_spu __((void *));
 static int
 free_spu(p)
      void *p;
@@ -368,7 +371,7 @@ free_spu(p)
 int
 main(argc, argv)
 	int argc;
-	char *argv[];
+	const char *argv[];
 {
 	char file[MAXPATHLEN+128];
 	char *s;
@@ -380,7 +383,7 @@ main(argc, argv)
 #endif
 	struct ctldesc *dp;
 
-	RETSIGTYPE (*oldsig)();
+	RETSIGTYPE (*oldsig) __((int));
 
 	SIGNAL_HANDLESAVE(SIGINT, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
@@ -399,7 +402,8 @@ main(argc, argv)
 
 	SIGNAL_IGNORE(SIGPIPE);
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
+	progname = strrchr(argv[0], '/');
+	if (progname == NULL)
 	  progname = argv[0];
 	else
 	  ++progname;
@@ -412,7 +416,10 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:Dgh:Hl:rRSMV8")) != EOF) {
+	while (1) {
+	  c = getopt(argc, (char*const*)argv, "abc:Dgh:Hl:rRSMV8");
+	  if (c == EOF)
+	    break;
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -466,7 +473,7 @@ main(argc, argv)
 	  }
 	}
 	if (errflg || optind != argc) {
-	  fprintf(stderr, "Usage: %s [-V8abDr] [-c channel] [-h host]\n",
+	  fprintf(stderr, "Usage: %s [-8abDgHMrSV] [-l logfile] [-c channel] [-h host]\n",
 		  argv[0]);
 	  exit(EX_USAGE);
 	}
@@ -577,7 +584,7 @@ main(argc, argv)
 	    free(nbp->user);
 	    free(nbp);
 	  }
-	  sp_scan(free_spu,NULL,spt_users);
+	  sp_scan( (int(*)__((struct spblk*))) free_spu, NULL, spt_users);
 	  free((void*)spt_users);
 	  spt_users = NULL;
 #endif	/* HAVE_SOCKET */
@@ -587,6 +594,7 @@ main(argc, argv)
 	return 0;
 }
 
+static void rfc822localize __((char *));
 static void
 rfc822localize(user)
 	char *user;
@@ -623,7 +631,7 @@ void
 process(dp)
 	struct ctldesc *dp;
 {
-	char *cp, *ts;
+	char *ts;
 	struct rcpt *rp;
 	time_t curtime;
 	int is_mime_qptext = 0;
@@ -645,6 +653,14 @@ process(dp)
 	 * and strip the quotes from around a name.
 	 */
 
+#ifndef HAVE_MMAP
+	readalready = 0; /* ignore any previous message data cache */
+#endif
+
+#ifndef HAVE_MMAP
+	readalready = 0; /* ignore any previous message data cache */
+#endif
+
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 
 	  userlen = strlen(rp->addr->user);
@@ -663,12 +679,11 @@ process(dp)
 
 	  rfc822localize(user);
 
-	  readalready = 0; /* ignore any previous message data cache */
 	  time(&curtime);
 	  ts = ctime(&curtime);
 
 	  /* seek to message body start on each iteration */
-	  if (lseek(dp->msgfd, (off_t)dp->msgbodyoffset, L_SET) < 0L) {
+	  if (lseek(dp->msgfd, (off_t)dp->msgbodyoffset, SEEK_SET) < 0L) {
 	    warning("Cannot seek to message body in %s! (%m)",
 		    dp->msgfile);
 	    DIAGNOSTIC(rp, user, EX_TEMPFAIL,
@@ -704,9 +719,10 @@ process(dp)
  * propably_x400() -- some heuristics to see if this is likely
  *		      a mis-written X.400 address
  */
+int propably_x400 __((const char *));
 int
 propably_x400(addr)
-	char *addr;
+	const char *addr;
 {
 	int slashes = 0;
 	int equs = 0;
@@ -740,18 +756,20 @@ propably_x400(addr)
 
 int havedotlock = 0, havemaillock = 0;
 
+int acquire_mboxlock __((struct rcpt *, const char *, int));
 int
-acquire_mboxlock(rp,file,uid)
+acquire_mboxlock(rp,file,iuid)
 struct rcpt *rp;
-char *file;
-uid_t uid;
+const char *file;
+int iuid;
 {
 #ifdef	HAVE_MAILLOCK_H
-	char *maillockuser;
+	const char *maillockuser;
 	struct passwd *pw;
 	int i;
 	char errbuf[30];
-	char *cp;
+	const char *cp;
+	uid_t uid = (uid_t) iuid;
 
 	if (*(rp->addr->user) == TO_FILE) {
 	  if ((pw = getpwuid(uid)) == NULL) {
@@ -767,7 +785,8 @@ uid_t uid;
 	else
 	  maillockuser = rp->addr->user;
 
-	switch ((i = maillock(maillockuser, 2))) {
+	i = maillock((char*)maillockuser, 2); /* Sigh.. proto vs. man-page */
+	switch (i) {
 	case L_SUCCESS:
 	  cp = NULL;
 	  break;
@@ -809,23 +828,34 @@ uid_t uid;
 	havedotlock = (dotlock(file) == 0);
 	if (!havedotlock) {
 	  char mbuf[256];
-	  sprintf(mbuf, "\"%s\": %s", file, strerror(errno));
+	  int rc, err = errno;
+	  sprintf(mbuf, "\"%s\": %s", file, strerror(err));
 	  notaryreport(file,"failed",
 		       "5.4.5 (File locking with dotlock failed)",
 		       "x-local; 550 (File locking with dotlock failed)");
-	  DIAGNOSTIC(rp, file, errno == EBUSY ? EX_TEMPFAIL :
-		     (errno == EACCES ? EX_NOPERM : EX_UNAVAILABLE),
-		     "can't dotlock %s", mbuf);
+	  rc = EX_UNAVAILABLE;
+	  switch (err) {
+	  case EBUSY:
+	    rc = EX_TEMPFAIL;
+	    break;
+	  case EACCES:
+	    rc = EX_NOPERM;
+	    break;
+	  default:
+	    break;
+	  }
+	  DIAGNOSTIC(rp, file, rc, "can't dotlock %s", mbuf);
 	  return 1;
 	}
 #endif  /* HAVE_DOTLOCK */
 	return 0;
 }
 
+int acquire_nfsmboxlock __((struct rcpt *, const char *));
 int
 acquire_nfsmboxlock(rp,file)
 struct rcpt *rp;
-char *file;
+const char *file;
 {
 #ifdef	USE_NFSMBOX
 	if (nfslock(file, LOCK_EX) != 0) {
@@ -842,15 +872,16 @@ char *file;
 }
 
 
+int acquire_lockflock __((int, struct rcpt *, const char *));
 int
 acquire_lockflock(fdmail,rp,file)
 int fdmail;
 struct rcpt *rp;
-char *file;
+const char *file;
 {
 #if defined(HAVE_LOCKF) && defined(F_LOCK)
 	/* Seek to begining for locking! [mea@utu.fi] */
-	if (lseek(fdmail, (off_t)0, L_SET) < 0 ||
+	if (lseek(fdmail, (off_t)0, SEEK_SET) < 0 ||
 	    lockf(fdmail, F_LOCK, 0) < 0) {
 	  alarm(0);
 	  notaryreport(file,"failed",
@@ -860,16 +891,17 @@ char *file;
 		     "can't lockf() \"%s\"", file);
 	  return 1;
 	}
-	lseek(fdmail, (off_t)0, L_XTND); /* To the end of the file */
+	lseek(fdmail, (off_t)0, SEEK_END); /* To the end of the file */
 #endif	/* HAVE_LOCKF */
 	return 0;
 }
 
+int acquire_flocklock __((int, struct rcpt *, const char *));
 int
 acquire_flocklock(fdmail,rp,file)
 int fdmail;
 struct rcpt *rp;
-char *file;
+const char *file;
 {
 #ifdef	HAVE_FLOCK
 	if (flock(fdmail, LOCK_EX) < 0) {
@@ -894,18 +926,19 @@ void
 deliver(dp, rp, usernam, timestring)
 	struct ctldesc *dp;
 	struct rcpt *rp;
-	char *usernam;
-	char *timestring;
+	const char *usernam;
+	const char *timestring;
 {
-	register char **maild;
+	register const char **maild;
 	int fdmail;
 	uid_t uid;
 	FILE *fp;
 	int hasdir;
 	struct stat st, s2;
-	char *file = NULL, *cp, *plus;
-	char *retrecptaddr;
-	int ismbox = 0, i;
+	const char *file = NULL;
+	char *cp, *plus;
+	const char *retrecptaddr;
+	int ismbox = 0;
 #if	defined(HAVE_SOCKET)
 	struct biffer *nbp = NULL;
 #ifdef	HAVE_PROTOCOLS_RWHOD_H
@@ -913,9 +946,9 @@ deliver(dp, rp, usernam, timestring)
 #endif
 #endif
 	struct passwd *pw;
-	char *mboxlocks = getzenv("MBOXLOCKS");
-	char *filelocks = NULL;
-	char *locks;
+	const char *mboxlocks = getzenv("MBOXLOCKS");
+	const char *filelocks = NULL;
+	const char *locks     = NULL;
 	time_t starttime, endtime;
 
 	time(&starttime);
@@ -1014,11 +1047,10 @@ deliver(dp, rp, usernam, timestring)
 	  if ((pw = getpwnam(usernam)) == NULL) {
 
 	    /* No match as is ?  Lowercasify, and try again! */
-	    for (cp = usernam ; *cp != '\0'; ++cp)
-	      if (isascii(*cp) && isupper(*cp))
-		*cp = tolower(*cp);
+	    strlower((char*)usernam);
 
-	    if ((pw = getpwnam(usernam)) == NULL) {
+	    pw = getpwnam(usernam);
+	    if (pw == NULL) {
 	      if (plus) *plus = '+';
 	      if (propably_x400(usernam)) {
 
@@ -1048,22 +1080,24 @@ deliver(dp, rp, usernam, timestring)
 	    }
 	  }
 	  hasdir = 0;
-	  for (maild = &maildirs[0]; *maild != 0; maild++) {
+	  for (maild = maildirs; *maild != 0; maild++) {
 	    if (stat(*maild,&st) < 0 ||
 		!S_ISDIR(st.st_mode))
 	      /* Does not exist, or is not a directory */
 	      continue;
 	    hasdir = 1;
-	    if ((file = exists(*maild, usernam, rp)) != NULL)
+	    file = exists(*maild, pw->pw_name, rp);
+	    if (file != NULL)
 	      break;		/* found it */
 	    if (rp->status != EX_OK)
 	      return;
 	  }
 
 	  if (hasdir && *maild == 0 &&
-	      !creatembox(rp, usernam, &file, &st.st_uid, &st.st_gid, pw))
+	      !creatembox(rp, pw->pw_name, (char**)&file,
+			  &st.st_uid, &st.st_gid, pw))
 	    return; /* creatembox() sets status */
-
+	  
 	  if (!hasdir) {	/* No directory ?? */
 
 	    char *mailbox = getzenv("MAILBOX");
@@ -1286,7 +1320,7 @@ deliver(dp, rp, usernam, timestring)
 	        case 'L':
 		case 'l':
 #if defined(HAVE_LOCKF) && defined(F_LOCK) /* If one, also the other */
-		  lseek(fdmail,(off_t)0,L_SET);
+		  lseek(fdmail,(off_t)0,SEEK_SET);
 		  lockf(fdmail, F_ULOCK, 0);
 #endif	/* HAVE_LOCKF */
 		  break;
@@ -1312,7 +1346,7 @@ deliver(dp, rp, usernam, timestring)
 
 
 	/* Where is the end of file -- before we write anything */
-	eofindex = lseek(fdmail, (off_t)0, L_XTND);
+	eofindex = lseek(fdmail, (off_t)0, SEEK_END);
 #if	defined(HAVE_SOCKET)
 	if (nbp != NULL)
 	  nbp->offset = eofindex;
@@ -1354,7 +1388,7 @@ deliver(dp, rp, usernam, timestring)
 	    case 'L':
 	    case 'l':
 #if defined(HAVE_LOCKF) && defined(F_LOCK)
-	      lseek(fdmail,(off_t)0,L_SET);
+	      lseek(fdmail,(off_t)0,SEEK_SET);
 	      lockf(fdmail, F_ULOCK, 0);
 #endif	/* HAVE_LOCKF */
 	      break;
@@ -1405,13 +1439,13 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	struct ctldesc *dp;
 	struct rcpt *rp;
 	int fdmail;
-	char *fdopmode, *timestring, *file;
+	const char *fdopmode, *timestring, *file;
 {
 	FILE *fp;
 	int len, rc, mw=0;
 	char buf[2];
 	struct stat st;
-	char *fromuser;
+	const char *fromuser;
 	int lastch = 0xFFF;
 
 	fstat(fdmail, &st);
@@ -1442,7 +1476,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	} else if (eofindex > 0L && eofindex < (sizeof "From x\n")) {
 	  /* A mail message *cannot* be this small.  It must be
 	     a corrupted mailbox file.  Ignore the trash bytes. */
-	  fseek(fp, (off_t)0, L_SET);
+	  fseek(fp, (off_t)0, SEEK_SET);
 	  eofindex = 0;
 	}
 
@@ -1564,9 +1598,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	   * login etc. can distinguish new mail from old.
 	   * The mtime will be set to now by the following write() calls.
 	   */
-	  fseek(fp, (off_t)-2, L_XTND);
+	  fseek(fp, (off_t)-2, SEEK_END);
 	  len = fread(buf, 1, 2, fp);
-	  fseek(fp, (off_t)0, L_XTND);	/* to end of file, again */
+	  fseek(fp, (off_t)0, SEEK_END);	/* to end of file, again */
 
 	  if (len == 1 || len == 2) {
 	    int err;
@@ -1633,6 +1667,9 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	fsync(fdmail);
 #endif
 
+
+      time_reset:
+
 	/* This is actually Linux-thing, we reset the last access time
 	   to be that of before we did read the file, last modification
 	   into current moment -- At Linux 1.3.8x the system is a bit
@@ -1655,8 +1692,6 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  fflush(logfp);
 	}
 
-      time_reset:
-
 	if (*(rp->addr->user) != TO_PIPE && keepatime) {
 #ifdef	HAVE_UTIME
 	  struct utimbuf tv;
@@ -1680,13 +1715,14 @@ void
 program(dp, rp, user, timestring)
 	struct ctldesc *dp;
 	struct rcpt *rp;
-	char *user;
-	char *timestring;
+	const char *user;
+	const char *timestring;
 {
 	int i, pid, in[2], out[2];
 	int uid = -1;
 	int gid = -1;
-	char *env[20], buf[8192], *cp, *s, *cpe;
+	const char *env[20], *s;
+	char buf[8192], *cp, *cpe;
 	int status;
 	struct passwd *pw;
 	FILE *errfp;
@@ -1717,7 +1753,8 @@ program(dp, rp, user, timestring)
 		     "Bad privilege for a pipe \"%s\"", rp->addr->misc);
 	  return;
 	}
-	if ((pw = getpwuid(uid)) == NULL) {
+	pw = getpwuid(uid);
+	if (pw == NULL) {
 	  gid = 0;
 	  env[i++] = "HOME=/tmp";
 	  env[i++] = "USER=anonymous";
@@ -1791,8 +1828,9 @@ program(dp, rp, user, timestring)
 	  return;
 	}
 
-	if ((pid = fork()) == 0) { /* child */
-	  environ = env;
+	pid = fork();
+	if (pid == 0) { /* child */
+	  environ = (char**)env;
 	  setgid(gid);
 	  setuid(uid);
 	  close(in[0]);
@@ -1891,13 +1929,13 @@ program(dp, rp, user, timestring)
 int
 creatembox(rp, uname, filep, uid, gid, pw)
 	struct rcpt *rp;
-	char *uname;
+	const char *uname;
 	char **filep;
 	uid_t *uid;
 	gid_t *gid;
 	struct passwd *pw;
 {
-	char **maild;
+	const char **maild;
 	int fd = -1;
 	char *s;
 
@@ -1928,7 +1966,8 @@ creatembox(rp, uname, filep, uid, gid, pw)
 	    }
 	  }
 	  strcat(s, uname);
-	  if ((fd = createfile(rp, *filep, *uid)) >= 0) {
+	  fd = createfile(rp, *filep, *uid);
+	  if (fd >= 0) {
 #ifdef	HAVE_FCHOWN
 	    fchown(fd, *uid, *gid);
 #else  /* !HAVE_FCHOWN */
@@ -1974,14 +2013,15 @@ creatembox(rp, uname, filep, uid, gid, pw)
 }
 
 int
-createfile(rp, file, uid)
+createfile(rp, file, iuid)
 	struct rcpt *rp;
-	char *file;
-	uid_t uid;
+	const char *file;
+	int iuid;
 {
 	int fd, i = 0, saverrno;
 	struct stat st;
 	char *cp, msg[BUFSIZ];
+	uid_t uid = iuid;
 
 	if ((fd = open(file, O_RDWR|O_CREAT|O_EXCL, MAILMODE)) < 0) {
 	  saverrno = errno;
@@ -2084,8 +2124,8 @@ setupuidgid(rp, uid, gid)
 
 char *
 exists(maildir, uname, rp)
-	char *maildir;
-	char *uname;
+	const char *maildir;
+	const char *uname;
 	struct rcpt *rp;
 {
 	char *file;
@@ -2138,14 +2178,14 @@ appendlet(dp, rp, fp, file, ismime)
 	struct rcpt *rp;
 	FILE *fp;
 	int ismime;
-	char *file;
+	const char *file;
 {
 	struct writestate WS;
-	char *s; /* message body start */
+	const char *s; /* message body start */
 
 #ifndef	HAVE_MMAP
 	char buffer[BUFSIZ+1];
-	char *bufend = buffer+BUFSIZ;
+	const char *bufend = buffer+BUFSIZ;
 	register int i;
 	register int bufferfull;
 	char iobuf[BUFSIZ]; /* For setvbuf(mfp) */
@@ -2167,7 +2207,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  /* Split it to lines.. */
 	  if (readalready > 0) {
 	    int linelen, readidx = 0;
-	    char *s0 = buffer, *s;
+	    const char *s0 = buffer, *s;
 	    while (readidx < readalready) {
 	      s = s0;
 	      linelen = 0;
@@ -2196,6 +2236,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  /* we are assuming to be positioned properly
 	     at the start of the message body */
 	  bufferfull = 0;
+	  /* We really can't use the 'let_buffer' cache here */
 	  readalready = 0;
 	  i = 0;
 	  while ((i = cfgets(buffer, BUFSIZ, mfp)) != EOF) {
@@ -2231,17 +2272,23 @@ appendlet(dp, rp, fp, file, ismime)
 	    return WS.lastch;
 	  }
 
-	  /* we are assumed to be positioned properly
-	     at the start of the message body */
+	  /* Make sure we are properly positioned at the start
+	     of the message body */
+	  lseek(mfd, dp->msgbodyoffset, SEEK_SET);
 	  bufferfull = 0;
-	  while ((i = read(mfd, buffer, BUFSIZ)) != 0) {
+	  while (1) {
+	    i = read(mfd, buffer, BUFSIZ);
+	    if (i == 0)
+	      break;
 	    if (i < 0) {
 	      DIAGNOSTIC(rp, file, EX_IOERR,
 			 "read error from message file", 0);
+	      readalready = 0;
 	      return -256;
 	    }
 	    if (writebuf(&WS, buffer, i) != i) {
 	      DIAGNOSTIC(rp, file, EX_IOERR, "write to \"%s\" failed", file);
+	      readalready = 0;
 	      return -256;
 	    }
 	    readalready = i;
@@ -2257,7 +2304,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  int i;
 
 	  while (s < dp->let_end) {
-	    char *s2 = s;
+	    const char *s2 = s;
 	    i = 0;
 	    while (s2 < dp->let_end && *s2 != '\n') ++s2, ++i;
 	    if (s2 < dp->let_end && *s2 == '\n') ++s2, ++i;
@@ -2286,9 +2333,9 @@ appendlet(dp, rp, fp, file, ismime)
 int
 exstat(rp, file, stbufp, statfcn)
 	struct rcpt *rp;
-	char *file;
+	const char *file;
 	struct stat *stbufp;
-	int (*statfcn)();
+	int (*statfcn) __((const char *, struct stat *));
 {
 	if (statfcn(file, stbufp) < 0) {
 	  if (TEMPFAIL(errno))
@@ -2304,7 +2351,7 @@ exstat(rp, file, stbufp, statfcn)
 #if	defined(HAVE_SOCKET)
 void
 biff(hostname, username, offset)
-	char *hostname, *username;
+	const char *hostname, *username;
 	long offset;
 {
 	int f;
@@ -2319,7 +2366,7 @@ biff(hostname, username, offset)
 	/* Could do this with  getservbyname() - but sometimes (with NIS
 	   on loaded Suns) it takes AGES...  Hardwiring works as well.   */
 
-	symid = symbol((void*)hostname);
+	symid = symbol((const void*)hostname);
 	if (spt_hosts == NULL)
 	  spt_hosts = sp_init();
 	spl = sp_lookup(symid, spt_hosts);
@@ -2357,16 +2404,14 @@ readrwho()
 #define NMAX sizeof(outmp.out_name)
 #define LMAX sizeof(outmp.out_line)
 
-	int cc, width, f, n, i;
+	int cc, f, n;
 	spkey_t symid;
 	register struct whod *w = &wd;
 	register struct whoent *we;
-	register char *tp, *s;
-	register FILE *fi;
 	long now;
 	char username[NMAX+1];
-	DIR *dirp, *opendir();
-	struct dirent *dp, *readdir();
+	DIR *dirp;
+	struct dirent *dp;
 	struct userhost *uhp;
 	struct spblk *spl;
 	
@@ -2404,7 +2449,8 @@ readrwho()
 	    /* add to data structure */
 
 	    symid = symbol((void*)username);
-	    if ((spl = sp_lookup(symid, spt_users)) == NULL)
+	    spl = sp_lookup(symid, spt_users);
+	    if (spl == NULL)
 	      continue;
 	    uhp = (struct userhost *)spl->data;
 	    if (uhp != NULL
@@ -2412,9 +2458,9 @@ readrwho()
 	      continue;
 
 	    uhp = (struct userhost *)emalloc(sizeof (struct userhost));
-	    uhp->next = (struct userhost *)spl->data;
+	    uhp->next     = (struct userhost *)spl->data;
 	    uhp->hostname = strdup(w->wd_hostname);
-	    spl->data = (unsigned char *)uhp;
+	    spl->data     = (void *)uhp;
 	  }
 	  close(f);
 	}
@@ -2447,25 +2493,15 @@ rbiff(nbp)
 int
 writebuf(WS, buf, len)
 	struct writestate *WS;
-	char *buf;
+	const char *buf;
 	int len;
 {
-	register char *cp;
+	register const char *cp;
 	register int n;
-	int wlen, tlen, i = 0;
+	int   tlen;
 	register char expect;
 	char *fromp;
-#if 0
-	static char save = '\0';
-	static char frombuf[8];
 
-	if (buf == NULL) {	/* magic initialization */
-	  save = mmdf_mode ? 0 : 'F';
-	  frombuf[0] = 0;
-	  fromp = frombuf;
-	  return 0;
-	}
-#endif
 	/* -------------------------------------------- */
 	/* Non-MIME processing				*/
 
@@ -2525,26 +2561,15 @@ writebuf(WS, buf, len)
 int
 writemimeline(WS, buf, len)
 	struct writestate *WS;
-	char *buf;
+	const char *buf;
 	int len;
 {
-	register char *cp, *s;
+	register char *s;
+	register const char *cp;
 	register int n;
-	int wlen, tlen, i = 0;
+	int   tlen = 0, i = 0;
 	char *buf2;
-#if 0
-	static char save = '\0';
 
-	static char *buf2 = NULL;
-	static int buf2len = 0;
-
-	if (buf == NULL) {	/* magic initialization */
-	  save = mmdf_mode ? 0 : 'F';
-	  frombuf[0] = 0;
-	  fromp = frombuf;
-	  return 0;
-	}
-#endif
 #ifdef	USE_ALLOCA
 	WS->buf2 = alloca(len+1);
 #else
@@ -2559,68 +2584,71 @@ writemimeline(WS, buf, len)
 #endif
 	buf2 = WS->buf2;
 
+#if 0 /* Not yet */
 	/* ------------------------------------------------------------ */
 	/* MIME format processing, the input is LINE BY LINE */
 	if (mime_boundary != NULL && len > mime_boundary_len+2 &&
 	    buf[0] == '-' && buf[1] == '-' &&
-	    strncmp(buf+2,mime_boundary,mime_boundary_len) == 0) {
+	    strncmp(buf+2, mime_boundary, mime_boundary_len) == 0) {
 	  /* ------------------------------------------------------------ */
 	  /* XX: process MIME boundary! */
 	  WS->lastch = buf[len-1];
 	  return fwrite(buf, 1, len, WS->fp);
 
-	} else if (convert_qp) {
-	  /* ------------------------------------------------------------ */
-	  /* Process Q-P chars of this line */
-	  int qp_hex = 0;
-	  int qp_chrs = 0;
-	  int tlen = 0;
+	} else
+#endif
+	  if (convert_qp) {
+	    /* ------------------------------------------------------------ */
+	    /* Process Q-P chars of this line */
+	    int qp_hex = 0;
+	    int qp_chrs = 0;
 
-	  for (cp = buf, s = buf2, n = len; n > 0; --n, ++cp) {
-	    register int c = *cp;
-	    /* At first, convert the possible QP character.. */
-	    if (!qp_chrs && c == '=') {
-	      qp_chrs = 2;
-	      qp_hex  = 0;
-	      continue;		/* '=' starts something */
+	    for (cp = buf, s = buf2, n = len; n > 0; --n, ++cp) {
+	      register int c = *cp;
+	      /* At first, convert the possible QP character.. */
+	      if (!qp_chrs && c == '=') {
+		qp_chrs = 2;
+		qp_hex  = 0;
+		continue;		/* '=' starts something */
+	      }
+	      if (qp_chrs && c == '\n') {
+		qp_chrs = 0;	/* It was "=[ \t]*\n", which was thrown away.*/
+		continue;
+	      }
+	      if (qp_chrs == 2 && (c == ' ' || c == '\t'))
+		continue;		/* It is "=[ \t]*\n", throw it away */
+	      /* Ok, this may be of  =HexHex ? */
+	      if (qp_chrs && ((c >= '0' && c <= '9') ||
+			      (c >= 'A' && c <= 'F') ||
+			      (c >= 'a' && c <= 'f'))) {
+		qp_hex <<= 4;
+		if (c >= '0' && c <= '9') qp_hex += (c - '0');
+		if (c >= 'A' && c <= 'F') qp_hex += (c - 'A' + 10);
+		if (c >= 'a' && c <= 'f') qp_hex += (c - 'a' + 10);
+		--qp_chrs;
+		if (!qp_chrs) c = qp_hex;
+		else continue;	/* Go collect another digit */
+	      } else if (qp_chrs)
+		qp_chrs = 0;	/* Failed a HEX check.. */
+	      /* Now we have a converted character at `c', save it! */
+	      *s++ = c;
+	      ++tlen;
 	    }
-	    if (qp_chrs && c == '\n') {
-	      qp_chrs = 0;	/* It was "=[ \t]*\n", which was thrown away.*/
-	      continue;
-	    }
-	    if (qp_chrs == 2 && (c == ' ' || c == '\t'))
-	      continue;		/* It is "=[ \t]*\n", throw it away */
-	    /* Ok, this may be of  =HexHex ? */
-	    if (qp_chrs && ((c >= '0' && c <= '9') ||
-			    (c >= 'A' && c <= 'F') ||
-			    (c >= 'a' && c <= 'f'))) {
-	      qp_hex <<= 4;
-	      if (c >= '0' && c <= '9') qp_hex += (c - '0');
-	      if (c >= 'A' && c <= 'F') qp_hex += (c - 'A' + 10);
-	      if (c >= 'a' && c <= 'f') qp_hex += (c - 'a' + 10);
-	      --qp_chrs;
-	      if (!qp_chrs) c = qp_hex;
-	      else continue;	/* Go collect another digit */
-	    } else if (qp_chrs)
-	      qp_chrs = 0;	/* Failed a HEX check.. */
-	    /* Now we have a converted character at `c', save it! */
-	    *s++ = c;
-	    ++tlen;
-	  }
-	  /* Uhh... Stored the conversion result into buf2[], length  tlen */
-	  if (!mmdf_mode && buf2[0] == 'F' && strncmp(buf2,"From ",5)==0)
-	    if (WS->lastch >= 256 || WS->lastch == '\n')
-	      if (putc('>',WS->fp) == EOF)
-		return -1;
-	  if (tlen > 0) {
-	    i = fwrite(buf2, 1, tlen, WS->fp);
-	    WS->lastch = buf2[tlen-1];
-	  } else
-	    i = 0;
-	  if (i != tlen) return -1;
-	  return len;		/* Return the incoming length,
+	    /* Uhh... Stored the conversion result into buf2[], length  tlen */
+	    if (!mmdf_mode && tlen >= 5 &&
+		buf2[0] == 'F' && memcmp(buf2,"From ",5)==0)
+	      if (WS->lastch >= 256 || WS->lastch == '\n')
+		if (putc('>',WS->fp) == EOF)
+		  return -1;
+	    if (tlen > 0) {
+	      i = fwrite(buf2, 1, tlen, WS->fp);
+	      WS->lastch = buf2[tlen-1];
+	    } else
+	      i = 0;
+	    if (i != tlen) return -1;
+	    return len;		/* Return the incoming length,
 				   NOT the true length! */
-	}
+	  }
 	/* ------------------------------------------------------------ */
 	/* Well, no other processings known.. */
 	if (!mmdf_mode && buf[0] == 'F' && strncmp(buf,"From ",5)==0)
@@ -2641,8 +2669,8 @@ qptext_check(rp)
 {
 	/* "Content-Transfer-Encoding: 8BIT" */
 
-	char **hdrs = *(rp->newmsgheader);
-	char *hdr;
+	const char **hdrs = *((const char ***)rp->newmsgheader);
+	const char *hdr;
 	int cte = 0;
 
 	is_mime = 0;
@@ -2665,14 +2693,14 @@ qptext_check(rp)
 	return cte;
 }
 
-static char *
+static const char *
 find_return_receipt_hdr (rp)
     struct rcpt *rp;
 {
-    char **ptr, *hdr;
+    const char **ptr, *hdr;
 
-    for (ptr = *(rp->newmsgheader); *ptr != NULL; ptr++) {
-        if (strncasecmp(*ptr, "Return-Receipt-To:", 18) == 0)
+    for (ptr = *((const char ***)rp->newmsgheader); *ptr != NULL; ptr++) {
+        if (cistrncmp(*ptr, "Return-Receipt-To:", 18) == 0)
             break;
     }
 
@@ -2686,9 +2714,10 @@ find_return_receipt_hdr (rp)
     return(hdr);
 }
 
+static void encodeXtext __((FILE *, const char *));
 static void encodeXtext(fp,str)
 FILE *fp;
-char *str;
+const char *str;
 {
 	while (*str) {
 	  u_char c = *str;
@@ -2700,11 +2729,11 @@ char *str;
 	}
 }
 
-
+static void decodeXtext __((FILE *, const char *));
 static void
 decodeXtext(fp,xtext)
 	FILE *fp;
-	char *xtext;
+	const char *xtext;
 {
 	for (;*xtext;++xtext) {
 	  if (*xtext == '+') {
@@ -2721,11 +2750,11 @@ decodeXtext(fp,xtext)
 static void
 return_receipt (dp, retrecptaddr, uidstr)
 	struct ctldesc *dp;
-	char *retrecptaddr;
-	char *uidstr;
+	const char *retrecptaddr;
+	const char *uidstr;
 {
 	char buf[BUFSIZ];
-	char **cpp, *mailshare, *mfpath;
+	const char **cpp, *mailshare, *mfpath;
 	FILE *mfp, *efp;
 	struct rcpt *rp;
 	int n;
@@ -2733,9 +2762,9 @@ return_receipt (dp, retrecptaddr, uidstr)
 	struct passwd *pw;
 	int uid;
 	struct stat stb;
-	char *username = "unknown";
+	const char *username = "unknown";
 
-	char *dfltform[] = {
+	const char *dfltform[] = {
 		"Subject: Returned mail: Return receipt",
 		"MIME-Version: 1.0",
 		"Priority: junk",
@@ -2750,7 +2779,8 @@ return_receipt (dp, retrecptaddr, uidstr)
 	if (pw)
 	  username = pw->pw_name;
 
-	if ((mfp = mail_open(MSG_RFC822)) == NULL) {
+	mfp = mail_open(MSG_RFC822);
+	if (mfp == NULL) {
 	  for (rp = dp->recipients; rp != NULL; rp = rp->next)
 	    DIAGNOSTIC(rp, "", EX_TEMPFAIL, "mail_open failure", 0);
 	  warning("Cannot open mail file!");
@@ -2770,15 +2800,18 @@ return_receipt (dp, retrecptaddr, uidstr)
 		     username);
 
 	/* copy error message file itself */
-	if ((mailshare = getzenv("MAILSHARE")) == NULL)
+	mailshare = getzenv("MAILSHARE");
+	if (mailshare == NULL)
 	  mailshare = MAILSHARE;
 
 	mfpath = emalloc(3+strlen(mailshare)+strlen(FORMSDIR)
 				 +strlen(RETURN_RECEIPT_FORM));
-	sprintf(mfpath, "%s/%s/%s", mailshare, FORMSDIR, RETURN_RECEIPT_FORM);
+	sprintf((char*)mfpath, "%s/%s/%s",
+		mailshare, FORMSDIR, RETURN_RECEIPT_FORM);
 
 
-	if ((efp = fopen(mfpath, "r")) != NULL) {
+	efp = fopen(mfpath, "r");
+	if (efp != NULL) {
 	  int inhdr = 1;
 	  buf[sizeof(buf)-1] = 0;
 	  while (fgets(buf,sizeof(buf)-1,efp) != NULL) {
@@ -2800,7 +2833,7 @@ return_receipt (dp, retrecptaddr, uidstr)
 	  } /* ... while() ends.. */
 	  fclose(efp);
 	} else {
-	  for (cpp = &dfltform[0]; *cpp != NULL; ++cpp)
+	  for (cpp = dfltform; *cpp != NULL; ++cpp)
 	    if (*cpp[0] == 0) {
 	      fprintf(mfp, "\tboundary=\"%s\"\n\n\n",boundarystr);
 	      fprintf(mfp, "--%s\n", boundarystr);
