@@ -44,9 +44,9 @@ extern int wait();
 # define WSIGNALSTATUS(s) ((s) & 0177)
 #endif
 
-#ifndef	L_SET
-#define	L_SET	0
-#endif	/* !L_SET */
+#ifndef	SEEK_SET
+#define	SEEK_SET 0
+#endif	/* !SEEK_SET */
 
 /* as in: SKIPWHILE(isascii,cp) */
 #define	SKIPWHILE(X,Y)	while (*Y != '\0' && isascii(*Y) && X(*Y)) { ++Y; }
@@ -60,7 +60,9 @@ extern int wait();
 char uucpname[MAXHOSTNAMELEN+1];
 
 char	*progname;
+#ifndef HAVE_MMAP
 int	readalready = 0;	/* does buffer contain valid message data? */
+#endif
 int	mimeqpnarrow = 0;	/* Can't send TAB thru without MIME-QP */
 FILE	*verboselog = NULL;
 FILE	*logfp   = NULL;
@@ -84,7 +86,7 @@ extern void prversion();
 extern void process __((struct ctldesc *dp, struct maildesc *mp, FILE *verboselog));
 extern void deliver __((struct ctldesc *dp, struct maildesc *mp, struct rcpt *startrp, struct rcpt *endrp, FILE *verboselog));
 extern int appendlet __((struct ctldesc *dp, struct maildesc *mp, FILE *fp, FILE *verboselog, int convertmode));
-extern int writebuf __((struct maildesc *mp, FILE *fp, char *buf, int len));
+extern int writebuf __((struct maildesc *mp, FILE *fp, const char *buf, int len));
 extern time_t time();
 #ifndef strchr
 extern char *strchr(), *strrchr();
@@ -99,6 +101,8 @@ struct maildesc {
 	char	*argv[20];
 };
 
+extern int writemimeline __(( struct maildesc *mp, FILE *fp, const char *buf, int len, int convertmode));
+
 #define	MO_FFROMFLAG		0x00001
 #define	MO_RFROMFLAG		0x00002
 #define	MO_NORESETUID		0x00004
@@ -119,10 +123,10 @@ struct maildesc {
 
 struct exmapinfo {
 	int	origstatus;
-	char	*statusmsg;
+	const char *statusmsg;
 	int	newstatus;
-	char	*dsnstatus;
-	char	*dsndiags;
+	const char *dsnstatus;
+	const char *dsndiags;
 } exmap[] = {
   { EX_USAGE,	"command line usage error",	EX_TEMPFAIL,	"5.3.0", "x-local; 500 (Command line usage error)"	},
   { EX_DATAERR,	"data format error",		EX_DATAERR,	"5.3.0", "x-local; 500 (Data format error)"	},
@@ -187,7 +191,11 @@ main(argc, argv)
 	host = channel = NULL;
 	myhostname[0] = 0;
 	cf = NULL;
-	while ((c = getopt(argc, argv, "f:c:h:HQvVw:8")) != EOF) {
+	while (1) {
+	  c = getopt(argc, argv, "f:c:h:HQvVw:8");
+	  if (c == EOF)
+	    break;
+
 	  switch (c) {
 	  case 'f':
 	    cf = optarg;
@@ -302,7 +310,9 @@ process(dp, mp, verboselog)
 {
 	struct rcpt *rp, *rphead;
 
+#ifndef HAVE_MMAP
 	readalready = 0; /* ignore any previous message data cache */
+#endif
 
 	if (mp->flags & MO_MANYUSERS) {
 	  for (rp = rphead = dp->recipients; rp != NULL; rp = rp->next) {
@@ -334,21 +344,22 @@ deliver(dp, mp, startrp, endrp, verboselog)
 {
 	struct rcpt *rp;
 	struct exmapinfo *exp;
-	char *exs, *exd;
+	const char *exs, *exd;
 	int i, j, pid, in[2], out[2];
 	unsigned int avsize;
 	FILE *tafp, *errfp;
-	char *cp, *ds, *s, buf[BUFSIZ], buf2[BUFSIZ];
-	char **av;
-	int	status;
+	char *cp, buf[BUFSIZ], buf2[BUFSIZ];
+	char *ws;
+	const char *ds, **av, *s;
+	int status;
 	int content_kind, conversion_prohibited, convertmode, ascii_clean = 0;
 
-	if (lseek(dp->msgfd, (off_t)(dp->msgbodyoffset), L_SET) < 0L)
+	if (lseek(dp->msgfd, (off_t)(dp->msgbodyoffset), SEEK_SET) < 0L)
 		warning("Cannot seek to message body! (%m)", (char *)NULL);
 
 	i = 0;
 	avsize = 5;
-	av = (char **)emalloc(sizeof av[0] * avsize);
+	av = (const char **)emalloc(sizeof av[0] * avsize);
 	av[i++] = mp->argv[0];
 	if (mp->flags & MO_FFROMFLAG) {
 	  av[i++] = "-f";
@@ -366,8 +377,8 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	for (j = 1; mp->argv[j] != NULL; ++j) {
 	  while (i+2 >= avsize) {
 	    avsize *= 2;
-	    av = (char **)erealloc((char *)av,
-				   sizeof av[0] * avsize);
+	    av = (const char **)erealloc((char *)av,
+					 sizeof av[0] * avsize);
 	  }
 	  if (strchr(mp->argv[j], '$') == 0) {
 	    av[i++] = mp->argv[j];
@@ -377,10 +388,10 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  do {
 	    while (i+2 >= avsize) {
 	      avsize <<= 1;
-	      av = (char **)erealloc((char *)av,
-				     sizeof av[0] * avsize);
+	      av = (const char **)erealloc((char *)av,
+					   sizeof av[0] * avsize);
 	    }
-	    for (cp = mp->argv[j], s = buf; *cp != '\0'; ++cp) {
+	    for (cp = mp->argv[j], ws = buf; *cp != '\0'; ++cp) {
 	      if (*cp == '$') {
 		switch (*++cp) {
 		case 'g':
@@ -395,9 +406,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 		  break;
 		case 'U':
 		  strcpy(buf2, rp->addr->user);
-		  for (ds = buf2; *ds != 0; ++ds)
-		    if (isupper(*ds))
-		      *ds = tolower(*ds);
+		  strlower(buf2);
 		  ds = buf2;
 		  rp = rp->next;
 		  break;
@@ -413,16 +422,16 @@ deliver(dp, mp, startrp, endrp, verboselog)
 			  *cp, dp->msgfile);
 		  warning(msg, mp->name);
 		} else {
-		  strcpy(s, ds);
-		  s += strlen(s);
+		  strcpy(ws, ds);
+		  ws += strlen(ds);
 		}
 	      } else
-		*s++ = *cp;
+		*ws++ = *cp;
 	    }
-	    *s = '\0';
+	    *ws = '\0';
 	    av[i] = emalloc((u_int)(strlen(buf)+1));
 	    /* not worth freeing this stuff */
-	    strcpy(av[i], buf);
+	    strcpy((char*)av[i], buf);
 	    ++i;
 	  } while (rp != startrp && rp != endrp);
 	}
@@ -475,7 +484,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	    dup2(in[1], 2);
 	    close(in[1]);
 	  }
-	  execv(mp->command, av);
+	  execv(mp->command, (char**) av);
 	  _exit(254);
 	} else if (pid < 0) {	/* couldn't fork, complain */
 	  for (rp = startrp; rp != endrp; rp = rp->next) {
@@ -553,7 +562,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	if (mp->flags & (MO_UNIXFROM|MO_REMOTEFROM)) {
 	  char *timestring;
 	  time_t now;
-	  char *uu = startrp->addr->link->user;
+	  const char *uu = startrp->addr->link->user;
 
 	  now = time((time_t *)0);
 	  timestring = ctime(&now);
@@ -628,7 +637,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	   exist.. */
 	if (mp->flags & MO_RETURNPATH) {
 	  char **hdrs = has_header(startrp,"Return-Path:");
-	  char *uu = startrp->addr->link->user;
+	  const char *uu = startrp->addr->link->user;
 	  if (hdrs) delete_header(startrp, hdrs);
 	  if (strcmp(startrp->addr->link->channel,"error")==0)
 	    uu = "";
@@ -661,9 +670,9 @@ deliver(dp, mp, startrp, endrp, verboselog)
 
 	if (mp->flags & MO_BSMTP) {
 	  if (mp->flags & MO_CRLF)
-	    fprintf(tafp,".\r\nQUIT\r\n");
+	    fprintf(tafp,".\r\n");
 	  else
-	    fprintf(tafp,".\nQUIT\n");
+	    fprintf(tafp,".\n");
 	}
 	
 	fclose(tafp);
@@ -748,9 +757,9 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	 */
 
 	register int i;
-	register int bufferfull;
 	int lastch;
 #ifndef HAVE_MMAP
+	register int bufferfull;
 	int mfd = dp->msgfd;
 #endif
 
@@ -769,14 +778,15 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	}
 #endif
 
-	/* we are assumed to be positioned properly at start of message body */
-	bufferfull = 0;
 	lastch = -1;
 	if (convertmode == _CONVERT_NONE) {
 #ifndef	HAVE_MMAP
+	  bufferfull = 0;
+	  readalready = 0;
+	  lseek(mfd, dp->msgbodyoffset, SEEK_SET);
 	  while ((i = read(mfd, let_buffer, sizeof let_buffer)) != 0) {
 #else
-	    char *let_buffer = dp->let_buffer + dp->msgbodyoffset;
+	    const char *let_buffer = dp->let_buffer + dp->msgbodyoffset;
 	    i = dp->let_end - (dp->let_buffer + dp->msgbodyoffset);
 #endif
 	    if (i < 0)
@@ -784,13 +794,10 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	    lastch = let_buffer[i-1];
 	    if (writebuf(mp, fp, let_buffer, i) != i)
 	      return EX_IOERR;
+#ifndef	HAVE_MMAP
 	    readalready = i;
 	    bufferfull++;
-#ifndef	HAVE_MMAP
 	  }
-#endif
-
-#ifndef HAVE_MMAP
 	  if (bufferfull > 1)	/* not all in memory, need to reread */
 	    readalready = 0;
 #endif
@@ -804,7 +811,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  char iobuf[BUFSIZ];
 	  FILE *mfp = fdopen(mfd,"r");
 
-	  fseek(mfp,(off_t)(dp->msgbodyoffset),L_SET);
+	  fseek(mfp,(off_t)(dp->msgbodyoffset),SEEK_SET);
 
 	  setvbuf(mfp, iobuf, _IOFBF, sizeof(iobuf));
 
@@ -813,9 +820,9 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  readalready = 0;
 #else
 #define MFPCLOSE
-	  char *s = dp->let_buffer + dp->msgbodyoffset;
+	  const char *s = dp->let_buffer + dp->msgbodyoffset;
 #endif
-	  writemimeline(mp, fp, (char *)NULL, 0, 0, NULL);
+	  writemimeline(mp, fp, (char *)NULL, 0, 0);
 
 	  /*
 	     if(verboselog) fprintf(verboselog,
@@ -826,7 +833,6 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 
 	  /* we are assuming to be positioned properly
 	     at the start of the message body */
-	  bufferfull = 0;
 	  lastch = -1;
 	  i = 0;
 
@@ -835,7 +841,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	    if ((i = cfgets(let_buffer, sizeof(let_buffer), mfp)) == EOF)
 	      break;
 #else
-	    char *let_buffer = s, *s2 = s;
+	    const char *let_buffer = s, *s2 = s;
 	    i = 0;
 	    if (s >= dp->let_end) break;	/* "EOF" */
 	    while (s2 < dp->let_end && *s2 != '\n')
@@ -847,8 +853,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	    /* It MAY be malformed -- if it has a BUFSIZ length
 	       line in it, IT CAN'T BE MIME  :-/		*/
 	    /* Ok, write the line */
-	    if (writemimeline(mp, fp, let_buffer, i,
-			      convertmode, &lastch) != i) {
+	    if (writemimeline(mp, fp, let_buffer, i, convertmode) != i) {
 	      return EX_IOERR;
 	    }
 	  }
@@ -877,10 +882,10 @@ int
 writebuf(mp, fp, buf, len)
 	struct maildesc *mp;
 	FILE *fp;
-	char *buf;
+	const char *buf;
 	int len;
 {
-	register char *cp;
+	register const char *cp;
 	register int n;
 	int tlen;
 	register char expect;
@@ -895,13 +900,10 @@ writebuf(mp, fp, buf, len)
 	  return 0;
 	}
 	expect = save;
-	if ((mp->flags & MO_STRIPHIBIT)) {
-	  for (cp = buf, n = len; n > 0; --n, ++cp)
-	    if (*cp & 0200)
-	      *cp &= 0177;
-	}
 	for (cp = buf, n = len, tlen = 0; n > 0; --n, ++cp) {
-	  int c = *(unsigned char *)cp;
+	  int c = (*cp) & 0xFF;
+	  if (mp->flags & MO_STRIPHIBIT)
+	    c &= 0x7F;
 	  ++tlen;
 	  if (c == '\n') {
 	    frombuf[0] = 0;
@@ -969,11 +971,11 @@ writebuf(mp, fp, buf, len)
 }
 
 int
-writemimeline(mp, fp, buf, len, convertmode, lastchp)
+writemimeline(mp, fp, buf, len, convertmode)
 	struct maildesc *mp;
 	FILE *fp;
-	char *buf;
-	int len, convertmode, *lastchp;
+	const char *buf;
+	int len, convertmode;
 {
 	/* `convertmode' controls the behaviour of the message conversion:
 	     _CONVERT_NONE (0): send as is
@@ -981,7 +983,7 @@ writemimeline(mp, fp, buf, len, convertmode, lastchp)
 	     _CONVERT_8BIT (2): Convert QP-encoded chars to 8-bit
 	     _CONVERT_UNKNOWN (3): Turn message to charset=UNKNOWN-8BIT, Q-P..
 	 */
-	register char *cp;
+	register const char *cp;
 	register int n;
 	static int  column;
 	register int qp_conv;
@@ -996,7 +998,7 @@ writemimeline(mp, fp, buf, len, convertmode, lastchp)
 		   convertmode == _CONVERT_UNKNOWN);
 
 	for (cp = buf, n = len; n > 0; --n, ++cp) {
-	  int c = *(unsigned char *)cp;
+	  int c = (*cp) & 0xFF;
 	  ++column;
 
 	  if (qp_conv) {
@@ -1237,7 +1239,7 @@ struct ctldesc *dp;
 {
 #ifdef	HAVE_MMAP
 	/* With MMAP()ed spool file it is sweet and simple.. */
-	register char *s = dp->let_buffer + dp->msgbodyoffset;
+	register const char *s = dp->let_buffer + dp->msgbodyoffset;
 	while (s < dp->let_end)
 	  if (128 & *s) {
     /*
@@ -1270,21 +1272,26 @@ struct ctldesc *dp;
 	while ((i = read(mfd, let_buffer, sizeof let_buffer)) != 0) {
 	  if (i < 0) {
 	    /* ERROR ?!?!? */
-	    if (bufferfull > 1) readalready = 0;
+	    if (errno == EINTR)
+	      continue;
+	    readalready = 0;
+	    lseek(mfd,dp->msgbodyoffset,0);
 	    return 0;
 	  }
 	  lastwasnl = (let_buffer[i-1] == '\n');
 	  readalready = i;
 	  bufferfull++;
-	  for (i=0; i<readalready; ++i)
+	  for (i=0; i < readalready; ++i)
 	    if (128 & (let_buffer[i])) {
-	      lseek(mfd,mfd_pos,0);
-	      if (bufferfull > 1) readalready = 0;
+	      lseek(mfd,dp->msgbodyoffset,0);
+	      /* We propably have not read everything of the file! */
+	      readalready = 0;
 	      return 0;		/* Not clean ! */
 	    }
 	}
 	/* Got to EOF, and still it is clean 7-BIT! */
 	lseek(mfd,dp->msgbodyoffset,0);
+
 	if (bufferfull > 1)	/* not all in memory, need to reread */
 	  readalready = 0;
 
