@@ -12,9 +12,8 @@ OUTFILE=$3
 
 if [ ! -f "$INFILE" ]; then exit 1; fi
 
-cat $INFILE |			\
 (
 	echo "# generated script from '$INFILE' into '$OUTFILE' at `date`"
 	echo "relation -lt incore -d pathalias -s 0 $DBASE"
 	awk '/^[^#]/{printf "db add '$DBASE' '\''%s'\'' '\''%s'\''\n",$1,$2}' 
-) > $OUTFILE
+) < $INFILE > $OUTFILE
